// Grand Roads
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// RobFoxroads stuff
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////

#define MAP_WIDTH 200
#define MAP_HEIGHT 200

#define m(x,y) a[x+wide*y]

#define WIDTH 2.5    // one half the width of a road

////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// Bridge Stuff
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

struct VEC3D
{

	float
		x,
		y,
		z;
};

typedef struct VEC3D vec3d;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct CONTOUR_PATH
{

	int
		from,
		to,
		type,
		count;

	float
		xmin,
		xmax,
		zmin,
		zmax;

	vec3d
		*points;
};

typedef struct CONTOUR_PATH contour_path;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct BRIDGE
{
	vec3d
		end1,
		end2;

	int
		road,
		start_node,
		end_node,
		type;

	float
		max_leg_height;
};

typedef struct BRIDGE bridge;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////


/*
struct BRIDGE_TEXT_DATA
{
	vec3d
		end1,
		end2;

	int
		start_node,
		end_node,
		type;

	float
		max_leg_height;
};

typedef struct BRIDGE_TEXT_DATA bridge_text_data;
*/


////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// terrain data structures
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

#define TERRAIN_3D_Y_SCALE ( 1.0 / 32767.0 );


struct TERRAIN_3D_POINT_XZ
{

	short int
		x,
		z;
};

typedef struct TERRAIN_3D_POINT_XZ terrain_3d_point_xz;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct FLOATING_POINT
{
	float
		x,
		z;
};

typedef struct FLOATING_POINT floating_point;


///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

#pragma pack(1)

struct TERRAIN_3D_POINT_Y
{

	union
	{

		struct
		{

			unsigned short int
				y:15,
				normal_change:1;
		};

		unsigned short int
			whole_value;

		short int
			whole_signed_value;
	};
};

typedef struct TERRAIN_3D_POINT_Y terrain_3d_point_y;


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// my stuff for road polys
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////

struct SECTOR_POINT					
{
	terrain_3d_point_xz		xz_point;

	terrain_3d_point_y		y_point;
};

typedef struct SECTOR_POINT sector_point;

//////////////////////////////////////////////////////////////////

struct POINT
{

	int
		x_sector, // this is the sector it's in
		z_sector,
		sec_x,    // this is the coordinates in the sector
		sec_z,
		sec_y,
		polygon_number; // this is the polygon # it's on

	double
		global_x,
		global_z,
		global_y;
	
};

typedef struct POINT point;

//////////////////////////////////////////////////////////////////////

struct SIMPLE_POINT
{
	//////////////////////////////////////////////////////////////////
	// the letter p-s is the same as the quad structure points p-s
	// the letter a-d is the sector corner points starting with 
	//	  a = the lower left and going counter clockwise
	// the letter R is for intersections with sector boundaries and the
	//    right side of the road
	// the letter L is for intersections with sector boundaries and the
	//    left side of the road
	// the letter e is for intersections with sector boundaries and the
	//    cross section line at the a end of the quad
	// the letter f is for intersections with sector boundaries and the
	//    cross section line at the b end of the quad
	// the letter j is for the three or four corner points of road junctions
	///////////////////////////////////////////////////////////////////
	char
		letter;

	int
		road,
		quad;

	double
		x,
		z,
		y;

	sector_point
		sec_pt;
};

typedef struct SIMPLE_POINT simple_point;

//////////////////////////////////////////////////////////////////////

struct LINE
{
	point
		point_a,
		point_b;
};

typedef struct LINE line;

//////////////////////////////////////////////////////////////////////

struct QUAD
{
	point
		p,
		q,
		r,
		s;
};

typedef struct QUAD quad;

///////////////////////////////////////////////////////////////////////

struct ROAD
{
	int
		number_of_center_points,
		number_of_quads;

	point
		center_points[1024];

	//quad
		//quads[512];
};

typedef struct ROAD road;

////////////////////////////////////////////////////////////////////////
// these are the road polys after they have been split at
// the sector boundaries
////////////////////////////////////////////////////////////////////////

typedef struct ROAD_POLY	
{
	int number_of_points;				

	simple_point	point_list[6];
};

typedef struct ROAD_POLY road_poly;

/////////////////////////////////////////////////////////////////////////
// these are the road polys after they have been split at
// the sector boundaries
/////////////////////////////////////////////////////////////////////////

typedef struct ROAD_POLY_LIST_PER_SECTOR 
{
	int number_of_polys;

	road_poly	poly_list[32];
};

typedef struct ROAD_POLY_LIST_PER_SECTOR road_poly_list_per_sector;

/////////////////////////////////////////////////////////////////////////
// this struct is used for fixing the road junctions
// by adding another polygon that is flat where the roads
// meet and adjusting the intersecting road's joining points
/////////////////////////////////////////////////////////////////////////

typedef struct JUNCTION
{
	int road_1,
		road_2,
		road_3,
		road_1_center_point, // these are the point numbers
		road_2_center_point,
		road_3_center_point,
		x_sector,
		z_sector,
		is_crossroads, // boolean, true is if 3 roads have a common point
		from_left;		// boolean, true if road 2 comes from the left side of road 1

	//////////////////////////////////////////////////
	// L0 and L1 are the intersection points for
	// junctions from the left
	// R0 and R1 are the intersection points for
	// junctions from the right
	//////////////////////////////////////////////////

	simple_point	intersect_L0,
					intersect_L1,
					intersect_R0,
					intersect_R1;
};

typedef struct JUNCTION junction;

////////////////////////////////////////////////////

typedef struct SECTOR
{
	int x,
		z;
};

typedef struct SECTOR sector;


/////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// terrain data structures
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR
{

	unsigned char
		red,
		green,
		blue;
};

typedef struct TERRAIN_3D_COLOUR terrain_3d_colour;

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_COLOUR_INDEX terrain_3d_colour_index;


#pragma pack(4)

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_WORD_REFERENCE
{

	unsigned short int
		point;
};

typedef struct TERRAIN_3D_POINT_WORD_REFERENCE terrain_3d_point_word_reference;

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_BYTE_REFERENCE
{

	unsigned char
		point;
};

typedef struct TERRAIN_3D_POINT_BYTE_REFERENCE terrain_3d_point_byte_reference;

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_FACE
{

	unsigned char
		number_of_points:7,
		surface_change:1;
};

typedef struct TERRAIN_3D_FACE terrain_3d_face;

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

struct TERRAIN_3D_SURFACE
{

	unsigned char
		surface_id;
};

typedef struct TERRAIN_3D_SURFACE terrain_3d_surface;

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

struct TERRAIN_3D_SECTOR
{

	unsigned short int
		number_of_points,
		number_of_polygons,
		radius,
		normal_changes_word;

	short int
		minimum_height,
		maximum_height,
		average_height,
		pad_byte,
		number_of_surface_ids;

	int
		colour_indices_offset,
		point_colours_offset,
		max_colour_index;

	struct TERRAIN_3D_POINT_XZ
		points_xz[1024];

	struct TERRAIN_3D_POINT_Y
		points_y[1024];

	struct TERRAIN_3D_FACE
		polygons[1024];

	union
	{

		struct TERRAIN_3D_POINT_WORD_REFERENCE
			point_word_references[1024][3];

		struct TERRAIN_3D_POINT_BYTE_REFERENCE
			point_byte_references[1024][3];
	};


	struct TERRAIN_3D_SURFACE
		surface_ids[256];

	struct TERRAIN_3D_SURFACE
		surface_id_each_face[1024];

	struct TERRAIN_3D_COLOUR
		point_colours[128];

	struct TERRAIN_3D_COLOUR_INDEX
		colour_indices[1024];
};

typedef struct TERRAIN_3D_SECTOR terrain_3d_sector;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define TERRAIN_3D_MAX_TREE_HEIGHT	(35)


struct TERRAIN_3D_TREE_DATA
{

	signed short int
		x,
		z;

	unsigned short int
		y;

	unsigned char
		colour;

	unsigned char
		height:7,
		type:1;
};

typedef struct TERRAIN_3D_TREE_DATA terrain_3d_tree_data;

///////////////////////////////////////////////////////////////////
// for point in poly function
///////////////////////////////////////////////////////////////////

#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define INSIDE 1
#define OUTSIDE 0

typedef struct {
   double x,y;
} Point;



