#include <stdio.h>
#include <stdlib.h>
#include <math.h>

//#include "terrain.h"
//#include "misc.h"

#include "export_roads.h"


terrain_3d_sector
	**terrain_sectors;

int
	colour_file_offset_counter;
	terrain_3d_map_width,
	terrain_3d_map_height,
	global_number_of_road_sectors,
	sectors_for_sector_road_poly_list [ 1024 ][ 2 ];   // [][0] = x    [][1] = z

float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;

road_poly_list_per_sector
		sector_road_poly_list[ 1024 ];


///////////////////////////////
// Functions
///////////////////////////////

void save_terrain_sector_as_lw_3d_object ( int x, int z, const char *filename );

//void load_3d_terrain (void);

void allocate_memory_for_sectors (void);

void iff_short_int_write ( short int value, FILE *fp );

void iff_int_write ( int value, FILE *fp );

void iff_float_write ( float value, FILE *fp );

void set_terrain_rendering_routines ( terrain_types type, terrain_polygon_type polygon_type);
													//void ( *render_word_clipped ) ( int ),
													//void ( *render_byte_clipped ) ( int ),
													//void ( *render_word_unclipped ) ( int ),
													//void ( *render_byte_unclipped ) ( int ) );

void initialize_terrain_type_information(void);

void print_sector_data(int sec_z, int sec_x);

void set_terrain_type_textures ( terrain_types type, int red, int green, int blue );

void load_road_polys_binary (void);

void format_road_poly_data_into_eech_structure (void);

road_poly	get_clockwise_ordered_poly(road_poly	in_poly);

void swap(simple_point *array, int a, int b);

int	isLeft( simple_point P0, simple_point P1, simple_point P2 );

void export_road_sectors_to_lw_objs (void);



int main(int argc, char ** argv)
{
	int
		x,
		z;

	char
		filename [16];

	//load_3d_terrain ();

	//print_sector_data(10, 10);

	initialize_terrain_type_information();

	//sprintf(filename, "%03dx%03d.lwo", 10, 10); 

	//save_terrain_sector_as_lw_3d_object ( 10, 10, filename );

	//for(x = 0; x < terrain_3d_map_width; x ++){
	//	for(z = 0; z < terrain_3d_map_height; z ++){

	//		sprintf(filename, "%03dx%03d.lwo", x, z); 

	//		save_terrain_sector_as_lw_3d_object ( x, z, filename );

	//	}// end for z loop
	//} // end for x loop


	load_road_polys_binary ();

	allocate_memory_for_sectors ();

	format_road_poly_data_into_eech_structure ();

	//print_sector_data (15, 84);

	export_road_sectors_to_lw_objs ();

	return 0;
} // end main

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		terrain_version,
		x,
		z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		rgb_4byte_header,
		ffp_4byte_header,
		four_byte_zero_word,
		i,
		print_loop;

	float
		min_height,
		max_height,
		terrain_3d_map_scaled_height_difference;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius,
		normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height,
		pad_word_zero;

	unsigned char
		pad_byte_zero,
		normal_changes_buf[1024];
	
	long
		current_file_position;

	////////////////////////////////////////////////////////////////////////////
	// initialize variables
	////////////////////////////////////////////////////////////////////////////

	colour_file_offset_counter = 0;

	

	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(z = 0; z < map_height; z ++){

		if( ! (terrain_sectors[z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( z=0 ; z < map_height ; z++){
		for( x=0 ; x < map_width ; x++){
		//printf("sector %d x %d\n", x, z);	
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			printf("\nSector %d x %d\n\n", z, x);

			fread(&terrain_sectors[z][x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			printf("%d points\n", terrain_sectors[z][x].number_of_points);
		
			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[z][x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			printf("%d polygons\n", terrain_sectors[z][x].number_of_polygons);

			fread(&terrain_sectors[z][x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[z][x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[z][x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[z][x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n",z,x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[z][x].points_y );

			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[z][x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[z][x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[z][x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[z][x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n",z,x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[z][x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[z][x].number_of_points, ffp_in);

			///////////////////////////////////////////////////////////////////////
			
					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[z][x].number_of_points <= 256){

				if(!(terrain_sectors[z][x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n",z,x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[z][x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[z][x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[z][x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n",z,x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[z][x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[z][x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[z][x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n",z,x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			printf("size of polygon chunk = %d\n",
				surface_changes_offset - polygons_offset);

			fread(terrain_sectors[z][x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[z][x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[z][x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[z][x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface changes - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[z][x].surface_changes = 
				malloc(normal_changes_offset - surface_changes_offset))){
				printf("No memory for surface_changes sector %d x %d\n",z,x);
				exit(1);
			}

			terrain_sectors[z][x].number_of_surface_changes_bytes = 
				normal_changes_offset - surface_changes_offset;

			printf("%d surface changes bytes\n",
				terrain_sectors[z][x].number_of_surface_changes_bytes);

			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[z][x].surface_changes, sizeof(terrain_3d_surface),
				terrain_sectors[z][x].number_of_surface_changes_bytes, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			if((z == map_height - 1) && (x == map_width - 1)){

				// this method for when data is the last chunk of the ffp file

				i = 0;

				//printf("normal changes fp = %d\n", (int)ftell(ffp_in));

				while(!feof(ffp_in)){

					//fread((normal_changes_buf + i), sizeof(unsigned char), 1, ffp_in);

					normal_changes_buf[i] = getc(ffp_in);

					i++;

					//printf("i = %d		%d\n", i,normal_changes_buf[i]);
					//if(i > 50)exit(1);

				} // end while

				terrain_sectors[z][x].number_of_normal_changes = i;

				if(!(terrain_sectors[z][x].normal_changes = malloc(i))){
					printf("No memory for normal_changes sector %d x %d\n",z,x);
					exit(1);
				}

				for(i = 0; i < terrain_sectors[z][x].number_of_normal_changes; i++){
					*(terrain_sectors[z][x].normal_changes + i) = normal_changes_buf[i];
				}

			} // end if
			else{

				// this method for when data is NOT the last chunk of the ffp file

				// need to get the next ffp file offset for the size 
				// of normal changes data calculation.
				// this doesn't work for original maps because they may jump back in the ffp
				// file to re-use a previous sector that I assume must be identical

				fseek(sec_in, 24, SEEK_CUR);
				fread(&offset, sizeof(int), 1, sec_in); // sector file
				fseek(sec_in, -28, SEEK_CUR);

				//printf("\nSector %d x %d\n\n", z, x);

				terrain_sectors[z][x].number_of_normal_changes = 
					offset - normal_changes_offset;

				//printf("normal changes fp = %d\n", (int)ftell(ffp_in));

				//printf("next points_y offset = %d\n", offset);

				printf("number_of_normal_changes = %d\n", 
					terrain_sectors[z][x].number_of_normal_changes); 

				if(!(terrain_sectors[z][x].normal_changes = 
					malloc(terrain_sectors[z][x].number_of_normal_changes))){
					printf("No memory for normal_changes sector %d x %d\n",z,x);
					exit(1);
				}

				
				fread(terrain_sectors[z][x].normal_changes,	
					sizeof(*terrain_sectors[z][x].normal_changes),
					terrain_sectors[z][x].number_of_normal_changes, ffp_in);

			} // end else

			// read in color offsets from sector file

			fread(&terrain_sectors[z][x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[z][x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[z][x].colour_indices = malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[z][x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n",z,x);
				exit(1);
			}

			fread(terrain_sectors[z][x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[z][x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[z][x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[z][x].number_of_points; i++){ 
				
				// there's one colour index per point

				if((terrain_sectors[z][x].colour_indices + i)->index > 
					terrain_sectors[z][x].max_colour_index){

					terrain_sectors[z][x].max_colour_index = 
						(terrain_sectors[z][x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[z][x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[z][x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void save_terrain_sector_as_lw_3d_object ( int x, int z, const char *filename )
{

	FILE
		*fp;

	char
		id[8];

	terrain_3d_sector
		*sector;

	int
		count,
		size,
		number_of_triangles,
		number_of_point_references;

	terrain_3d_face
		*polygon;

	terrain_3d_point_xz
		*points_xz;

	terrain_3d_point_y
		*points_y;

	
	if(( fp = fopen(filename,"wb"))==NULL){
		printf("cannot open %s\n", filename);
		exit(1);
	}
	else printf("%s opened ...\n", filename );

	sector = &terrain_sectors[z][x];

	points_y = sector->points_y;

	//points_xz = ( terrain_3d_point_xz * ) points_y + sector->number_of_points;

	points_xz = sector->points_xz; // Craig changed from line above

	fwrite ( "FORM", 4, 1, fp );

	//
	// Work out the size of the file.
	//

	//
	//
	//
	// LWOB identifier
	//			4 bytes
	//
	// Surface names:
	//			8 bytes + 8 * number of surfaces
	//
	// Surface descriptions:
	//			26 * number of surfaces
	//
	// Points:
	//			8 bytes + 12 * number of points
	//
	// Faces:
	//			8 bytes + 4 * number of faces + 2 * number of point references
	//
	// Total:
	//			28 + 12 * Points + 4 * faces + 2 * point refs
	//

	polygon = sector->polygons;

	number_of_point_references = 0;

	number_of_triangles = 0;

	
	
	for ( count = 0; count < sector->number_of_polygons; count++ )
	{
		int
			number_of_polygon_triangles;

		number_of_polygon_triangles = ( polygon->number_of_points - 2 );

		number_of_triangles += number_of_polygon_triangles;

		number_of_point_references += 3 * number_of_polygon_triangles;

		polygon++;
	}

	size = 28 +
			( 26 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 8 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 12 * sector->number_of_points ) +
			( 4 * number_of_triangles ) +
			( 2 * number_of_point_references );

	iff_int_write ( size, fp );

	fwrite ( "LWOB", 4, 1, fp );

	//
	// Detail the surface names
	//

	fwrite ( "SRFS", 4, 1, fp );

	iff_int_write ( ( ( TERRAIN_TYPE_LAST -1 ) * 8 ), fp );

	for ( count = 1; count < TERRAIN_TYPE_LAST; count++ )
	{

		sprintf ( id, "%07d", count );

		//printf("%07d", count ); // Craig

		fwrite ( id, 8, 1, fp );
	}

	//
	// Describe the 3d points
	//

	fwrite ( "PNTS", 4, 1, fp );

	iff_int_write ( ( 12 * sector->number_of_points ), fp );

	for ( count = 0; count < sector->number_of_points; count++ )
	{

		float
			x,
			y,
			z;

		x =  points_xz[count].x;
		y =  points_y[count].y;
		z =  points_xz[count].z;
/*
		x *= ( 2048.0 / 65535.0 );
		z *= ( 2048.0 / 65535.0 );
		y = ( sector->maximum_height - sector->minimum_height );

		if ( y == 0 )
		{

			y = sector->maximum_height;
		}
		else
		{

			y = sector->minimum_height + ( ( float ) points_y[count].y / 32767.0 ) * y;
		}
*/

		iff_float_write ( x, fp );
		iff_float_write ( y, fp );
		iff_float_write ( z, fp );

		//printf("%d  %f  %f  %f\n", count,  x, z, y); // Craig
	}

	//
	// Describe the polygons
	//

	{

		int
			points_offset;

		terrain_3d_surface
			*surface;

		fwrite ( "POLS", 4, 1, fp );

		iff_int_write ( ( ( 4 * number_of_triangles ) + ( 2 * number_of_point_references ) ), fp );

		points_offset = 0;

		polygon = sector->polygons;

		surface = sector->surface_changes;

		for ( count = 0; count < sector->number_of_polygons; count++ )
		{

			int
				point_count;

			if ( polygon->surface_change )
			{

				surface++;
			}

			if ( terrain_type_information[surface->surface_id].polygon_type == POLYGON_TYPE_FAN )
			{

				if ( sector->number_of_points < 257 )
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						iff_short_int_write ( 3, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset].point, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
						iff_short_int_write ( ( short int ) surface->surface_id, fp );
					}
				}
				else
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						iff_short_int_write ( 3, fp );
						iff_short_int_write ( sector->point_word_references[points_offset].point, fp );
						iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
						iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
						iff_short_int_write ( ( short int ) surface->surface_id, fp );
					}
				}
			}
			else
			{

				if ( sector->number_of_points < 257 )
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						if ( point_count & 1 )
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
						else
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
					}
				}
				else
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						if ( point_count & 1 )
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
						else
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
					}
				}
			}



			points_offset += polygon->number_of_points;

			polygon++;
		}
	}

	//
	// Describe the surface colours
	//

	for ( count = 1; count < TERRAIN_TYPE_LAST; count++ )
	{

		unsigned char
			value;

		fwrite ( "SURF", 4, 1, fp );

		iff_int_write ( 18, fp );

		sprintf ( id, "%07d", count );

		fwrite ( id, 8, 1, fp );

		fwrite ( "COLR", 4, 1, fp );

		iff_short_int_write ( 4, fp );

		value = terrain_type_information[count].red;
		fwrite ( &value, 1, 1, fp );

		value = terrain_type_information[count].green;
		fwrite ( &value, 1, 1, fp );

		value = terrain_type_information[count].blue;
		fwrite ( &value, 1, 1, fp );

		value = 0;
		fwrite ( &value, 1, 1, fp );
	}

	fclose ( fp );




} // end save_terrain_sector_as_lw_3d_object

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void iff_float_write ( float value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[4];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[3];
	buffer[1] = ptr[2];
	buffer[2] = ptr[1];
	buffer[3] = ptr[0];

	fwrite ( buffer, 4, 1, fp );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void iff_int_write ( int value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[4];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[3];
	buffer[1] = ptr[2];
	buffer[2] = ptr[1];
	buffer[3] = ptr[0];

	fwrite ( buffer, 4, 1, fp );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void iff_short_int_write ( short int value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[2];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[1];
	buffer[1] = ptr[0];

	fwrite ( buffer, 2, 1, fp );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////


void set_terrain_rendering_routines ( terrain_types type, terrain_polygon_type polygon_type)
													//void ( *render_word_clipped ) ( int ),
													//void ( *render_byte_clipped ) ( int ),
													//void ( *render_word_unclipped ) ( int ),
													//void ( *render_byte_unclipped ) ( int ) )
{

	terrain_type_information[type].initialised = TRUE;

	terrain_type_information[type].polygon_type = polygon_type;

	//terrain_type_information[type].render_word_clipped = render_word_clipped;
	//terrain_type_information[type].render_byte_clipped = render_byte_clipped;
	//terrain_type_information[type].render_word_unclipped = render_word_unclipped;
	//terrain_type_information[type].render_byte_unclipped = render_byte_unclipped;
}

void initialize_terrain_type_information(void){

	set_terrain_rendering_routines ( TERRAIN_TYPE_SEA, POLYGON_TYPE_FAN );

   set_terrain_rendering_routines ( TERRAIN_TYPE_BEACH, POLYGON_TYPE_STRIP);

   set_terrain_rendering_routines ( TERRAIN_TYPE_LAND, POLYGON_TYPE_FAN );

	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_FLOOR, POLYGON_TYPE_FAN );

	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_MID_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_TOP_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_MID_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_TOP_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_TOP, POLYGON_TYPE_FAN );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA1, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA2, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA3, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA4, POLYGON_TYPE_FAN );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_ROAD, POLYGON_TYPE_STRIP );
 	
	set_terrain_rendering_routines ( TERRAIN_TYPE_TRACK, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RIVER, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RESERVOIR, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RAIL, POLYGON_TYPE_STRIP );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_ROAD_BANK, POLYGON_TYPE_STRIP );
 	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RIVER_BANK, POLYGON_TYPE_STRIP );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_RAIL_BANK, POLYGON_TYPE_STRIP );

	//
	// Field types
	//

   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD1, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD2, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD3, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD4, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD5, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD6, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD7, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD8, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD9, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD10, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD11, POLYGON_TYPE_FAN );

	//
	// Varied Land Types (e.g. snow, rock etc.)
	//
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND1, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND2, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND3, POLYGON_TYPE_FAN );

	//
	// Hedges / Walls
	//

   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_TOP, POLYGON_TYPE_STRIP );
  
   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_SIDE_X, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_SIDE_Z, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_TOP, POLYGON_TYPE_STRIP );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_SIDE_X, POLYGON_TYPE_STRIP );
  
   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_SIDE_Z, POLYGON_TYPE_STRIP );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH_SIDE_X, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH_SIDE_Z, POLYGON_TYPE_STRIP );

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   // now do the terrain_type_textures
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   set_terrain_type_textures ( TERRAIN_TYPE_SEA, 4, 103, 115 );

   set_terrain_type_textures ( TERRAIN_TYPE_BEACH, 199, 172, 120 );
   set_terrain_type_textures ( TERRAIN_TYPE_LAND,  61, 135, 75);

	//
	// Forest textures
	//

	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_FLOOR, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_MID_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_TOP_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_MID_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_TOP_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_TOP, 190, 170, 15 );

	//
	// Builtup areas
	//


   set_terrain_type_textures ( TERRAIN_TYPE_ROAD, 255, 255, 255 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRACK, 255, 255, 255 );


	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA1, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA2, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA3, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA4, 255, 255, 255 );


   set_terrain_type_textures ( TERRAIN_TYPE_RIVER, 4, 103, 115 );
   set_terrain_type_textures ( TERRAIN_TYPE_RESERVOIR, 4, 103, 115 );
   set_terrain_type_textures ( TERRAIN_TYPE_RAIL, 38, 38, 38 );
   set_terrain_type_textures ( TERRAIN_TYPE_ROAD_BANK, 61, 135, 75 );
   set_terrain_type_textures ( TERRAIN_TYPE_RIVER_BANK, 141, 122, 84 );
   set_terrain_type_textures ( TERRAIN_TYPE_RAIL_BANK, 141, 122, 84 );

	//
	// Field types
	//

   set_terrain_type_textures ( TERRAIN_TYPE_FIELD1, 255, 255, 80 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD2, 100, 70, 40 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD3, 140, 70, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD4, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD5, 170, 140, 0 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD6, 81, 165, 85 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD7, 41, 115, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD8, 255, 255, 80 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD9, 100, 70, 40 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD10, 140, 70, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD11, 190, 170, 15 );

	//
	// Varied Land Types (e.g. snow, rock etc.)
	//
	
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND1, 170, 140, 0 );
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND2, 81, 165, 85 );
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND3, 255, 255, 255 );


	//
	// Hedges / Walls
	//

   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_TOP, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_SIDE_X, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_SIDE_Z, 190, 170, 15 );

	set_terrain_type_textures ( TERRAIN_TYPE_WALL_TOP, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_WALL_SIDE_X, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_WALL_SIDE_Z, 255, 255, 255 );

   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH_SIDE_X, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH_SIDE_Z, 190, 170, 15 );

	//
	// All the types that are used in the terrain generation tool
	//

   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_BEACH_MID,			0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_FRINGE,				0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_TELEPHONE_LINE,	0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_SEA_BANK,			0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_COASTAL_RIVER,	0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );



} // end initialize_terrain_type_information(void)

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void print_sector_data(int sec_z, int sec_x){

	int
		number_of_points,
		number_of_polys,
		point_loop,
		poly_loop;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys =  terrain_sectors[sec_z][sec_x].number_of_polygons;

	printf("\n\nSector %d x %d has %d points\n\n", sec_x, sec_z, number_of_points);

	printf("\nX  Z  Y\n\n");

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		printf("%d  %d  %d  %d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].x,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].z,
			terrain_sectors[sec_z][sec_x].points_y[point_loop].y);
	}

	printf("\n%d Polygons\n\n", number_of_polys);

	printf("\nPolygon  Number of points  Surface change  Point references\n\n");

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		printf("%d  %d  %d  %d  %d  %d\n", poly_loop,
			terrain_sectors[sec_z][sec_x].polygons [ poly_loop ].number_of_points,
			terrain_sectors[sec_z][sec_x].polygons [ poly_loop ].surface_change,
			terrain_sectors[sec_z][sec_x].point_byte_references[ poly_loop * 3 ].point,
			terrain_sectors[sec_z][sec_x].point_byte_references[ poly_loop * 3 + 1].point,
			terrain_sectors[sec_z][sec_x].point_byte_references[ poly_loop * 3 + 2 ].point);

	} // end for poly loop


} // end print_sector_data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void set_terrain_type_textures ( terrain_types type, int red, int green, int blue )
{

	terrain_type_information[type].red = red;
	terrain_type_information[type].green = green;
	terrain_type_information[type].blue = blue;

}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void load_road_polys_binary (void){

	// File format
	//
	// int		number_of_sectors
	//
	// loop through sectors
	//		int		x sector
	//		int		z sector
	//
	//			one   road_poly_list_per_sector   structure
	//

	FILE
		*fp;

	int
		number_of_sectors,
		sector_loop,
		sec_x,
		sec_z,
		number_of_polys,
		poly_loop,
		point_loop,
		number_of_points;

	road_poly_list_per_sector
		temp_road_polys_per_sector;

	number_of_sectors = 0;

	if(( fp = fopen("road_polys_binary.bin","rb"))==NULL){
		printf("cannot open road_polys_binary.bin\n");
		exit(1);
	}
	else printf("road_polys_binary.bin opened ...\n");

	//printf("sizeof road_poly_list_per_sector is %d bytes\n", sizeof (road_poly_list_per_sector));
	//printf("sizeof road_poly is %d bytes\n", sizeof (road_poly));
	//printf("sizeof simple_point is %d bytes\n", sizeof (simple_point));
	//printf("sizeof sector_point is %d bytes\n", sizeof (sector_point));

	fread(&number_of_sectors, sizeof(int), 1, fp);

	global_number_of_road_sectors = number_of_sectors;

	for(sector_loop = 0; sector_loop < number_of_sectors; sector_loop ++){

		fread(&sec_x, sizeof(int), 1, fp);

		fread(&sec_z, sizeof(int), 1, fp);

		//printf("%d  sector %d x %d\n", sector_loop, sec_x, sec_z);

		sectors_for_sector_road_poly_list [ sector_loop ][ 0 ] = sec_x;

		sectors_for_sector_road_poly_list [ sector_loop ][ 1 ] = sec_z;

		fread( &temp_road_polys_per_sector, sizeof( road_poly_list_per_sector ), 1, fp);

		sector_road_poly_list[ sector_loop ] = temp_road_polys_per_sector;

/*
		/////////////////////////////////////////////////////
		// start print statement
		/////////////////////////////////////////////////////

		number_of_polys = sector_road_poly_list[ sector_loop ].number_of_polys;

		for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

			number_of_points = 	sector_road_poly_list[ sector_loop ].
				poly_list[ poly_loop ].number_of_points;

			printf("\npoly %d has %d points\n\n", poly_loop, number_of_points);

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				printf("%d  %d  %d\n", sector_road_poly_list[ sector_loop ].poly_list [ poly_loop ].
					point_list[ point_loop ].sec_pt.xz_point.x,

				sector_road_poly_list[ sector_loop ].poly_list [ poly_loop ].
					point_list[ point_loop ].sec_pt.xz_point.z,

				sector_road_poly_list[ sector_loop ].poly_list [ poly_loop ].
					point_list[ point_loop ].sec_pt.y_point.y);

			} // end for point loop
		} // end for poly loop

		/////////////////////////////////////////////////////
		// end print statement
		/////////////////////////////////////////////////////
*/
	} // end for sector loop

	fclose(fp);

} // end load road polys binary


//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void format_road_poly_data_into_eech_structure (void){

	int
		sec_x,
		sec_z,
		number_of_sectors,
		number_of_polys,
		number_of_points,
		sector_loop,
		poly_loop,
		point_loop,
		point_loop2,
		point_counter,
		point_byte_reference_counter,
		triangle_loop,
		triangle_counter,
		number_of_points_with_duplicates,
		number_of_points_without_duplicates,
		number_of_clockwise_triangle_polys;

	sector_point
		point_list [ 128 ],
		points_without_duplicates [ 128 ],
		current_point;

	road_poly
		clockwise_poly,
		clockwise_triangle_polys [ 64 ];

	///////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////
	// point section
	///////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////
	// get a list of all poly points with duplicates
	///////////////////////////////////////////////////////////////

	number_of_sectors = global_number_of_road_sectors;

	for(sector_loop = 0; sector_loop < number_of_sectors; sector_loop ++){

		sec_x = sectors_for_sector_road_poly_list [ sector_loop ][ 0 ];

		sec_z = sectors_for_sector_road_poly_list [ sector_loop ][ 1 ];

		number_of_polys = sector_road_poly_list[ sector_loop ].number_of_polys;

		//printf("\n\nSector %d x %d\n\n", sec_x, sec_z);

		point_counter = 0;

		
		for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

			number_of_points = sector_road_poly_list[ sector_loop ].
				poly_list[ poly_loop ].number_of_points;

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				point_list [ point_counter ] = sector_road_poly_list[ sector_loop ].
					poly_list[ poly_loop ].point_list[ point_loop ].sec_pt;

				point_counter ++;

			} // end for point loop
		} // end for poly loop

		number_of_points_with_duplicates = point_counter;

		///////////////////////////////////////////////////////////////
		// get a list of all points without duplicates
		///////////////////////////////////////////////////////////////

		number_of_points_without_duplicates = 0;

		point_counter = 0;

		for(point_loop = 0; point_loop < number_of_points_with_duplicates; point_loop ++){

			current_point = point_list [ point_loop ];

			if(current_point.y_point.y != 0){

				points_without_duplicates [ point_counter ] = current_point;

				number_of_points_without_duplicates ++;

				point_counter ++;
			}

			//
			// mark any duplicates of current point
			//

			for(point_loop2 = point_loop + 1; 
				point_loop2 < number_of_points_with_duplicates;
				point_loop2 ++){

				if(current_point.xz_point.x == point_list [ point_loop2 ].xz_point.x &&
					current_point.xz_point.z == point_list [ point_loop2 ].xz_point.z){

					point_list [ point_loop2 ].y_point.y = 0;

				} // end if
			} // end for point_loop2
		} // end for point loop

		////////////////////////////////////////////////
		// copy the points into the global EECH data
		////////////////////////////////////////////////

		////////////////////
		// number of points
		////////////////////

		terrain_sectors[sec_z][sec_x].number_of_points = number_of_points_without_duplicates;

		/////////////////////////////////
		// points_y and points_xz []
		/////////////////////////////////

		for(point_loop = 0; point_loop < number_of_points_without_duplicates; point_loop ++){

			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x =
				(short int)points_without_duplicates [ point_loop ].xz_point.x;

			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z =
				(short int)points_without_duplicates [ point_loop ].xz_point.z;

			terrain_sectors[sec_z][sec_x].points_y[ point_loop ].y =
				(unsigned short int)points_without_duplicates [ point_loop ].y_point.y;

		} // end for point loop

/*
		//////////////////////////////////////////////
		// start print section for points
		//////////////////////////////////////////////

		printf("\nSector %d x %d\n\n", sec_x, sec_z);

		printf("%d points with duplicates\n\n",	number_of_points_with_duplicates);

		for(point_loop = 0; point_loop < number_of_points_with_duplicates; point_loop ++){

			printf("%d  %d  %d\n", point_list [ point_loop ].xz_point.x,
				point_list [ point_loop ].xz_point.z,
				point_list [ point_loop ].y_point.y);
		} // end for point loop

		printf("\n%d points without duplicates\n\n",	number_of_points_without_duplicates);

		for(point_loop = 0; point_loop < number_of_points_without_duplicates; point_loop ++){

			printf("%d  %d  %d\n", points_without_duplicates [ point_loop ].xz_point.x,
				points_without_duplicates [ point_loop ].xz_point.z,
				points_without_duplicates [ point_loop ].y_point.y);
		} // end for point loop

		///////////////////////////////////////////////
		// end print section
		///////////////////////////////////////////////
*/
	//} // end for sector loop

	//////////////////////////////////////////////////////
	//////////////////////////////////////////////////////
	// polygon section
	//////////////////////////////////////////////////////
	//////////////////////////////////////////////////////

	//for(sector_loop = 0; sector_loop < number_of_sectors; sector_loop ++){

		//sec_x = sectors_for_sector_road_poly_list [ sector_loop ][ 0 ];

		//sec_z = sectors_for_sector_road_poly_list [ sector_loop ][ 1 ];

		

		number_of_polys = sector_road_poly_list[ sector_loop ].number_of_polys;

		number_of_clockwise_triangle_polys = 0;

		///////////////////////////
		// surface changes []
		///////////////////////////
		
		terrain_sectors[sec_z][sec_x].surface_changes [ 0 ].surface_id = TERRAIN_TYPE_ROAD;

		triangle_counter = 0;

		for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

			number_of_points = sector_road_poly_list[ sector_loop ].
				poly_list[ poly_loop ].number_of_points;
/*
			printf("\npoly %d original order\n\n", poly_loop);

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				printf("%d  %d  %d\n", sector_road_poly_list[ sector_loop ].
					poly_list[ poly_loop ].point_list[ point_loop ].sec_pt.xz_point.x,
					sector_road_poly_list[ sector_loop ].
					poly_list[ poly_loop ].point_list[ point_loop ].sec_pt.xz_point.z,
					sector_road_poly_list[ sector_loop ].
					poly_list[ poly_loop ].point_list[ point_loop ].sec_pt.y_point.y);
			} // end for point loop
*/
			clockwise_poly = get_clockwise_ordered_poly( sector_road_poly_list[ sector_loop ].
				poly_list[ poly_loop ] );
/*
			printf("\nclockwise ordered poly %d\n\n", poly_loop);

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				printf("%d  %d  %d\n", clockwise_poly.point_list[ point_loop ].sec_pt.xz_point.x,
					clockwise_poly.point_list[ point_loop ].sec_pt.xz_point.z,
					clockwise_poly.point_list[ point_loop ].sec_pt.y_point.y);
			} // end for point loop
*/

			number_of_clockwise_triangle_polys += number_of_points - 2;

			//////////////////////////////////////////////////
			// divide polygons into triangle polygons
			//////////////////////////////////////////////////

			//printf("\n\nTriangles\n\n");

			for(triangle_loop = 0; triangle_loop < number_of_points - 2; triangle_loop ++){

				clockwise_triangle_polys [ triangle_counter ].number_of_points = 3;

				clockwise_triangle_polys [ triangle_counter ].point_list[ 0 ] = 
					clockwise_poly.point_list[ 0 ];

				clockwise_triangle_polys [ triangle_counter ].point_list[ 1 ] = 
					clockwise_poly.point_list[ triangle_loop + 1 ];

				clockwise_triangle_polys [ triangle_counter ].point_list[ 2 ] = 
					clockwise_poly.point_list[ triangle_loop + 2 ];

				triangle_counter ++;

/*
				for(point_loop = 0; point_loop < 3; point_loop ++){

					printf("triangle counter = %d triangle loop = %d  %d  %d  %d\n",
						triangle_counter, triangle_loop, 

						clockwise_triangle_polys[ triangle_counter ].
						point_list[ point_loop ].sec_pt.xz_point.x,

						clockwise_triangle_polys[ triangle_counter ].
						point_list[ point_loop ].sec_pt.xz_point.z,

						clockwise_triangle_polys[ triangle_counter ].
						point_list[ point_loop ].sec_pt.y_point.y);
				} // end for point loop
*/
			} // end for triangle loop

			

		} // end for poly loop

		//////////////////////////////////////////////////
		// copy the polygon data into EECH structure
		//////////////////////////////////////////////////

		/////////////////////////
		// number of polygons
		/////////////////////////

		//printf("\n%d total number_of_clockwise_triangle_polys\n\n",
		//	number_of_clockwise_triangle_polys);

		terrain_sectors[sec_z][sec_x].number_of_polygons = 
			(unsigned short int) number_of_clockwise_triangle_polys;

		point_byte_reference_counter = 0;

		for(triangle_loop = 0;
			triangle_loop < number_of_clockwise_triangle_polys;
			triangle_loop ++){

			/////////////////////
			// polygons []
			/////////////////////

			terrain_sectors[sec_z][sec_x].polygons [ triangle_loop ].number_of_points = 3;

			terrain_sectors[sec_z][sec_x].polygons [ triangle_loop ].surface_change = 0;
			
			///////////////////////////////////////
			// point_byte_references []
			///////////////////////////////////////

			//printf("\ntriangle loop %d\n", triangle_loop);

			for(point_loop = 0; point_loop < number_of_points_without_duplicates; point_loop ++){

				if(points_without_duplicates [ point_loop ].xz_point.x ==
					clockwise_triangle_polys [ triangle_loop ].point_list[ 0 ].
					sec_pt.xz_point.x &&
					points_without_duplicates [ point_loop ].xz_point.z ==
					clockwise_triangle_polys [ triangle_loop ].point_list[ 0 ].
					sec_pt.xz_point.z){

					terrain_sectors[sec_z][sec_x].
						point_byte_references[ point_byte_reference_counter ].point =
						(unsigned char)point_loop;

					//printf("%d  %d  %d\n", triangle_loop, point_byte_reference_counter, point_loop);

					point_byte_reference_counter ++;

					break;

				} // end if
			} // end for point loop

			for(point_loop = 0; point_loop < number_of_points_without_duplicates; point_loop ++){

				if(points_without_duplicates [ point_loop ].xz_point.x ==
					clockwise_triangle_polys [ triangle_loop ].point_list[ 1 ].
					sec_pt.xz_point.x &&
					points_without_duplicates [ point_loop ].xz_point.z ==
					clockwise_triangle_polys [ triangle_loop ].point_list[ 1 ].
					sec_pt.xz_point.z){

					terrain_sectors[sec_z][sec_x].
						point_byte_references[ point_byte_reference_counter ].point =
						(unsigned char)point_loop;

					//printf("%d  %d  %d\n", triangle_loop, point_byte_reference_counter, point_loop);

					point_byte_reference_counter ++;

					break;

				} // end if
			} // end for point loop

			for(point_loop = 0; point_loop < number_of_points_without_duplicates; point_loop ++){

				if(points_without_duplicates [ point_loop ].xz_point.x ==
					clockwise_triangle_polys [ triangle_loop ].point_list[ 2 ].
					sec_pt.xz_point.x &&
					points_without_duplicates [ point_loop ].xz_point.z ==
					clockwise_triangle_polys [ triangle_loop ].point_list[ 2 ].
					sec_pt.xz_point.z){

					terrain_sectors[sec_z][sec_x].
						point_byte_references[ point_byte_reference_counter ].point =
						(unsigned char)point_loop;

					//printf("%d  %d  %d\n", triangle_loop, point_byte_reference_counter, point_loop);

					point_byte_reference_counter ++;

					break;

				} // end if
			} // end for point loop

		} // end for triangle loop
	
	} // end for sector loop

} // end format_road_poly_data_into_eech_structure

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void allocate_memory_for_sectors (void){

	int
		x,
		z;

	printf("Allocating memory for sectors ...\n");

	terrain_sectors = malloc ( MAP_HEIGHT * sizeof ( terrain_3d_sector * ) );

	for(z = 0; z < MAP_HEIGHT; z ++){

		if( ! (terrain_sectors[z] = malloc( MAP_WIDTH * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for


	for( z=0 ; z < MAP_HEIGHT ; z++){
		for( x=0 ; x < MAP_WIDTH ; x++){

			if(!(terrain_sectors[z][x].points_y = malloc(sizeof(terrain_3d_point_y) * 128 ))){
				printf("No memory for points_y sector %d x %d\n",z,x);
				exit(1);
			}

			if(!(terrain_sectors[z][x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 128 ))){
				printf("No memory for points_xz sector %d x %d\n",z,x);
				exit(1);
			}

			if(!(terrain_sectors[z][x].point_byte_references = malloc( 256 ))){
					printf("No memory for point_byte_references sector %d x %d\n",z,x);
					exit(1);
			}

			if(!(terrain_sectors[z][x].polygons = malloc( 64 ))){
				printf("No memory for polygons sector %d x %d\n",z,x);
				exit(1);
			}

			if(!(terrain_sectors[z][x].surface_changes = malloc( 8 ))){
				printf("No memory for surface_changes sector %d x %d\n",z,x);
				exit(1);
			}
		} // end for x loop
	} // end for z loop

} // end allocate memory for sectors

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////
	
road_poly	get_clockwise_ordered_poly(road_poly	in_poly){

	int	
		a,
		b,
		c,
		d,
		e,
		f,
		lowest_rightmost_pt,
		leftmost,
		number_of_points;

	short int
		lowest_z;

	road_poly
		out_poly;

	//printf("\n*** Start of get_clockwise_ordered_poly\n");


	//////////////////////////////////////////////////////////////////////////////
	// find the lowest and rightmost point
	//////////////////////////////////////////////////////////////////////////////

	number_of_points = in_poly.number_of_points;

	out_poly.number_of_points = number_of_points;

	//printf("in_poly befor processing\n");
	//for(a = 0; a < number_of_points; a++){
	//	printf("%f  %f  %f  %c  %d  %d\n", in_poly.point_list[a].x,
	//		in_poly.point_list[a].z, in_poly.point_list[a].y,
	//		in_poly.point_list[a].letter,in_poly.point_list[a].road,
	//		in_poly.point_list[a].quad);
	//}
	//printf("\n");

	lowest_z = in_poly.point_list[0].sec_pt.xz_point.z;

	//printf("lowest z = %d\n", lowest_z);

	for(a = 1; a < number_of_points; a++){

		if(in_poly.point_list[a].sec_pt.xz_point.z < lowest_z){
			lowest_z = in_poly.point_list[a].sec_pt.xz_point.z;
		}

		//printf("a = %d  lowest z = %d\n", a, lowest_z);
	}
	
	// find the first point in the array at the lowest z

	for(a = 0; a < number_of_points; a++){

		if(in_poly.point_list[a].sec_pt.xz_point.z == lowest_z) {
			lowest_rightmost_pt = a;

		//printf("z = %d  lowest_z = %d  a = %d  lowest_rightmost_pt = %d\n",
		//	in_poly.point_list[a].sec_pt.xz_point.z,
		//	lowest_z, a, lowest_rightmost_pt);
		break;
		}
	}

	// find if any other points are equal to the lowest z and also further to the right

	for(a = lowest_rightmost_pt + 1; a < number_of_points; a++){

		if((in_poly.point_list[a].sec_pt.xz_point.z == lowest_z) && 
			(in_poly.point_list[a].sec_pt.xz_point.x > 
			in_poly.point_list[lowest_rightmost_pt].sec_pt.xz_point.x)){

			lowest_rightmost_pt = a;
		}

		//printf("a = %d  lowest_rightmost_pt = %d\n", a, lowest_rightmost_pt);
	} // end for

	// the first point is the lowest and rightmost point

	out_poly.point_list[0] = in_poly.point_list[lowest_rightmost_pt];

	
	// copy the rest of the points to out poly

	b = 1;

	for(a = 0; a < number_of_points; a++){

		if(a != lowest_rightmost_pt){
			
			out_poly.point_list[b] = in_poly.point_list[a];

			b++;
		}
	} // end for

	//printf("out_poly with lowest rightmost point in position 0\n");
	//for(c = 0; c < number_of_points; c++){
	//	printf("%d  %d\n", out_poly.point_list[c].sec_pt.xz_point.x, 
	//		out_poly.point_list[c].sec_pt.xz_point.z);
	//}
	//printf("\n");


	/////////////////////////////////////////////////////////////////////////////
	// sort points 1 - number_of_points in clockwise order
	/////////////////////////////////////////////////////////////////////////////

	for(d = 1; d <= number_of_points - 2; d++){

		leftmost = d;

		//printf("d = %d\n", d);

		for(e = d; e <= number_of_points - 2; e++){

			//printf("e = %d\n", e);

			if(isLeft( out_poly.point_list[0],
				out_poly.point_list[leftmost],
				out_poly.point_list[e + 1]) > 0){

				//printf("isLeft is called with 0  %d  %d  and  %d  is returned\n", leftmost, e+1,
				//	isLeft( out_poly.point_list[0],
				//	out_poly.point_list[leftmost],
				//	out_poly.point_list[e + 1]));

				leftmost = e + 1;

				//printf("new leftmost = %d\n", leftmost);
			} // end for

			//if(leftmost != d){

			//	swap(out_poly.point_list, leftmost, d);

				//printf("swap has been called with %d and %d\n", leftmost, d);
				//printf("new point list after swap\n");
				//for(c = 0; c < number_of_points; c++){
				//	printf("%f  %f\n", out_poly.point_list[c].x, out_poly.point_list[c].z);
				//}
				//printf("\n");
			//} // end if

		} // end for

		if(leftmost != d){

			swap(out_poly.point_list, leftmost, d);
		}
	} // end for

	//printf("\n*** End of get_clockwise_ordered_poly\n");


	return out_poly;
} // end get_clockwise_ordered_poly()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


void swap(simple_point *array, int a, int b){

	simple_point temp;

	temp = array[a];

	array[a] = array[b];

	array[b] = temp;
/*
	temp.x = array[a].x;
	temp.z = array[a].z;
	temp.y = array[a].y;
	temp.letter = array[a].letter;
	temp.road = array[a].road;
	temp.quad = array[a].quad;

	array[a].x = array[b].x;
	array[a].z = array[b].z;
	array[a].y = array[b].y;
	array[a].letter = array[b].letter;
	array[a].road = array[b].road;
	array[a].quad = array[b].quad;

	array[b].x = temp.x;
	array[b].z = temp.z;
	array[b].y = temp.y;
	array[b].letter = temp.letter;
	array[b].road = temp.road;
	array[b].quad = temp.quad;
*/
} // end swap()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// isLeft(): tests if a point is Left|On|Right of an infinite line.
//    Input:  three points P0, P1, and P2
//    Return: >0 for P2 left of the line through P0 and P1
//            =0 for P2 on the line
//            <0 for P2 right of the line

int	isLeft( simple_point P0, simple_point P1, simple_point P2 )
{
    return (int)((P1.sec_pt.xz_point.x - P0.sec_pt.xz_point.x) *
		(P2.sec_pt.xz_point.z - P0.sec_pt.xz_point.z) - 
		(P2.sec_pt.xz_point.x - P0.sec_pt.xz_point.x) *
		(P1.sec_pt.xz_point.z - P0.sec_pt.xz_point.z));
}

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void export_road_sectors_to_lw_objs (void){

	int
		sector_loop,
		sec_x,
		sec_z;

	char
		filename [16];

	for(sector_loop = 0; sector_loop < global_number_of_road_sectors; sector_loop ++){

		sec_x = sectors_for_sector_road_poly_list [ sector_loop ][ 0 ];

		sec_z = sectors_for_sector_road_poly_list [ sector_loop ][ 1 ];

		sprintf(filename, "road_lw_objs\\%03dx%03d.lwo", sec_x, sec_z); 

		save_terrain_sector_as_lw_3d_object ( sec_x, sec_z, filename );

	} // end for sector loop
} // end export_road_sectors_to_lw_objs