#include <stdio.h>
//#include <ctype.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>

#include "georgia.h"

//////////////////////////////
// global data
//////////////////////////////

	//////////////////////////////////////////
	//				height x width	
	//
	// Thailand	map1  200 x 180
	// Cuba		map2  128 x 240
	// Georgia	map3  116 x 244
	// Taiwan	map4  192 x 124
	// Lebanon	map5  92 x 76
	//////////////////////////////////////////


terrain_3d_sector
	**terrain_sectors;

unsigned char
	*terrain_3d_sector_data,
	*terrain_3d_geometry_data,
	*terrain_3d_colour_data;

int
	terrain_version,
	rgb_4byte_header,
	ffp_4byte_header,
	size_of_ffp_file,
	size_of_rgb_file,
	max_forest_sector_colors,
	global_repetition_count;

unsigned short int
	map_width, 
	map_height, 
	normal_changes_word;

float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;

///////////////////////////////
// Functions
///////////////////////////////

void load_3d_terrain (void);

void print_sector_data (int sec_z, int sec_x);

void get_data_sizes (void);

void write_new_terrain_files(void);

void find_forest_sectors (void);

void print_forest_sector_data (int sec_z, int sec_x);

void get_points_directly_below_forest_top_points (int sec_z, int sec_x);

void remove_forest_sides (int sec_z, int sec_x);

void lower_forest_top (int sec_z, int sec_x);

void create_tree_files(int max_z, int max_x);

int edge_test_ok(long long x, long long z, long long min_space, long long poly[][3], int num_points);

int InsidePolygon(Point *polygon,int N,Point p);

void find_new_min_max_height(void);

int get_y_value_from_adjacent_corner_sector ( int sec_z, int sec_x, int top_point_z, int top_point_x, int top_point_y );

void fix_steep_polys (int sec_z, int sec_x );

double normal_angle_with_y ( int sec_z, int sec_x, int a_ref, int b_ref, int c_ref );

void match_adjacent_sector_point ( int sec_z, int sec_x, int match_point_z, int match_point_x, int match_point_y );

void find_number_of_colors ( void );

void backup_files ( void );

void exit_function(void);

void change_forest_canopy_to_land (int sec_z, int sec_x );

void lighten_colors ( void );

void add_new_colors (int sec_z, int sec_x );

int file_exist (char *filename);

void create_winter_rgb_file ( void );

void find_all_sea_sectors ( void );

void fix_steep_polys_2 (int sec_z, int sec_x );

int find_closest_point (int sec_z, int sec_x, int target_ref);

void get_slopes_above_x (float slope_x);

int steep_check ( int sec_z, int sec_x, float slope_x );



int main(int argc, char ** argv)
{
	int
		sec_x,
		sec_z,
		return_value;

	//backup_files ();

	load_3d_terrain();

	get_data_sizes ();

//	print_sector_data (26, 28);
//	print_sector_data (27, 28);

	find_number_of_colors ();

	find_forest_sectors ();

	lighten_colors ();

	printf("removing forest sides ...\n");

	for(sec_z = 0; sec_z < map_height; sec_z ++){
		for(sec_x = 0; sec_x < map_width; sec_x ++){

			if( terrain_sectors[sec_z][sec_x].forest_sector ){

				remove_forest_sides ( sec_z, sec_x );
			}
			
		}// end for z loop
	} // end for x loop


	printf("lowering forest canopies ...\n");

	for(sec_z = 0; sec_z < map_height; sec_z ++){
		for(sec_x = 0; sec_x < map_width; sec_x ++){

			if( terrain_sectors[sec_z][sec_x].forest_sector ){

				lower_forest_top ( sec_z, sec_x );
			}
			
		}// end for z loop
	} // end for x loop

	get_slopes_above_x (60.0);
	get_slopes_above_x (70.0);
	get_slopes_above_x (80.0);

	printf("fixing steep forest polys ...\n");

	for(sec_z = 1; sec_z < map_height - 1; sec_z ++){
		for(sec_x = 1; sec_x < map_width - 1; sec_x ++){

			global_repetition_count = 0;

			if( terrain_sectors[sec_z][sec_x].forest_sector ){

				fix_steep_polys_2 ( sec_z, sec_x );
			}
			
		}// end for z loop
	} // end for x loop

	get_slopes_above_x (60.0);
	get_slopes_above_x (70.0);
	get_slopes_above_x (80.0);

	printf("adding tree colors ...\n");

	for(sec_z = 0; sec_z < map_height; sec_z ++){
		for(sec_x = 0; sec_x < map_width; sec_x ++){

			if( terrain_sectors[sec_z][sec_x].forest_sector ){

				add_new_colors ( sec_z, sec_x );
			}
			
		}// end for z loop
	} // end for x loop

	//create_tree_files( map_height, map_width); 

	write_new_terrain_files();

	if( map_width == 244 ){
		
		create_winter_rgb_file ();
	}

	//printf("\nThank You for trying the Block Forest Replacement Mod\n");

	//exit_function ();

	return 0;
}

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	long
		file_size;

	int
		sec_x,
		sec_z,
		offset;

	short int
		pad_word_zero;

	if(( sec_in = fopen("original_default.sec","rb"))==NULL){
		printf("cannot open original_default.sec\n");
		exit_function ();
	}
	else printf("original_default.sec opened\n");


	if(( ffp_in = fopen("original_terrain.ffp","rb"))==NULL){
		printf("cannot open original_terrain.ffp\n");
		fclose(sec_in);
		exit_function ();
	}
	else printf("original_terrain.ffp opened\n");


	if(( rgb_in = fopen("original_default.rgb","rb"))==NULL){
		printf("cannot open original_default.rgb\n");
		exit_function ();
	}
	else printf("original_default.rgb opened\n");

	//
	// load terrain.ffp file
	//

	fseek( ffp_in, 0L, SEEK_END );
	file_size = ftell( ffp_in );
	fseek( ffp_in, 0L, SEEK_SET );

	size_of_ffp_file = file_size;

	printf("Allocating memory for terrain data ...\n");

	if ( ! (terrain_3d_geometry_data = malloc ( file_size ) ) ){

		printf("No memory for terrain data ...\n");
		exit_function ();
	}

	printf("Reading terrain data ...\n");

	fread ( terrain_3d_geometry_data, file_size, 1, ffp_in);

	//
	// load rgb file
	//

	fseek( rgb_in, 0L, SEEK_END );
	file_size = ftell( rgb_in );
	fseek( rgb_in, 0L, SEEK_SET );

	size_of_rgb_file = file_size;

	printf("Allocating memory for rgb data ...\n");

	if ( ! (terrain_3d_colour_data = malloc ( file_size ) ) ){

		printf("No memory for rgb data ...\n");
		exit_function ();
	}

	printf("Reading rgb data ...\n");

	fread ( terrain_3d_colour_data, file_size, 1, rgb_in);

	//
	// Get ffp and rgb headers
	//

	fseek( ffp_in, 0L, SEEK_SET );

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	fseek( rgb_in, 0L, SEEK_SET );

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);


	///////////////////////////////////////////////////////
	// read in map dimensions from sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;

	//
	// exit with error if map size is incorrect
	//

	if( ! ( ( map_width == 244 && map_height == 116 ) ||	// Georgia	map3
		( map_width == 180 && map_height == 200 ) ||		// Thailand	map1
		( map_width == 240 && map_height == 128 ) ||		// Cuba		map2
		( map_width == 124 && map_height == 192 ) ||		// Taiwan	map4
		( map_width == 76 && map_height == 92 ) ) ){		// Lebanon	map5

		printf("Error: map size is incorrect!\n");

		exit_function ();
	}

	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	if ( ! (terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) ) ) ){

		printf("No memory for terrain_sectors ...\n");
		exit_function ();
	}

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory for terrain_sectors[%d]\n", sec_z);
			exit_function ();
		}

	} // end for

	printf("Memory for sectors allocated ...\n");

	//map_height = 2;

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			//printf("\nSector %d x %d\n\n", sec_z, sec_x);
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////
			// Get the offsets into the geometry file
			///////////////////////////////////////////////////////////////

			//////////////////////////
			// points y
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpoints y offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].points_y = ( terrain_3d_point_y * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// points xz
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpoints xz offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].points_xz = ( terrain_3d_point_xz * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// point references
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpoint references offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].point_byte_references = ( terrain_3d_point_byte_reference * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// polygons
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpolygons offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].polygons = ( terrain_3d_face * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// surface changes
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tsurface changes offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].surface_changes = ( terrain_3d_surface * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// normal changes
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tnormal changes offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].normal_changes = ( terrain_3d_normal_index * ) ( terrain_3d_geometry_data + offset );

			//
			// Get the offsets into the colours file
			//

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			terrain_sectors[sec_z][sec_x].colour_indices_offset = offset;
			
			terrain_sectors[sec_z][sec_x].colour_indices = ( terrain_3d_colour_index * ) ( terrain_3d_colour_data + offset );

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			terrain_sectors[sec_z][sec_x].point_colours_offset = offset;
			
			terrain_sectors[sec_z][sec_x].point_colours = ( terrain_3d_colour * ) ( terrain_3d_colour_data + offset );
		}
	}

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain loaded ...\n");

	
} // end load 3d terrain

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void get_data_sizes (void){

	int
		last_sector = 0,
		sec_z,
		sec_x,
		next_sec_x,
		next_sec_z,
		temp_sec_x,
		temp_sec_z,
		points_y_offset,
		normal_changes_offset,
		polygons_offset,
		point_references_offset,
		number_of_points,
		number_of_polygons,
		poly_loop,
		surface_change_count;

	terrain_3d_point_y
		*all_sea_sector_points_y;

	printf("finding data sizes ...\n");

	if( map_width == 244 ){ // Georgia

		all_sea_sector_points_y = terrain_sectors[ 0 ][ 0 ].points_y;
	}
	else if ( map_width == 76 ){ // Lebanon

		all_sea_sector_points_y = terrain_sectors[ 4 ][ 23 ].points_y;
	}
	else if( map_width == 240 ){ // Cuba

		all_sea_sector_points_y = terrain_sectors[ 0 ][ 0 ].points_y;
	}
	else if( map_width == 124 ){ // Taiwan

		all_sea_sector_points_y = terrain_sectors[ 0 ][ 0 ].points_y;
	}
	else {

		all_sea_sector_points_y = NULL;
	}

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			number_of_points = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_points;

			number_of_polygons = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_polygons;

			surface_change_count = 0;

			//////////////////////////////////////////////////////////////////////////////
			// tag each sector as all sea sector or not
			// 
			//////////////////////////////////////////////////////////////////////////////

			if( terrain_sectors[ sec_z ][ sec_x ].points_y == all_sea_sector_points_y ){

				terrain_sectors[ sec_z ][ sec_x ].all_sea_sector = 1;

				//
				// an all sea sector always has 2 normal changes
				//

				terrain_sectors[sec_z][sec_x].number_of_normal_changes = 2;

			}
			else{

				terrain_sectors[ sec_z ][ sec_x ].all_sea_sector = 0;
			}

			///////////////////////////////////////////////////////////////////////////////////
			// find the next sector in memory
			//
			// the first sector is all sea and all sectors that are all sea use the same
			// chunck of data so need to skip over these to find the next position in
			// the sector memory
			///////////////////////////////////////////////////////////////////////////////////

			temp_sec_x = sec_x;
			temp_sec_z = sec_z;

			while ( 1 ){

				if( temp_sec_z == (int)( map_height - 1 ) && temp_sec_x == (int)( map_width - 1 ) ){

					last_sector = 1;
				}
				else if ( temp_sec_x == (int)( map_width - 1 ) ){

					next_sec_x = 0;
					next_sec_z = temp_sec_z + 1;
				}
				else{

					next_sec_x = temp_sec_x + 1;
					next_sec_z = temp_sec_z;
				}

				if( terrain_sectors[ next_sec_z ][ next_sec_x ].points_y == all_sea_sector_points_y ){

					temp_sec_x = next_sec_x;
					temp_sec_z = next_sec_z;

					continue;
				}
				else{

					break;
				}
			}

			////////////////////////////////////////////////////////////////////////////
			// get the number of point references
			////////////////////////////////////////////////////////////////////////////

			polygons_offset = (int)terrain_sectors[sec_z][sec_x].polygons;

			point_references_offset = (int)terrain_sectors[sec_z][sec_x].point_byte_references;

			if( number_of_points <= 256 ){

				terrain_sectors[sec_z][sec_x].number_of_point_references =
					( ( polygons_offset - point_references_offset ) /
					sizeof ( terrain_3d_point_byte_reference ) );
			}
			else{

				terrain_sectors[sec_z][sec_x].number_of_point_references =
					( ( polygons_offset - point_references_offset ) /
					sizeof ( terrain_3d_point_word_reference ) );
			}
			

			// /////////////////////////////////////////////////////////////////////////
			// get the number of surface changes
			//
			// there is a zero pad byte at the end of the surface changes
			// if the number is odd, so need to loop thru all polygons
			// and count the surface changes
			////////////////////////////////////////////////////////////////////////////

			for( poly_loop = 0; poly_loop < number_of_polygons; poly_loop ++ ){

				if( terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change ){

					surface_change_count ++;
				}
			}

			//
			// since the surface change bit is not set for the first surface of
			// a sector, the number of surfaces is the surface change count + 1
			//

			terrain_sectors[ sec_z ][ sec_x ].number_of_surface_changes = surface_change_count;

			terrain_sectors[ sec_z ][ sec_x ].number_of_surfaces = surface_change_count + 1;

			//printf ("%d	%d	%d\n", sec_z, sec_x, terrain_sectors[sec_z][sec_x].number_of_surface_changes);

			/////////////////////////////////////////////////////////////////////////////
			// get the number of normal changes
			/////////////////////////////////////////////////////////////////////////////				

			if ( !last_sector ){

				if( terrain_sectors[ sec_z ][ sec_x ].all_sea_sector ){

					// already set to 2
				}
				else{

					points_y_offset = (int)terrain_sectors[next_sec_z][next_sec_x].points_y;

					normal_changes_offset = (int)terrain_sectors[sec_z][sec_x].normal_changes;

					terrain_sectors[sec_z][sec_x].number_of_normal_changes =
						( points_y_offset - normal_changes_offset ) /
						sizeof ( terrain_3d_normal_index );
				}
			}
			else{ // if last sector

				terrain_sectors[sec_z][sec_x].number_of_normal_changes =
					( ( size_of_ffp_file - ( (int)terrain_sectors[sec_z][sec_x].normal_changes - (int)terrain_3d_geometry_data )  ) /
					sizeof ( terrain_3d_normal_index ) );
				
			}

			//printf ("%d x %d   all_sea = %d    next sector %d  %d  	%d\n", sec_z, sec_x,
			//	terrain_sectors[ sec_z ][ sec_x ].all_sea_sector,
			//	next_sec_z, next_sec_x, terrain_sectors[sec_z][sec_x].number_of_normal_changes);

		}
	}
} // end get data sizes

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void print_sector_data(int sec_z, int sec_x){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		color_index,
		number_of_colors,
		color_loop;

	number_of_colors =  terrain_sectors[sec_z][sec_x].number_of_colors;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	//printf("\nSector %d x %d\n\n", sec_z, sec_x);

	//printf("colour_indices_offset = %d   point_colours_offset = %d\n",
	//	terrain_sectors[sec_z][sec_x].colour_indices_offset,
	//	terrain_sectors[sec_z][sec_x].point_colours_offset);
/*
	for (color_loop = 0; color_loop < number_of_colors; color_loop ++ ){

		printf("   %3d      %3d  %3d  %3d\n", color_loop,
			terrain_sectors[sec_z][sec_x].point_colours[ color_loop ].red,
			terrain_sectors[sec_z][sec_x].point_colours[ color_loop ].green,
			terrain_sectors[sec_z][sec_x].point_colours[ color_loop ].blue);
	} // end color loop
*/
	printf("\nSector %d x %d --- %d points  %d polygons \n\n",
		sec_z, sec_x, number_of_points, number_of_polys);

	printf("Point  X  Z  Y  Color index  Color\n\n");

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		//
		// there is a color index for each point
		//

		color_index = terrain_sectors[sec_z][sec_x].colour_indices [ point_loop ].index;

		printf("%3d    %6d  %6d  %6d     %3d     %3d  %3d  %3d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x,
			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z,
			terrain_sectors[sec_z][sec_x].points_y[ point_loop ].y,
			color_index,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].red,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].green,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].blue);
	}
/*
	printf("Point  X  Z  Y  \n\n");

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		color_index = terrain_sectors[sec_z][sec_x].colour_indices [ point_loop ].index;

		printf("%3d    %6d  %6d  %6d \n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].x,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].z,
			terrain_sectors[sec_z][sec_x].points_y[point_loop].y);
	}
*/
	printf("\nPolygon	Number of points	Surface\n\n");

	point_ref_counter = 0;

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}

		printf("Poly	%3d		has %d points	suface = %3d\n\n", poly_loop, number_of_poly_points, surface);

		printf("	Point references\n\n");

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

			printf("\t%2d	%3d\n", poly_point_loop, point_reference);
		} // end poly point loop

		printf("\n");

	} // end for poly loop


} // end print sector data


///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////


void write_new_terrain_files(void){

	FILE
		*sec_out,
		*ffp_out,
		*rgb_out;

	int
		sec_x,
		sec_z,
		offset,
		all_sea_points_y_offset,
		all_sea_points_xz_offset,
		all_sea_point_references_offset,
		all_sea_polygons_offset,
		all_sea_surface_changes_offset,
		all_sea_normal_changes_offset,
		four_byte_zero_word = 0,
		number_of_points,
		rgb_offset;

	short int
		pad_word_zero = 0;

	unsigned char
		pad_byte_zero = 0;

	/////////////////////////////////////////////////////////

	printf("Writing new terrain files ...\n");
/*
	if(( sec_out = fopen("new_default.sec","wb"))==NULL){
		printf("cannot open new_default.sec\n");
		exit_function ();
	}
	else printf("new_default.sec opened\n");


	if(( ffp_out = fopen("new_terrain.ffp","wb"))==NULL){
		printf("cannot open new_terrain.ffp\n");
		fclose(sec_out);
		exit_function ();
	}
	else printf("new_terrain.ffp opened\n");
*/

	if(( sec_out = fopen("default.sec","wb"))==NULL){
		printf("cannot open default.sec\n");
		exit_function ();
	}
	else printf("default.sec opened\n");


	if(( ffp_out = fopen("terrain.ffp","wb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_out);
		exit_function ();
	}
	else printf("terrain.ffp opened\n");

	if(( rgb_out = fopen("default.rgb","wb"))==NULL){
		printf("cannot open default.rgb\n");
		exit_function ();
	}
	else printf("default.rgb opened\n");

	fwrite(&terrain_version, sizeof(int), 1, sec_out);
	fwrite(&map_width, sizeof(unsigned short int), 1, sec_out);
	fwrite(&map_height, sizeof(unsigned short int), 1, sec_out);
	fwrite(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_out);
	fwrite(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_out);

	fwrite(&ffp_4byte_header, sizeof(int), 1, ffp_out);

	fwrite(&rgb_4byte_header, sizeof(int), 1, rgb_out);


	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			//printf("\nSector %d x %d\n\n", sec_z, sec_x);

			number_of_points = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_points;

			/////////////////////////////////////////////////////////////////
			// write all the stats for each sector to sector file
			/////////////////////////////////////////////////////////////////

			fwrite(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_out);
		
			//printf("%d points\n", terrain_sectors[sec_z][sec_x].number_of_points); 

			fwrite(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_out);

			//printf("%d pad_2_bytes\n", terrain_sectors[sec_z][sec_x].pad_2_bytes); 

			fwrite(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_out);

			//printf("%d polygons\n", terrain_sectors[sec_z][sec_x].number_of_polygons); 

			fwrite(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_out);

			//printf("%d radius\n", terrain_sectors[sec_z][sec_x].radius);

			fwrite(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_out);

			//printf("%d minimum_height\n", terrain_sectors[sec_z][sec_x].minimum_height);

			fwrite(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_out);

			//printf("%d maximum_height\n", terrain_sectors[sec_z][sec_x].maximum_height);

			fwrite(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_out);

			//printf("%d average_height\n", terrain_sectors[sec_z][sec_x].average_height);

			fwrite(&pad_word_zero, sizeof(short int), 1, sec_out);

			///////////////////////////////////////////////////////////////////
			///////////////////////////////////////////////////////////////////
			// write the geometry data to the ffp file
			///////////////////////////////////////////////////////////////////
			///////////////////////////////////////////////////////////////////

			//
			// this section is for the first all sea sector 
			// which is used for every all sea sector
			//

			if( ( map_width == 244 && sec_z == 0 && sec_x == 0 ) || // Georgia
				( map_width == 76 && sec_z == 4 && sec_x == 23 ) || // Lebanon
				( map_width == 240 && sec_z == 0 && sec_x == 0 ) || // Cuba
				( map_width == 124 && sec_z == 0 && sec_x == 0 )){  // Taiwan

				//
				// this chunk of data is used for all sectors
				// that are all sea
				//

				//////////////////////////
				// points y
				//////////////////////////

				offset = (int)ftell(ffp_out);

				all_sea_points_y_offset = offset;

				//printf("\tpoints y offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
					terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

				/////////////////////////
				// points xz
				/////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpoints xz offset %d\n", offset);

				all_sea_points_xz_offset = offset;

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
					terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

				//////////////////////////
				// point references
				//////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpoint references offset %d\n", offset);

				all_sea_point_references_offset = offset;

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].point_byte_references,
					sizeof(terrain_3d_point_byte_reference),
					terrain_sectors[sec_z][sec_x].number_of_point_references, ffp_out);

				//////////////////////////
				// polygons
				//////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpolygons offset %d\n", offset);

				all_sea_polygons_offset = offset;

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face), // num points & surface changes
					terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_out); 
			
				//
				// if number of polygons is odd need to add a 0 pad byte
				//

				if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){

					fwrite( &pad_byte_zero, 1, 1, ffp_out);

				}

				/////////////////////////////
				// surface changes
				/////////////////////////////
			
				offset = (int)ftell(ffp_out);

				//printf("\tsurface changes offset %d\n", offset);

				all_sea_surface_changes_offset = offset;

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].surface_changes, 
					sizeof(terrain_3d_surface),
					terrain_sectors[sec_z][sec_x].number_of_surfaces,
					ffp_out);

				//printf("number of surfacs = %d\n", 
				//	terrain_sectors[sec_z][sec_x].number_of_surfaces);

				//
				// if the number of surfaces is odd
				// need to add a pad byte zero at the end
				//

				if(terrain_sectors[sec_z][sec_x].number_of_surfaces % 2 != 0){

					fwrite(&pad_byte_zero, sizeof(pad_byte_zero), 1, ffp_out);
				}

				//////////////////////////////
				// normal changes
				//////////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tnormal changes offset %d\n", offset);

				all_sea_normal_changes_offset = offset;

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].normal_changes, 
					sizeof(terrain_3d_normal_index),
					terrain_sectors[sec_z][sec_x].number_of_normal_changes,
					ffp_out);

				//
				// write rgb offsets
				//
/*
				fwrite(&terrain_sectors[sec_z][sec_x].colour_indices_offset,
					sizeof(int), 1, sec_out); // sector file

				fwrite(&terrain_sectors[sec_z][sec_x].point_colours_offset,
					sizeof(int), 1, sec_out); // sector file
*/
				rgb_offset = (int)ftell(rgb_out);

				//
				// write rgb offset
				//

				fwrite(&rgb_offset,	sizeof(int), 1, sec_out); // sector file

				//
				// write the color file
				//

				fwrite(terrain_sectors[sec_z][sec_x].colour_indices, 
					sizeof(terrain_3d_colour_index),
					terrain_sectors[sec_z][sec_x].number_of_points, rgb_out);

				rgb_offset = (int)ftell(rgb_out);

				//
				// write rgb offset
				//

				fwrite(&rgb_offset,	sizeof(int), 1, sec_out); // sector file

				//
				// write the color file
				//

				fwrite(terrain_sectors[sec_z][sec_x].point_colours, 
					sizeof(terrain_3d_colour),
					terrain_sectors[sec_z][sec_x].number_of_colors, rgb_out);

			} // end if sec_z == 0 && sec_x ==0

			//
			// this section is used for all other all sea sectors
			// which use the same data as the first sector
			// the offsets are the same as for the first all sea sector
			//

			else if ( terrain_sectors[sec_z][sec_x].all_sea_sector ){

				//
				// this section is for all remaining
				// sectors that are all sea
				// which points to sector 0 x 0
				//

				//////////////////////////
				// points y
				//////////////////////////

				offset = all_sea_points_y_offset;

				//printf("\tpoints y offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				/////////////////////////
				// points xz
				/////////////////////////

				offset = all_sea_points_xz_offset;

				//printf("\tpoints xz offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				//////////////////////////
				// point references
				//////////////////////////

				offset = all_sea_point_references_offset;

				//printf("\tpoint references offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				//////////////////////////
				// polygons
				//////////////////////////

				offset = all_sea_polygons_offset;

				//printf("\tpolygons offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file
				
				/////////////////////////////
				// surface changes
				/////////////////////////////
			
				offset = all_sea_surface_changes_offset;

				//printf("\tsurface changes offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				//////////////////////////////
				// normal changes
				//////////////////////////////

				offset = all_sea_normal_changes_offset;

				//printf("\tnormal changes offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				//
				// write rgb offsets
				//
/*
				fwrite(&terrain_sectors[sec_z][sec_x].colour_indices_offset,
					sizeof(int), 1, sec_out); // sector file

				fwrite(&terrain_sectors[sec_z][sec_x].point_colours_offset,
					sizeof(int), 1, sec_out); // sector file
*/
				
				rgb_offset = (int)ftell(rgb_out);

				//
				// write rgb offset
				//

				fwrite(&rgb_offset,	sizeof(int), 1, sec_out); // sector file

				//
				// write the color file
				//

				fwrite(terrain_sectors[sec_z][sec_x].colour_indices, 
					sizeof(terrain_3d_colour_index),
					terrain_sectors[sec_z][sec_x].number_of_points, rgb_out);

				rgb_offset = (int)ftell(rgb_out);

				//
				// write rgb offset
				//

				fwrite(&rgb_offset,	sizeof(int), 1, sec_out); // sector file

				//
				// write the color file
				//

				fwrite(terrain_sectors[sec_z][sec_x].point_colours, 
					sizeof(terrain_3d_colour),
					terrain_sectors[sec_z][sec_x].number_of_colors, rgb_out);

			} // end else all sea sector

			//
			// this section is for all others sectors
			// which have some land and need their own data
			//

			else{


				//////////////////////////
				// points y
				//////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpoints y offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
					terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

				/////////////////////////
				// points xz
				/////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpoints xz offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
					terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

				//////////////////////////
				// point references
				//////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpoint references offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				if( number_of_points <= 256 ){

					fwrite(terrain_sectors[sec_z][sec_x].point_byte_references,
						sizeof(terrain_3d_point_byte_reference),
						terrain_sectors[sec_z][sec_x].number_of_point_references, ffp_out);
				}
				else{

					fwrite(terrain_sectors[sec_z][sec_x].point_word_references,
						sizeof(terrain_3d_point_word_reference),
						terrain_sectors[sec_z][sec_x].number_of_point_references, ffp_out);
				}

				//////////////////////////
				// polygons
				//////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tpolygons offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face), // num points & surface changes
					terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_out); 
			
				//
				// if number of polygons is odd need to add a 0 pad byte
				//

				if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){

					fwrite( &pad_byte_zero, 1, 1, ffp_out);

				}

				/////////////////////////////
				// surface ids
				/////////////////////////////
			
				offset = (int)ftell(ffp_out);

				//printf("\tsurface changes offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].surface_changes, 
					sizeof(terrain_3d_surface),
					terrain_sectors[sec_z][sec_x].number_of_surfaces,
					ffp_out);

				//printf("number of surfacs = %d\n", 
				//	terrain_sectors[sec_z][sec_x].number_of_surfaces);

				//
				// if the number of surfaces is odd
				// need to add a pad byte zero at the end
				//

				if(terrain_sectors[sec_z][sec_x].number_of_surfaces % 2 != 0){

					fwrite(&pad_byte_zero, sizeof(pad_byte_zero), 1, ffp_out);
				}

				//////////////////////////////
				// normal changes
				//////////////////////////////

				offset = (int)ftell(ffp_out);

				//printf("\tnormal changes offset %d\n", offset);

				fwrite(&offset, sizeof(int), 1, sec_out); // sector file

				fwrite(terrain_sectors[sec_z][sec_x].normal_changes, 
					sizeof(terrain_3d_normal_index),
					terrain_sectors[sec_z][sec_x].number_of_normal_changes,
					ffp_out);
/*
				//
				// write rgb offsets - this is for copying the original rgb date
				// which includes some colors that aren't used
				//

				fwrite(&terrain_sectors[sec_z][sec_x].colour_indices_offset,
					sizeof(int), 1, sec_out); // sector file

				fwrite(&terrain_sectors[sec_z][sec_x].point_colours_offset,
					sizeof(int), 1, sec_out); // sector file
*/
				rgb_offset = (int)ftell(rgb_out);

				//
				// write rgb offset
				//

				fwrite(&rgb_offset,	sizeof(int), 1, sec_out); // sector file

				//
				// write the color file
				//

				fwrite(terrain_sectors[sec_z][sec_x].colour_indices, 
					sizeof(terrain_3d_colour_index),
					terrain_sectors[sec_z][sec_x].number_of_points, rgb_out);

				rgb_offset = (int)ftell(rgb_out);

				//
				// write rgb offset
				//

				fwrite(&rgb_offset,	sizeof(int), 1, sec_out); // sector file

				//
				// write the color file
				//

				fwrite(terrain_sectors[sec_z][sec_x].point_colours, 
					sizeof(terrain_3d_colour),
					terrain_sectors[sec_z][sec_x].number_of_colors, rgb_out);
			

			} // end else for sectors that have some land

			
		}
	}

	// write out the last four bytes of sector file (four bytes of zeros)

	fwrite(&four_byte_zero_word, sizeof(int), 1, sec_out);

	
	fclose(sec_out);
	
	fclose(ffp_out);

	fclose(rgb_out);

} // end write new terrain files

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void find_forest_sectors (void){

	int
		sec_x,
		sec_z,
		number_of_surfaces,
		surface_loop,
		surface;

	printf("finding forest sectors ...\n");
	
	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			number_of_surfaces = terrain_sectors[sec_z][sec_x].number_of_surfaces;

			for( surface_loop = 0; surface_loop < number_of_surfaces; surface_loop ++ ){

				surface = (int)terrain_sectors[sec_z][sec_x].surface_changes [ surface_loop ].surface_id;

				if( surface == TERRAIN_TYPE_FOREST_FLOOR ||
					surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
					surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
					surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
					surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
					surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
					surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ||
					surface == TERRAIN_TYPE_FOREST_TOP ){

					terrain_sectors[sec_z][sec_x].forest_sector = 1;
				}
				else{

					terrain_sectors[sec_z][sec_x].forest_sector = 0;
				}
			} // end surface loop

			//if( terrain_sectors[sec_z][sec_x].forest_sector == 1 ){

			//	print_forest_sector_data ( sec_z, sec_x );
			//}

		} // end sec x loop
	} // end sec z loop
} // end find forest sectors

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void print_forest_sector_data (int sec_z, int sec_x){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		color_index,
		FOREST_FLOOR = 0,
		FOREST_SIDE_BOTTOM_X = 0,
		FOREST_SIDE_BOTTOM_Z = 0,
		FOREST_SIDE_MID_X = 0,
		FOREST_SIDE_MID_Z = 0,
		FOREST_SIDE_TOP_X = 0,
		FOREST_SIDE_TOP_Z = 0,
		FOREST_TOP = 0;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	//
	// print section
	//

	printf("\nSector %d x %d --- %d points  %d polygons \n\n",
		sec_z, sec_x, number_of_points, number_of_polys);
/*
	printf("Point  X  Z  Y  Color index  Color\n\n");

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		color_index = terrain_sectors[sec_z][sec_x].colour_indices [ point_loop ].index;

		printf("%3d    %6d  %6d  %6d     %3d     %3d  %3d  %3d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].x,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].z,
			terrain_sectors[sec_z][sec_x].points_y[point_loop].y,
			color_index,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].red,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].green,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].blue);
	}

	printf("\nPolygon	Number of points	Surface\n\n");
*/
	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
/*
		if( surface == TERRAIN_TYPE_FOREST_FLOOR ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ||
			surface == TERRAIN_TYPE_FOREST_TOP ){

			printf("Poly	%3d		has %d points	suface = %3d\n\n", poly_loop, number_of_poly_points, surface);

			printf("\tpoly point   point reference   X   Z   Y\n\n");

		} // end if surface is forest
*/

		printf("Poly	%3d		has %d points	suface = %3d\n\n", poly_loop, number_of_poly_points, surface);

		printf("\tpoly point   point reference   X   Z   Y\n\n");

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

		//
		// print section
		//
/*
			if( surface == TERRAIN_TYPE_FOREST_FLOOR ||
				surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
				surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
				surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
				surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
				surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
				surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ||
				surface == TERRAIN_TYPE_FOREST_TOP ){

				printf("\t%2d	%3d    %6d  %6d  %6d\n", poly_point_loop, point_reference,
					terrain_sectors[sec_z][sec_x].points_xz[point_reference].x,
					terrain_sectors[sec_z][sec_x].points_xz[point_reference].z,
					terrain_sectors[sec_z][sec_x].points_y[point_reference].y);

				printf("\n");

			} // end if forest sector
*/
		printf("\t%2d	%3d    %6d  %6d  %6d\n", poly_point_loop, point_reference,
			terrain_sectors[sec_z][sec_x].points_xz[point_reference].x,
			terrain_sectors[sec_z][sec_x].points_xz[point_reference].z,
			terrain_sectors[sec_z][sec_x].points_y[point_reference].y);

		printf("\n");

		} // end poly point loop

	} // end poly loop
/*
		//
		// test if forest floor is used
		// and if sides by themselves poke across sector boundaries
		//

		if( surface == TERRAIN_TYPE_FOREST_FLOOR ) FOREST_FLOOR = 1;
		else if( surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ) FOREST_SIDE_BOTTOM_X = 1;
		else if( surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ) FOREST_SIDE_BOTTOM_Z = 1;
		else if( surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ) FOREST_SIDE_MID_X = 1;
		else if( surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ) FOREST_SIDE_MID_Z = 1;
		else if( surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ) FOREST_SIDE_TOP_X = 1;
		else if( surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ) FOREST_SIDE_TOP_Z = 1;
		else if( surface == TERRAIN_TYPE_FOREST_TOP ) FOREST_TOP = 1;

	} // end for poly loop

	if( FOREST_FLOOR ){

		printf("forest floor in sector z = %d   x = %d\n", sec_z, sec_x);
	}

	if( ( FOREST_SIDE_BOTTOM_X ||
		FOREST_SIDE_BOTTOM_Z ||
		FOREST_SIDE_MID_X ||
		FOREST_SIDE_MID_Z ||
		FOREST_SIDE_TOP_X ||
		FOREST_SIDE_TOP_Z ) &&
		!FOREST_TOP ){

		printf("forest side and no top in sector z = %d   x = %d\n", sec_z, sec_x);
	}
*/
} // end print forest sector data

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void get_points_directly_below_forest_top_points (int sec_z, int sec_x){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		top_point_x,
		top_point_z,
		top_point_y,
		point_x,
		point_z,
		point_y,
		forest_height,
		bottom_point_found,
		number_of_below_points = 0,
		total_forest_height = 0,
		average_forest_height;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	printf("\nSector %d x %d Below Points --- %d points  %d polygons \n\n",
		sec_z, sec_x, number_of_points, number_of_polys);

	//
	// find forest top polys
	//

	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

			if ( surface == TERRAIN_TYPE_FOREST_TOP ){

				top_point_x = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_reference ].x;
				top_point_z = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_reference ].z;
				top_point_y = (int)terrain_sectors[sec_z][sec_x].points_y[ point_reference ].y;

				bottom_point_found = 0;

				printf("forest top point %d\n", point_reference);

				for(point_loop = 0; point_loop < number_of_points; point_loop ++){

					point_x = (int)terrain_sectors[sec_z][sec_x].points_xz[point_loop].x;
					point_z = (int)terrain_sectors[sec_z][sec_x].points_xz[point_loop].z;
					point_y = (int)terrain_sectors[sec_z][sec_x].points_y[point_loop].y;

					forest_height = top_point_y - point_y;

					if( ( point_x == top_point_x && point_z == top_point_z ) &&
						( point_reference != point_loop ) ){

						bottom_point_found = 1;

						number_of_below_points ++;

						total_forest_height += forest_height;

						printf("    below point %d ---  %d - %d = %d\n\n",
							point_loop, top_point_y, point_y, forest_height);
					}
					
				} // end point loop

				if(  ( ! bottom_point_found ) &&
					( (top_point_x  == 16384 && top_point_z == 16384) ||
					(top_point_x == 16384 && top_point_z == -16384) ||
					(top_point_x  == -16384 && top_point_z == 16384) ||
					(top_point_x  == -16384 && top_point_z == -16384) )  ){

					printf("no bottom point in sector %d x %d for top point %d  %d\n", sec_z, sec_x,
						top_point_x, top_point_z);
				}

			} // end if point is a forest top point

		} // end poly point loop

	} // end poly loop
/*
	if( number_of_below_points ){

		average_forest_height = total_forest_height / number_of_below_points;

		printf("sector %d x %d  average forest height %d\n", sec_z, sec_x, average_forest_height);
	}
	else{

		printf("Sector %d x %d no below points\n");
	}
*/
	

} // end get_points_directly_below_forest_top_points

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void remove_forest_sides (int sec_z, int sec_x){

	terrain_3d_face
		new_polygons [ 2048 ];

	terrain_3d_point_word_reference
		//new_point_word_references [ 2048 ];
		new_point_word_references [ 4096 ];

	terrain_3d_point_byte_reference
		//new_point_byte_references [ 2048 ];
		new_point_byte_references [ 4096 ];

	terrain_3d_surface
		new_surface_changes [ 64 ];

	int
		number_of_surfaces,
		number_of_polygons,
		number_of_poly_points,
		number_of_point_references,
		new_poly_count = 0,
		new_surface_count = 0,
		point_ref_counter = 0,
		point_reference,
		new_point_ref_counter = 0,
		point_ref_loop,
		poly_loop,
		surface_loop,
		surface,
		surface_changes_index = 0,
		surface_change,
		poly_point_loop;

	number_of_surfaces = terrain_sectors[sec_z][sec_x].number_of_surfaces;

	number_of_polygons = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	//printf("%d x %d  surf %d  polys %d  ref %d", sec_z, sec_x, number_of_surfaces, number_of_polygons,
	//	terrain_sectors[sec_z][sec_x].number_of_point_references);

	for( poly_loop = 0; poly_loop < number_of_polygons; poly_loop ++ ){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;


		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			if( ! ( surface == TERRAIN_TYPE_FOREST_FLOOR ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ) ){

				///////////////////////////////////////
				// new_surface_changes (surface id's)
				///////////////////////////////////////

				new_surface_changes [ new_surface_count ] =
					terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ];

				//surface_changes_index ++;

				new_surface_count ++;
			}

			surface_changes_index ++;
			
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			if( ! ( surface == TERRAIN_TYPE_FOREST_FLOOR ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ) ){

				/////////////////////////////////////////
				// new_surface_changes (surface id's)
				/////////////////////////////////////////

				new_surface_changes [ new_surface_count ] =
					terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ];

				//surface_changes_index ++;

				new_surface_count ++;
			}

			surface_changes_index ++;
		}

		if( ! ( surface == TERRAIN_TYPE_FOREST_FLOOR ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
			surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ) ){

			////////////////////////////////////////////////////////////
			// new_polygons (surface change bit and number of points)
			////////////////////////////////////////////////////////////

			new_polygons [ new_poly_count ] = terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ];

			new_poly_count ++;

		} // end if surface is NOT forest side

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				//point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				//point_ref_counter ++;
			}

		
			if( ! ( surface == TERRAIN_TYPE_FOREST_FLOOR ||
				surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X ||
				surface == TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z ||
				surface == TERRAIN_TYPE_FOREST_SIDE_MID_X ||
				surface == TERRAIN_TYPE_FOREST_SIDE_MID_Z ||
				surface == TERRAIN_TYPE_FOREST_SIDE_TOP_X ||
				surface == TERRAIN_TYPE_FOREST_SIDE_TOP_Z ) ){

				if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

					////////////////////////////////////////////
					// new_point_byte_references
					////////////////////////////////////////////

					new_point_byte_references [ new_point_ref_counter ]  =
						terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ];

					new_point_ref_counter ++;
				}
				else{

					////////////////////////////////////////////
					// new_point_word_references
					////////////////////////////////////////////

					new_point_word_references [ new_point_ref_counter ] =
						terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ];

					new_point_ref_counter ++;
				}

			} // end if surface is NOT forest side

			point_ref_counter ++;

		} // end poly point loop

	} // end poly loop

	//////////////////////////////////////////////////
	// copy new data back to original arrays
	// and change the counts
	//////////////////////////////////////////////////

	//////////////////////////////////////////
	// new_surface_changes  ( surface id's )
	//////////////////////////////////////////

	for( surface_loop = 0; surface_loop < new_surface_count; surface_loop ++ ){

		terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_loop ] =
			new_surface_changes [ surface_loop ];

	} // end surface loop

	terrain_sectors[ sec_z ][ sec_x ].number_of_surfaces = new_surface_count;

	////////////////////////////////////////////////////////////////
	// new_polygons  ( surface change bit and number of points )
	////////////////////////////////////////////////////////////////

	for( poly_loop = 0; poly_loop < new_poly_count; poly_loop ++ ){

		terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ] =
			new_polygons [ poly_loop ];

		if( poly_loop == 0 ){

			terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change = 0;
		}

	} // end poly loop

	terrain_sectors[ sec_z ][ sec_x ].number_of_polygons = (unsigned short int) new_poly_count;

	//printf("   new surf %d   new polys %d", new_surface_count, new_poly_count);

	////////////////////////////////////////////
	// new_point_references
	////////////////////////////////////////////

	for( point_ref_loop = 0; point_ref_loop < new_point_ref_counter; point_ref_loop ++ ){

		if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

			////////////////////////////////////////////
			// new_point_byte_references
			////////////////////////////////////////////

			terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_loop ] =
				new_point_byte_references [ point_ref_loop ];

		}
		else{

			////////////////////////////////////////////
			// new_point_word_references
			////////////////////////////////////////////

			terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_loop ] =
				new_point_word_references [ point_ref_loop ];

		}

	} // end point ref loop

	terrain_sectors[ sec_z ][ sec_x ].number_of_point_references = new_point_ref_counter;

	//printf("  new_ref %d\n", new_point_ref_counter);

} // end remove forest sides

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void lower_forest_top (int sec_z, int sec_x){

	
	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		top_point_x,
		top_point_z,
		top_point_y,
		point_x,
		point_z,
		point_y,
		forest_height,
		below_point_found,
		adjacent_sector_y_value,
		corner_corrected,
		top_point_reference,
		below_point_references [ 4 ],
		below_point_reference_count,
		below_point_loop,
		lowest_point_reference,
		point_ref_lowered[ 2048 ],
		below_delta = 100;
	
	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		point_ref_lowered[ point_loop ] = 0;
	}

	//printf("\nSector %d x %d --- %d points  %d polygons \n\n",
	//	sec_z, sec_x, number_of_points, number_of_polys);

	//
	// find forest top polys
	//

	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

			if ( surface == TERRAIN_TYPE_FOREST_TOP ){

				top_point_reference = point_reference;

				top_point_x = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_reference ].x;
				top_point_z = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_reference ].z;
				top_point_y = (int)terrain_sectors[sec_z][sec_x].points_y[ point_reference ].y;

				below_point_found = 0;

				corner_corrected = 0;

				below_point_reference_count = 0;

				//printf("forest top point %d\n", point_reference);

				for(point_loop = 0; point_loop < number_of_points; point_loop ++){

					point_x = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x;
					point_z = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z;
					point_y = (int)terrain_sectors[sec_z][sec_x].points_y[ point_loop ].y;

					//
					// it is possible to have more than one bottom point
					// at the edge of walls
					// need to find both and pick the lowest
					//

					if( ( point_x == top_point_x && point_z == top_point_z ) &&
						( point_loop != top_point_reference ) ){

						below_point_found = 1;

						below_point_references [ below_point_reference_count ] = point_loop;

						below_point_reference_count ++;

					} // end if bottom point found
					
				} // end point loop

				/////////////////////////////////////////////////////////
				// change the forest top point reference
				// to the lowest below point reference
				// if below point is found
				/////////////////////////////////////////////////////////

				if( below_point_found ){

					//
					// find the lowest point
					//

					lowest_point_reference = below_point_references [ 0 ];

					for( below_point_loop = 1; below_point_loop < below_point_reference_count; below_point_loop ++ ){

						if( terrain_sectors[sec_z][sec_x].points_y[ below_point_references [ below_point_loop ] ].y <
							terrain_sectors[sec_z][sec_x].points_y[ lowest_point_reference ].y ){

							lowest_point_reference = below_point_references [ below_point_loop ];
						}
					}
					
					//
					// make the change
					//

					if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

						terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter - 1 ].point =
							(unsigned char) lowest_point_reference;
					}
					else{

						terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter - 1 ].point =
							(unsigned short int) lowest_point_reference;
					}

					below_delta = top_point_y - (int)terrain_sectors[sec_z][sec_x].points_y[ lowest_point_reference ].y;

				} // end if below point found

				/////////////////////////////////////////////////////
				// if the bottom point wasn't found and the point
				// is a corner point need to check adjacent sector
				// to see if y values match up
				/////////////////////////////////////////////////////

				if( ! below_point_found &&
					( top_point_x == 16384 || top_point_x == -16384 ) && 
					( top_point_z == 16384 || top_point_z == -16384 ) ){

					adjacent_sector_y_value = 
						get_y_value_from_adjacent_corner_sector ( sec_z, sec_x, top_point_z, top_point_x, top_point_y );

					if ( adjacent_sector_y_value && ( adjacent_sector_y_value != top_point_y ) ){

						terrain_sectors[sec_z][sec_x].points_y[point_reference].y = 
							(unsigned short int)adjacent_sector_y_value;

						corner_corrected = 1;

						//printf("corrected corner point %d x %d in sector %d x %d\n",
						//	top_point_z, top_point_x, sec_z, sec_x);
					}
				}

				/////////////////////////////////////////////////////
				// lower the rest of the forest top points
				// an average value of 100 if they aren't edge points
				/////////////////////////////////////////////////////
/*
				if( ! below_point_found && 
					! corner_corrected &&
					top_point_x != 16384 &&
					top_point_x != -16384 &&
					top_point_z != 16384 &&
					top_point_z != -16384 ){
*/

				if( ! below_point_found && 
					! corner_corrected &&
					! point_ref_lowered [ point_reference ] ){

					terrain_sectors[sec_z][sec_x].points_y[ point_reference ].y -= below_delta;

					point_ref_lowered [ point_reference ] = 1;
/*
					if( sec_z < (map_height - 1) && sec_x < (map_width - 1) &&
						( top_point_x == 16384 || top_point_x == -16384 || top_point_z == 16384 || top_point_z == -16384 ) ){

						//printf("changed y value of edge point %d  %d in sector %d x %d\n", bz, bx, sec_z, sec_x);

						match_adjacent_sector_point ( sec_z, sec_x, top_point_z, top_point_x, 
							terrain_sectors[sec_z][sec_x].points_y[ point_reference ].y);
					}
*/
				}

			} // end if surface == TERRAIN_TYPE_FOREST_TOP

		} // end poly point loop

	} // end poly loop

} // end lower forest top

///////////////////////////
///////////////////////////
//						 //
//						 //
//						 //
//						 //
///////////////////////////
///////////////////////////

void create_tree_files(int max_z, int max_x){

	FILE
		*tree_sec,
		*tree_data,
		*fp;

	int	
		offset_counter_next = 0,
		offset_counter_last = 0,
		sec_x,
		sec_z,
		i,
		j,
		while_runaway_check,
		tempPoint,
		points_this_poly,
		poly_point_loop,
		point_ref_counter,
		triangle_counter,
		surface_changes_counter,
		poly_loop,
		polys_this_sector,
		trees_this_sector,
		trees_this_poly,
		x,
		z,
		min_y,
		max_y,
		average_y,
		x_bound_max,
		x_bound_min,
		z_bound_max,
		z_bound_min,
		tree_index,
		space_ok,
		num_trees,
		face_orientation,
		locality_count,
		dissolve_ratio_factor,
		dissolve_trees_this_sector,
		solid_trees_this_sector,
		highest_tree_count,
		highest_x_sector,
		highest_z_sector,
		dissolve_ratio_counter,
		elevation_and_slope_okay,
		number_of_sectors_with_max_trees,
		random_number,
		tree_count;

	
	unsigned int	
			total_tree_count = 0;
			
	long long
		k, // plane constant
		poly_points[128][3], // [][0]=x, [][1]=z, [][2]=y
		triangle_points[3][3], // [][0]=x, [][1]=z, [][2]=y
		px,pz,	// randon point
		Ax,Az,Ay,Bx,Bz,By, // vectors
		Nx,Nz,Ny, // normal vector
		min_space = 400,
		density_factor = 10, // bigger = less trees
		space,
		tree_points[1024][3]; // [][0]=x, [][1]=z, [][2]=y
			

	float
		terrain_3d_map_scaled_height_difference,
		elevation,
		slope;
	
	terrain_3d_tree_data
		local_tree_data;

	Point
		inside_poly_array[3],
		random_point;

	terrain_3d_surface
		current_surface;

	////////////////////////////////////////////////////
	// end of variables
	////////////////////////////////////////////////////

	printf("planting trees ...\n");
		
	number_of_sectors_with_max_trees = 0;

	dissolve_ratio_factor = DISSOLVE_TO_SOLID_RATIO + 1;

	highest_tree_count = 0;

	dissolve_ratio_counter = 0;

	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height )
		* TERRAIN_3D_Y_SCALE;

	local_tree_data.colour = 0;
	local_tree_data.height = 35;
	local_tree_data.type = 1;

	//////////////////////////////////////////////////////////////////////////////////
	// my output files
	//////////////////////////////////////////////////////////////////////////////////
/*
	if(( tree_sec = fopen("C:\\Razorworks\\common\\maps\\map3\\terrain\\tree_pos.sec","wb"))==NULL){
		printf("cannot open C:\\Razorworks\\common\\maps\\map3\\terrain\\tree_pos.sec\n");
		return;
	}
	else printf("C:\\Razorworks\\common\\maps\\map3\\terrain\\tree_pos.sec opened\n");

	if(( tree_data = fopen("C:\\Razorworks\\common\\maps\\map3\\terrain\\tree_pos.dat","wb"))==NULL){
		fclose(tree_sec);
		printf("cannot open C:\\Razorworks\\common\\maps\\map3\\terrain\\tree_pos.dat\n");
		return;
	}
	else printf("C:\\Razorworks\\common\\maps\\map3\\terrain\\tree_pos.dat opened\n");
*/

	if(( tree_sec = fopen("tree_pos.sec","wb"))==NULL){
		printf("cannot open tree_pos.sec\n");
		exit_function ();
	}
	else printf("tree_pos.sec opened\n");

	if(( tree_data = fopen("tree_pos.dat","wb"))==NULL){
		fclose(tree_sec);
		printf("cannot open tree_pos.dat\n");
		exit_function ();
	}
	else printf("tree_pos.dat opened\n");


	if(( fp = fopen("TREE_PLANT.log","w"))==NULL){
		printf("cannot open TREE_PLANT.log\n");
		exit_function ();
	}
	else printf("TREE_PLANT.log opened\n");

	printf("this could take a few minutes ...\n");
	printf("Please Wait ...\n");

	//
	// write map size to tree sector file
	//

	fwrite(&max_x, sizeof(int), 1, tree_sec);
	fwrite(&max_z, sizeof(int), 1, tree_sec);

	
	fprintf(fp, "%-34s%-10d\n", "DISSOLVE_TO_SOLID_RATIO", DISSOLVE_TO_SOLID_RATIO);
	fprintf(fp, "%-34s%-10d\n", "MAX_TREES_PER_SECTOR", MAX_TREES_PER_SECTOR);
	fprintf(fp, "\n");
	fprintf(fp, "%-15s%-15s%-15s%-15s\n\n",
		"Sector X x Z", "Total Trees", "Solid Trees", "Dissolve Trees");

		
	//////////////////////////////////////////////////////////////////////////
	// Loop through all sectors
	//////////////////////////////////////////////////////////////////////////

	for(sec_z = 0; sec_z < max_z; sec_z++){
		for(sec_x = 0; sec_x < max_x; sec_x++){
	//for(sec_z = 9; sec_z < 10; sec_z++){
	//	for(sec_x = 48; sec_x < 49; sec_x++){

			trees_this_sector = 0;

			if ( ! ( terrain_sectors[sec_z][sec_x].forest_sector ) ){

				///////////////////////////////////////////////////////
				// write out the tree sector file
				///////////////////////////////////////////////////////

				fwrite(&trees_this_sector, sizeof(int), 1, tree_sec);
				fwrite(&offset_counter_last, sizeof(int), 1, tree_sec);
			
				offset_counter_last = offset_counter_next;

				continue;

			}

			//printf("\nPLANTING trees in sector %d x %d\n\n", sec_z, sec_x);

			surface_changes_counter = 0;

			current_surface.surface_id = terrain_sectors[sec_z][sec_x].surface_changes[0].surface_id;
			
			polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

			dissolve_trees_this_sector = 0;

			solid_trees_this_sector = 0;

			//////////////////////////////////////////////////////////////////////////
			//////////////////////////////////////////////////////////////////////////
			// START Polygon Loop
			//////////////////////////////////////////////////////////////////////////
			//////////////////////////////////////////////////////////////////////////

			point_ref_counter = 0;

			for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop++){

				//
				// check for next surface id and update current_surface
				//

				if(terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change){ 
					surface_changes_counter ++;
					current_surface.surface_id = 
						terrain_sectors[sec_z][sec_x].surface_changes[surface_changes_counter].surface_id;
				}

				points_this_poly = (int)terrain_sectors[sec_z][sec_x].polygons[poly_loop].number_of_points;

				////////////////////////////////////////////////////////////////////////////////////////////////
				// assign the points for this polygon
				// need to assign the points whether the surface id is right or not
				// because of the sequential way the points are stored
				////////////////////////////////////////////////////////////////////////////////////////////////

				for( poly_point_loop = 0; poly_point_loop < points_this_poly; poly_point_loop ++ ){

					if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

						poly_points[ poly_point_loop ][ 0 ] = (long long)terrain_sectors[sec_z][sec_x].
							points_xz[(int)terrain_sectors[sec_z][sec_x].
							point_byte_references[point_ref_counter].point].x;
						poly_points[ poly_point_loop ][ 1 ] = (long long)terrain_sectors[sec_z][sec_x].
							points_xz[(int)terrain_sectors[sec_z][sec_x].
							point_byte_references[point_ref_counter].point].z;
						poly_points[ poly_point_loop ][ 2 ] = (long long)terrain_sectors[sec_z][sec_x].
							points_y[(int)terrain_sectors[sec_z][sec_x].
							point_byte_references[point_ref_counter].point].y;

					}
					else{

						poly_points[ poly_point_loop ][ 0 ] = (long long)terrain_sectors[sec_z][sec_x].
							points_xz[(int)terrain_sectors[sec_z][sec_x].
							point_word_references[point_ref_counter].point].x;
						poly_points[ poly_point_loop ][ 1 ] = (long long)terrain_sectors[sec_z][sec_x].
							points_xz[(int)terrain_sectors[sec_z][sec_x].
							point_word_references[point_ref_counter].point].z;
						poly_points[ poly_point_loop ][ 2 ] = (long long)terrain_sectors[sec_z][sec_x].
							points_y[(int)terrain_sectors[sec_z][sec_x].
							point_word_references[point_ref_counter].point].y;

					}

					point_ref_counter++;

				} // end for assigning points_this_poly

				/////////////////////////////////////////////////////////////////////////////////////////
				// now start checking if we want any trees on this polygon
				/////////////////////////////////////////////////////////////////////////////////////////

				trees_this_poly = 0;
				tree_index = 0;

				if(current_surface.surface_id == TERRAIN_TYPE_FOREST_TOP &&
					trees_this_sector <= MAX_TREES_PER_SECTOR &&
					points_this_poly > 3){

					trees_this_poly = 1;
					//printf("POLYGON %d  points this poly = %d\n", poly_loop, points_this_poly);
				}

				///////////////////////////////////////////////////////////////////////////////////
				// start calculations and planting trees if we want any
				///////////////////////////////////////////////////////////////////////////////////

				if( trees_this_poly ){

					//
					// break 3d_face into triangles because 3d_faces aren't always flat
					//

					triangle_points[0][0] = poly_points[0][0];
					triangle_points[0][1] = poly_points[0][1];
					triangle_points[0][2] = poly_points[0][2];

					for(triangle_counter = 0; triangle_counter < (points_this_poly - 2); triangle_counter++){

						//printf("triangle loop %d\n", triangle_counter);

						triangle_points[1][0] = poly_points[triangle_counter + 1][0];
						triangle_points[1][1] = poly_points[triangle_counter + 1][1];
						triangle_points[1][2] = poly_points[triangle_counter + 1][2];
						triangle_points[2][0] = poly_points[triangle_counter + 2][0];
						triangle_points[2][1] = poly_points[triangle_counter + 2][1];
						triangle_points[2][2] = poly_points[triangle_counter + 2][2];

						// vectors
						Ax = triangle_points[0][0] - triangle_points[1][0]; // ax - bx
						Ay = triangle_points[0][2] - triangle_points[1][2]; // ay - by
						Az = triangle_points[0][1] - triangle_points[1][1]; // az - bz
						Bx = triangle_points[2][0] - triangle_points[1][0]; // cx - bx
						By = triangle_points[2][2] - triangle_points[1][2]; // cy - by
						Bz = triangle_points[2][1] - triangle_points[1][1]; // cz - bz

						// normal
						Nx = Ay * Bz - Az * By;
						Ny = Az * Bx - Ax * Bz;
						Nz = Ax * By - Ay * Bx;
				
						// plane constant
						//k = ax * Nx + ay * Ny + az * Nz;
						k = triangle_points[0][0] * Nx + triangle_points[0][2] * Ny + triangle_points[0][1] * Nz;

						//
						// find the bounding rectangle for the triangle
						//

						x_bound_min = (int)triangle_points[0][0];
						x_bound_max = (int)triangle_points[0][0];
						z_bound_min = (int)triangle_points[0][1];
						z_bound_max = (int)triangle_points[0][1];

						for(i = 1; i < 3; i++){
							if((int)triangle_points[i][0] < x_bound_min) x_bound_min = (int)triangle_points[i][0];
							if((int)triangle_points[i][0] > x_bound_max) x_bound_max = (int)triangle_points[i][0];
							if((int)triangle_points[i][1] < z_bound_min) z_bound_min = (int)triangle_points[i][1];
							if((int)triangle_points[i][1] > z_bound_max) z_bound_max = (int)triangle_points[i][1];
						}

						//
						// assign points to inside_poly_array[]
						//

						for(i = 0; i < 3; i++){
							inside_poly_array[i].x = (double)triangle_points[i][0];
							inside_poly_array[i].y = (double)triangle_points[i][1];
						}

						//
						// find how many trees for this size of triangle
						//

						num_trees = (int)((x_bound_max - x_bound_min) * (z_bound_max - z_bound_min)) /
							(min_space * min_space * density_factor);

						//printf("num_trees = %d  tree_index = %d\n", num_trees, tree_index);

						while_runaway_check = 0;

						tree_count = 0;

						while(tree_count < num_trees){

							//printf("while check %d  tree index %d\n", while_runaway_check, tree_index);

							//
							// get a random point
							//

							px = x_bound_min + (rand() % (x_bound_max - x_bound_min));
							pz = z_bound_min + (rand() % (z_bound_max - z_bound_min));

							random_point.x = (double)px;
							random_point.y = (double)pz;

							//
							// check to see if this point is a good one
							//

							if(InsidePolygon(inside_poly_array, 3, random_point)){

								if(edge_test_ok(px, pz, min_space, poly_points, points_this_poly)){

									//
									// check for min_space against tree list
									//

									space_ok = 1;

									for(x = 0; x < tree_index; x++){

										space = (long long)sqrt(((px - tree_points[x][0]) * (px - tree_points[x][0])) + 
											((pz - tree_points[x][1]) * (pz - tree_points[x][1])));

										if(space < min_space){

											space_ok = 0;
										}
									} // end for
					
									//
									// add point to tree list if space_ok
									//

									if(space_ok){

										tree_points[tree_index][0] = px;
										tree_points[tree_index][1] = pz;
										tree_points[tree_index][2] = ((k - px * Nx - pz * Nz) / Ny);
										tree_index ++;
										tree_count ++;
									}

								} // end if edge_test_ok
							} // end if Inside Polygon

							if(while_runaway_check > 1000){

								//printf("while_runaway_check planting trees in sector %d x %d\n", sec_z, sec_x);

								//fflush( stdout );

								break;
							}
							while_runaway_check++;

						} // end while
						
					} // end for triangles loop
						
				} // end if trees_this_poly

				///////////////////////////////////////////////
				// write out tree data
				///////////////////////////////////////////////

				for(i = 0; i < tree_index; i++){

					dissolve_ratio_counter ++;

					if(dissolve_ratio_counter % dissolve_ratio_factor == 0){ 
															
						//
						// these are the solid trees
						//

						random_number = rand();

						if ( random_number % 3 == 2 ){

							local_tree_data.colour = 0; // lightest
						}
						else if ( random_number % 3 == 1 ){

							local_tree_data.colour = 1;
						}
						else {

							local_tree_data.colour = 2; //darkest
						}
								
						local_tree_data.height = (rand() % 40) + 80;
						local_tree_data.type = 1;
						local_tree_data.x = (short int) tree_points[i][0] * 2;
						local_tree_data.z = (short int) tree_points[i][1] * 2;
						local_tree_data.y = (unsigned short int) tree_points[i][2] * 2;
						fwrite(&local_tree_data, sizeof(terrain_3d_tree_data), 1, tree_data);
						trees_this_sector += 1;
						offset_counter_next += 1;
						solid_trees_this_sector += 1;

						//fprintf(fp, "%d  %d  %d \n",
						//	local_tree_data.x,
						//	local_tree_data.z,
						//	local_tree_data.y);
					} // end if 
					else{
							
						//
						// these are the dissolving trees
						//

						random_number = rand();

						if ( random_number % 3 == 2 ){

							local_tree_data.colour = 0; // lightest
						}
						else if ( random_number % 3 == 1 ){

							local_tree_data.colour = 1;
						}
						else {

							local_tree_data.colour = 2; //darkest
						}

						local_tree_data.height = (rand() % 20) + 30;
						local_tree_data.type = 0;
						local_tree_data.x = (short int) tree_points[i][0] * 2;
						local_tree_data.z = (short int) tree_points[i][1] * 2;
						local_tree_data.y = (unsigned short int) tree_points[i][2] * 2;
						fwrite(&local_tree_data, sizeof(terrain_3d_tree_data), 1, tree_data);
						trees_this_sector += 1;
						offset_counter_next += 1;
						dissolve_trees_this_sector += 1;

						//fprintf(fp, "%d  %d  %d \n",
						//	local_tree_data.x,
						//	local_tree_data.z,
						//	local_tree_data.y);
					}

				} // end for tree index
			
			} // end for poly loop

			/////////////////////////////////////////////////
			// END POLY LOOP
			/////////////////////////////////////////////////

			total_tree_count += trees_this_sector;

			if(highest_tree_count < trees_this_sector){

				highest_tree_count = trees_this_sector;
				highest_x_sector = sec_x;
				highest_z_sector = sec_z;
			}

			if(trees_this_sector > MAX_TREES_PER_SECTOR){

				number_of_sectors_with_max_trees ++;
			}

			if(trees_this_sector > 0){	
				fprintf(fp, "%3d x %-3d      %-15d%-15d%-15d\n",
					sec_z, sec_x, trees_this_sector,
					solid_trees_this_sector, dissolve_trees_this_sector);
			}

            ///////////////////////////////////////////////////////
			// write out the tree sector file
			///////////////////////////////////////////////////////

			fwrite(&trees_this_sector, sizeof(int), 1, tree_sec);
			fwrite(&offset_counter_last, sizeof(int), 1, tree_sec);
			
			offset_counter_last = offset_counter_next;

		} // end x loop
	} // end z loop

	fprintf(fp, "\nHighest tree count is %d in sector %d x %d\n", 
		highest_tree_count, highest_x_sector, highest_z_sector);

	fprintf(fp, "\n%d Sectors out of %d have maximum trees\n",
		number_of_sectors_with_max_trees, max_x * max_z);

	fprintf(fp, "\nTotal trees is %u\n", total_tree_count);

	printf("\n\tTotal trees is %u\n", total_tree_count);
	printf("\t%d Sectors out of %d have the maximum trees of %d per sector\n",
		number_of_sectors_with_max_trees, max_x * max_z, MAX_TREES_PER_SECTOR + 8);

		
	fclose(tree_sec);
	fclose(tree_data);
	fclose(fp);

} // end create_tree_files

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

// function returns 1 if point is at least min_space/2 from all edges of n-sided polygon
// otherwise returns 0

int edge_test_ok(long long x, long long z, long long min_space, long long poly[][3], int num_points){

	int	distance_ok,
		i;

	long long	a,
				b,
				c,
				d,
				distance;

	distance_ok = 1;

	for(i = 0; i < num_points; i++){

		a = poly[i][0];
		b = poly[i][1];
		c = poly[(i+1) % num_points][0];
		d = poly[(i+1) % num_points][1];

		distance = (long long)(labs((long)((x - a) * (d - b) - (z - b) * (c - a))) /
			(long)sqrt((double)((c - a) * (c - a) + (d - b) * (d - b))));

		if(distance < (min_space / 2)) distance_ok = 0;
	} // end for

	return distance_ok;

} // end edge_test

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(Point *polygon,int N,Point p)
{
  int counter = 0;
  int i;
  double xinters;
  Point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.y > MIN(p1.y,p2.y)) {
      if (p.y <= MAX(p1.y,p2.y)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.y != p2.y) {
            xinters = (p.y-p1.y)*(p2.x-p1.x)/(p2.y-p1.y)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void find_new_min_max_height(void){

	int
		sec_z,
		sec_x,
		number_of_points_this_sector,
		point_loop,
		orig_min_height,
		orig_max_height,
		local_point_height,
		new_local_min_height,
		new_local_max_height,
		scaled_point_height;

	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - 
		terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
	//for( sec_z = 0 ; sec_z < 1 ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			new_local_min_height = 9999999;

			new_local_max_height = 0;

			number_of_points_this_sector = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_points;

			orig_min_height = (int)terrain_sectors[ sec_z ][ sec_x ].minimum_height;

			orig_max_height = (int)terrain_sectors[ sec_z ][ sec_x ].maximum_height;

			//printf ("Sector %d x %d min height = %d    max height = %d\n", sec_z, sec_x, 
			//	orig_min_height, orig_max_height );
			
			for(point_loop = 0; point_loop < number_of_points_this_sector; point_loop ++){

				local_point_height = (int)terrain_sectors[ sec_z ][ sec_x ].points_y[ point_loop ].y;

				scaled_point_height = (int)( terrain_3d_map_minimum_height + 
					( local_point_height * terrain_3d_map_scaled_height_difference ) );

				//printf("%d     scaled_point_height = %d\n", point_loop, scaled_point_height);
				
				if( scaled_point_height < new_local_min_height ){

					new_local_min_height = scaled_point_height;
				}

				if( scaled_point_height > new_local_max_height ){

					new_local_max_height = scaled_point_height;

					//printf("scaled_point_height = %d  new_local_max_height = %d\n", 
					//	scaled_point_height, new_local_max_height);
				}
			} // end point loop

			new_local_min_height ++;

			if( new_local_min_height < orig_min_height ){

				terrain_sectors[ sec_z ][ sec_x ].minimum_height = (short int) new_local_min_height;

				//printf("new min height found in sector %d x %d   %d   %d\n", sec_z, sec_x,
				//	orig_min_height, new_local_min_height);
			}

			if( new_local_max_height > orig_max_height ){

				terrain_sectors[ sec_z ][ sec_x ].maximum_height = (short int) new_local_max_height;

				//printf("new max height found in sector %d x %d   %d   %d\n", sec_z, sec_x,
				//	orig_max_height, new_local_max_height);
			}


		} // end sec_x loop
	} // end sec_z loop
} // end find new min max height

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int get_y_value_from_adjacent_corner_sector ( int sec_z, int sec_x, int top_point_z, int top_point_x, int top_point_y ){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		point_x,
		point_z,
		point_y,
		search_point_x,
		search_point_z,
		search_point_y,
		search_sec_x,
		search_sec_z,
		return_value,
		lowest_y_value;

	if( top_point_x == -16384 && top_point_z == -16384 ){

		search_sec_x = sec_x - 1;
		search_sec_z = sec_z - 1;
		search_point_x = 16384;
		search_point_z = 16384;
	}
	else if( top_point_x == -16384 && top_point_z == 16384 ){

		search_sec_x = sec_x - 1;
		search_sec_z = sec_z + 1;
		search_point_x = 16384;
		search_point_z = -16384;
	}
	else if( top_point_x == 16384 && top_point_z == 16384 ){

		search_sec_x = sec_x + 1;
		search_sec_z = sec_z + 1;
		search_point_x = -16384;
		search_point_z = -16384;
	}
	else if( top_point_x == 16384 && top_point_z == -16384 ){

		search_sec_x = sec_x + 1;
		search_sec_z = sec_z - 1;
		search_point_x = -16384;
		search_point_z = 16384;
	}
	else{

		printf("error in get_y_value_from_adjacent_corner_sector\n");
		exit_function ();
	}

	lowest_y_value = top_point_y;

				
	number_of_points =  terrain_sectors[ search_sec_z ][ search_sec_x ].number_of_points;

	number_of_polys = terrain_sectors[ search_sec_z ][ search_sec_x ].number_of_polygons;


	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].
			polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].
			polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].
				surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].
				surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[ search_sec_z ][ search_sec_x ].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].
					point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].
					point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

			point_x = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].points_xz[ point_reference ].x;
			point_z = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].points_xz[ point_reference ].z;
			point_y = (int)terrain_sectors[ search_sec_z ][ search_sec_x ].points_y[ point_reference ].y;

			if( point_x == search_point_x &&
				point_z == search_point_z &&
				point_y < lowest_y_value){

				lowest_y_value = point_y;
			}

		} // end poly point loop

	} // end poly loop

	if( lowest_y_value < top_point_y ){

		return lowest_y_value;
	}
	else{

		return 0;
	}

	//printf("no adjacent sectors points found for top_points %d x %d in sector %d x %d\n",
	//	top_point_z, top_point_x, sec_z, sec_x);
	
	return 0;

} // end get_y_value_from_adjacent_corner_sector

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void fix_steep_polys (int sec_z, int sec_x ){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		point_x,
		point_z,
		point_y,
		poly_point_references [ 128 ],
		ax, az, ay,
		bx, bz, by,
		cx, cz, cy,
		side_ab,
		side_bc,
		side_ca,
		least_distance;

	float
		slope;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	//
	// find forest top polys
	//

	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

			poly_point_references [ poly_point_loop ] = point_reference;

		} // end poly point loop

		if ( surface == TERRAIN_TYPE_FOREST_TOP ){

			if( number_of_poly_points == 3 ){

				slope = (float)normal_angle_with_y ( sec_z, sec_x, poly_point_references [ 0 ],
					poly_point_references [ 1 ], poly_point_references [ 2 ] );

				if( slope > 55.0 ){

					ax = terrain_sectors[sec_z][sec_x].points_xz[ poly_point_references [ 0 ] ].x;
					az = terrain_sectors[sec_z][sec_x].points_xz[ poly_point_references [ 0 ] ].z;
					ay = terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 0 ] ].y;

					bx = terrain_sectors[sec_z][sec_x].points_xz[ poly_point_references [ 1 ] ].x;
					bz = terrain_sectors[sec_z][sec_x].points_xz[ poly_point_references [ 1 ] ].z;
					by = terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 1 ] ].y;

					cx = terrain_sectors[sec_z][sec_x].points_xz[ poly_point_references [ 2 ] ].x;
					cz = terrain_sectors[sec_z][sec_x].points_xz[ poly_point_references [ 2 ] ].z;
					cy = terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 2 ] ].y;

					/////////////////////////////////////////////////
					// find the two closest points
					/////////////////////////////////////////////////

					side_ab = sqrt ( ( (ax - bx) * (ax - bx) + (az - bz) * (az - bz) ) );

					side_bc = sqrt ( ( (bx - cx) * (bx - cx) + (bz - cz) * (bz - cz) ) );

					side_ca = sqrt ( ( (cx - ax) * (cx - ax) + (cz - az) * (cz - az) ) );

					least_distance = side_ab;

					if( side_bc < least_distance ){

						least_distance = side_bc;
					}

					if( side_ca < least_distance ){

						least_distance = side_ca;
					}

					////////////////////////////////////////////////////////
					// lower the highest y value of the two closest points
					// to the lower point value
					////////////////////////////////////////////////////////

					if( side_ab == least_distance ){

						if( ay < by ){ // change by

							terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 1 ] ].y = (unsigned short int) ay;

							if( bx == 16384 || bx == -16384 || bz == 16384 || bz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", bz, bx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, bz, bx, ay );
							}
						}
						else{ // change ay

							terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 0 ] ].y = (unsigned short int) by;

							if( ax == 16384 || ax == -16384 || az == 16384 || az == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", az, ax, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, az, ax, by );
							}
						}

					} // end if side ab

					else if( side_bc == least_distance ){

						if( by < cy ){ // change cy

							terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 2 ] ].y = (unsigned short int) by;

							if( cx == 16384 || cx == -16384 || cz == 16384 || cz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", cz, cx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, cz, cx, by );
							}
						}
						else{ // change by

							terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 1 ] ].y = (unsigned short int) cy;

							if( bx == 16384 || bx == -16384 || bz == 16384 || bz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", bz, bx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, bz, bx, cy );
							}
						}

					} // end if side bc

					else if( side_ca == least_distance ){

						if( cy < ay ){ // change ay

							terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 0 ] ].y = (unsigned short int) cy;

							if( ax == 16384 || ax == -16384 || az == 16384 || az == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", az, ax, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, az, ax, cy );
							}
						}
						else{ // change cy

							terrain_sectors[sec_z][sec_x].points_y[ poly_point_references [ 2 ] ].y = (unsigned short int) ay;

							if( cx == 16384 || cx == -16384 || cz == 16384 || cz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", cz, cx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, cz, cx, ay );
							}
						}

					} // end if side ca

					else{

						printf("couldn't find point to correct in fix_steep_polys sector %d x %d\n", sec_z, sec_x);
					}

/*
					printf("%d x %d   slope %f  %d  %d  %d * %d  %d  %d * %d  %d  %d", sec_z, sec_x, slope,
						ax, az, ay, bx, bz, by, cx, cz, cy);

					if( ax == 16384 || ax == -16384 || az == 16384 || az == -16384 ||
						bx == 16384 || bx == -16384 || bz == 16384 || bz == -16384 ||
						cx == 16384 || cx == -16384 || cz == 16384 || cz == -16384 ){

						printf("   edge point\n");
					}
					else{

						printf("\n");
					}
*/
				} // end if slope is steep
				
			} // end if triangle
			
		} // end if surface == TERRAIN_TYPE_FOREST_TOP

	} // end poly loop

} // end fix steep polys

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

double normal_angle_with_y ( int sec_z, int sec_x, int a_ref, int b_ref, int c_ref ){

	// returns the angle between the poly normal and the Y axis (bigger = steeper)
	// it only works for triangles

	// points

	double
		ax,
		bx,
		cx,
		az,
		bz,
		cz,
		ay,
		by,
		cy;

	// vectors

	double 
		abx,
		abz,
		aby, 
		acx, 
		acz, 
		acy,
		nx,
		nz, 
		ny;
	
	int
		number_of_points;

	double
		radian_angle,
		pi = 3.1415926535,
		normalized_ny,
		angle;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	ax = (double)terrain_sectors[sec_z][sec_x].points_xz[ a_ref ].x;

	bx = (double)terrain_sectors[sec_z][sec_x].points_xz[ b_ref ].x;

	cx = (double)terrain_sectors[sec_z][sec_x].points_xz[ c_ref ].x;

	az = (double)terrain_sectors[sec_z][sec_x].points_xz[ a_ref ].z;

	bz = (double)terrain_sectors[sec_z][sec_x].points_xz[ b_ref ].z;

	cz = (double)terrain_sectors[sec_z][sec_x].points_xz[ c_ref ].z;

	ay = (double)terrain_sectors[sec_z][sec_x].points_y[ a_ref ].y;

	by = (double)terrain_sectors[sec_z][sec_x].points_y[ b_ref ].y;

	cy = (double)terrain_sectors[sec_z][sec_x].points_y[ c_ref ].y;

	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_ny = ny / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_ny *= -1.0;
	
	radian_angle = acos(normalized_ny); // normalized_ny / 1 = arcos of angle with y axis
	
	angle = (180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_y()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void match_adjacent_sector_point ( int sec_z, int sec_x, int match_point_z, int match_point_x, int match_point_y ){

	int
		number_of_points,
		point_loop,
		point_x,
		point_z,
		point_y,
		search_points_x [ 3 ], 
		search_points_z [ 3 ],
		number_of_search_points,
		search_sectors_x [ 3 ],
		search_sectors_z [ 3 ], 
		number_of_search_sectors,
		sector_loop,
		this_sec_x,
		this_sec_z,
		this_search_point_x,
		this_search_point_z;

	//
	// if matching to a corner point
	// need to match in 3 sectors
	//

	if( ( match_point_x == 16384 || match_point_x == -16384 ) && ( match_point_z == 16384 || match_point_z == -16384 ) ){

		number_of_search_sectors = 3;
		number_of_search_points = 3;

		//
		// find sectors and points going clockwise
		//

		//
		// upper right corner
		//

		if( match_point_x == 16384 && match_point_z == 16384 ){ 

			search_sectors_x [ 0 ] = sec_x;
			search_sectors_z [ 0 ] = sec_z + 1;

			search_points_x [ 0 ] = 16384;
			search_points_z [ 0 ] = -16384;

			////////////////////////////////////////////////

			search_sectors_x [ 1 ] = sec_x + 1;
			search_sectors_z [ 1 ] = sec_z + 1;

			search_points_x [ 1 ] = -16384;
			search_points_z [ 1 ] = -16384;

			////////////////////////////////////////////////

			search_sectors_x [ 2 ] = sec_x + 1;
			search_sectors_z [ 2 ] = sec_z;

			search_points_x [ 2 ] = -16384;
			search_points_z [ 2 ] = 16384;
		}

		//
		// lower right corner
		//

		else if( match_point_x == 16384 && match_point_z == -16384 ){ 

			search_sectors_x [ 0 ] = sec_x + 1;
			search_sectors_z [ 0 ] = sec_z;

			search_points_x [ 0 ] = -16384;
			search_points_z [ 0 ] = -16384;

			////////////////////////////////////////////////

			search_sectors_x [ 1 ] = sec_x + 1;
			search_sectors_z [ 1 ] = sec_z - 1;

			search_points_x [ 1 ] = -16384;
			search_points_z [ 1 ] = 16384;

			////////////////////////////////////////////////

			search_sectors_x [ 2 ] = sec_x;
			search_sectors_z [ 2 ] = sec_z - 1;

			search_points_x [ 2 ] = 16384;
			search_points_z [ 2 ] = 16384;
		}

		//
		// lower left corner
		//

		else if( match_point_x == -16384 && match_point_z == -16384 ){ 

			search_sectors_x [ 0 ] = sec_x;
			search_sectors_z [ 0 ] = sec_z - 1;

			search_points_x [ 0 ] = -16384;
			search_points_z [ 0 ] = 16384;

			////////////////////////////////////////////////

			search_sectors_x [ 1 ] = sec_x - 1;
			search_sectors_z [ 1 ] = sec_z - 1;

			search_points_x [ 1 ] = 16384;
			search_points_z [ 1 ] = 16384;

			////////////////////////////////////////////////

			search_sectors_x [ 2 ] = sec_x - 1;
			search_sectors_z [ 2 ] = sec_z;

			search_points_x [ 2 ] = 16384;
			search_points_z [ 2 ] = -16384;
		}

		//
		// upper left corner
		//

		else if( match_point_x == -16384 && match_point_z == 16384 ){														

			search_sectors_x [ 0 ] = sec_x - 1;
			search_sectors_z [ 0 ] = sec_z;

			search_points_x [ 0 ] = 16384;
			search_points_z [ 0 ] = 16384;

			////////////////////////////////////////////////

			search_sectors_x [ 1 ] = sec_x - 1;
			search_sectors_z [ 1 ] = sec_z + 1;

			search_points_x [ 1 ] = 16384;
			search_points_z [ 1 ] = -16384;

			////////////////////////////////////////////////

			search_sectors_x [ 2 ] = sec_x;
			search_sectors_z [ 2 ] = sec_z + 1;

			search_points_x [ 2 ] = -16384;
			search_points_z [ 2 ] = -16384;
		}
		else{

			printf("error 1 in match_adjacent_sector_point\n");
		}

	} // end if corner point

	//
	// match only a single side point
	// 

	else{

		number_of_search_sectors = 1;
		number_of_search_points = 1;

		if( match_point_x == 16384 ){

			search_sectors_x [ 0 ] = sec_x + 1;
			search_sectors_z [ 0 ] = sec_z;

			search_points_x [ 0 ] = -16384;
			search_points_z [ 0 ] = match_point_z;
		}

		/////////////////////////////////////////////////

		else if( match_point_x == -16384 ){

			search_sectors_x [ 0 ] = sec_x - 1;
			search_sectors_z [ 0 ] = sec_z;

			search_points_x [ 0 ] = 16384;
			search_points_z [ 0 ] = match_point_z;
		}

		/////////////////////////////////////////////////

		else if( match_point_z == 16384 ){

			search_sectors_x [ 0 ] = sec_x;
			search_sectors_z [ 0 ] = sec_z + 1;

			search_points_x [ 0 ] = match_point_x;
			search_points_z [ 0 ] = -16384;
		}

		/////////////////////////////////////////////////

		else if( match_point_z == -16384 ){

			search_sectors_x [ 0 ] = sec_x;
			search_sectors_z [ 0 ] = sec_z - 1;

			search_points_x [ 0 ] = match_point_x;
			search_points_z [ 0 ] = 16384;
		}

		else{

			printf("error 2 in match_adjacent_sector_point\n");
		}

	} // end else match only a single side point

	////////////////////////////////////////////////////////////////
	// find the corresponding point in single 
	// side sector or 3 corner sectors
	////////////////////////////////////////////////////////////////

	for( sector_loop = 0; sector_loop < number_of_search_sectors; sector_loop ++ ){

		this_sec_x = search_sectors_x [ sector_loop ];
		this_sec_z = search_sectors_z [ sector_loop ];

		this_search_point_x = search_points_x [ sector_loop ];
		this_search_point_z = search_points_z [ sector_loop ];

		number_of_points =  terrain_sectors[ this_sec_z ] [ this_sec_x ].number_of_points;

		for(point_loop = 0; point_loop < number_of_points; point_loop ++){

			point_x = (int)terrain_sectors[ this_sec_z ][ this_sec_x ].points_xz[ point_loop ].x;
			point_z = (int)terrain_sectors[ this_sec_z ][ this_sec_x ].points_xz[ point_loop ].z;
			
			if( point_x == this_search_point_x && point_z == this_search_point_z ){

				terrain_sectors[ this_sec_z ][ this_sec_x ].points_y[ point_loop ].y = (unsigned short int)match_point_y;

				//printf("	point matched in sector %d x %d\n", this_sec_z, this_sec_x);

				break;
			}

		} // end point loop

	} // end sector loop

} // end match_adjacent_sector_point

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void find_number_of_colors ( void ){

	int
		sec_z,
		sec_x,
		number_of_points,
		point_loop,
		color_index,
		max_color_index,
		global_max_color_index = 0;

	printf("finding number of colors ...\n");

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			number_of_points = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_points;

			max_color_index = 0;

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				color_index = terrain_sectors[sec_z][sec_x].colour_indices [ point_loop ].index;

				if( color_index > max_color_index ){

					max_color_index = color_index;
				}

			} // end point loop

			terrain_sectors[ sec_z ][ sec_x ].number_of_colors = max_color_index + 1;

			//if( max_color_index > global_max_color_index ){

			//	global_max_color_index = max_color_index;
			//}

		} // end sec x loop
	} // end sec z loop

	//printf("global_max_color_index = %d\n", global_max_color_index);

} // end find number of colors

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void backup_files ( void ){

	char
		input,
		orig_ffp[] = "original_terrain.ffp",
		orig_sec[] = "original_default.sec",
		orig_rgb[] = "original_default.rgb",
		orig_tree_dat[] = "original_tree_pos.dat",
		orig_tree_sec[] = "original_tree_pos.sec";

	//
	// terrain.ffp
	//

	if( file_exist ( orig_ffp ) ){

		printf("original_terrain.ffp already exists\ndo you want to continue without backing up files? y/n ");

		fflush(stdout);
		fflush(stdin);

		input = getche();

		printf("\n");
		
		if(input == 'y'){}
		else{ exit_function (); }
	}
	else if( system ("copy terrain.ffp original_terrain.ffp") == 1 ){

		printf("Error: can not copy terrain.ffp PROGRAM TERMINATED\n");
		exit_function ();
	}
	else{

		printf("terrain.ffp copied to original_terrain.ffp\n");
	}

	//
	// default.sec
	//

	if( file_exist ( orig_sec ) ){

		printf("original_default.sec already exists ...\ncontinuing without backing up ...\n");
	} 
	else{

		if( system ("copy default.sec original_default.sec") == 1 ){

			printf("Error: can not copy default.sec PROGRAM TERMINATED\n");
			exit_function ();
		}
		else{

			printf("default.sec copied to original_default.sec\n");
		}
	}

	//
	// default.rgb
	//

	if( file_exist ( orig_rgb ) ){

		printf("original_default.rgb already exists ...\ncontinuing without backing up ...\n");
	} 
	else{

		if( system ("copy default.rgb original_default.rgb") == 1 ){

			printf("Error: can not copy default.rgb PROGRAM TERMINATED\n");
			exit_function ();
		}
		else{

			printf("default.rgb copied to original_default.rgb\n");
		}
	}

	//
	// tree_pos.dat
	//

	if( file_exist ( orig_tree_dat ) ){

		printf("original_tree_pos.dat already exists ...\ncontinuing without backing up ...\n");
	} 
	else{

		if( system ("copy tree_pos.dat original_tree_pos.dat") == 1 ){

			printf("Error: can not copy tree_pos.dat PROGRAM TERMINATED\n");
			exit_function ();
		}
		else{

			printf("tree_pos.dat copied to original_tree_pos.dat\n");
		}
	}

	//
	// tree_pos.sec
	//

	if( file_exist ( orig_tree_sec ) ){

		printf("original_tree_pos.sec already exists ...\ncontinuing without backing up ...\n");
	} 
	else{

		if( system ("copy tree_pos.sec original_tree_pos.sec") == 1 ){

			printf("Error: can not copy tree_pos.sec PROGRAM TERMINATED\n");
			exit_function ();
		}
		else{

			printf("tree_pos.sec copied to original_tree_pos.sec\n");
		}
	}

} // end backup files

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void exit_function(void){

	char
		input;

	printf("\nPress any key to continue ... ");

	fflush( stdout );

	fflush( stdin );

	//scanf("%c", &input);

	input = getche();

	exit(1);
	
} // end exit function

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void change_forest_canopy_to_land (int sec_z, int sec_x ){

	int
		number_of_surfaces,
		surface_loop;

	number_of_surfaces = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_surfaces;

	for(surface_loop = 0; surface_loop < number_of_surfaces; surface_loop ++){

		if( terrain_sectors[sec_z][sec_x].surface_changes [ surface_loop ].surface_id == TERRAIN_TYPE_FOREST_TOP ){

			terrain_sectors[sec_z][sec_x].surface_changes [ surface_loop ].surface_id = TERRAIN_TYPE_LAND;
		}
	} // end surface loop

} // end change forest canopy to land

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void lighten_colors ( void ){

	int
		sec_z,
		sec_x,
		number_of_points,
		color_loop,
		number_of_colors,
		general_lighten_factor,			// 0 - 100 ( 0 = no change,  100 = 255 )
		forest_lighten_factor,
		red,
		green,
		blue,
		point_ref_counter,
		poly_loop,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		poly_point_loop,
		surface_changes_index,
		color_index,
		point_reference,
		point_loop;

	int
		forest_colors [ 512 ],
		land_colors [ 512 ],
		water_colors [ 512 ];

	printf("lightening colors ...\n");

	general_lighten_factor = 70;
	forest_lighten_factor = 60;

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			point_ref_counter = 0;

			surface_changes_index = 0;

			number_of_colors =  terrain_sectors[sec_z][sec_x].number_of_colors;

			number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

			for(color_loop = 0; color_loop < number_of_colors; color_loop ++){

				forest_colors [ color_loop ] = 0;

				land_colors [ color_loop ] = 0;

				water_colors [ color_loop ] = 0;
		
			} // end color loop
			
			for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

				number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

				surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

				if( poly_loop == 0 ){

					surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

					surface_changes_index ++;
				}
				else if( surface_change ){

					surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

					surface_changes_index ++;
				}

				//printf("Poly	%3d		has %d points	suface = %3d\n\n", poly_loop, number_of_poly_points, surface);

				//printf("	Point references\n\n");

				for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

					if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

						point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

						point_ref_counter ++;
					}
					else{

						point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

						point_ref_counter ++;
					}

					//
					// there is a color index for each point
					//

					color_index = terrain_sectors[sec_z][sec_x].colour_indices [ point_reference ].index;

					if( surface == TERRAIN_TYPE_LAND ){

						land_colors [ color_index ] = 1;
					}
					else if( surface == TERRAIN_TYPE_FOREST_TOP ){

						forest_colors [ color_index ] = 1;
					}
					else if( surface == TERRAIN_TYPE_SEA ||
						     surface == TERRAIN_TYPE_RIVER ||
							 surface == TERRAIN_TYPE_RESERVOIR){

						water_colors [ color_index ] = 1;
					}

				} // end poly point loop

				//printf("\n");

			} // end for poly loop
/*
			if ( sec_z == 1 && sec_x == 46 ){

				for(color_loop = 0; color_loop < number_of_colors; color_loop ++){

					red = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].red;
					green = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].green;
					blue = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].blue;

					printf("%d      %d  %d  %d", color_loop, red, green, blue);

					if( land_colors [ color_loop ] ){

						printf("\tland");
					}
					if( forest_colors [ color_loop ] ){

						printf("\tforest");
					}

					printf("\n");
				}
			}
*/

			for(color_loop = 0; color_loop < number_of_colors; color_loop ++){

				if( water_colors [ color_loop ] ){

					// don't change water colors
				}
				else if( forest_colors [ color_loop ] ){

					red = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].red;
					green = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].green;
					blue = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].blue;

					red = red + ( forest_lighten_factor * ( 255 - red ) ) / 100;
					green = green + ( forest_lighten_factor * ( 255 - green ) ) / 100;
					blue = blue + ( forest_lighten_factor * ( 255 - blue ) ) / 100;

					terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].red = (unsigned char)red;
					terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].green = (unsigned char)green;
					terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].blue = (unsigned char)blue;
				}
				else{

					red = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].red;
					green = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].green;
					blue = (int)terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].blue;

					red = red + ( general_lighten_factor * ( 255 - red ) ) / 100;
					green = green + ( general_lighten_factor * ( 255 - green ) ) / 100;
					blue = blue + ( general_lighten_factor * ( 255 - blue ) ) / 100;

					terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].red = (unsigned char)red;
					terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].green = (unsigned char)green;
					terrain_sectors[sec_z][sec_x].point_colours [ color_loop ].blue = (unsigned char)blue;
				}

			} // end color loop

		}
	}
} // end lighten colors

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void add_new_colors (int sec_z, int sec_x ){

	int
		number_of_colors,
		number_of_indices,
		color_loop,
		index_loop;

	struct TERRAIN_3D_COLOUR
		*point_colours;

	terrain_3d_colour
		tree_colour_0,
		tree_colour_1,
		tree_colour_2;


	tree_colour_0.red = 122; // lightest green
	tree_colour_0.green = 182;
	tree_colour_0.blue = 99;

	tree_colour_1.red = 105; // middle green
	tree_colour_1.green = 164;
	tree_colour_1.blue = 82;
	
	tree_colour_2.red = 68; // darkest green
	tree_colour_2.green = 127;
	tree_colour_2.blue = 47;

	number_of_colors = terrain_sectors[sec_z][sec_x].number_of_colors;

	number_of_indices = (int)terrain_sectors[sec_z][sec_x].number_of_points;

	if ( number_of_colors > max_forest_sector_colors ){

		max_forest_sector_colors = number_of_colors;
	}

	if( ! (point_colours = malloc( sizeof (terrain_3d_colour) * (number_of_colors + 3) ) ) ){
		printf("No memory for point_colours in add_new_colors sector %d x %d\n", sec_z, sec_x);
		exit_function ();
	}

	//
	// move the indices 3 to the right
	//

	for( index_loop = 0; index_loop < number_of_indices; index_loop ++){

		terrain_sectors[sec_z][sec_x].colour_indices [ index_loop ].index += 3;

	} // end index loop

	//
	// add the 3 tree colors to new array
	//

	point_colours [ 0 ] = tree_colour_0;
	point_colours [ 1 ] = tree_colour_1;
	point_colours [ 2 ] = tree_colour_2;

	//
	// add the original colors, 3 places to the right
	//

	for( color_loop = 0; color_loop < number_of_colors; color_loop ++){

		point_colours [ color_loop + 3 ] = terrain_sectors[sec_z][sec_x].point_colours [ color_loop ];
	}

	//
	// swap pointer
	//

	terrain_sectors[sec_z][sec_x].point_colours = point_colours;

	terrain_sectors[sec_z][sec_x].number_of_colors += 3;


} // end add new colors

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int file_exist (char *filename)
{
	char fn[32];

	FILE
		*file_ptr;

	if (file_ptr = fopen (filename, "r"))
	{

		fclose (file_ptr);

		return TRUE;
	}

	return FALSE;
}

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void create_winter_rgb_file ( void ){

	FILE
		*rgb_out;

	int
		sec_x,
		sec_z,
		number_of_colors,
		color_loop;

	terrain_3d_colour
		white_colour;


	white_colour.red = 255; 
	white_colour.green = 255;
	white_colour.blue = 255;

	printf("Writing Winter rgb file ...\n");

	if(( rgb_out = fopen("default-W.rgb","wb"))==NULL){
		printf("cannot open default-W.rgb\n");
		exit_function ();
	}
	else printf("default-W.rgb opened\n");

	fwrite(&rgb_4byte_header, sizeof(int), 1, rgb_out);


	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			number_of_colors = terrain_sectors[sec_z][sec_x].number_of_colors;

			//
			// the colour indices don't change
			//

			fwrite(terrain_sectors[sec_z][sec_x].colour_indices, 
				sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_out);

			//
			// write all white colors
			//

			for( color_loop = 0; color_loop < number_of_colors; color_loop ++ ){

				fwrite(&white_colour, sizeof(terrain_3d_colour), 1, rgb_out);
			}
		}
	}

	if( system ("copy default.sec default-W.sec") == 1 ){

		printf("Error: can not copy default.sec to default-W.sec PROGRAM TERMINATED\n");
		exit_function ();
	}
	else{

		printf("default.sec copied to default-W.sec\n");
	}

} // end create winter rgb file

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void find_all_sea_sectors ( void ){

	int
		sec_z,
		sec_x,
		number_of_surfaces,
		first_surface;

	terrain_3d_point_y
		*sea_sector_points_y;

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			number_of_surfaces = (int)terrain_sectors[ sec_z ][ sec_x ].number_of_surfaces;

			first_surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes[ 0 ].surface_id;

			if( number_of_surfaces == 1 && first_surface == TERRAIN_TYPE_SEA ){

				terrain_sectors[ sec_z ][ sec_x ].all_sea_sector = 1;
			}
			else{

				terrain_sectors[ sec_z ][ sec_x ].all_sea_sector = 0;
			}

			if( terrain_sectors[ sec_z ][ sec_x ].all_sea_sector ){

				printf("%d x %d   %p\n", sec_z, sec_x, terrain_sectors[ sec_z ][ sec_x ].points_y );
			}
		}
	}
} // end find all sea sectors

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void fix_steep_polys_2 (int sec_z, int sec_x ){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index = 0,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		point_x,
		point_z,
		point_y,
		poly_point_references [ 128 ],
		ax, az, ay,
		bx, bz, by,
		cx, cz, cy,
		side_ab,
		side_bc,
		side_ca,
		least_distance,
		triangle_counter,
		triangle_point_reference_0,
		triangle_point_reference_1,
		triangle_point_reference_2;

	float
		slope;

	

	global_repetition_count ++;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	//
	// find forest top polys
	//

	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;

		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;

		if( poly_loop == 0 ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}
		else if( surface_change ){

			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;

			surface_changes_index ++;
		}

		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////

		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){

			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}
			else{

				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;

				point_ref_counter ++;
			}

			poly_point_references [ poly_point_loop ] = point_reference;

		} // end poly point loop

		if ( surface == TERRAIN_TYPE_FOREST_TOP ){

			//printf("\nPolygon %d has %d points\n\n",poly_loop, number_of_poly_points);

			//
			// need to do all polys and break into triangles
			//

			triangle_point_reference_0 = poly_point_references [ 0 ];

			for(triangle_counter = 0; triangle_counter < (number_of_poly_points - 2); triangle_counter++){

				triangle_point_reference_1 = poly_point_references [ triangle_counter + 1 ];

				triangle_point_reference_2 = poly_point_references [ triangle_counter + 2 ];

				slope = (float)normal_angle_with_y ( sec_z, sec_x, triangle_point_reference_0,
					triangle_point_reference_1, triangle_point_reference_2 );

				//printf("triangle %d slope %f\n", triangle_counter, slope);

				if( slope > 50.0 ){

					ax = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_0 ].x;
					az = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_0 ].z;
					ay = terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_0 ].y;

					bx = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_1 ].x;
					bz = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_1 ].z;
					by = terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_1 ].y;

					cx = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_2 ].x;
					cz = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_2 ].z;
					cy = terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_2 ].y;

					/////////////////////////////////////////////////
					// find the two closest points
					/////////////////////////////////////////////////

					side_ab = sqrt ( ( (ax - bx) * (ax - bx) + (az - bz) * (az - bz) ) );

					side_bc = sqrt ( ( (bx - cx) * (bx - cx) + (bz - cz) * (bz - cz) ) );

					side_ca = sqrt ( ( (cx - ax) * (cx - ax) + (cz - az) * (cz - az) ) );

					least_distance = side_ab;

					if( side_bc < least_distance ){

						least_distance = side_bc;
					}

					if( side_ca < least_distance ){

						least_distance = side_ca;
					}

					////////////////////////////////////////////////////////
					// lower the highest y value of the two closest points
					// to the lower point value
					////////////////////////////////////////////////////////

					if( side_ab == least_distance ){

						if( ay < by ){ // change by

							terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_1 ].y = (unsigned short int) ay;

							if( bx == 16384 || bx == -16384 || bz == 16384 || bz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", bz, bx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, bz, bx, ay );
							}
						}
						else{ // change ay

							terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_0 ].y = (unsigned short int) by;

							if( ax == 16384 || ax == -16384 || az == 16384 || az == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", az, ax, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, az, ax, by );
							}
						}

					} // end if side ab

					else if( side_bc == least_distance ){

						if( by < cy ){ // change cy

							terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_2 ].y = (unsigned short int) by;

							if( cx == 16384 || cx == -16384 || cz == 16384 || cz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", cz, cx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, cz, cx, by );
							}
						}
						else{ // change by

							terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_1 ].y = (unsigned short int) cy;

							if( bx == 16384 || bx == -16384 || bz == 16384 || bz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", bz, bx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, bz, bx, cy );
							}
						}

					} // end if side bc

					else if( side_ca == least_distance ){

						if( cy < ay ){ // change ay

							terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_0 ].y = (unsigned short int) cy;

							if( ax == 16384 || ax == -16384 || az == 16384 || az == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", az, ax, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, az, ax, cy );
							}
						}
						else{ // change cy

							terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_2 ].y = (unsigned short int) ay;

							if( cx == 16384 || cx == -16384 || cz == 16384 || cz == -16384 ){

								//printf("changed y value of edge point %d  %d in sector %d x %d\n", cz, cx, sec_z, sec_x);

								match_adjacent_sector_point ( sec_z, sec_x, cz, cx, ay );
							}
						}

					} // end if side ca

					else{

						printf("couldn't find point to correct in fix_steep_polys sector %d x %d\n", sec_z, sec_x);
					}

/*
					printf("%d x %d   slope %f  %d  %d  %d * %d  %d  %d * %d  %d  %d", sec_z, sec_x, slope,
						ax, az, ay, bx, bz, by, cx, cz, cy);

					if( ax == 16384 || ax == -16384 || az == 16384 || az == -16384 ||
						bx == 16384 || bx == -16384 || bz == 16384 || bz == -16384 ||
						cx == 16384 || cx == -16384 || cz == 16384 || cz == -16384 ){

						printf("   edge point\n");
					}
					else{

						printf("\n");
					}
*/
				} // end if slope is steep
				
			} // end triangle loop
			
		} // end if surface == TERRAIN_TYPE_FOREST_TOP

	} // end poly loop

	if ( global_repetition_count > 16 ){

		printf("global_repetition_count > 16 in sector %d x %d\n", sec_z, sec_x);

		return;
	}
	else if ( steep_check ( sec_z, sec_x, 80.0 ) ){

		fix_steep_polys_2 ( sec_z, sec_x );
	}
	else{

		return;
	}



} // end fix steep polys2

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int find_closest_point (int sec_z, int sec_x, int target_ref){

	int
		point_loop,
		number_of_points,
		point_x,
		point_z,
		point_y,
		target_x,
		target_z,
		distance,
		least_distance,
		least_distance_ref;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	target_x = (int)terrain_sectors[sec_z][sec_x].points_xz[ target_ref ].x;

	target_z = (int)terrain_sectors[sec_z][sec_x].points_xz[ target_ref ].z;

	least_distance = sqrt(( 32768  *  32768 ) + ( 32768  *  32768 ));

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		if( point_loop != target_ref ){

			point_x = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x;
			point_z = (int)terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z;
			
			distance = sqrt((( target_x - point_x ) * ( target_x - point_x )) + 
							(( target_z - point_z ) * ( target_z - point_z )));

			if( distance < least_distance ){

				least_distance = distance;

				least_distance_ref = point_loop;
			}
		}
	} // end point loop

	return least_distance_ref;

} // end find closest point

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void get_slopes_above_x (float slope_x){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		point_x,
		point_z,
		point_y,
		poly_point_references [ 128 ],
		ax, az, ay,
		bx, bz, by,
		cx, cz, cy,
		side_ab,
		side_bc,
		side_ca,
		least_distance,
		triangle_counter,
		triangle_point_reference_0,
		triangle_point_reference_1,
		triangle_point_reference_2,
		sector_count = 0,
		triangle_count = 0,
		sector_bool,
		sec_z, sec_x;

	float
		slope;

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			sector_bool = 0;

			surface_changes_index = 0;

			number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;
		
			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;
		
			//
			// find forest top polys
			//
		
			point_ref_counter = 0;
			
			for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){
		
				number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;
		
				surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;
		
				if( poly_loop == 0 ){
		
					surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;
		
					surface_changes_index ++;
				}
				else if( surface_change ){
		
					surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;
		
					surface_changes_index ++;
				}
		
				//////////////////////////////////////////////////////////////////////
				// need to go through the points of all polys
				// even if they aren't forest polys because
				// of the sequential way the points are stored
				//////////////////////////////////////////////////////////////////////
		
				for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){
		
					if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){
		
						point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;
		
						point_ref_counter ++;
					}
					else{
		
						point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;
		
						point_ref_counter ++;
					}
		
					poly_point_references [ poly_point_loop ] = point_reference;
		
				} // end poly point loop
		
				if ( surface == TERRAIN_TYPE_FOREST_TOP ){
		
					//printf("\nPolygon %d has %d points\n\n",poly_loop, number_of_poly_points);
		
		
					//
					// break 3d_face into triangles because 3d_faces aren't always flat
					//
		
					triangle_point_reference_0 = poly_point_references [ 0 ];
		
					for(triangle_counter = 0; triangle_counter < (number_of_poly_points - 2); triangle_counter++){
		
						triangle_point_reference_1 = poly_point_references [ triangle_counter + 1 ];
		
						triangle_point_reference_2 = poly_point_references [ triangle_counter + 2 ];
		
						slope = (float)normal_angle_with_y ( sec_z, sec_x, triangle_point_reference_0,
							triangle_point_reference_1, triangle_point_reference_2 );
		
						//printf("triangle %d slope %f\n", triangle_counter, slope);
		
						if( slope > slope_x ){

							sector_bool = 1;

							triangle_count ++;
		
							ax = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_0 ].x;
							az = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_0 ].z;
							ay = terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_0 ].y;
		
							bx = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_1 ].x;
							bz = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_1 ].z;
							by = terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_1 ].y;
		
							cx = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_2 ].x;
							cz = terrain_sectors[sec_z][sec_x].points_xz[ triangle_point_reference_2 ].z;
							cy = terrain_sectors[sec_z][sec_x].points_y[ triangle_point_reference_2 ].y;
		
							
						} // end if slope is steep
						
					} // end triangle loop
					
				} // end if surface == TERRAIN_TYPE_FOREST_TOP
		
			} // end poly loop

			if( sector_bool ){

				sector_count ++;

				printf("Sector %d x %d\n", sec_z, sec_x);
			}

		} // end sec_x loop
	} // end sec_z loop

	printf("\n%d sectors have slopes above %f\n", sector_count, slope_x);

	printf("%d triangles have slopes above %f\n\n", triangle_count, slope_x);


} // end get_slopes_above_x

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int steep_check ( int sec_z, int sec_x, float slope_x ){

	int
		number_of_points,
		number_of_polys,
		number_of_poly_points,
		surface_change,
		surface,
		surface_changes_index,
		point_reference,
		point_ref_counter,
		point_loop,
		poly_loop,
		poly_point_loop,
		poly_point_references [ 128 ],
		triangle_counter,
		triangle_point_reference_0,
		triangle_point_reference_1,
		triangle_point_reference_2,
		triangle_count = 0;
		
	float
		slope;

	
	surface_changes_index = 0;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;
	
	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;
	
	//
	// find forest top polys
	//
	
	point_ref_counter = 0;
	
	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){
	
		number_of_poly_points = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].number_of_points;
	
		surface_change = (int)terrain_sectors[ sec_z ][ sec_x ].polygons [ poly_loop ].surface_change;
	
		if( poly_loop == 0 ){
	
			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;
	
			surface_changes_index ++;
		}
		else if( surface_change ){
	
			surface = (int)terrain_sectors[ sec_z ][ sec_x ].surface_changes [ surface_changes_index ].surface_id;
	
			surface_changes_index ++;
		}
	
		//////////////////////////////////////////////////////////////////////
		// need to go through the points of all polys
		// even if they aren't forest polys because
		// of the sequential way the points are stored
		//////////////////////////////////////////////////////////////////////
	
		for(poly_point_loop = 0; poly_point_loop < number_of_poly_points; poly_point_loop ++){
	
			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){
	
				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_byte_references[ point_ref_counter ].point;
	
				point_ref_counter ++;
			}
			else{
	
				point_reference = (int)terrain_sectors[ sec_z ][ sec_x ].point_word_references[ point_ref_counter ].point;
	
				point_ref_counter ++;
			}
	
			poly_point_references [ poly_point_loop ] = point_reference;
	
		} // end poly point loop
	
		if ( surface == TERRAIN_TYPE_FOREST_TOP ){
	
			//
			// break 3d_face into triangles because 3d_faces aren't always flat
			//
	
			triangle_point_reference_0 = poly_point_references [ 0 ];
	
			for(triangle_counter = 0; triangle_counter < (number_of_poly_points - 2); triangle_counter++){
	
				triangle_point_reference_1 = poly_point_references [ triangle_counter + 1 ];
	
				triangle_point_reference_2 = poly_point_references [ triangle_counter + 2 ];
	
				slope = (float)normal_angle_with_y ( sec_z, sec_x, triangle_point_reference_0,
					triangle_point_reference_1, triangle_point_reference_2 );
	
				//printf("triangle %d slope %f\n", triangle_counter, slope);
	
				if( slope > slope_x ){

					return 1;
					
				} // end if slope is steep
				
			} // end triangle loop
			
		} // end if surface == TERRAIN_TYPE_FOREST_TOP
	
	} // end poly loop

	return 0;

} // end steep check

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////







