
#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define INSIDE 1
#define OUTSIDE 0
#define  TRUE 1
#define  FALSE 0


// Tree parameters
// smaller numbers = greater chance for CHANCE parameters
// larger numbers = more trees for NUMBER parameters


#define DISSOLVE_TO_SOLID_RATIO				1		// X to 1	0 = no dissolve trees
#define MAX_TREES_PER_SECTOR				700



//
// Terrain data is stored in free-form polygons for now.
//

//
// The terrain grid is broken down into sectors.
//

#define TERRAIN_3D_SECTOR_SIDE_LENGTH 2048
#define TERRAIN_3D_SECTOR_SIDE_LENGTH_ADJUSTMENT 2048

//#define TERRAIN_3D_SECTOR_X_MAX 128

//#define TERRAIN_3D_SECTOR_Z_MAX 128

#define TERRAIN_3D_MIN_MAP_X	(0.0)
#define TERRAIN_3D_MIN_MAP_Z	(0.0)
#define TERRAIN_3D_MAX_MAP_X	((float) (TERRAIN_3D_SECTOR_X_MAX * TERRAIN_3D_SECTOR_SIDE_LENGTH) - 0.001)
#define TERRAIN_3D_MAX_MAP_Z	((float) (TERRAIN_3D_SECTOR_Z_MAX * TERRAIN_3D_SECTOR_SIDE_LENGTH) - 0.001)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// Terrain geometry data scaling factors
//

#define TERRAIN_3D_XZ_SCALE ( ( ( float ) ( TERRAIN_3D_SECTOR_SIDE_LENGTH + TERRAIN_3D_SECTOR_SIDE_LENGTH_ADJUSTMENT ) ) / 65536.0 )

#define TERRAIN_3D_Y_SCALE ( 1.0 / 32767.0 )

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_XZ
{

	short int
		x,
		z;
};

typedef struct TERRAIN_3D_POINT_XZ terrain_3d_point_xz;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma pack(1)

struct TERRAIN_3D_POINT_Y
{

	union
	{

		struct
		{

			unsigned short int
				y:15,
				normal_change:1;
		};

		unsigned short int
			whole_value;

		short int
			whole_signed_value;
	};
};

typedef struct TERRAIN_3D_POINT_Y terrain_3d_point_y;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR
{

	unsigned char
		red,
		green,
		blue;
};

typedef struct TERRAIN_3D_COLOUR terrain_3d_colour;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_COLOUR_INDEX terrain_3d_colour_index;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_NORMAL_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_NORMAL_INDEX terrain_3d_normal_index;

#pragma pack(4)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_WORD_REFERENCE
{

	unsigned short int
		point;
};

typedef struct TERRAIN_3D_POINT_WORD_REFERENCE terrain_3d_point_word_reference;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_BYTE_REFERENCE
{

	unsigned char
		point;
};

typedef struct TERRAIN_3D_POINT_BYTE_REFERENCE terrain_3d_point_byte_reference;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_FACE
{

	unsigned char
		number_of_points:7,
		surface_change:1;
};

typedef struct TERRAIN_3D_FACE terrain_3d_face;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_SURFACE
{

	unsigned char
		surface_id;
};

typedef struct TERRAIN_3D_SURFACE terrain_3d_surface;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_SECTOR
{
	int
		number_of_surface_changes,
		number_of_surfaces,
		number_of_normal_changes,
		number_of_point_references,
		colour_indices_offset,
		point_colours_offset,
		number_of_colors,
		all_sea_sector,
		forest_sector;

	unsigned short int
		number_of_points,
		number_of_polygons,
		radius;

	short int
		minimum_height,
		maximum_height,
		average_height,
		pad_2_bytes;

	struct TERRAIN_3D_POINT_XZ
		*points_xz;

	struct TERRAIN_3D_POINT_Y
		*points_y;

	struct TERRAIN_3D_FACE
		*polygons;

	union
	{

		struct TERRAIN_3D_POINT_WORD_REFERENCE
			*point_word_references;

		struct TERRAIN_3D_POINT_BYTE_REFERENCE
			*point_byte_references;
	};


	struct TERRAIN_3D_SURFACE
		*surface_changes;

	struct TERRAIN_3D_NORMAL_INDEX
		*normal_changes;

	struct TERRAIN_3D_COLOUR
		*point_colours;

	struct TERRAIN_3D_COLOUR_INDEX
		*colour_indices;

	//
	// Pointer to a sector approximation
	//

//	struct TERRAIN_3D_SECTOR
//		*approximation;

	//
	// If the sector has any terrain objects, they're on this list
	//

	struct TERRAIN_3D_OBJECT
		*objects;
};

typedef struct TERRAIN_3D_SECTOR terrain_3d_sector;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

//
// Terrain types are ordered according to their relative heights in a sector
//

enum //TERRAIN_TYPES
{

	TERRAIN_TYPE_INVALID, // 0

	// Sunken features tops
   TERRAIN_TYPE_RIVER,
   TERRAIN_TYPE_RESERVOIR,
   TERRAIN_TYPE_TRENCH,

	// Sunken features banks
   TERRAIN_TYPE_RIVER_BANK,
   TERRAIN_TYPE_TRENCH_SIDE_X,
   TERRAIN_TYPE_TRENCH_SIDE_Z,

	// Regular terrain types
   TERRAIN_TYPE_SEA,

   TERRAIN_TYPE_BEACH,

   TERRAIN_TYPE_LAND,// 9

	TERRAIN_TYPE_FIELD1,
	TERRAIN_TYPE_FIELD2,
	TERRAIN_TYPE_FIELD3,
	TERRAIN_TYPE_FIELD4,
	TERRAIN_TYPE_FIELD5,
	TERRAIN_TYPE_FIELD6,
	TERRAIN_TYPE_FIELD7,
	TERRAIN_TYPE_FIELD8,
	TERRAIN_TYPE_FIELD9,
	TERRAIN_TYPE_FIELD10,
	TERRAIN_TYPE_FIELD11,

	TERRAIN_TYPE_ALTERED_LAND1,
	TERRAIN_TYPE_ALTERED_LAND2,
	TERRAIN_TYPE_ALTERED_LAND3,

	TERRAIN_TYPE_FOREST_FLOOR,

	// Raised features banks
   TERRAIN_TYPE_ROAD_BANK,
   TERRAIN_TYPE_RAIL_BANK, // 26

	// Raised features sides
	TERRAIN_TYPE_HEDGE_SIDE_X,
	TERRAIN_TYPE_HEDGE_SIDE_Z,
	TERRAIN_TYPE_WALL_SIDE_X,
	TERRAIN_TYPE_WALL_SIDE_Z,
   TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X,
   TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z,
   TERRAIN_TYPE_FOREST_SIDE_MID_X,
   TERRAIN_TYPE_FOREST_SIDE_MID_Z,
   TERRAIN_TYPE_FOREST_SIDE_TOP_X,
   TERRAIN_TYPE_FOREST_SIDE_TOP_Z,

	// Raised features tops
   TERRAIN_TYPE_BUILT_UP_AREA1,
   TERRAIN_TYPE_BUILT_UP_AREA2,
   TERRAIN_TYPE_BUILT_UP_AREA3,
   TERRAIN_TYPE_BUILT_UP_AREA4,

   TERRAIN_TYPE_ROAD,  // 41
   TERRAIN_TYPE_TRACK,
   TERRAIN_TYPE_RAIL,
	TERRAIN_TYPE_WALL_TOP,
	TERRAIN_TYPE_HEDGE_TOP,

   TERRAIN_TYPE_FOREST_TOP, // 46

	// Temporary types used in the terrain generation, but not in the visual
	TEMP_TERRAIN_TYPE_FRINGE,
	TEMP_TERRAIN_TYPE_POWER_LINE,
	TEMP_TERRAIN_TYPE_TELEPHONE_LINE,
	TEMP_TERRAIN_TYPE_SEA_BANK,
	TEMP_TERRAIN_TYPE_COASTAL_RIVER,

	TEMP_TERRAIN_TYPE_BEACH_MID,

   TEMP_TERRAIN_TYPE_STEPPED_RIVER_BANK3,
   TEMP_TERRAIN_TYPE_STEPPED_RIVER_BANK5,
   TEMP_TERRAIN_TYPE_STEPPED_RIVER_BANK7,

   TEMP_TERRAIN_TYPE_LAKE,

   TEMP_TERRAIN_TYPE_FOREST_SIDE_X,
   TEMP_TERRAIN_TYPE_FOREST_SIDE_Z,

	TEMP_TERRAIN_TYPE_OFFROAD,
	TEMP_TERRAIN_TYPE_POLITICAL_BOUNDARY,
	TEMP_TERRAIN_TYPE_BUILT_UP_AREA_BANK,

	TERRAIN_TYPE_LAST, // 62
};


//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

typedef struct {
   double x,y;
} Point;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define TERRAIN_3D_MAX_TREE_HEIGHT	(35)


struct TERRAIN_3D_TREE_DATA
{

	signed short int
		x,
		z;

	unsigned short int
		y;

	unsigned char
		colour;

	unsigned char
		height:7,
		type:1;
};

typedef struct TERRAIN_3D_TREE_DATA terrain_3d_tree_data;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



