// Grand Roads
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "GrandRoads.h"

////////////////////////////////////////
// Globals
////////////////////////////////////////

terrain_3d_sector
	terrain_sectors[MAP_HEIGHT][MAP_WIDTH];

//terrain_3d_sector
//	**terrain_sectors;

road
	roads[64];

road_poly_list_per_sector
		sector_road_poly_list[MAP_HEIGHT][MAP_WIDTH]; // world and sector coords

int
	terrain_3d_map_width,
	terrain_3d_map_height,
	rgb_4byte_header,
	ffp_4byte_header,
	terrain_version,

	number_of_roads,
	max_road_poly_points,
	min_road_poly_points,
	max_road_polys_in_sector[3];

float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height;

double
	terrain_3d_map_scaled_height_difference;

/////////////////////////////////////////
// functions
/////////////////////////////////////////

void load_3d_terrain (void);

void process_roads(void);

int InsidePolygon(simple_point *polygon,int N,simple_point p);

int get_y_value(int sec_x, int sec_z, int x, int z);

static int solve_array(double *a,				 /* input array of wide by high */
		       double *b,				 /* result array of size high */
		       int wide,				 /* width of array */
		       int high);				 /* height of array */

simple_point	get_line_intersection(simple_point p1,
							  simple_point p2,
							  simple_point p3,
							  simple_point p4);

simple_point	get_segment_intersection(simple_point p1,
									  simple_point p2,
									  simple_point p3,
									  simple_point p4);

int get_sector_quad_poly_intersection(int z_sector,
								  int x_sector,
								  int road,
								  int quad);

int		get_sector_coords(int z_sector, int x_sector);

sector_point		get_sector_coordinates(simple_point in_pt, int sec_x, int sec_z);

double		isLeft( simple_point P0, simple_point P1, simple_point P2 );

road_poly	get_clockwise_ordered_poly(road_poly	in_poly);

sector		get_sector_of_world_coord_point(simple_point point);

void	write_road_poly_binary(void);

void	add_intermediate_points_to_road_file_points(void);

void	print_road_file_points (void);

int		get_distance(int x, int x1, int z, int z1);

void	load_road_file (void);



int main(int argc, char ** argv)
{
	
	load_3d_terrain ();

	load_road_file();

	//add_intermediate_points_to_road_file_points();

	process_roads();

	//print_road_file_points ();

	write_road_poly_binary();

	return 0;

} // end main

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


void process_roads(){

	int
		i,
		j,
		k,
		l,
		number_of_points_for_this_road,
		number_of_quads,
		//x_int_pos,		// Rob Fox uses ints for floats (int/100 = float)
		//z_int_pos,
		return_value;

	double 
		ax,
		az,
		bx,
		bz;

	double
		theta;

	simple_point
		p1,
		p2,
		p3,
		p4,
		p_intersect;

	printf("processing roads\n");

	max_road_poly_points = 0;
	min_road_poly_points = 6;
	max_road_polys_in_sector[0] = 0;


	///////////////////////////////////////////////////////////////////////////
	// initialize global array
	///////////////////////////////////////////////////////////////////////////

	for(i = 0; i < MAP_HEIGHT; i++){
		for(j = 0; j < MAP_WIDTH; j++){

			sector_road_poly_list[i][j].number_of_polys = 0;

			for(k = 0; k < 32; k++){

				sector_road_poly_list[i][j].
						poly_list[k].
						number_of_points = 0;
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////
	// do calculations on the center points
	///////////////////////////////////////////////////////////////////////////

	// the global value of terrain_3d_map_scaled_height_difference is set here

	terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	for(i = 0; i < number_of_roads; i++){

		number_of_points_for_this_road = roads[i].number_of_center_points;

		for(j = 0; j < number_of_points_for_this_road; j++){

			/////////////////////////////////////////////////////////////
			// find the sector
			/////////////////////////////////////////////////////////////

			roads[i].center_points[j].x_sector = (int)roads[i].center_points[j].global_x / 2048;
			roads[i].center_points[j].z_sector = (int)roads[i].center_points[j].global_z / 2048;

			//////////////////////////////////////////////////////////////
			// find the sector coordinates
			//////////////////////////////////////////////////////////////

			roads[i].center_points[j].sec_x = (int)((roads[i].center_points[j].global_x -
				(roads[i].center_points[j].x_sector * 2048 + 1024)) * 16);

			roads[i].center_points[j].sec_z = (int)((roads[i].center_points[j].global_z -
				(roads[i].center_points[j].z_sector * 2048 + 1024)) * 16);

			//printf("%f    %d    %d\n", roads[i].center_points[j].global_x,
			//	roads[i].center_points[j].x_sector,
			//	roads[i].center_points[j].sec_x);

			///////////////////////////////////////////////////////////////
			// get the y value in sector coordinates
			///////////////////////////////////////////////////////////////

			roads[i].center_points[j].sec_y = get_y_value(
				roads[i].center_points[j].x_sector,
				roads[i].center_points[j].z_sector,
				roads[i].center_points[j].sec_x,
				roads[i].center_points[j].sec_z);

			////////////////////////////////////////////////////////////////
			// calculate the global y value
			////////////////////////////////////////////////////////////////

			roads[i].center_points[j].global_y = ((double)terrain_3d_map_minimum_height + 
				((double)roads[i].center_points[j].sec_y * terrain_3d_map_scaled_height_difference ));
	

		} // end for points per road
	} // end for roads

	////////////////////////////////////////////////////////////////////////
	// do calculations on center points to get four points (quad) for
	// each road section
	////////////////////////////////////////////////////////////////////////

	for(i = 0; i < number_of_roads; i++){

		number_of_quads = roads[i].number_of_quads;

		//printf("Road %d has %d quads\n\n", i, number_of_quads);

		for(j = 0; j < number_of_quads; j++){

			///////////////////////////////////////////////////////////
			// find the quad points p, q, r, s in world coordinates
			// a and b are the center points for each road section
			///////////////////////////////////////////////////////////

			ax = roads[i].center_points[j].global_x;
			az = roads[i].center_points[j].global_z;
			bx = roads[i].center_points[j + 1].global_x;
			bz = roads[i].center_points[j + 1].global_z;

			theta = atan((ax - bx) / (bz - az));

			if(az <= bz){

				roads[i].quads[j].p.global_x = WIDTH * (cos(theta)) + ax;
				roads[i].quads[j].p.global_z = WIDTH * (sin(theta)) + az;

				roads[i].quads[j].q.global_x = ax - WIDTH * (cos(theta));
				roads[i].quads[j].q.global_z = az - WIDTH * (sin(theta));

				roads[i].quads[j].r.global_x = bx - WIDTH * (cos(theta));
				roads[i].quads[j].r.global_z = bz - WIDTH * (sin(theta));

				roads[i].quads[j].s.global_x = WIDTH * (cos(theta)) + bx;
				roads[i].quads[j].s.global_z = WIDTH * (sin(theta)) + bz;
			}
			else{ // need different calculations for cases 2, 4, 6

				roads[i].quads[j].p.global_x = ax - WIDTH * (cos(theta));
				roads[i].quads[j].p.global_z = az - WIDTH * (sin(theta));

				roads[i].quads[j].q.global_x = WIDTH * (cos(theta)) + ax;
				roads[i].quads[j].q.global_z = WIDTH * (sin(theta)) + az;

				roads[i].quads[j].r.global_x = WIDTH * (cos(theta)) + bx;
				roads[i].quads[j].r.global_z = WIDTH * (sin(theta)) + bz;

				roads[i].quads[j].s.global_x = bx - WIDTH * (cos(theta));
				roads[i].quads[j].s.global_z = bz - WIDTH * (sin(theta));
			}


		} // end for point for this road
	} // end for number of roads

	/////////////////////////////////////////////////////////////////////
	// change all quad points except the end points to
	// the intersections of adjacent road section lines
	/////////////////////////////////////////////////////////////////////

	printf("quads are built\n");

	for(i = 0; i < number_of_roads; i++){

		number_of_quads = roads[i].number_of_quads;

		for(j = 0; j < number_of_quads - 1; j++){

			// line 1 points p and s

			p1.x = roads[i].quads[j].p.global_x;
			p1.z = roads[i].quads[j].p.global_z;
			p2.x = roads[i].quads[j].s.global_x;
			p2.z = roads[i].quads[j].s.global_z;

			// line 2 points p and s

			p3.x = roads[i].quads[j + 1].p.global_x;
			p3.z = roads[i].quads[j + 1].p.global_z;
			p4.x = roads[i].quads[j + 1].s.global_x;
			p4.z = roads[i].quads[j + 1].s.global_z;

			// intersection of line 1 and line 2

			p_intersect = get_line_intersection(p1, p2, p3, p4);

			//////////////////////////////////////////////////////////////////////////////////
			// check if lines were parallel
			// if they are we don't want to change the intersection points
			// because they are already correct because right angle
			// cross lines meet flush together
			// the returned value of the intersection algorithm is 0 anyway
			//////////////////////////////////////////////////////////////////////////////////

			if(p_intersect.x == 0 && p_intersect.z == 0){
				printf("Road %d Sections %d and %d are parallel\n\n", i, j, j+1);
				//exit (1);
				continue; // leave these cross point alone and go to next loop
			}

			// change quad point s of first section
			// and quad point p of next section

			roads[i].quads[j].s.global_x = p_intersect.x;
			roads[i].quads[j].s.global_z = p_intersect.z;

			roads[i].quads[j + 1].p.global_x = p_intersect.x;
			roads[i].quads[j + 1].p.global_z = p_intersect.z;


			///////////////////////////////////////////////////////////////
			// do the same thing for the other lines of the two quads
			///////////////////////////////////////////////////////////////

			// line 1 points q and r

			p1.x = roads[i].quads[j].q.global_x;
			p1.z = roads[i].quads[j].q.global_z;
			p2.x = roads[i].quads[j].r.global_x;
			p2.z = roads[i].quads[j].r.global_z;

			// line 2 points q and r

			p3.x = roads[i].quads[j + 1].q.global_x;
			p3.z = roads[i].quads[j + 1].q.global_z;
			p4.x = roads[i].quads[j + 1].r.global_x;
			p4.z = roads[i].quads[j + 1].r.global_z;

			// intersection of line 1 and line 2

			p_intersect = get_line_intersection(p1, p2, p3, p4);

			// change quad point r of first section
			// and quad point q of next section

			roads[i].quads[j].r.global_x = p_intersect.x;
			roads[i].quads[j].r.global_z = p_intersect.z;

			roads[i].quads[j + 1].q.global_x = p_intersect.x;
			roads[i].quads[j + 1].q.global_z = p_intersect.z;


		} // end for point for this road
	} // end for number of roads

	printf("line intersections are finished\n");

	///////////////////////////////////////////////////////////////////
	// get the sector coordinates for the quads
	///////////////////////////////////////////////////////////////////

	for(i = 0; i < number_of_roads; i++){

		number_of_quads = roads[i].number_of_quads;

		for(j = 0; j < number_of_quads; j++){

			////////////////////////////////////////////////////////////
			// find the sector
			////////////////////////////////////////////////////////////

			roads[i].quads[j].p.x_sector = (int)roads[i].quads[j].p.global_x / 2048;
			roads[i].quads[j].p.z_sector = (int)roads[i].quads[j].p.global_z / 2048;

			roads[i].quads[j].q.x_sector = (int)roads[i].quads[j].q.global_x / 2048;
			roads[i].quads[j].q.z_sector = (int)roads[i].quads[j].q.global_z / 2048;

			roads[i].quads[j].r.x_sector = (int)roads[i].quads[j].r.global_x / 2048;
			roads[i].quads[j].r.z_sector = (int)roads[i].quads[j].r.global_z / 2048;

			roads[i].quads[j].s.x_sector = (int)roads[i].quads[j].s.global_x / 2048;
			roads[i].quads[j].s.z_sector = (int)roads[i].quads[j].s.global_z / 2048;

			/////////////////////////////////////////////////////////////
			// find sector coords
			/////////////////////////////////////////////////////////////

			roads[i].quads[j].p.sec_x = (int)((roads[i].quads[j].p.global_x -
				(roads[i].quads[j].p.x_sector * 2048 + 1024)) * 16);

			roads[i].quads[j].p.sec_z = (int)((roads[i].quads[j].p.global_z -
				(roads[i].quads[j].p.z_sector * 2048 + 1024)) * 16);


			roads[i].quads[j].q.sec_x = (int)((roads[i].quads[j].q.global_x -
				(roads[i].quads[j].q.x_sector * 2048 + 1024)) * 16);

			roads[i].quads[j].q.sec_z = (int)((roads[i].quads[j].q.global_z -
				(roads[i].quads[j].q.z_sector * 2048 + 1024)) * 16);


			roads[i].quads[j].r.sec_x = (int)((roads[i].quads[j].r.global_x -
				(roads[i].quads[j].r.x_sector * 2048 + 1024)) * 16);

			roads[i].quads[j].r.sec_z = (int)((roads[i].quads[j].r.global_z -
				(roads[i].quads[j].r.z_sector * 2048 + 1024)) * 16);


			roads[i].quads[j].s.sec_x = (int)((roads[i].quads[j].s.global_x -
				(roads[i].quads[j].s.x_sector * 2048 + 1024)) * 16);

			roads[i].quads[j].s.sec_z = (int)((roads[i].quads[j].s.global_z -
				(roads[i].quads[j].s.z_sector * 2048 + 1024)) * 16);

			///////////////////////////////////////////////////////////////////
			// assign the y values for the quads which are equal to
			// the center point y value
			///////////////////////////////////////////////////////////////////

			roads[i].quads[j].p.sec_y = roads[i].center_points[j].sec_y;
			roads[i].quads[j].p.global_y = roads[i].center_points[j].global_y;

			roads[i].quads[j].q.sec_y = roads[i].center_points[j].sec_y;
			roads[i].quads[j].q.global_y = roads[i].center_points[j].global_y;

			roads[i].quads[j].r.sec_y = roads[i].center_points[j + 1].sec_y;
			roads[i].quads[j].r.global_y = roads[i].center_points[j + 1].global_y;

			roads[i].quads[j].s.sec_y = roads[i].center_points[j + 1].sec_y;
			roads[i].quads[j].s.global_y = roads[i].center_points[j + 1].global_y;


			///////////////////////////////////
			// print section
			///////////////////////////////////


			printf("\n\tRoad %d Quad %d\n", i, j);

			printf("p sector = %d x %d  x = %d  z = %d  y = %d\n",
				roads[i].quads[j].p.x_sector,
				roads[i].quads[j].p.z_sector,
				roads[i].quads[j].p.sec_x,
				roads[i].quads[j].p.sec_z,
				roads[i].quads[j].p.sec_y);

			printf("q sector = %d x %d  x = %d  z = %d  y = %d\n",
				roads[i].quads[j].q.x_sector,
				roads[i].quads[j].q.z_sector,
				roads[i].quads[j].q.sec_x,
				roads[i].quads[j].q.sec_z,
				roads[i].quads[j].q.sec_y);

			printf("r sector = %d x %d  x = %d  z = %d  y = %d\n",
				roads[i].quads[j].r.x_sector,
				roads[i].quads[j].r.z_sector,
				roads[i].quads[j].r.sec_x,
				roads[i].quads[j].r.sec_z,
				roads[i].quads[j].r.sec_y);

			printf("s sector = %d x %d  x = %d  z = %d  y = %d\n",
				roads[i].quads[j].s.x_sector,
				roads[i].quads[j].s.z_sector,
				roads[i].quads[j].s.sec_x,
				roads[i].quads[j].s.sec_z,
				roads[i].quads[j].s.sec_y);


			ax = roads[i].center_points[j].global_x;
			az = roads[i].center_points[j].global_z;
			bx = roads[i].center_points[j + 1].global_x;
			bz = roads[i].center_points[j + 1].global_z;



			printf("section %d  center points ax = %f  az = %f  bx = %f  bz = %f\n",
				j, ax, az, bx, bz);
			//printf("length = %f\n", (float)sqrt(pow((bx-ax), 2) + pow((bz - az), 2))); // double convert
			printf("length = %f\n", sqrt(pow((bx-ax), 2) + pow((bz - az), 2)));
			printf("px = %f  pz = %f  qx = %f  qz = %f\n", 
				roads[i].quads[j].p.global_x,
				roads[i].quads[j].p.global_z,
				roads[i].quads[j].q.global_x,
				roads[i].quads[j].q.global_z);

			printf("rx = %f  rz = %f  sx = %f  sz = %f\n\n", 
				roads[i].quads[j].r.global_x,
				roads[i].quads[j].r.global_z,
				roads[i].quads[j].s.global_x,
				roads[i].quads[j].s.global_z);

			if(roads[i].quads[j].p.global_z == roads[i].quads[j].s.global_z){
				printf("Road %d Quad %d is Horizontal\n", i, j);
			}
			
			if(roads[i].quads[j].p.global_x == roads[i].quads[j].s.global_x){
				printf("Road %d Quad %d is Vertical\n", i, j);
			}

			////////////////////////////////////////////
			// end print section
			////////////////////////////////////////////

			
		} // end for quads for this road
	} // end for number of roads


	/////////////////////////////////////////////////////////////////////////////////
	// In world coordinates, need to cut the road quads into pieces where they
	// cross sector boundaries. The get_sector_quad_poly_intersection() function
	// determines if a quad line crosses a sector boundary and if so finds these
	// intersection points, and then adds the full quads or partial quads cut at
	// the sector boundaries into a global array of road polys for each sector
	// road_poly_list_per_sector sector_road_poly_list[MAP_HEIGHT][MAP_WIDTH];
	//
	// !!!! always call these functions( z, x ) !!!
	/////////////////////////////////////////////////////////////////////////////////

	for(i = 0; i < MAP_HEIGHT; i++){
		for(j = 0; j < MAP_WIDTH; j++){
			for(k = 0; k < number_of_roads; k++){
				for(l = 0; l < roads[k].number_of_quads; l++){

					return_value = get_sector_quad_poly_intersection(i,j,k,l);

				}
			}
		}
	}


	///////////////////////////////////////////////////////////////////////////////////
	// Calculate the sector coordinates for all the road polys
	///////////////////////////////////////////////////////////////////////////////////

	for(i = 0; i < MAP_HEIGHT; i++){
		for(j = 0; j < MAP_WIDTH; j++){

			return_value = get_sector_coords(i, j);
		}
	}

	//return_value = get_sector_quad_poly_intersection(16,30,0,194);

	//return_value = get_sector_coords(16, 30);

	//return_value = get_sector_quad_poly_intersection(3,25,0,223);

	//return_value = get_sector_coords(3, 25);


	/////////////////////////////////////////////////////////////////////////////////
	// find the max number of polys in a sector
	/////////////////////////////////////////////////////////////////////////////////

	for(i = 0; i < MAP_HEIGHT; i++){
		for(j = 0; j < MAP_WIDTH; j++){

			if(sector_road_poly_list[i][j].number_of_polys > max_road_polys_in_sector[0]){

				max_road_polys_in_sector[0] = sector_road_poly_list[i][j].number_of_polys;
				max_road_polys_in_sector[1] = j;
				max_road_polys_in_sector[2] = i;
			}
		}
	}

	printf("Max Polys per sector = %d in Sector %d x %d\n\n", 
		max_road_polys_in_sector[0],
		max_road_polys_in_sector[1],
		max_road_polys_in_sector[2]);

	printf("Min points per road poly %d  Max points per road poly %d\n\n", 
		min_road_poly_points, max_road_poly_points);

	printf("end process_roads()\n");

} // end process_roads()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(simple_point *polygon,int N,simple_point p)
{
  int counter = 0;
  int i;
  double xinters;
  simple_point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.z > MIN(p1.z,p2.z)) {
      if (p.z <= MAX(p1.z,p2.z)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.z != p2.z) {
            xinters = (p.z-p1.z)*(p2.x-p1.x)/(p2.z-p1.z)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


int get_y_value(int sec_x, int sec_z, int x, int z){

	// This function returns the y value in sector coordinates
	// for the sector and x z sector coord pair

	int		i,
			polys_this_sector,
			poly_counter,
			poly_number,
			y_value;

	long long	k, // plane constant
				triangle_points[3][3], // [][0]=x, [][1]=z, [][2]=y
				Ax,Az,Ay,Bx,Bz,By,	// vectors
				Nx,Nz,Ny;			// normal vector

	double	terrain_3d_map_scaled_height_difference;

	simple_point	inside_poly_array[3],
					test_point;

	terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height )
		* TERRAIN_3D_Y_SCALE;


	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	test_point.x = x;
	test_point.z = z;

	// Find which polygon of this sector the point is on
	// by testing for point_in_poly with all polys until it is found

	for(poly_counter = 0; poly_counter < polys_this_sector; poly_counter++){

		poly_number = 1000;

		triangle_points[0][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][0].point].x,
		triangle_points[1][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][1].point].x,
		triangle_points[2][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][2].point].x,
		triangle_points[0][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][0].point].z,
		triangle_points[1][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][1].point].z,
		triangle_points[2][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][2].point].z,
		triangle_points[0][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][0].point].y,
		triangle_points[1][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][1].point].y,
		triangle_points[2][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][2].point].y;

		// assign points to inside_poly_array[]

		for(i = 0; i < 3; i++){
			inside_poly_array[i].x = (double)triangle_points[i][0];
			inside_poly_array[i].z = (double)triangle_points[i][1]; 
		}

		if(InsidePolygon(inside_poly_array, 3, test_point)){

			poly_number = poly_counter;
			break;
		}

	} // end for poly loop

	if(poly_number == 1000){
		printf("can't find poly for point %d x %d in sector %d x %d\n", x, z, sec_x, sec_z);
		exit(1);
	}

	// vectors
	Ax = triangle_points[0][0] - triangle_points[1][0]; // ax - bx
	Ay = triangle_points[0][2] - triangle_points[1][2]; // ay - by
	Az = triangle_points[0][1] - triangle_points[1][1]; // az - bz
	Bx = triangle_points[2][0] - triangle_points[1][0]; // cx - bx
	By = triangle_points[2][2] - triangle_points[1][2]; // cy - by
	Bz = triangle_points[2][1] - triangle_points[1][1]; // cz - bz

	// normal
	Nx = Ay * Bz - Az * By;
	Ny = Az * Bx - Ax * Bz;
	Nz = Ax * By - Ay * Bx;
				
	// plane constant
	//k = ax * Nx + ay * Ny + az * Nz;
	k = triangle_points[0][0] * Nx + triangle_points[0][2] * Ny + triangle_points[0][1] * Nz;

	y_value = (int)((k - x * Nx - z * Nz) / Ny);

	//printf("%d   %d   %d   %d   %d\n", poly_number, (int)triangle_points[0][2],
	//	(int)triangle_points[1][2], (int)triangle_points[2][2], y_value);


	return y_value;
} // end get_y_value()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


// this function solves a system of N variables with N equations using gaussian elimination

static int solve_array(double *a,				 /* input array of wide x high */
		       double *b,				 /* result array of size high */
		       int wide,				 /* width of array */
		       int high)				 /* height of array */
{
  int i, j, k;							 /* loop variables */
  double t;							 /* temporary (scaling) value */
  
  /*
   * Step 1: Make array upper triangular
   */
  for (i = 0; i < high; i++) {					 /* for each row in the array */
    t = m(i,i);							 /* pick the scale factor */
    if (t == 0.0) {						 /* if it is 0.0, the array is singular */
      printf("Singular: Error in the Solve_array() function\n");
      //print_array(a, wide, high);
      return 0;							 /* error here */
    }
    for (j = i; j < wide; j++)					 /* scale the row until diagonal point is 1.0 */
      m(j,i) = m(j,i)/t;					 /* narrow the loop to reduce load since left of i = 0.0 */
    
    for (j = i+1; j < high; j++) {				 /* zero out all lower entries in this column */
      t = m(i, j);						 /* factor to solve by */
      for (k = 0; k < wide; k++)				 /* subtract ith row from jth row t times */
	m(k,j) = m(k,j) - t * m(k,i);				 /* to zero out next column */
    }
  }
  /* 
   * Step 2: Make array into identity matrix by eliminating non-diagonal entries in upper triangle.
   */
  for (i = high - 1; i > 0; i--) {				 /* solve for identity matrix */
    for (j = i - 1; j >= 0; j--) {				 /* take away one column */
      t = m(i,j);						 /* get the factor */
      for (k = 0; k < wide; k++)				 /* subtract the t times the i'th equation from the j'th equation*/
	m(k,j) = m(k,j) - t * m(k,i);
    }
  }
  /*
   * Step 3: Get the last column which contains the solution (since the
   *         array is now the identity matrix).
   */
  for (i = 0; i < high; i++)					 
    b[i] = m(wide-1,i);
  return 1;							 /* return success */
} // end solve_array()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


simple_point get_line_intersection(simple_point p1,
							  simple_point p2,
							  simple_point p3,
							  simple_point p4){

	int		return_value;

	simple_point	result,
					temp;

	/////////////////////////////////////////////////////////////////////
	// arrays are used for input/output to solve array()
	// which uses gausian elimination to solve two
	// simultaneous equations
	/////////////////////////////////////////////////////////////////////

	double input[6];
	double output[2];

	double	x1,
			y1,
			x2,
			y2,
			a,
			b,
			c,
			d,
			e,
			f;

	///////////////////////////////////////////////////////////////////////
	// test to see if both lines are vertical because you can't
	// divide by zero
	///////////////////////////////////////////////////////////////////////

	if((p1.x == p2.x) && (p3.x == p4.x)){

		result.x = 0.0;
		result.z = 0.0;
		return result;
	}

	///////////////////////////////////////////////////////////////////////
	// test to see if lines are parallel
	///////////////////////////////////////////////////////////////////////

	if(((p2.z - p1.z) / (p2.x - p1.x)) == ((p4.z - p3.z) / (p4.x - p3.x))){

		result.x = 0.0;
		result.z = 0.0;
		return result;
	}

	///////////////////////////////////////////////////////////////////////////
	// check for special case if first line is vertical
	// if x2 == x1 slope is infinite and you can't divide by zero
	// if this is the case we convert the other line to the slope-intercept
	// form y = ax + b where b is the y-intercept and a is the slope
	///////////////////////////////////////////////////////////////////////////

	if(p1.x == p2.x){

		a = (p4.z - p3.z) / (p4.x - p3.x); // slope

		////////////////////////////////////////////////////////////////////////
		// now just need to transpose a point of line 2 to the left
		// by p1.x and plug the new values into y = ax + b and solve for b
		// b = y - a*x
		////////////////////////////////////////////////////////////////////////

		temp = p3;
		
		temp.x = temp.x - p1.x;

		b = temp.z - a * temp.x;

		result.x = p1.x;
		result.z = b;

		return result;

	} // end if

	///////////////////////////////////////////////////////////////////////////
	// check for special case if second line is vertical
	///////////////////////////////////////////////////////////////////////////

	if(p3.x == p4.x){

		a = (p2.z - p1.z) / (p2.x - p1.x); // slope

		////////////////////////////////////////////////////////////////////////
		// now just need to transpose a point of line 2 to the left
		// by x1 and plug the new values into y = ax + b and solve for b
		// b = y - a*x
		////////////////////////////////////////////////////////////////////////

		temp.x = p1.x;
		temp.z = p1.z;
	
		temp.x = temp.x - p3.x;

		b = temp.z - a * temp.x;

		result.x = p3.x;
		result.z = b;

		return result;

	} // end if

	///////////////////////////////////////////////////////////////////////////
	// this section converts a line formula of the form
	// y - y1 = ((y2 - y1) / (x2 - x1)) * (x - x1) to the form
	// d * y + e * x = f
	// d, e, and f are the matrix values for the gaussian elimination
	// method of solving two variables with two equations
	///////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////////
	// convert the first line
	///////////////////////////////////////////////////////////////////////////

	x1 = p1.x;
	y1 = p1.z;
	x2 = p2.x;
	y2 = p2.z;


	a = y1;
	b = (y2 - y1) / (x2 - x1); // slope
	c = x1;

	d = 1;
	e = -1 * b;
	f = a + (-1 * b * c);

	input[0] = d;
	input[1] = e;
	input[2] = f;

	//////////////////////////////////////////////////////////////////////////////
	// convert the second line
	//////////////////////////////////////////////////////////////////////////////

	x1 = p3.x;
	y1 = p3.z;
	x2 = p4.x;
	y2 = p4.z;

	a = y1;
	b = (y2 - y1) / (x2 - x1);  // slope
	c = x1;

	d = 1;
	e = -1 * b;
	f = a + (-1 * b * c);

	input[3] = d;
	input[4] = e;
	input[5] = f;

	return_value = solve_array(input, output, 3, 2);

	result.x = output[1];
	result.z = output[0];

	return result;
} // end get_line_intersection()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


simple_point get_segment_intersection(simple_point p1,
									simple_point p2,
									simple_point p3,
									simple_point p4){

	simple_point result;

	double	segment_1_length,
			intersect_p1_distance,
			intersect_p2_distance,
			segment_2_length,
			intersect_p3_distance,
			intersect_p4_distance;

	result = get_line_intersection(p1, p2, p3, p4);

	segment_1_length = sqrt((pow((p2.x - p1.x), 2)) + (pow((p2.z - p1.z), 2)));
	intersect_p1_distance = sqrt((pow((result.x - p1.x), 2)) + (pow((result.z - p1.z), 2)));
	intersect_p2_distance = sqrt((pow((result.x - p2.x), 2)) + (pow((result.z - p2.z), 2)));

	segment_2_length = sqrt((pow((p4.x - p3.x), 2)) + (pow((p4.z - p3.z), 2)));
	intersect_p3_distance = sqrt((pow((result.x - p3.x), 2)) + (pow((result.z - p3.z), 2)));
	intersect_p4_distance = sqrt((pow((result.x - p4.x), 2)) + (pow((result.z - p4.z), 2)));


	if((intersect_p1_distance <= segment_1_length &&
		intersect_p2_distance <= segment_1_length) &&
		(intersect_p3_distance <= segment_2_length &&
		intersect_p4_distance <= segment_2_length)){

		return result;
	}
	else{

		result.x = 0.0;
		result.z = 0.0;
		return result;
	}

} // end get_segment_intersection

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

/////////////////////////////////////////////////////////////
// Cut road polys at sector boundaries
// and build global list of road polys
/////////////////////////////////////////////////////////////

int get_sector_quad_poly_intersection(int z_sector,
								  int x_sector,
								  int road,
								  int quad){

	int	counter,
		quad_point_counter,
		sector_point_counter,
		number_of_road_quad_points_inside_sector = 0,
		number_of_sector_points_inside_road_quad = 0,
		number_of_line_intersection_points =0;


	simple_point	road_quad_points_inside_sector[4],
					sector_points_inside_road_quad[4],
					sector_points[4],
					road_quad_points[4],
					line_intersection_points[6],
					poly_intersection_points[6],
					temp_point;

	/////////////////////////////////////////////////////////////////////////////
	// Assign road quad points
	/////////////////////////////////////////////////////////////////////////////

	road_quad_points[0].x = roads[road].quads[quad].p.global_x;
	road_quad_points[0].z = roads[road].quads[quad].p.global_z;
	road_quad_points[0].y = roads[road].quads[quad].p.global_y;
	road_quad_points[0].letter = 'p';
	road_quad_points[0].road = road;
	road_quad_points[0].quad = quad;

	road_quad_points[1].x = roads[road].quads[quad].q.global_x;
	road_quad_points[1].z = roads[road].quads[quad].q.global_z;
	road_quad_points[1].y = roads[road].quads[quad].q.global_y;
	road_quad_points[1].letter = 'q';
	road_quad_points[1].road = road;
	road_quad_points[1].quad = quad;

	road_quad_points[2].x = roads[road].quads[quad].r.global_x;
	road_quad_points[2].z = roads[road].quads[quad].r.global_z;
	road_quad_points[2].y = roads[road].quads[quad].r.global_y;
	road_quad_points[2].letter = 'r';
	road_quad_points[2].road = road;
	road_quad_points[2].quad = quad;

	road_quad_points[3].x = roads[road].quads[quad].s.global_x;
	road_quad_points[3].z = roads[road].quads[quad].s.global_z;
	road_quad_points[3].y = roads[road].quads[quad].s.global_y;
	road_quad_points[3].letter = 's';
	road_quad_points[3].road = road;
	road_quad_points[3].quad = quad;

	/////////////////////////////////////////////////////////////////////////////
	// Asign sector points starting at lower left and going clockwise
	/////////////////////////////////////////////////////////////////////////////

	sector_points[0].x = x_sector * 2048.0;
	sector_points[0].z = z_sector * 2048.0;
	sector_points[0].letter = 'a';
	sector_points[0].road = road;
	sector_points[0].quad = quad;

	sector_points[1].x = x_sector * 2048.0;
	sector_points[1].z = z_sector * 2048.0 + 2048.0;
	sector_points[1].letter = 'b';
	sector_points[1].road = road;
	sector_points[1].quad = quad;

	sector_points[2].x = x_sector * 2048.0 + 2048.0;
	sector_points[2].z = z_sector * 2048.0 + 2048.0;
	sector_points[2].letter = 'c';
	sector_points[2].road = road;
	sector_points[2].quad = quad;

	sector_points[3].x = x_sector * 2048.0 + 2048.0;
	sector_points[3].z = z_sector * 2048.0;
	sector_points[3].letter = 'd';
	sector_points[3].road = road;
	sector_points[3].quad = quad;

	/////////////////////////////////////////////////////////////////////////////
	// find any road quad points inside sector
	/////////////////////////////////////////////////////////////////////////////

	for(quad_point_counter = 0; quad_point_counter < 4; quad_point_counter++){

		if(InsidePolygon(sector_points,4,road_quad_points[quad_point_counter]) == INSIDE){

			road_quad_points_inside_sector[number_of_road_quad_points_inside_sector] =
				road_quad_points[quad_point_counter];

			number_of_road_quad_points_inside_sector++;

			//////////////////////////////////////////////////////////////////////
			// add this point into the global array
			//////////////////////////////////////////////////////////////////////

			sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				point_list[sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points] = road_quad_points[quad_point_counter];

			sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points++;
			
		} // end if
	} // end for

	////////////////////////////////////////////////////////////////////////////
	// find any sector corner points inside road quad
	////////////////////////////////////////////////////////////////////////////

	for(sector_point_counter = 0; sector_point_counter < 4; sector_point_counter++){

		if(InsidePolygon(road_quad_points,4,sector_points[sector_point_counter]) == INSIDE){

			sector_points_inside_road_quad[number_of_sector_points_inside_road_quad] =
				sector_points[sector_point_counter];

			number_of_sector_points_inside_road_quad++;

			//////////////////////////////////////////////////////////////////////
			// add this point into the global array
			//////////////////////////////////////////////////////////////////////

			sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				point_list[sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points] = sector_points[sector_point_counter];

			sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points++;
			
		} // end if
	} // end for


	/////////////////////////////////////////////////////////////////////////////
	// find intersections between quad lines and sector boundary lines
	/////////////////////////////////////////////////////////////////////////////


	for(quad_point_counter = 0; quad_point_counter < 4; quad_point_counter++){
		for(sector_point_counter = 0; sector_point_counter < 4; sector_point_counter++){

			temp_point = get_segment_intersection(road_quad_points[quad_point_counter],
												  road_quad_points[(quad_point_counter + 1) % 4],
												  sector_points[sector_point_counter],
												  sector_points[(sector_point_counter + 1) % 4]);

			if(temp_point.x != 0.0){ // if this is true there was an intersection

				//////////////////////////////////////////////////////////////////
				// determine the letter value for this point
				// p-s quad points = r intersection points (right side of road)
				// q-r quad points = l intersection points (left side of road)
				// p-q quad points = e intersection points (cross section line a end of road)
				// r-s quad points = f intersection points (cross section line b end of road)
				//////////////////////////////////////////////////////////////////

				if(quad_point_counter == 0){

					temp_point.letter = 'e';
					temp_point.road = road;
					temp_point.quad = quad;
				} // end if

				if(quad_point_counter == 1){

					temp_point.letter = 'L';
					temp_point.road = road;
					temp_point.quad = quad;
					
				} // end if

				if(quad_point_counter == 2){

					temp_point.letter = 'f';
					temp_point.road = road;
					temp_point.quad = quad;
				} // end if

				if(quad_point_counter == 3){

					temp_point.letter = 'R';
					temp_point.road = road;
					temp_point.quad = quad;
				} // end if

				//////////////////////////////////////////////////////////
				// calculate the global y value for the intersection
				// using the line formula form:
				// ay - y1 = (y2 - y1 / x2 - x1) * (ax - x1)
				//////////////////////////////////////////////////////////

				//////////////////////////////////////////////////////////
				// first check if line is vertical and then use
				// the y and z coordinate system (vertical is infinite slope)
				//////////////////////////////////////////////////////////

				if((road_quad_points[(quad_point_counter + 1) % 4].x) == (road_quad_points[quad_point_counter].x)){

					temp_point.y = ((road_quad_points[(quad_point_counter + 1) % 4].y - 
								road_quad_points[quad_point_counter].y) /
								(road_quad_points[(quad_point_counter + 1) % 4].z - 
								road_quad_points[quad_point_counter].z)) *
								(temp_point.z - road_quad_points[quad_point_counter].z) +
								road_quad_points[quad_point_counter].y;
				}

				else{  // use x and y coordinate system

					temp_point.y = ((road_quad_points[(quad_point_counter + 1) % 4].y - 
								road_quad_points[quad_point_counter].y) /
								(road_quad_points[(quad_point_counter + 1) % 4].x - 
								road_quad_points[quad_point_counter].x)) *
								(temp_point.x - road_quad_points[quad_point_counter].x) +
								road_quad_points[quad_point_counter].y;
				}
				
				///////////////////////////////////////////////////////////
				// add point to the array
				///////////////////////////////////////////////////////////

				line_intersection_points[number_of_line_intersection_points] = temp_point;

				//printf("%f  %f  %f\n", road_quad_points[quad_point_counter].y,
				//	line_intersection_points[number_of_line_intersection_points].y,
				//	road_quad_points[(quad_point_counter + 1) % 4].y);

				number_of_line_intersection_points++;

				sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					point_list[sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					number_of_points] = temp_point;

				sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					number_of_points++;
				
			} // end if

/*

			printf("*** Test %d ***\n", (quad_point_counter * 4 + sector_point_counter));

			printf("quad pts %d and %d		sector pts %d and %d\n", quad_point_counter, (quad_point_counter + 1) % 4, sector_point_counter, (sector_point_counter + 1) % 4); 

			printf("p1x = %f	p1z  = %f\n", road_quad_points[a].x, road_quad_points[quad_point_counter].z);
			printf("p2x = %f	p2z  = %f\n\n", road_quad_points[(quad_point_counter + 1) % 4].x, road_quad_points[(quad_point_counter + 1) % 4].z);

			printf("p3x = %f	p3z  = %f\n", sector_points[b].x, sector_points[sector_point_counter].z);
			printf("p4x = %f	p4z  = %f\n\n", sector_points[(sector_point_counter + 1) % 4].x, sector_points[(sector_point_counter + 1) % 4].z);

			printf("intersection  x = %f	z = %f\n", temp_point.x, temp_point.z);

			printf("\n");
*/
		} // end for
	} // end for

	if(sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					number_of_points > 0){

		printf("Sector %d x %d  Road %d  Quad %d\n\n", x_sector, z_sector, road, quad);

		printf("Number of points in this poly %d\n\n", sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					number_of_points);

		for(counter = 0; counter < sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					number_of_points; counter++){

			printf("%f  %f  %f\n", sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				point_list[counter].x,
				sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				point_list[counter].z,
				sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				point_list[counter].y);

			
		} // end for

		printf("\n");

		/////////////////////////////////////////////////////////////////////////////
		// The next two if's just get the min and max points for all polys on map
		/////////////////////////////////////////////////////////////////////////////

		if(sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points < min_road_poly_points){

			min_road_poly_points = sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points;
		}

		if(sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points > max_road_poly_points){

			max_road_poly_points = sector_road_poly_list[z_sector][x_sector].
				poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
				number_of_points;
		}
			
			

	} // end if

	//////////////////////////////////////////////////////////////////////////////
	// check if a poly was added and increment poly count
	//////////////////////////////////////////////////////////////////////////////

	if(sector_road_poly_list[z_sector][x_sector].
					poly_list[sector_road_poly_list[z_sector][x_sector].number_of_polys].
					number_of_points > 0){
		
		sector_road_poly_list[z_sector][x_sector].number_of_polys++;
	}

	return 0;

} // end get_sector_quad_poly_intersection

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

sector_point	get_sector_coordinates(simple_point in_pt, int sec_z, int sec_x){

	sector_point	out_pt;

	/////////////////////////////////////////////////////////////////////////////////
	// need to pass sectors because points exactly on sector boundaries
	// are actually in both sectors
	/////////////////////////////////////////////////////////////////////////////////

	//out_pt.xz_point.x = (short int)((in_pt.x - ((int)(in_pt.x / 2048) * 2048 + 1024)) * 16);
	out_pt.xz_point.x = (short int)((in_pt.x - (sec_x * 2048 + 1024)) * 16);

	//out_pt.xz_point.z = (short int)((in_pt.z - ((int)(in_pt.z / 2048) * 2048 + 1024)) * 16);
	out_pt.xz_point.z = (short int)((in_pt.z - (sec_z * 2048 + 1024)) * 16);

	out_pt.y_point.y = (unsigned short int)(in_pt.y - (double)terrain_3d_map_minimum_height) /
							terrain_3d_map_scaled_height_difference;

	return out_pt;

} // end get_sector_coordinates

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// isLeft(): tests if a point is Left|On|Right of an infinite line.
//    Input:  three points P0, P1, and P2
//    Return: >0 for P2 left of the line through P0 and P1
//            =0 for P2 on the line
//            <0 for P2 right of the line

double	isLeft( simple_point P0, simple_point P1, simple_point P2 )
{
    return (P1.x - P0.x)*(P2.z - P0.z) - (P2.x - P0.x)*(P1.z - P0.z);
}
//===================================================================

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void swap(simple_point *array, int a, int b){

	simple_point temp;

	temp = array[a];

	array[a] = array[b];

	array[b] = temp;
/*
	temp.x = array[a].x;
	temp.z = array[a].z;
	temp.y = array[a].y;
	temp.letter = array[a].letter;
	temp.road = array[a].road;
	temp.quad = array[a].quad;

	array[a].x = array[b].x;
	array[a].z = array[b].z;
	array[a].y = array[b].y;
	array[a].letter = array[b].letter;
	array[a].road = array[b].road;
	array[a].quad = array[b].quad;

	array[b].x = temp.x;
	array[b].z = temp.z;
	array[b].y = temp.y;
	array[b].letter = temp.letter;
	array[b].road = temp.road;
	array[b].quad = temp.quad;
*/
} // end swap()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////
 

road_poly	get_clockwise_ordered_poly(road_poly	in_poly){

	int		a,
			b,
			c,
			d,
			e,
			f,
			lowest_rightmost_pt,
			leftmost,
			number_of_points;

	double	lowest_z;

	road_poly		out_poly;

	//printf("\n*** Start of get_clockwise_ordered_poly\n");


	//////////////////////////////////////////////////////////////////////////////
	// find the lowest and rightmost point
	//////////////////////////////////////////////////////////////////////////////

	number_of_points = in_poly.number_of_points;

	out_poly.number_of_points = number_of_points;

	//printf("in_poly befor processing\n");
	//for(a = 0; a < number_of_points; a++){
	//	printf("%f  %f  %f  %c  %d  %d\n", in_poly.point_list[a].x,
	//		in_poly.point_list[a].z, in_poly.point_list[a].y,
	//		in_poly.point_list[a].letter,in_poly.point_list[a].road,
	//		in_poly.point_list[a].quad);
	//}
	//printf("\n");

	lowest_z = in_poly.point_list[0].z;

	//printf("lowest z = %f\n", lowest_z);

	for(a = 1; a < number_of_points; a++){

		if(in_poly.point_list[a].z < lowest_z) lowest_z = in_poly.point_list[a].z;

		//printf("a = %d  lowest z = %f\n", a, lowest_z);
	}
	
	// find the first point in the array at the lowest z

	for(a = 0; a < number_of_points; a++){

		if(in_poly.point_list[a].z == lowest_z) {
			lowest_rightmost_pt = a;

		//printf("z = %f  lowest_z = %f  a = %d  lowest_rightmost_pt = %d\n", in_poly.point_list[a].z,
		//	lowest_z, a, lowest_rightmost_pt);
		break;
		}
	}

	// find if any other points are equal to the lowest z and also further to the right

	for(a = lowest_rightmost_pt + 1; a < number_of_points; a++){

		if((in_poly.point_list[a].z == lowest_z) && 
			(in_poly.point_list[a].x > in_poly.point_list[lowest_rightmost_pt].x)){

			lowest_rightmost_pt = a;
		}

		//printf("a = %d  lowest_rightmost_pt = %d\n", a, lowest_rightmost_pt);
	} // end for

	// the first point is the lowest and rightmost point

	out_poly.point_list[0] = in_poly.point_list[lowest_rightmost_pt];

	
	// copy the rest of the points to out poly

	b = 1;

	for(a = 0; a < number_of_points; a++){

		if(a != lowest_rightmost_pt){
			
			out_poly.point_list[b] = in_poly.point_list[a];

			b++;
		}
	} // end for

	//printf("out_poly with lowest rightmost point in position 0\n");
	//for(c = 0; c < number_of_points; c++){
	//	printf("%f  %f\n", out_poly.point_list[c].x, out_poly.point_list[c].z);
	//}
	//printf("\n");


	/////////////////////////////////////////////////////////////////////////////
	// sort points 1 - number_of_points in clockwise order
	/////////////////////////////////////////////////////////////////////////////

	for(d = 1; d <= number_of_points - 2; d++){

		leftmost = d;

		//printf("d = %d\n", d);

		for(e = d; e <= number_of_points - 2; e++){

			//printf("e = %d\n", e);

			if(isLeft( out_poly.point_list[0],
				out_poly.point_list[leftmost],
				out_poly.point_list[e + 1]) > 0){

				//printf("isLeft is called with 0  %d  %d  and  %f  is returned\n", leftmost, e+1,
				//	isLeft( out_poly.point_list[0],
				//	out_poly.point_list[leftmost],
				//	out_poly.point_list[e + 1]));

				leftmost = e + 1;

				//printf("new leftmost = %d\n", leftmost);
			} // end for

			//if(leftmost != d){

			//	swap(out_poly.point_list, leftmost, d);

				//printf("swap has been called with %d and %d\n", leftmost, d);
				//printf("new point list after swap\n");
				//for(c = 0; c < number_of_points; c++){
				//	printf("%f  %f\n", out_poly.point_list[c].x, out_poly.point_list[c].z);
				//}
				//printf("\n");
			//} // end if

		} // end for

		if(leftmost != d){

			swap(out_poly.point_list, leftmost, d);
		}
	} // end for

	//printf("\n*** End of get_clockwise_ordered_poly\n");


	return out_poly;
} // end get_clockwise_ordered_poly()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int get_sector_coords(int z_sector, int x_sector){

	/////////////////////////////////////////////////////////////////////////////////////////////
	// This function calculates the sector coordinates for the road poly list per sector
	// road_poly_list_per_sector	sector_road_poly_list[MAP_HEIGHT][MAP_WIDTH]; // world and sector coords
	/////////////////////////////////////////////////////////////////////////////////////////////

	int		a,
			b,
			c,
			number_of_polys,
			number_of_points;

	number_of_polys = sector_road_poly_list[z_sector][x_sector].number_of_polys;

	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	// process all road polys for this sector
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////

	if(!number_of_polys){
		
		return 0;		
	}
	
	///////////////////////////////////////////////////////////////////////////////
	// Calculate sector coordinates for all road polys for this sector
	///////////////////////////////////////////////////////////////////////////////

	for(a = 0; a < number_of_polys; a++){

		number_of_points = sector_road_poly_list[z_sector][x_sector].
					poly_list[a].number_of_points;

		for(b = 0; b < number_of_points; b++){

			sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].sec_pt =
				get_sector_coordinates(
					sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b],
					z_sector, x_sector);

		} // end for

	} // end for

	////////////////////////////////////////////////////////////////////////////////
	// Print all the road poly stuff in both coordinate systems
	////////////////////////////////////////////////////////////////////////////////
/*
	printf("*** Sector *** %d x %d has %d road polys\n\n", x_sector, z_sector, number_of_polys);

	for(a = 0; a < number_of_polys; a++){

		number_of_points = sector_road_poly_list[z_sector][x_sector].
					poly_list[a].number_of_points;

		printf("Poly %d has %d points\n\n", a, number_of_points);

		for(b = 0; b < number_of_points; b++){

			printf("%d  %d  %c  %f  %f  %f    %d  %d  %d\n",
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].road,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].quad,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].letter,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].x,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].z,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].y,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].sec_pt.xz_point.x,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].sec_pt.xz_point.z,
				sector_road_poly_list[z_sector][x_sector].poly_list[a].point_list[b].sec_pt.y_point.y);

		} // end for
		printf("\n");
	} // end for
*/
	return 0;
} // end get_sector_coords

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

sector get_sector_of_world_coord_point(simple_point point){

	sector	sec;

	sec.x = floor(point.x / 2048);

	sec.z = floor(point.z /2048);

	return sec;
}

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void write_road_poly_binary(void){

	// File format
	//
	// int		number_of_sectors
	//
	// loop through sectors
	//		int		x sector
	//		int		z sector
	//
	//			 one road_poly_list_per_sector structure
	//

	FILE
		*fp;

	int
		number_of_sectors,
		sector_loop,
		sec_x_loop,
		sec_z_loop,
		number_of_polys,
		poly_loop,
		point_loop,
		number_of_points;

	road_poly_list_per_sector
		temp_road_polys_per_sector;

	number_of_sectors = 0;

	if(( fp = fopen("C:\\EechDev\\Export_roads\\road_polys_binary.bin","wb"))==NULL){
		printf("cannot open road_polys_binary.bin\n");
		exit(1);
	}
	else printf("road_polys_binary.bin opened ...\n");

	//printf("sizeof road_poly_list_per_sector is %d bytes\n", sizeof (road_poly_list_per_sector));
	//printf("sizeof road_poly is %d bytes\n", sizeof (road_poly));
	//printf("sizeof simple_point is %d bytes\n", sizeof (simple_point));
	//printf("sizeof sector_point is %d bytes\n", sizeof (sector_point));

	//
	// find how many sectors have road polys
	//

	for( sec_z_loop = 0; sec_z_loop < MAP_HEIGHT; sec_z_loop ++ ){
		for( sec_x_loop = 0; sec_x_loop < MAP_WIDTH; sec_x_loop ++ ){

			number_of_polys = sector_road_poly_list[ sec_z_loop ][ sec_x_loop ].number_of_polys;

			if(number_of_polys){

				number_of_sectors ++;
			}


		} // end sec_x_loop
	} // end sec_z_loop

	printf("number of sectors with road polys %d\n", number_of_sectors);

	fwrite(&number_of_sectors, sizeof(int), 1, fp);

	for( sec_z_loop = 0; sec_z_loop < MAP_HEIGHT; sec_z_loop ++ ){
		for( sec_x_loop = 0; sec_x_loop < MAP_WIDTH; sec_x_loop ++ ){

			number_of_polys = sector_road_poly_list[ sec_z_loop ][ sec_x_loop ].number_of_polys;

			if(number_of_polys){

				printf("\nsector %d x %d has %d road polys\n\n", sec_x_loop, sec_z_loop,
					number_of_polys);

				temp_road_polys_per_sector = sector_road_poly_list[ sec_z_loop ][ sec_x_loop ];

				fwrite(&sec_x_loop, sizeof(int), 1, fp);

				fwrite(&sec_z_loop, sizeof(int), 1, fp);

				fwrite( &temp_road_polys_per_sector, sizeof( road_poly_list_per_sector ), 1, fp);

				/////////////////////////////////////////////////////
				// start print statement
				/////////////////////////////////////////////////////

				for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

					number_of_points = 
						temp_road_polys_per_sector.poly_list [ poly_loop ].number_of_points;

					printf("\npoly %d has %d points\n\n", poly_loop, number_of_points);

					for(point_loop = 0; point_loop < number_of_points; point_loop ++){

						printf("%d  %d  %d\n", temp_road_polys_per_sector.poly_list [ poly_loop ].
							point_list[ point_loop ].sec_pt.xz_point.x,

							temp_road_polys_per_sector.poly_list [ poly_loop ].
							point_list[ point_loop ].sec_pt.xz_point.z,

							temp_road_polys_per_sector.poly_list [ poly_loop ].
							point_list[ point_loop ].sec_pt.y_point.y);

					} // end for point loop
				} // end for poly loop

				/////////////////////////////////////////////////////
				// end print statement
				/////////////////////////////////////////////////////

			} // end if number of polys


		} // end sec_x_loop
	} // end sec_z_loop

	fclose( fp );

} // end write road poly binary

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void add_intermediate_points_to_road_file_points(void){

	int
		a,
		b,
		c,
		d,
		road_loop,
		point_loop,
		points_this_road,
		temp_points_this_road,
		max_distance_ok,
		while_runaway_check,
		for_runaway_check;

	double
		distance,
		max_distance = 500.0,
		x,
		x1,
		z,
		z1;

	vec3d
		temp_points_1[1024],
		temp_points_2[1024],
		middle_point,
		original_last_point;

	printf("Adding intermediate points\n");

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		//printf("road loop = %d\n", road_loop);

		while_runaway_check = 0;
		max_distance_ok = 0;

		points_this_road = roads[road_loop].number_of_center_points; 

		// copy points into temp array

		for(a = 0; a < points_this_road; a++){

			temp_points_1[ a ].x = (float)roads[ road_loop ].center_points[ a ].global_x;
			temp_points_1[ a ].z = (float)roads[ road_loop ].center_points[ a ].global_z;
		} // end for

		// save the original last point
/*
		original_last_point.x = 
			(float)roads[road_loop].center_points[roads[road_loop].
			number_of_center_points - 1].global_x;
		
		original_last_point.z = 
			(float)roads[road_loop].center_points[roads[road_loop].
			number_of_center_points - 1].global_z;
*/
		//////////////////////////////////////////////////////////////////
		// check distance between all points and add inbetween point
		// if distance is too far
		// repeat loop until distance checks okay for all distances
		//////////////////////////////////////////////////////////////////

		max_distance_ok = 0; // not okay

		temp_points_this_road = points_this_road;

		while(!max_distance_ok){

			while_runaway_check ++;

			if(while_runaway_check > 500){

				printf("while runaway check blew past 4000 while adding inbetween pylons\n");

				exit(1);
			}

			for_runaway_check = 0;

			for(point_loop = 0; point_loop < temp_points_this_road - 1; point_loop++){

				for_runaway_check ++;

				if(for_runaway_check > 1024){

					printf("point_loop blew past 1024 while adding pylons\n");

					exit(1);
				}

				x = temp_points_1[ point_loop ].x;
				x1 = temp_points_1[ point_loop + 1 ].x;
				z = temp_points_1[ point_loop ].z;
				z1 = temp_points_1[ point_loop + 1 ].z;
				
				distance = sqrt(((x1 - x) * ((x1 - x))) + ((z1 - z) * (z1 - z)));

				if(distance <= max_distance && (point_loop == (temp_points_this_road - 2))){

					max_distance_ok = 1;

					break;
				} 
				else if(distance <= max_distance){

					continue;
				}
				else{

					//////////////////////////////////////////////////////
					// need to add another point inbetween
					//
					// I'm not sure why I had to force these middle points
					// to an integer value because the get_line_intersection
					// function was not detecting that the lines were parallel
					// that should have been.
					// There still seems to be a lack of parallel lines 
					// detected but it's working so far
					//////////////////////////////////////////////////////

					middle_point.x = (int)(x + x1) / 2;
					middle_point.z = (int)(z + z1) / 2;

					/////////////////////////////////////////////////////////////
					// copy temp_points_1 into temp_points_2 adding middle point
					/////////////////////////////////////////////////////////////

					for(a = 0; a <= point_loop; a++){

						temp_points_2[a] = temp_points_1[a];
					} // end for

					temp_points_2[a] = middle_point;

					temp_points_this_road ++;

					a ++;

					for( ; a < temp_points_this_road; a++){

						temp_points_2[ a ] = temp_points_1[ a - 1 ];

					} // end for

					

					/////////////////////////////////////////////////////////////
					// copy temp_points_2 back into temp_points_1
					// with new middle point and number of points increased by 1
					/////////////////////////////////////////////////////////////

					for(a = 0; a < temp_points_this_road; a++){

						temp_points_1[a] = temp_points_2[a];
					} // end for

				} // end else

				break;

			} // end for
			
		} // end while

		//////////////////////////////////////////////////////////////////////
		// copy the new temp array with added points back to global data
		//////////////////////////////////////////////////////////////////////

		// first point doesn't need to be changed

		for(a = 1, b = 0; b < temp_points_this_road; a++, b++){

			roads[road_loop].center_points[b].global_x = (double)temp_points_1[b].x;
			roads[road_loop].center_points[b].global_z = (double)temp_points_1[b].z;
		} // end for

		//roads[road_loop].center_points[a].global_x = (double)temp_points_1[b].x;
		//roads[road_loop].center_points[a].global_z = (double)temp_points_1[b].z;

		//roads[road_loop].center_points[a + 1].global_x = (double)original_last_point.x;
		//roads[road_loop].center_points[a + 1].global_z = (double)original_last_point.z;

		roads[road_loop].number_of_center_points = temp_points_this_road;

		roads[road_loop].number_of_quads = temp_points_this_road - 1;

	} // end for road_loop

} // end add_intermediate_points_to_road_file points

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void	print_road_file_points (void){

	int
		number_of_points_this_road,
		road_loop,
		point_loop,
		distance;

	vec3d
		this_point,
		next_point;

	for(road_loop = 0; road_loop < number_of_roads; road_loop ++){

		number_of_points_this_road = roads [road_loop].number_of_center_points;

		printf("\n\tRoad %d has %d points\n\n", road_loop, number_of_points_this_road);

		for(point_loop = 0; point_loop < number_of_points_this_road - 1; point_loop ++){

			this_point.x = (float)roads [ road_loop ].center_points[ point_loop ].global_x;
			this_point.z = (float)roads [ road_loop ].center_points[ point_loop ].global_z;

			next_point.x = (float)roads [ road_loop ].center_points[ point_loop + 1 ].global_x;
			next_point.z = (float)roads [ road_loop ].center_points[ point_loop + 1 ].global_z;

			printf("%d  %f  %f\n", point_loop, this_point.x, this_point.z);

			distance = get_distance ((int)this_point.x, (int)next_point.x,
				(int)this_point.z, (int)next_point.z);

			printf("\t%d\n", distance);
		} // end for point loop

		this_point.x = (float)roads [ road_loop ].center_points[ point_loop ].global_x;
		this_point.z = (float)roads [ road_loop ].center_points[ point_loop ].global_z;

		printf("%d  %f  %f\n", point_loop, this_point.x, this_point.z);

	} // end road loop

} // end print road file points

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int get_distance(int x, int x1, int z, int z1){

	int
		d;

	d = (int)sqrt((double)(((x1 - x) * (x1 - x)) + ((z1 - z) * (z1 - z))));

	return d;

} // end get_distance

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void load_road_file (void){

	FILE
		*road_fp;

	int
		i,
		j,
		weird_int_one,
		weird_int_two,
		number_of_points_for_this_road,
		x_int_pos,
		z_int_pos;

		if(( road_fp = fopen("C:\\aGrand\\Grand.ROAD","rb"))==NULL){
		printf("cannot open Grand.ROAD\n");
		exit(1);
	}
	else printf("Grand.ROAD opened\n");

	//////////////////////////////////////////////////////////////////////
	// read in all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fread ( &number_of_roads, sizeof ( int ), 1, road_fp );

	//printf("number of roads = %d\n\n", number_of_roads);

	for(i = 0; i < number_of_roads; i++){

		fread ( &weird_int_one, sizeof ( int ), 1, road_fp );
		fread ( &weird_int_two, sizeof ( int ), 1, road_fp );
		fread ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );
		
		roads[i].number_of_center_points = number_of_points_for_this_road;

		roads[i].number_of_quads = number_of_points_for_this_road - 1;

		for(j = 0; j < number_of_points_for_this_road; j++){

			fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			roads[i].center_points[j].global_x = (double)x_int_pos / 100.0;
			roads[i].center_points[j].global_z = (double)z_int_pos / 100.0;
		}
	}
	
	fclose ( road_fp );
} // end load road file


//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void load_3d_terrain (void){
	FILE
		*sec_in,
		*ffp_in;

	int
		terrain_version,
		x,
		z,
		offset,
		ffp_4byte_header,
		four_byte_zero_word,
		sec_z,
		sec_x;

	float
		min_height,
		max_height,
		terrain_3d_map_scaled_height_difference;

	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius,
		normal_changes_word;

	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height,
		pad_word_zero,
		pad_word_1536;

	unsigned char
		pad_byte_zero;

	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");

	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		exit(1);
	}
	else printf("terrain.ffp opened\n");

	
	////////////////////////////////////////////////////////
	// read in and write out map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);
	
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	
	for( z=0 ; z < map_height ; z++){
		for( x=0 ; x < map_width ; x++){

			//printf("sector %d x %d\n", z, x);

			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[z][x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);
		
			fread(&pad_word_1536, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[z][x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[z][x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[z][x].number_of_points, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[z][x].number_of_points, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].point_word_references,
				(sizeof(terrain_3d_point_word_reference) * 3),
				terrain_sectors[z][x].number_of_polygons, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[z][x].number_of_polygons, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].surface_changes,
				sizeof(terrain_3d_surface),	1, ffp_in);

			// read in the zero byte at the end of surface changes

			fread(&pad_byte_zero, sizeof(pad_byte_zero), 1, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// color offsets 

			fread(&terrain_sectors[z][x].colour_indices_offset,
				sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[z][x].point_colours_offset,
				sizeof(int), 1, sec_in); // sector file


		} // end for
	} // end for

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	printf("Terrain data has been read in\n");


} // end load 3d terrain


