// Grand Roads
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
// RobFoxroads stuff
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////

#define MAP_WIDTH 106
#define MAP_HEIGHT 62

#define m(x,y) a[x+wide*y]

#define WIDTH 25    // one half the width of a road

////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// terrain data structures
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

#define TERRAIN_3D_Y_SCALE ( 1.0 / 32767.0 );


struct TERRAIN_3D_POINT_XZ
{

	short int
		x,
		z;
};

typedef struct TERRAIN_3D_POINT_XZ terrain_3d_point_xz;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct VEC3D
{

	float
		x,
		y,
		z;
};

typedef struct VEC3D vec3d; 

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct FLOATING_POINT
{
	float
		x,
		z;
};

typedef struct FLOATING_POINT floating_point;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

#pragma pack(1)

struct TERRAIN_3D_POINT_Y
{

	union
	{

		struct
		{

			unsigned short int
				y:15,
				normal_change:1;
		};

		unsigned short int
			whole_value;

		short int
			whole_signed_value;
	};
};

typedef struct TERRAIN_3D_POINT_Y terrain_3d_point_y;


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// my stuff for road polys
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////

struct SECTOR_POINT					
{
	terrain_3d_point_xz		xz_point;

	terrain_3d_point_y		y_point;
};

typedef struct SECTOR_POINT sector_point;

//////////////////////////////////////////////////////////////////

struct POINT
{

	int
		x_sector, // this is the sector it's in
		z_sector,
		sec_x,    // this is the coordinates in the sector
		sec_z,
		sec_y,
		polygon_number; // this is the polygon # it's on

	double
		global_x,
		global_z,
		global_y;
	
};

typedef struct POINT point;

//////////////////////////////////////////////////////////////////////

struct SIMPLE_POINT
{
	//////////////////////////////////////////////////////////////////
	// the letter p-s is the same as the quad structure points p-s
	// the letter a-d is the sector corner points starting with 
	//	  a = the lower left and going counter clockwise
	// the letter R is for intersections with sector boundaries and the
	//    right side of the road
	// the letter L is for intersections with sector boundaries and the
	//    left side of the road
	// the letter e is for intersections with sector boundaries and the
	//    cross section line at the a end of the quad
	// the letter f is for intersections with sector boundaries and the
	//    cross section line at the b end of the quad
	// the letter j is for the three or four corner points of road junctions
	///////////////////////////////////////////////////////////////////
	char
		letter;

	int
		road,
		quad;

	double
		x,
		z,
		y;

	sector_point
		sec_pt;
};

typedef struct SIMPLE_POINT simple_point;

//////////////////////////////////////////////////////////////////////

struct LINE
{
	point
		point_a,
		point_b;
};

typedef struct LINE line;

//////////////////////////////////////////////////////////////////////

struct QUAD
{
	point
		p,
		q,
		r,
		s;
};

typedef struct QUAD quad;

///////////////////////////////////////////////////////////////////////

struct ROAD
{
	int
		number_of_center_points,
		number_of_quads;

	point
		center_points[512];

	quad
		quads[512];
};

typedef struct ROAD road;

////////////////////////////////////////////////////////////////////////
// these are the road polys after they have been split at
// the sector boundaries
////////////////////////////////////////////////////////////////////////

typedef struct ROAD_POLY	
{
	int number_of_points;				

	simple_point	point_list[6];
};

typedef struct ROAD_POLY road_poly;

/////////////////////////////////////////////////////////////////////////
// these are the road polys after they have been split at
// the sector boundaries
/////////////////////////////////////////////////////////////////////////

typedef struct ROAD_POLY_LIST_PER_SECTOR 
{
	int number_of_polys;

	road_poly	poly_list[32];
};

typedef struct ROAD_POLY_LIST_PER_SECTOR road_poly_list_per_sector;

////////////////////////////////////////////////////

typedef struct SECTOR
{
	int x,
		z;
};

typedef struct SECTOR sector;


/////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// terrain data structures
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR
{

	unsigned char
		red,
		green,
		blue;
};

typedef struct TERRAIN_3D_COLOUR terrain_3d_colour;

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_COLOUR_INDEX terrain_3d_colour_index;


#pragma pack(4)

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_WORD_REFERENCE
{

	unsigned short int
		point;
};

typedef struct TERRAIN_3D_POINT_WORD_REFERENCE terrain_3d_point_word_reference;

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_BYTE_REFERENCE
{

	unsigned char
		point;
};

typedef struct TERRAIN_3D_POINT_BYTE_REFERENCE terrain_3d_point_byte_reference;

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_FACE
{

	unsigned char
		number_of_points:7,
		surface_change:1;
};

typedef struct TERRAIN_3D_FACE terrain_3d_face;

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

struct TERRAIN_3D_SURFACE
{

	unsigned char
		surface_id;
};

typedef struct TERRAIN_3D_SURFACE terrain_3d_surface;

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

struct TERRAIN_3D_SECTOR
{

	unsigned short int
		number_of_points,
		number_of_polygons,
		radius;

	short int
		minimum_height,
		maximum_height,
		average_height,
		number_of_surface_changes;

	int
		colour_indices_offset,
		point_colours_offset;

	struct TERRAIN_3D_POINT_XZ
		points_xz[512];

	struct TERRAIN_3D_POINT_Y
		points_y[512];

	struct TERRAIN_3D_FACE
		polygons[1024];

	union
	{

		struct TERRAIN_3D_POINT_WORD_REFERENCE
			point_word_references[1024][3];

		struct TERRAIN_3D_POINT_BYTE_REFERENCE
			point_byte_references[1024][3];
	};


	struct TERRAIN_3D_SURFACE
		surface_changes[32];

	struct TERRAIN_3D_SURFACE
		surface_id_each_face[1024];

	struct TERRAIN_3D_COLOUR
		point_colours[64];

	struct TERRAIN_3D_COLOUR_INDEX
		colour_indices[512];
};

typedef struct TERRAIN_3D_SECTOR terrain_3d_sector;


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define TERRAIN_3D_MAX_TREE_HEIGHT	(35)


struct TERRAIN_3D_TREE_DATA
{

	signed short int
		x,
		z;

	unsigned short int
		y;

	unsigned char
		colour;

	unsigned char
		height:7,
		type:1;
};

typedef struct TERRAIN_3D_TREE_DATA terrain_3d_tree_data;

///////////////////////////////////////////////////////////////////
// for point in poly function
///////////////////////////////////////////////////////////////////

#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define INSIDE 1
#define OUTSIDE 0

typedef struct {
   double x,y;
} Point;



