#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "import.h"

//////////////////////////////
// global data
//////////////////////////////


terrain_3d_sector
	**terrain_sectors;

int
	colour_file_offset_counter = 0,
	terrain_3d_map_width,
	terrain_3d_map_height,
	rgb_4byte_header,
	ffp_4byte_header,
	terrain_version,
	number_of_textures_used = 10;

//
// these are terrain types used for the import because that's what
// what was used for the export, probably change these eventually
// the new types are assigned locally for sorting by surface for the
// whole map
//

int
	used_texture_list[] = {
		TERRAIN_TYPE_RIVER,
		TERRAIN_TYPE_TRENCH,
		TERRAIN_TYPE_LAND,
		TERRAIN_TYPE_FIELD1,
		TERRAIN_TYPE_ALTERED_LAND1,
		TERRAIN_TYPE_ALTERED_LAND2,
		TERRAIN_TYPE_ALTERED_LAND3,
		TERRAIN_TYPE_FOREST_FLOOR,
		TERRAIN_TYPE_ROAD,
		TERRAIN_TYPE_FOREST_TOP				
	};

short int
	pad_word_zero;

unsigned short int
	normal_changes_word;



float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;

///////////////////////////////
// Functions
///////////////////////////////

void save_terrain_sector_as_lw_3d_object ( int sec_x, int sec_z, const char *filename );

void load_3d_terrain (void);

void iff_short_int_write ( short int value, FILE *fp );

void iff_int_write ( int value, FILE *fp );

void iff_float_write ( float value, FILE *fp );

int iff_int_read ( FILE *fp );

short int iff_short_int_read ( FILE *fp );

float iff_float_read ( FILE *fp );

void set_terrain_rendering_routines ( terrain_types type, terrain_polygon_type polygon_type);
													
void initialize_terrain_type_information(void);

void print_sector_data(int sec_z, int sec_x);

void set_terrain_type_textures ( terrain_types type, int red, int green, int blue );

void test_function_for_reading_lwo_file(int sec_z, int sec_x);

void import_mod_lw_sector(int sec_z, int sec_x);

void sort_mod_sector_polys_by_surface( terrain_3d_sector   *sector);

int file_exist (char *filename);

void write_new_terrain_files(void);

void assign_colour_data(int sec_z, int sec_x);

// added the next 3 functions debugging grand_terraform

void assign_surface_ids(int sec_z, int sec_x);

void sort_sector_polys_by_surface( int sec_z, int sec_x );

double normal_angle_with_y(int sec_z, int sec_x, int poly);

void print_terrain_data_of_sector(int sec_z, int sec_x);



int main(int argc, char ** argv)
{
	int
		sec_x,
		sec_z;

	char
		filename [16];

	load_3d_terrain ();

	// better check loop nesting I think z needs to be on the outside
	// for the color file to be correct

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			sprintf(filename, "mod_lw_objs\\%03dx%03d.lwo", sec_x, sec_z); 

			if( file_exist ( filename ) ){

				import_mod_lw_sector ( sec_z, sec_x );
			}

			//assign_colour_data( sec_z,  sec_x);

		}// end for z loop
	} // end for x loop

	//print_terrain_data_of_sector(56, 99);
/*
	printf("Assigning surface IDs ...\n");

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			assign_surface_ids( sec_z, sec_x );

		}// end for z loop
	} // end for x loop

	//print_terrain_data_of_sector(56, 99);

	printf("Sorting by surface ...\n");

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			sort_sector_polys_by_surface ( sec_z, sec_x );

		}// end for z loop
	} // end for x loop

	//print_terrain_data_of_sector(56, 99);

	//printf("Assigning color data ...\n");

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			//assign_colour_data( sec_z,  sec_x);

		}// end for z loop
	} // end for x loop
*/
	write_new_terrain_files();
/*
	//print_terrain_data_of_sector(30, 50);
	print_terrain_data_of_sector(53, 99);
	print_terrain_data_of_sector(54, 99);
	print_terrain_data_of_sector(55, 99);
	print_terrain_data_of_sector(56, 99);
	print_terrain_data_of_sector(57, 99);
	print_terrain_data_of_sector(56, 100);
	print_terrain_data_of_sector(57, 100);
	print_terrain_data_of_sector(58, 100);
	print_terrain_data_of_sector(59, 100);
	print_terrain_data_of_sector(59, 101);
	print_terrain_data_of_sector(60, 101);
*/
	return 0;
} // end main

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void assign_colour_data(int sec_z, int sec_x){

	int
		poly_loop,
		number_of_polys,
		average_y, // real elevation in meters
		y0,
		y1,
		y2;

	terrain_3d_colour
		surface_0_colour,	
		surface_1_colour,
		surface_2_colour,
		surface_3_colour,
		surface_4_colour,	
		surface_5_colour,
		surface_6_colour,
		surface_7_colour,	
		surface_8_colour,
		tree_colour_0,
		tree_colour_1,
		tree_colour_2,
		tree_colour_3;

	/////////////////////////////////////////////////////////////

	surface_0_colour.red = 244; // lighter flat terrain color
	surface_0_colour.green = 178;
	surface_0_colour.blue = 103;

	surface_1_colour.red = 240; // middle flat terrain color
	surface_1_colour.green = 155;
	surface_1_colour.blue = 78;

	surface_2_colour.red = 219; // darker flat terrain color
	surface_2_colour.green = 131;
	surface_2_colour.blue = 71;

	surface_3_colour.red = 178; // lighter steep terrain color
	surface_3_colour.green = 102;
	surface_3_colour.blue = 63;

	surface_4_colour.red = 157; // middle steep terrain color
	surface_4_colour.green = 93;
	surface_4_colour.blue = 66;

	surface_5_colour.red = 157; // darker steep terrain color
	surface_5_colour.green = 72;
	surface_5_colour.blue = 43;

	//surface_5_colour.red = 10; // darker steep terrain color
	//surface_5_colour.green = 10;
	//surface_5_colour.blue = 10;

	surface_6_colour.red = 255; // white
	surface_6_colour.green = 255;
	surface_6_colour.blue = 255;

	surface_7_colour.red = 157; // spare
	surface_7_colour.green = 93;
	surface_7_colour.blue = 66;

	surface_8_colour.red = 0; // water
	surface_8_colour.green = 79;
	surface_8_colour.blue = 105;

	tree_colour_0.red = 184; // green
	tree_colour_0.green = 225;
	tree_colour_0.blue = 131;

	tree_colour_1.red = 246; // orange
	tree_colour_1.green = 163;
	tree_colour_1.blue = 8;

	tree_colour_2.red = 246; // red
	tree_colour_2.green = 58;
	tree_colour_2.blue = 8;

	tree_colour_3.red = 246; // yellow
	tree_colour_3.green = 240;
	tree_colour_3.blue = 8;

	terrain_sectors[sec_z][sec_x].point_colours[0] = surface_0_colour;
	terrain_sectors[sec_z][sec_x].point_colours[1] = surface_1_colour;
	terrain_sectors[sec_z][sec_x].point_colours[2] = surface_2_colour;
	terrain_sectors[sec_z][sec_x].point_colours[3] = surface_3_colour;
	terrain_sectors[sec_z][sec_x].point_colours[4] = surface_4_colour;
	terrain_sectors[sec_z][sec_x].point_colours[5] = surface_5_colour;
	terrain_sectors[sec_z][sec_x].point_colours[6] = surface_6_colour;
	terrain_sectors[sec_z][sec_x].point_colours[7] = surface_7_colour;
	terrain_sectors[sec_z][sec_x].point_colours[8] = surface_8_colour;
	terrain_sectors[sec_z][sec_x].point_colours[9] = tree_colour_0;
	terrain_sectors[sec_z][sec_x].point_colours[10] = tree_colour_1;
	terrain_sectors[sec_z][sec_x].point_colours[11] = tree_colour_2;
	terrain_sectors[sec_z][sec_x].point_colours[12] = tree_colour_3;

	////////////////////////////////////////////////////////////////////////
	// loop through all polygons
	////////////////////////////////////////////////////////////////////////

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){ 

		y0 = (int)(terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[poly_loop * 3].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y1 = (int)(terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[poly_loop * 3 + 1].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y2 = (int)(terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[poly_loop * 3 + 2].point].y * 
			terrain_3d_map_scaled_height_difference ));

		average_y = (int)(y0 + y1 + y2) / 3;

		
		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND1){
			
			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 4;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 3;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 5;
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_LAND){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 2;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 2;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 1;
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND2){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 3;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 4;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 3;
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND3){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 5;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 5;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 5;
	
		}
		else{	// TERRAIN_TYPE_RIVER

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 8;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 8;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 8;
		}

/*
		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_TRENCH){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 0;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 0;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 0;
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND3){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 2;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 2;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 2;
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_FOREST_FLOOR){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 4;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 4;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 4;
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_FOREST_TOP){	

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 6;
	
		}
		else{	// TERRAIN_TYPE_RIVER

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 8;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 8;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 8;
	
		}
*/
	} // end for poly loop

	//////////////////////////////////////////////////////////////
	// assign colour file offset values
	//////////////////////////////////////////////////////////////

	terrain_sectors[sec_z][sec_x].colour_indices_offset = colour_file_offset_counter;

	colour_file_offset_counter += terrain_sectors[sec_z][sec_x].number_of_points
		* sizeof(terrain_3d_colour_index);

	terrain_sectors[sec_z][sec_x].point_colours_offset = colour_file_offset_counter;

	// multiple needs to be the number of colors used

	colour_file_offset_counter += 13 * sizeof(terrain_3d_colour); 

} // end assign colour data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void write_new_terrain_files(void){

	FILE
		*sec_out,
		*ffp_out,
		*rgb_out;

	int
		sec_x,
		sec_z,
		offset,
		four_byte_zero_word = 0;

	unsigned short int
		map_width,
		map_height;

	unsigned char
		pad_byte_zero = 0;

	/////////////////////////////////////////////////////////

	printf("Writing new terrain files ...\n");

	map_width = (unsigned short int) terrain_3d_map_width;
	map_height = (unsigned short int) terrain_3d_map_height;

	//colour_file_offset_counter = 0;

	if(( sec_out = fopen("C:\\EechDev\\grand_terraform\\default.sec","wb"))==NULL){
		printf("cannot open C:\\EechDev\\grand_terraform\\default.sec\n");
		exit(1);
	}
	else printf("C:\\EechDev\\grand_terraform\\default.sec opened\n");

	if(( ffp_out = fopen("C:\\EechDev\\grand_terraform\\terrain.ffp","wb"))==NULL){
		printf("cannot open C:\\EechDev\\grand_terraform\\terrain.ffp\n");
		exit(1);
	}
	else printf("C:\\EechDev\\grand_terraform\\terrain.ffp opened\n");

	if(( rgb_out = fopen("C:\\EechDev\\grand_terraform\\default.rgb","wb"))==NULL){
		printf("cannot open C:\\EechDev\\grand_terraform\\default.rgb\n");
		exit(1);
	}
	else printf("C:\\EechDev\\grand_terraform\\default.rgb opened\n");

	fwrite(&terrain_version, sizeof(int), 1, sec_out);
	fwrite(&map_width, sizeof(unsigned short int), 1, sec_out);
	fwrite(&map_height, sizeof(unsigned short int), 1, sec_out);
	fwrite(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_out);
	fwrite(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_out);
	fwrite(&ffp_4byte_header, sizeof(int), 1, ffp_out);


	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){


			//printf("\nSECTOR  %d x %d\n\n", sec_x, sec_z);


			/////////////////////////////////////////////////////////////////
			// write all the stats for each sector to sector file
			/////////////////////////////////////////////////////////////////

			fwrite(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_out);
			
			fwrite(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_out);

			fwrite(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_out);

			fwrite(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_out);

			fwrite(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_out);

			fwrite(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_out);

			fwrite(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_out);

			fwrite(&pad_word_zero, sizeof(short int), 1, sec_out);

			///////////////////////////////////////////////////////////////////
			// write the geometry data to the ffp file
			///////////////////////////////////////////////////////////////////

			offset = (int)ftell(ffp_out);

			//printf("offset to points y= %d\n", offset);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			fwrite(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

			offset = (int)ftell(ffp_out);

			//printf("offset to points_xz = %d\n", offset);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			fwrite(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to point_word_references = %d\n", offset);

			fwrite(terrain_sectors[sec_z][sec_x].point_word_references,
				(sizeof(terrain_3d_point_word_reference) * 3),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_out);

			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to polygons = %d\n", offset);

			fwrite(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face), // num points & surface changes
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_out); 
			
			//
			// if number of polygons is odd need to add a 0 pad byte
			//

			// probably need this when adding roads and changing number of polygons

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){

				fwrite( &pad_byte_zero, 1, 1, ffp_out);

			}
			
			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to surface_ids = %d\n", offset);

			fwrite(terrain_sectors[sec_z][sec_x].surface_ids, // better check if & is needed
				sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids,
				ffp_out);

			//printf("number of surface ids = %d\n", 
			//	terrain_sectors[sec_z][sec_x].number_of_surface_ids);

			// write the zero byte at the end of surface changes

			fwrite(&pad_byte_zero, sizeof(pad_byte_zero), 1, ffp_out);

			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to normal_changes = %d\n", offset);

			//fwrite(terrain_sectors[sec_z][sec_x].normal_changes, sizeof(normal_changes_word),
			//	terrain_sectors[sec_z][sec_x].number_of_normal_changes, ffp_out);

			fwrite(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_out);

			// write color offsets to sector file


			//printf("colour indices offset %d\n", 
			//	terrain_sectors[sec_z][sec_x].colour_indices_offset);

			//printf("point colours offset %d\n", 
			//	terrain_sectors[sec_z][sec_x].point_colours_offset);



			fwrite(&terrain_sectors[sec_z][sec_x].colour_indices_offset,
				sizeof(int), 1, sec_out); // sector file

			fwrite(&terrain_sectors[sec_z][sec_x].point_colours_offset,
				sizeof(int), 1, sec_out); // sector file

			// write out colour file


			// had a bad bug because I had a & in front of terrain_sectors ... colour_indices
			// some day I need to figure this & thing out !!!!


			fwrite(terrain_sectors[sec_z][sec_x].colour_indices, 
				sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_out);

			fwrite(&terrain_sectors[sec_z][sec_x].point_colours, 
				sizeof(terrain_3d_colour),	13, rgb_out);
			// multiple needs to be the number of colors used ( 13 )


		} // end for sec_x loop
	} // end for sec_z loop

	// write out the last four bytes of sector file (four bytes of zeros)

	fwrite(&four_byte_zero_word, sizeof(int), 1, sec_out);


} // end write new terrain files

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void import_mod_lw_sector(int sec_z, int sec_x){

	FILE
		*fp;

	int
		point_loop,
		poly_loop,
		surface_loop,
		iff_byte_offset,
		number_of_points,
		number_of_poly_points,
		number_of_polygons,
		number_of_surfaces,
		surface_ids[64],
		word_reference_index = 0;

	char
		filename [32],
		iff_id[16],
		surface_id_string[8];

	float
		value;

	long
		position;

	terrain_3d_sector
		*lw_mod_sector;
	

	///////////////////////////////////////////////////////////////

	sprintf(filename, "mod_lw_objs\\%03dx%03d.lwo", sec_x, sec_z); 

	if(( fp = fopen(filename,"rb"))==NULL){
		printf("cannot open %s\n", filename);
		exit(1);
	}
	else printf("%s opened ...\n", filename );

	//
	// allocate memory for sector
	//

	if( ! (lw_mod_sector = malloc( sizeof(terrain_3d_sector)))){

			printf("Error: No memory for lw_mod_sector %d x %d\n", sec_x, sec_z);

			exit(1);
	}

	fread ( iff_id, 4, 1, fp ); // FORM

	iff_byte_offset = iff_int_read ( fp );

	////////////////////////////////////
	// read the SRFS chunk
	////////////////////////////////////

	fread ( iff_id, 8, 1, fp ); // LWOBSRFS

	iff_byte_offset = iff_int_read ( fp );

	number_of_surfaces = iff_byte_offset / 8;

	if(!(lw_mod_sector->surface_ids = malloc(sizeof(terrain_3d_surface) * number_of_surfaces))){
				printf("No memory for surface_ids lw_mod_sector %d x %d\n",sec_x, sec_z);
				exit(1);
	}

	for(surface_loop = 0; surface_loop < number_of_surfaces; surface_loop ++){

		fread ( surface_id_string, 1, 8, fp );

		//printf("%s\n", surface_id);

		surface_id_string[7] = ' ';

		//add a 1 to index because lightwave starts the index with 1 instead with 0

		surface_ids[surface_loop + 1] = atoi ( surface_id_string ); 

		//printf ("%d\n", surface_ids[surface_loop + 1] );


	} // end for surface loop

	////////////////////////////////////
	// read the PNTS chunk
	////////////////////////////////////

	fread ( iff_id, 4, 1, fp ); // PNTS

	iff_byte_offset = iff_int_read ( fp );

	/////////////////////////////////////////////////////////////////////////////
	// there are three 4-byte floats for each point X Y Z
	/////////////////////////////////////////////////////////////////////////////

	number_of_points = iff_byte_offset / 12; 

	lw_mod_sector->number_of_points = number_of_points;

	if(!(lw_mod_sector->points_y = malloc(sizeof(terrain_3d_point_y) * number_of_points))){
				printf("No memory for points_y lw_mod_sector %d x %d\n",sec_x, sec_z);
				exit(1);
	}

	if(!(lw_mod_sector->points_xz = malloc(sizeof(terrain_3d_point_xz) * number_of_points))){
				printf("No memory for points_xz lw_mod_sector %d x %d\n",sec_x, sec_z);
				exit(1);
	}

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		 lw_mod_sector->points_xz[point_loop].x = (short int) iff_float_read ( fp );
		 lw_mod_sector->points_y[point_loop].y = (unsigned short int) iff_float_read ( fp );
		 lw_mod_sector->points_xz[point_loop].z = (short int) iff_float_read ( fp );

		 //
		 // always set normal change to zero
		 //

		 lw_mod_sector->points_y[point_loop].normal_change = 0;

	} // end for point loop

	//for(point_loop = 0; point_loop < number_of_points; point_loop ++){

	//	printf("%d  %d  %d  %d\n", point_loop,
	//		lw_mod_sector->points_xz[point_loop].x,
	//		lw_mod_sector->points_xz[point_loop].z,
	//		lw_mod_sector->points_y[point_loop].y);
	//} // end for point loop

	////////////////////////////////////
	// read the POLS chunk
	////////////////////////////////////

	fread ( iff_id, 4, 1, fp ); // POLS

	iff_byte_offset = iff_int_read ( fp );

	///////////////////////////////////////////////////////////////////////////////
	// file format:
	//
	// This chunk contains a list of all the polygons in an object.  Each
    // entry consists of a short integer specifying the number of vertices
    // in a polygon followed by that many short integers specifying the
    // vertices themselves (as indices into the points list) followed by a
    // short integer specifying which surface is used by the polygon (as
    // an index into the surfaces list).
	//
	// All polys are guaranteed to have 3 points 
	// so there are 10 bytes for each poly
	////////////////////////////////////////////////////////////////////////////////

	number_of_polygons = iff_byte_offset / 10;

	lw_mod_sector->number_of_polygons = number_of_polygons;

	if(!(lw_mod_sector->polygons = malloc(sizeof(terrain_3d_face) * number_of_polygons))){
				printf("No memory for polygons lw_mod_sector %d x %d\n",sec_x, sec_z);
				exit(1);
	}

	if(!(lw_mod_sector->point_word_references = malloc(sizeof(terrain_3d_point_word_reference) * 
		number_of_polygons * 3))){
				printf("No memory for point_word_references lw_mod_sector %d x %d\n",sec_x, sec_z);
				exit(1);
	}

	if(!(lw_mod_sector->surface_id_each_face = malloc(sizeof(terrain_3d_surface) * 
		number_of_polygons))){
				printf("No memory for surface_id_each_face lw_mod_sector %d x %d\n",sec_x, sec_z);
				exit(1);
	}

	for(poly_loop = 0; poly_loop < number_of_polygons; poly_loop ++){

		number_of_poly_points = iff_short_int_read ( fp );

		lw_mod_sector->polygons[poly_loop].number_of_points = number_of_poly_points;

		lw_mod_sector->polygons[poly_loop].surface_change = 0; // initialize to zero

		for(point_loop = 0; point_loop < number_of_poly_points; point_loop ++){

			lw_mod_sector->point_word_references[word_reference_index].point = 
				(unsigned short int)iff_short_int_read ( fp );

			word_reference_index ++;
		} // end for point loop

		lw_mod_sector->surface_id_each_face[poly_loop].surface_id = 
			surface_ids [ iff_short_int_read ( fp ) ];
		
	} // end for poly loop

	word_reference_index = 0;

	//for(poly_loop = 0; poly_loop < number_of_polygons; poly_loop ++){

	//	printf("%d  %d  %d  %d  %d\n", poly_loop,
	//		lw_mod_sector->surface_id_each_face[poly_loop].surface_id,
	//		lw_mod_sector->point_word_references[word_reference_index].point,
	//		lw_mod_sector->point_word_references[word_reference_index + 1].point,
	//		lw_mod_sector->point_word_references[word_reference_index + 2].point);

	//	word_reference_index += 3;
	//} // end poly loop

	fclose(fp);

	sort_mod_sector_polys_by_surface( lw_mod_sector );

	///////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////
	// change relevant original sector data to the imported data
	///////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////

	terrain_sectors[sec_z][sec_x].number_of_points = lw_mod_sector->number_of_points;

	terrain_sectors[sec_z][sec_x].number_of_polygons = lw_mod_sector->number_of_polygons;

	//terrain_sectors[sec_z][sec_x].number_of_surface_changes_bytes = 
	//	number_of_surfaces; // one for each surface

	terrain_sectors[sec_z][sec_x].number_of_surface_ids = 
		number_of_surfaces; // one for each surface
	
	///////////////////////////////////////////////////////////////////
	// free original memory for 
	// points_y
	// points_xz
	// polygons
	// point_word_references
	// surface_id_each_face
	///////////////////////////////////////////////////////////////////

	free ( terrain_sectors[sec_z][sec_x].points_y );
	free ( terrain_sectors[sec_z][sec_x].points_xz );
	free ( terrain_sectors[sec_z][sec_x].polygons );
	free ( terrain_sectors[sec_z][sec_x].point_word_references );
	free ( terrain_sectors[sec_z][sec_x].surface_id_each_face );
	free ( terrain_sectors[sec_z][sec_x].surface_ids );

	///////////////////////////////////////////////////////////////////
	// swap pointers for arrays
	///////////////////////////////////////////////////////////////////

	terrain_sectors[sec_z][sec_x].points_y = lw_mod_sector->points_y;
	terrain_sectors[sec_z][sec_x].points_xz = lw_mod_sector->points_xz;
	terrain_sectors[sec_z][sec_x].polygons = lw_mod_sector->polygons;
	terrain_sectors[sec_z][sec_x].point_word_references = lw_mod_sector->point_word_references;
	terrain_sectors[sec_z][sec_x].surface_id_each_face = lw_mod_sector->surface_id_each_face;
	terrain_sectors[sec_z][sec_x].surface_ids = lw_mod_sector->surface_ids;

	//print_sector_data(sec_z, sec_x);

} // end read in mod lw sector

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void sort_mod_sector_polys_by_surface( terrain_3d_sector   *sector ){

	int
		texture_loop,
		poly_loop,
		number_of_polys,
		temp_poly_array_index = 0,
		word_ref_index = 0,
		current_surface_id,
		surface_changes_count;

	terrain_3d_face
		temp_sorted_polygons[1024];

	terrain_3d_point_word_reference
		temp_sorted_word_ref[3072];

	terrain_3d_surface
		temp_sorted_surface_id_each_face[1024];

	number_of_polys = sector->number_of_polygons;

	for(texture_loop = 0; texture_loop < number_of_textures_used; texture_loop ++){

		for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

			if(sector->surface_id_each_face[poly_loop].surface_id == 
				used_texture_list [ texture_loop ]){

				//printf("if entered\n");

				temp_sorted_polygons [ temp_poly_array_index ].number_of_points =
					sector->polygons [ poly_loop ].number_of_points;

				temp_sorted_polygons [ temp_poly_array_index ].surface_change =
					sector->polygons [ poly_loop ].surface_change;

				////////////////////////////////////////

				temp_sorted_surface_id_each_face [ temp_poly_array_index ].surface_id = 
					sector->surface_id_each_face [ poly_loop ].surface_id;

				////////////////////////////////////////

				temp_poly_array_index ++;

				////////////////////////////////////////

				temp_sorted_word_ref [ word_ref_index ].point = 
					sector->point_word_references [ poly_loop * 3 ].point;

				word_ref_index ++;

				temp_sorted_word_ref [ word_ref_index ].point = 
					sector->point_word_references [ poly_loop * 3 + 1 ].point;

				word_ref_index ++;

				temp_sorted_word_ref [ word_ref_index ].point = 
					sector->point_word_references [ poly_loop * 3 + 2 ].point;

				word_ref_index ++;
		
			} // end if
		} // end for poly loop 
	} // end for texture loop

	/////////////////////////////////////////////////////////
	// copy temp arrays back to primary data
	/////////////////////////////////////////////////////////

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		sector->polygons [ poly_loop ].number_of_points =
			temp_sorted_polygons [ poly_loop ].number_of_points;

		sector->polygons [ poly_loop ].surface_change =
			temp_sorted_polygons [ poly_loop ].surface_change;

		/////////////////////////////////////////////

		sector->surface_id_each_face[poly_loop].surface_id = 
			temp_sorted_surface_id_each_face[poly_loop].surface_id;

		/////////////////////////////////////////////

		sector->point_word_references[poly_loop * 3].point =
			temp_sorted_word_ref[poly_loop * 3].point;

		sector->point_word_references[poly_loop * 3 + 1].point =
			temp_sorted_word_ref[poly_loop * 3 + 1].point;

		sector->point_word_references[poly_loop * 3 + 2].point =
			temp_sorted_word_ref[poly_loop * 3 + 2].point;
	} // end for poly loop

	/////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////
	// add surface id's to the surface_changes array and
	// change the surface change bits at proper spot
	/////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////

	sector->number_of_surface_ids = 0;

	sector->surface_ids[0].surface_id = 
		sector->surface_id_each_face[0].surface_id;

	surface_changes_count = 0;

	current_surface_id = sector->surface_ids[0].surface_id;

	//////////////////////////////////////////////////////////////////////////////////////
	// loop thru polys 1 - last
	// find surface changes
	// set sector->surface_ids[surface_changes_count].surface_id
	// toggle terrain_sectors[sec_z][sec_x].polygons[x].surface_change
	//////////////////////////////////////////////////////////////////////////////////////

	//
	// it's weird I don't know why the first surface_change is getting
	// set to 1
	//


	sector->polygons[0].surface_change = 0;
	
	for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

		if(sector->surface_id_each_face[poly_loop].surface_id != current_surface_id){

			current_surface_id = sector->surface_id_each_face[poly_loop].surface_id;

			sector->number_of_surface_ids ++;

			surface_changes_count ++;

			sector->surface_ids[surface_changes_count].surface_id = current_surface_id;

			sector->polygons[poly_loop].surface_change = 1;
		} // end if
	} // end for

} // end sort mod sector polys by surface


//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		sec_x,
		sec_z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		four_byte_zero_word,
		i,
		print_loop,		
		surface_changes_index,
		poly_loop,
		number_of_polys;

	float
		min_height,
		max_height;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius;
		//normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height;

	unsigned char
		pad_byte_zero,
		current_surface_id,
		normal_changes_buf[1024];
	
	long
		current_file_position;

	////////////////////////////////////////////////////////////////////////////
	// initialize variables
	////////////////////////////////////////////////////////////////////////////

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			//printf("sector %d x %d\n\n", sec_x, sec_z);
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[sec_z][sec_x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[sec_z][sec_x].points_y );
			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[sec_z][sec_x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				if(!(terrain_sectors[sec_z][sec_x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[sec_z][sec_x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[sec_z][sec_x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface ids - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].surface_ids = 
				malloc(normal_changes_offset - surface_changes_offset))){
				printf("No memory for surface_ids sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			// there is always a zero pad byte at the end of the surface ids
			// if you are not going to recalculate the number of surface ids
			// for every sector then you need to take this into account

			terrain_sectors[sec_z][sec_x].number_of_surface_ids = 
				normal_changes_offset - surface_changes_offset - 1;

			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].surface_ids, sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids + 1, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// initialize the surface_id_each_face array
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].surface_id_each_face = 
				malloc(sizeof(terrain_3d_surface) * terrain_sectors[sec_z][sec_x].number_of_polygons))){
				printf("No memory for surface_id_each_face sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

			current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

			terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id = current_surface_id;
			
			surface_changes_index = 0;

			for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

				if(terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change == 1){

					surface_changes_index ++;

					current_surface_id = terrain_sectors[sec_z][sec_x].
						surface_ids[surface_changes_index].surface_id;

					terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id =
						current_surface_id;
				}
				else{

					terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id =
						current_surface_id;

				} // end if else
			} // end for poly loop

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// read in color offsets from sector file

			fread(&terrain_sectors[sec_z][sec_x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[sec_z][sec_x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[sec_z][sec_x].colour_indices = malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			fread(terrain_sectors[sec_z][sec_x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[sec_z][sec_x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[sec_z][sec_x].number_of_points; i++){ 
				
				// there is a colour index for each point

				if((terrain_sectors[sec_z][sec_x].colour_indices + i)->index > 
					terrain_sectors[sec_z][sec_x].max_colour_index){

					terrain_sectors[sec_z][sec_x].max_colour_index = 
						(terrain_sectors[sec_z][sec_x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[sec_z][sec_x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[sec_z][sec_x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void print_sector_data(int sec_z, int sec_x){

	int
		number_of_points,
		number_of_polys,
		point_loop,
		poly_loop;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	printf("\nSector %d x %d --- %d points  %d polygons \n\n",
		sec_x, sec_z, number_of_points, number_of_polys);

	printf("Point  X  Z  Y\n\n");

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		printf("%d  %d  %d  %d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].x,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].z,
			terrain_sectors[sec_z][sec_x].points_y[point_loop].y);
	}

	printf("\nPolygon  Surface  Reference 0  Reference 1  Reference 2\n\n");

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		printf("%d  %d  %d  %d  %d\n", poly_loop,
			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id,
			terrain_sectors[sec_z][sec_x].point_word_references[poly_loop * 3].point,
			terrain_sectors[sec_z][sec_x].point_word_references[poly_loop * 3 + 1].point,
			terrain_sectors[sec_z][sec_x].point_word_references[poly_loop * 3 + 2].point);

	} // end for poly loop


} // end print sector data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void save_terrain_sector_as_lw_3d_object ( int sec_x, int sec_z, const char *filename )
{

	FILE
		*fp;

	char
		id[8];

	terrain_3d_sector
		*sector;

	int
		count,
		size,
		number_of_triangles,
		number_of_point_references;

	terrain_3d_face
		*polygon;

	terrain_3d_point_xz
		*points_xz;

	terrain_3d_point_y
		*points_y;

	
	if(( fp = fopen(filename,"wb"))==NULL){
		printf("cannot open %s\n", filename);
		exit(1);
	}
	else printf("%s opened ...\n", filename );

	sector = &terrain_sectors[sec_z][sec_x];

	points_y = sector->points_y;

	//points_xz = ( terrain_3d_point_xz * ) points_y + sector->number_of_points;

	points_xz = sector->points_xz; // Craig changed from line above

	fwrite ( "FORM", 4, 1, fp );

	//
	// Work out the size of the file.
	//

	//
	//
	//
	// LWOB identifier
	//			4 bytes
	//
	// Surface names:
	//			8 bytes + 8 * number of surfaces
	//
	// Surface descriptions:
	//			26 * number of surfaces
	//
	// Points:
	//			8 bytes + 12 * number of points
	//
	// Faces:
	//			8 bytes + 4 * number of faces + 2 * number of point references
	//
	// Total:
	//			28 + 12 * Points + 4 * faces + 2 * point refs
	//

	polygon = sector->polygons;

	number_of_point_references = 0;

	number_of_triangles = 0;

	
	
	for ( count = 0; count < sector->number_of_polygons; count++ )
	{
		int
			number_of_polygon_triangles;

		number_of_polygon_triangles = ( polygon->number_of_points - 2 );

		number_of_triangles += number_of_polygon_triangles;

		number_of_point_references += 3 * number_of_polygon_triangles;

		polygon++;
	}

	size = 28 +
			( 26 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 8 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 12 * sector->number_of_points ) +
			( 4 * number_of_triangles ) +
			( 2 * number_of_point_references );

	iff_int_write ( size, fp );

	fwrite ( "LWOB", 4, 1, fp );

	//
	// Detail the surface names
	//

	fwrite ( "SRFS", 4, 1, fp );

	iff_int_write ( ( ( TERRAIN_TYPE_LAST -1 ) * 8 ), fp );

	for ( count = 1; count < TERRAIN_TYPE_LAST; count++ )
	{

		sprintf ( id, "%07d", count );

		//printf("%07d", count ); // Craig

		fwrite ( id, 8, 1, fp );
	}

	//
	// Describe the 3d points
	//

	fwrite ( "PNTS", 4, 1, fp );

	iff_int_write ( ( 12 * sector->number_of_points ), fp );

	for ( count = 0; count < sector->number_of_points; count++ )
	{

		float
			x,
			y,
			z;

		x =  points_xz[count].x;
		y =  points_y[count].y;
		z =  points_xz[count].z;
/*
		x *= ( 2048.0 / 65535.0 );
		z *= ( 2048.0 / 65535.0 );
		y = ( sector->maximum_height - sector->minimum_height );

		if ( y == 0 )
		{

			y = sector->maximum_height;
		}
		else
		{

			y = sector->minimum_height + ( ( float ) points_y[count].y / 32767.0 ) * y;
		}
*/

		iff_float_write ( x, fp );
		iff_float_write ( y, fp );
		iff_float_write ( z, fp );

		//printf("%d  %f  %f  %f\n", count,  x, z, y); // Craig
	}

	//
	// Describe the polygons
	//

	{

		int
			points_offset;

		terrain_3d_surface
			*surface;

		fwrite ( "POLS", 4, 1, fp );

		iff_int_write ( ( ( 4 * number_of_triangles ) + ( 2 * number_of_point_references ) ), fp );

		points_offset = 0;

		polygon = sector->polygons;

		surface = sector->surface_ids;

		for ( count = 0; count < sector->number_of_polygons; count++ )
		{

			int
				point_count;

			if ( polygon->surface_change )
			{

				surface++;
			}

			if ( terrain_type_information[surface->surface_id].polygon_type == POLYGON_TYPE_FAN )
			{

				if ( sector->number_of_points < 257 )
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						iff_short_int_write ( 3, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset].point, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
						iff_short_int_write ( ( short int ) surface->surface_id, fp );
					}
				}
				else
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						iff_short_int_write ( 3, fp );
						iff_short_int_write ( sector->point_word_references[points_offset].point, fp );
						iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
						iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
						iff_short_int_write ( ( short int ) surface->surface_id, fp );
					}
				}
			}
			else
			{

				if ( sector->number_of_points < 257 )
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						if ( point_count & 1 )
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
						else
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
					}
				}
				else
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						if ( point_count & 1 )
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
						else
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
					}
				}
			}



			points_offset += polygon->number_of_points;

			polygon++;
		}
	}

	//
	// Describe the surface colours
	//

	for ( count = 1; count < TERRAIN_TYPE_LAST; count++ )
	{

		unsigned char
			value;

		fwrite ( "SURF", 4, 1, fp );

		iff_int_write ( 18, fp );

		sprintf ( id, "%07d", count );

		fwrite ( id, 8, 1, fp );

		fwrite ( "COLR", 4, 1, fp );

		iff_short_int_write ( 4, fp );

		value = terrain_type_information[count].red;
		fwrite ( &value, 1, 1, fp );

		value = terrain_type_information[count].green;
		fwrite ( &value, 1, 1, fp );

		value = terrain_type_information[count].blue;
		fwrite ( &value, 1, 1, fp );

		value = 0;
		fwrite ( &value, 1, 1, fp );
	}

	fclose ( fp );

} // end save terrain

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void iff_float_write ( float value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[4];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[3];
	buffer[1] = ptr[2];
	buffer[2] = ptr[1];
	buffer[3] = ptr[0];

	fwrite ( buffer, 4, 1, fp );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void iff_int_write ( int value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[4];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[3];
	buffer[1] = ptr[2];
	buffer[2] = ptr[1];
	buffer[3] = ptr[0];

	fwrite ( buffer, 4, 1, fp );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void iff_short_int_write ( short int value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[2];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[1];
	buffer[1] = ptr[0];

	fwrite ( buffer, 2, 1, fp );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

float iff_float_read ( FILE *fp )
{

	unsigned char
		buffer1[4];

	unsigned char
		buffer2[4];

	float
		*result;

	fread ( buffer1, 4, 1, fp );

	buffer2[0] = buffer1[3];
	buffer2[1] = buffer1[2];
	buffer2[2] = buffer1[1];
	buffer2[3] = buffer1[0];

	result = ( float * ) buffer2;

	return ( *result );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

short int iff_short_int_read ( FILE *fp )
{

	short int
		value;

	unsigned char
		val;

	fread ( &val, 1, 1, fp );
	value = ( val<<8 );

	fread ( &val, 1, 1, fp );
	value |= ( val );

	return ( value );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

int iff_int_read ( FILE *fp )
{

	unsigned int
		value;

	unsigned char
		val;

	fread ( &val, 1, 1, fp );
	value = ( val<<24 );

	fread ( &val, 1, 1, fp );
	value += ( val<<16 );

	fread ( &val, 1, 1, fp );
	value += ( val<<8 );

	fread ( &val, 1, 1, fp );
	value += ( val );

	return ( value );
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void set_terrain_rendering_routines ( terrain_types type, terrain_polygon_type polygon_type)
{

	terrain_type_information[type].initialised = TRUE;

	terrain_type_information[type].polygon_type = polygon_type;

}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void set_terrain_type_textures ( terrain_types type, int red, int green, int blue )
{

	terrain_type_information[type].red = red;
	terrain_type_information[type].green = green;
	terrain_type_information[type].blue = blue;

}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void initialize_terrain_type_information(void){

	set_terrain_rendering_routines ( TERRAIN_TYPE_SEA, POLYGON_TYPE_FAN );

   set_terrain_rendering_routines ( TERRAIN_TYPE_BEACH, POLYGON_TYPE_STRIP);

   set_terrain_rendering_routines ( TERRAIN_TYPE_LAND, POLYGON_TYPE_FAN );

	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_FLOOR, POLYGON_TYPE_FAN );

	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_MID_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_TOP_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_MID_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_TOP_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_TOP, POLYGON_TYPE_FAN );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA1, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA2, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA3, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA4, POLYGON_TYPE_FAN );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_ROAD, POLYGON_TYPE_STRIP );
 	
	set_terrain_rendering_routines ( TERRAIN_TYPE_TRACK, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RIVER, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RESERVOIR, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RAIL, POLYGON_TYPE_STRIP );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_ROAD_BANK, POLYGON_TYPE_STRIP );
 	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RIVER_BANK, POLYGON_TYPE_STRIP );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_RAIL_BANK, POLYGON_TYPE_STRIP );

	//
	// Field types
	//

   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD1, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD2, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD3, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD4, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD5, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD6, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD7, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD8, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD9, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD10, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD11, POLYGON_TYPE_FAN );

	//
	// Varied Land Types (e.g. snow, rock etc.)
	//
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND1, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND2, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND3, POLYGON_TYPE_FAN );

	//
	// Hedges / Walls
	//

   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_TOP, POLYGON_TYPE_STRIP );
  
   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_SIDE_X, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_SIDE_Z, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_TOP, POLYGON_TYPE_STRIP );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_SIDE_X, POLYGON_TYPE_STRIP );
  
   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_SIDE_Z, POLYGON_TYPE_STRIP );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH_SIDE_X, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH_SIDE_Z, POLYGON_TYPE_STRIP );

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   // now do the terrain_type_textures
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   set_terrain_type_textures ( TERRAIN_TYPE_SEA, 4, 103, 115 );

   set_terrain_type_textures ( TERRAIN_TYPE_BEACH, 199, 172, 120 );
   set_terrain_type_textures ( TERRAIN_TYPE_LAND,  61, 135, 75);

	//
	// Forest textures
	//

	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_FLOOR, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_MID_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_TOP_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_MID_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_TOP_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_TOP, 190, 170, 15 );

	//
	// Builtup areas
	//


   set_terrain_type_textures ( TERRAIN_TYPE_ROAD, 255, 255, 255 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRACK, 255, 255, 255 );


	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA1, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA2, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA3, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA4, 255, 255, 255 );


   set_terrain_type_textures ( TERRAIN_TYPE_RIVER, 4, 103, 115 );
   set_terrain_type_textures ( TERRAIN_TYPE_RESERVOIR, 4, 103, 115 );
   set_terrain_type_textures ( TERRAIN_TYPE_RAIL, 38, 38, 38 );
   set_terrain_type_textures ( TERRAIN_TYPE_ROAD_BANK, 61, 135, 75 );
   set_terrain_type_textures ( TERRAIN_TYPE_RIVER_BANK, 141, 122, 84 );
   set_terrain_type_textures ( TERRAIN_TYPE_RAIL_BANK, 141, 122, 84 );

	//
	// Field types
	//

   set_terrain_type_textures ( TERRAIN_TYPE_FIELD1, 255, 255, 80 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD2, 100, 70, 40 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD3, 140, 70, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD4, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD5, 170, 140, 0 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD6, 81, 165, 85 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD7, 41, 115, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD8, 255, 255, 80 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD9, 100, 70, 40 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD10, 140, 70, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD11, 190, 170, 15 );

	//
	// Varied Land Types (e.g. snow, rock etc.)
	//
	
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND1, 170, 140, 0 );
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND2, 81, 165, 85 );
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND3, 255, 255, 255 );


	//
	// Hedges / Walls
	//

   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_TOP, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_SIDE_X, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_SIDE_Z, 190, 170, 15 );

	set_terrain_type_textures ( TERRAIN_TYPE_WALL_TOP, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_WALL_SIDE_X, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_WALL_SIDE_Z, 255, 255, 255 );

   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH_SIDE_X, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH_SIDE_Z, 190, 170, 15 );

	//
	// All the types that are used in the terrain generation tool
	//


} // end initialize_terrain_type_information(void)

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void test_function_for_reading_lwo_file(int sec_z, int sec_x){

	FILE
		*fp;

	int
		point_loop,
		poly_loop,
		iff_byte_offset;

	char
		filename [32],
		iff_id[16];

	///////////////////////////////////////////////////////////////

	sprintf(filename, "mod_lw_objs\\%03dx%03d.lwo", sec_x, sec_z); 

	if(( fp = fopen(filename,"rb"))==NULL){
		printf("cannot open %s\n", filename);
		exit(1);
	}
	else printf("%s opened ...\n", filename );

	fread ( iff_id, 4, 1, fp ); // FORM

	iff_id[4] = NULL;

	printf ("%s\n",iff_id);

	iff_byte_offset = iff_int_read ( fp );

	printf ("size of file = %d\n", iff_byte_offset);

	////////////////////////////////////////////

	fread ( iff_id, 8, 1, fp ); // LWOBSRFS

	iff_id[8] = NULL;

	printf ("%s\n",iff_id);

	iff_byte_offset = iff_int_read ( fp );

	printf ("offset = %d\n", iff_byte_offset);

	/////////////////////////////////////////////

	fseek (fp, iff_byte_offset, SEEK_CUR);

	fread ( iff_id, 4, 1, fp ); // PNTS

	iff_id[4] = NULL;

	printf ("%s\n",iff_id);

	iff_byte_offset = iff_int_read ( fp );

	printf ("offset = %d\n", iff_byte_offset);

	/////////////////////////////////////////////

	fseek (fp, iff_byte_offset, SEEK_CUR);

	fread ( iff_id, 4, 1, fp ); // POLS  

	iff_id[4] = NULL;

	printf ("%s\n",iff_id);

	iff_byte_offset = iff_int_read ( fp );

	printf ("offset = %d\n", iff_byte_offset);

	fclose(fp);

} // end test function for reading lwo file

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

int file_exist (char *filename)
{
	char fn[32];

	FILE
		*file_ptr;

	if (file_ptr = fopen (filename, "r"))
	{

		fclose (file_ptr);

		return TRUE;
	}

	return FALSE;
}

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void assign_surface_ids(int sec_z, int sec_x){

	// this function is called once for each sector

	int poly_loop,
		polys_this_sector,
		slope,
		y_point0, //sector coordinates
		y_point1,
		y_point2,
		field11_found_for_this_sector = 0;

	double	elevation,
			min_elevation,
			max_elevation,
			y0,  // world coordinates, actual elevation in meters
			y1,
			y2;

	float	terrain_3d_map_scaled_height_difference;
	
	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;

	//////////////////////////////////////////////////////////////////////
	// loop thru all polys and assign surface ids
	//////////////////////////////////////////////////////////////////////

	//printf("sector %d x %d\n", sec_x, sec_z);

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){
		
		// real world elevation

		y0 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y1 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y2 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point].y * 
			terrain_3d_map_scaled_height_difference ));
		
		// sector coordinate elevation
	
		y_point0 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point].y;

		y_point1 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point].y;

		y_point2 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point].y;

		// find the average elevation of the poly

		min_elevation = y0;
		if(y1 < min_elevation) min_elevation = y1;
		if(y2 < min_elevation) min_elevation = y2;

		max_elevation = y0;
		if(y1 > max_elevation) max_elevation = y1;
		if(y2 > max_elevation) max_elevation = y2;


		//elevation = (int)(min_elevation + max_elevation) / 2;
		elevation = min_elevation;

		slope = normal_angle_with_y(sec_z, sec_x, poly_loop);

		////////////////////////////////////////////////////////////////////////////////////////////////
		// this 'if else' chain is where you select what id you want for each face
		// the order is important ... think about what you want eliminated from the selection first
		//  the surface_id_each_face[ poly_loop ] array isn't used by EECH, it's only temporary for this program
		////////////////////////////////////////////////////////////////////////////////////////////////

		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_RIVER){ 

			// no change
		}

		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND3){ 

			// no change it's really a river polygon
		}

		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ROAD){ 

			// no change
		}
/*
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND3){ 

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND2;
		}
*/		
        else if(slope > 30 ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND1;
		}
/*
		else if(slope < 40){ 

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND1;
		}
		else if(slope < 60){ 

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND3;
		}
		else{ // everything else
			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND3;
		}
*/
		else{

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_LAND;
		}


	} // end for poly loop

} // end assign_surface_ids


//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void sort_sector_polys_by_surface( int sec_z, int sec_x ){

	int
		texture_loop,
		poly_loop,
		number_of_polys,
		temp_poly_array_index = 0,
		word_ref_index = 0,
		current_surface_id,
		//surface_changes_count,
		surface_ids_count;

	terrain_3d_face
		temp_sorted_polygons[1024];

	terrain_3d_point_word_reference
		temp_sorted_word_ref[3072];

	terrain_3d_surface
		temp_sorted_surface_id_each_face[1024];
/*
	int used_texture_list[] = {
		TERRAIN_TYPE_RIVER,
		TERRAIN_TYPE_TRENCH,
		TERRAIN_TYPE_LAND,
		TERRAIN_TYPE_FIELD1,
		TERRAIN_TYPE_ALTERED_LAND1,
		TERRAIN_TYPE_ALTERED_LAND2,
		TERRAIN_TYPE_ALTERED_LAND3,
		TERRAIN_TYPE_FOREST_FLOOR,
		TERRAIN_TYPE_ROAD,
		TERRAIN_TYPE_FOREST_TOP
		
	};
*/
	number_of_polys = terrain_sectors [sec_z][sec_x].number_of_polygons;

	for(texture_loop = 0; texture_loop < number_of_textures_used; texture_loop ++){

		for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

			if(terrain_sectors [sec_z][sec_x].surface_id_each_face[poly_loop].surface_id == 
				used_texture_list [ texture_loop ]){

				temp_sorted_polygons [ temp_poly_array_index ].number_of_points =
					terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].number_of_points;

				temp_sorted_polygons [ temp_poly_array_index ].surface_change =
					terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].surface_change;

				////////////////////////////////////////

				temp_sorted_surface_id_each_face [ temp_poly_array_index ].surface_id = 
					terrain_sectors [sec_z][sec_x].surface_id_each_face [ poly_loop ].surface_id;

				////////////////////////////////////////

				temp_poly_array_index ++;

				////////////////////////////////////////

				temp_sorted_word_ref [ word_ref_index ].point = 
					terrain_sectors [sec_z][sec_x].
					point_word_references [ poly_loop * 3 ].point;

				word_ref_index ++;

				temp_sorted_word_ref [ word_ref_index ].point = 
					terrain_sectors [sec_z][sec_x].
					point_word_references [ poly_loop * 3 + 1 ].point;

				word_ref_index ++;

				temp_sorted_word_ref [ word_ref_index ].point = 
					terrain_sectors [sec_z][sec_x].
					point_word_references [ poly_loop * 3 + 2 ].point;

				word_ref_index ++;
		
			} // end if
		} // end for poly loop 
	} // end for texture loop

	/////////////////////////////////////////////////////////
	// copy temp arrays back to primary data
	/////////////////////////////////////////////////////////

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].number_of_points =
			temp_sorted_polygons [ poly_loop ].number_of_points;

		terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].surface_change =
			temp_sorted_polygons [ poly_loop ].surface_change;

		/////////////////////////////////////////////

		terrain_sectors [sec_z][sec_x].surface_id_each_face[poly_loop].surface_id = 
			temp_sorted_surface_id_each_face[poly_loop].surface_id;

		/////////////////////////////////////////////

		terrain_sectors [sec_z][sec_x].point_word_references[poly_loop * 3].point =
			temp_sorted_word_ref[poly_loop * 3].point;

		terrain_sectors [sec_z][sec_x].point_word_references[poly_loop * 3 + 1].point =
			temp_sorted_word_ref[poly_loop * 3 + 1].point;

		terrain_sectors [sec_z][sec_x].point_word_references[poly_loop * 3 + 2].point =
			temp_sorted_word_ref[poly_loop * 3 + 2].point;

	} // end for poly loop

	/////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////
	// add surface id's and change the surface change bits at proper spot
	/////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////

	terrain_sectors[sec_z][sec_x].number_of_surface_ids = 1;

	terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id = 
		terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id;

	current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

	surface_ids_count = 0;

	//////////////////////////////////////////////////////////////////////////////////////
	// loop thru polys 1-polys_this_sector
	// find surface changes
	// set terrain_sectors[sec_z][sec_x].surface_ids[surface_ids_count].surface_id
	// toggle terrain_sectors[sec_z][sec_x].polygons[x].surface_change
	//////////////////////////////////////////////////////////////////////////////////////

	//
	// it's weird I don't know why the first surface_change for the
	// imported sectors is getting set to 1
	// so I'm setting it back to 0 here
	//

	terrain_sectors[sec_z][sec_x].polygons[ 0 ].surface_change = 0;

	for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

		if(terrain_sectors[sec_z][sec_x].
			surface_id_each_face[poly_loop].surface_id != current_surface_id){

			current_surface_id = terrain_sectors[sec_z][sec_x].
				surface_id_each_face[poly_loop].surface_id;

			terrain_sectors[sec_z][sec_x].number_of_surface_ids++;

			surface_ids_count++;

			terrain_sectors[sec_z][sec_x].
				surface_ids[surface_ids_count].surface_id = current_surface_id;

			terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change = 1;
		} // end if
		else{

			// get rid of original surface_change bits set because order is certain to be different

			terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change = 0;
		}
	} // end for

} // end sort sector polys by surface


/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

double normal_angle_with_y(int sec_z, int sec_x, int poly){

	// returns the angle between the poly normal and the Y axis (bigger = steeper)

	// points

	double ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	double abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	//int angle;
	double	radian_angle,
			pi = 3.1415926535;

	double	normalized_ny,
			angle;

	ax = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].x;

	bx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].x;

	cx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].x;

	az = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].z;

	bz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].z;

	cz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].z;

	ay = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].y;

	by = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].y;

	cy = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].y;


	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_ny = ny / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_ny *= -1.0;
	
	radian_angle = acos(normalized_ny); // normalized_ny / 1 = arcos of angle with y axis
	
	angle = (180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_y()


/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////


void print_terrain_data_of_sector(int sec_z, int sec_x){

	int poly_loop,
		polys_this_sector,
		point_loop,
		points_this_sector;

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	points_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_points;

	printf("\nSector %d x %d\n\n", sec_x, sec_z);

	printf("%d surface ids\n", (int)terrain_sectors[sec_z][sec_x].number_of_surface_ids);

	printf("%d polygons\n", polys_this_sector);

	printf("%d points\n", points_this_sector);

	printf("Polygon   Surface Id Each Face   Surface Change   Word References\n\n");
	
	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		printf("%d  %d  %d      %d  %d  %d\n", poly_loop,
			terrain_sectors[sec_z][sec_x].surface_id_each_face[ poly_loop ].surface_id,
			terrain_sectors[sec_z][sec_x].polygons[ poly_loop ].surface_change,
			terrain_sectors[sec_z][sec_x].point_word_references[ poly_loop * 3 ].point,
			terrain_sectors[sec_z][sec_x].point_word_references[ poly_loop * 3 + 1 ].point,
			terrain_sectors[sec_z][sec_x].point_word_references[ poly_loop * 3 + 2 ].point);
	}
/*
	printf("\n\nPoint  x  z  y  normal change\n\n");

	for(point_loop = 0; point_loop < points_this_sector; point_loop ++){

		printf("%d  %d  %d  %d  %d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x,
			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z,
			terrain_sectors[sec_z][sec_x].points_y[ point_loop ].y,
			terrain_sectors[sec_z][sec_x].points_y[ point_loop ].normal_change);
	}
*/	
} // end print terrain data of sector