#include <stdio.h>
#include <stdlib.h>
#include <math.h>

//#include "terrain.h"
//#include "misc.h"

#include "export.h"


terrain_3d_sector
	**terrain_sectors;

unsigned char
	*terrain_3d_sector_data,
	*terrain_3d_geometry_data,
	*terrain_3d_colour_data;

int
	terrain_version,
	rgb_4byte_header,
	ffp_4byte_header,
	size_of_ffp_file,
	size_of_rgb_file;

unsigned short int
	map_width, 
	map_height, 
	normal_changes_word;

int
	colour_file_offset_counter;

int
	terrain_3d_map_width,
	terrain_3d_map_height;

float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;


///////////////////////////////
// Functions
///////////////////////////////

void save_terrain_sector_as_lw_3d_object ( int x, int z, const char *filename );

void load_3d_terrain (void);

void iff_short_int_write ( short int value, FILE *fp );

void iff_int_write ( int value, FILE *fp );

void iff_float_write ( float value, FILE *fp );

void set_terrain_rendering_routines ( terrain_types type, terrain_polygon_type polygon_type);
													//void ( *render_word_clipped ) ( int ),
													//void ( *render_byte_clipped ) ( int ),
													//void ( *render_word_unclipped ) ( int ),
													//void ( *render_byte_unclipped ) ( int ) );

void initialize_terrain_type_information(void);

void print_sector_data(int z, int x);

void set_terrain_type_textures ( terrain_types type, int red, int green, int blue );



int main(int argc, char ** argv)
{
	int
		x,
		z;

	char
		filename [16];

	load_3d_terrain ();

	//print_sector_data(10, 10);

	initialize_terrain_type_information();

	z = atoi ( argv [ 1 ] );

	x = atoi ( argv [ 2 ] );

	sprintf(filename, "%03dx%03d.lwo", z, x); 

	save_terrain_sector_as_lw_3d_object ( x, z, filename );

/*
	for(x = 0; x < terrain_3d_map_width; x ++){
		for(z = 0; z < terrain_3d_map_height; z ++){

			//sprintf(filename, "%03dx%03d.lwo", x, z); 
			sprintf(filename, "terrain_lw_objs\\%03dx%03d.lwo", x, z); 

			save_terrain_sector_as_lw_3d_object ( x, z, filename );

		}// end for z loop
	} // end for x loop
*/

	

	return 0;
} // end main

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////


void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	long
		file_size;

	int
		sec_x,
		sec_z,
		offset;

	short int
		pad_word_zero;

	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");

	//
	// load terrain.ffp file
	//

	fseek( ffp_in, 0L, SEEK_END );
	file_size = ftell( ffp_in );
	fseek( ffp_in, 0L, SEEK_SET );

	size_of_ffp_file = file_size;

	printf("Allocating memory for terrain data ...\n");

	if ( ! (terrain_3d_geometry_data = malloc ( file_size ) ) ){

		printf("No memory for terrain data ...\n");
		exit(1);
	}

	printf("Reading terrain data ...\n");

	fread ( terrain_3d_geometry_data, file_size, 1, ffp_in);

	//
	// load rgb file
	//

	fseek( rgb_in, 0L, SEEK_END );
	file_size = ftell( rgb_in );
	fseek( rgb_in, 0L, SEEK_SET );

	size_of_rgb_file = file_size;

	printf("Allocating memory for rgb data ...\n");

	if ( ! (terrain_3d_colour_data = malloc ( file_size ) ) ){

		printf("No memory for rgb data ...\n");
		exit(1);
	}

	printf("Reading rgb data ...\n");

	fread ( terrain_3d_colour_data, file_size, 1, rgb_in);

	//
	// Get ffp and rgb headers
	//

	fseek( ffp_in, 0L, SEEK_SET );

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	fseek( rgb_in, 0L, SEEK_SET );

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);


	///////////////////////////////////////////////////////
	// read in map dimensions from sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;

	terrain_3d_map_width = (int)map_width;

	terrain_3d_map_height = (int)map_height;

	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	if ( ! (terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) ) ) ){

		printf("No memory for terrain_sectors ...\n");
		exit(1);
	}

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory for terrain_sectors[%d]\n", sec_z);
			exit(1);
		}

	} // end for

	printf("Memory for sectors allocated ...\n");

	//map_height = 2;

	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

			//printf("\nSector %d x %d\n\n", sec_z, sec_x);
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////
			// Get the offsets into the geometry file
			///////////////////////////////////////////////////////////////

			//////////////////////////
			// points y
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpoints y offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].points_y = ( terrain_3d_point_y * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// points xz
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpoints xz offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].points_xz = ( terrain_3d_point_xz * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// point references
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpoint references offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].point_byte_references = ( terrain_3d_point_byte_reference * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// polygons
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tpolygons offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].polygons = ( terrain_3d_face * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// surface changes
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tsurface changes offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].surface_changes = ( terrain_3d_surface * ) ( terrain_3d_geometry_data + offset );

			//////////////////////////
			// normal changes
			//////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			//printf("\tnormal changes offset %d\n", offset);

			terrain_sectors[sec_z][sec_x].normal_changes = ( terrain_3d_normal_index * ) ( terrain_3d_geometry_data + offset );

			//
			// Get the offsets into the colours file
			//

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			terrain_sectors[sec_z][sec_x].colour_indices_offset = offset;
			
			terrain_sectors[sec_z][sec_x].colour_indices = ( terrain_3d_colour_index * ) ( terrain_3d_colour_data + offset );

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			terrain_sectors[sec_z][sec_x].point_colours_offset = offset;
			
			terrain_sectors[sec_z][sec_x].point_colours = ( terrain_3d_colour * ) ( terrain_3d_colour_data + offset );
		}
	}

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain loaded ...\n");

	
} // end load 3d terrain

/*
void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		terrain_version,
		x,
		z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		rgb_4byte_header,
		ffp_4byte_header,
		four_byte_zero_word,
		i,
		print_loop;

	float
		min_height,
		max_height,
		terrain_3d_map_scaled_height_difference;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius,
		normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height,
		pad_word_zero;

	unsigned char
		pad_byte_zero,
		normal_changes_buf[1024];
	
	long
		current_file_position;

	////////////////////////////////////////////////////////////////////////////
	// initialize variables
	////////////////////////////////////////////////////////////////////////////

	colour_file_offset_counter = 0;

	

	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(z = 0; z < map_height; z ++){

		if( ! (terrain_sectors[z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( z=0 ; z < map_height ; z++){
		for( x=0 ; x < map_width ; x++){
		//printf("sector %d x %d\n", x, z);	
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			//printf("\nSector %d x %d\n\n", z, x);

			fread(&terrain_sectors[z][x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			//printf("%d points\n", terrain_sectors[z][x].number_of_points);
		
			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[z][x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			//printf("%d polygons\n", terrain_sectors[z][x].number_of_polygons);

			fread(&terrain_sectors[z][x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[z][x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[z][x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[z][x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n",z,x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[z][x].points_y );

			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[z][x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[z][x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[z][x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[z][x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n",z,x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[z][x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[z][x].number_of_points, ffp_in);

			///////////////////////////////////////////////////////////////////////
			
					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[z][x].number_of_points <= 256){

				if(!(terrain_sectors[z][x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n",z,x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[z][x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[z][x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[z][x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n",z,x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[z][x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[z][x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[z][x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n",z,x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			//printf("size of polygon chunk = %d\n",
			//	surface_changes_offset - polygons_offset);

			fread(terrain_sectors[z][x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[z][x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[z][x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[z][x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface changes - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[z][x].surface_changes = 
				malloc(normal_changes_offset - surface_changes_offset))){
				printf("No memory for surface_changes sector %d x %d\n",z,x);
				exit(1);
			}

			terrain_sectors[z][x].number_of_surface_changes_bytes = 
				normal_changes_offset - surface_changes_offset;

			//printf("%d surface changes bytes\n",
			//	terrain_sectors[z][x].number_of_surface_changes_bytes);

			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[z][x].surface_changes, sizeof(terrain_3d_surface),
				terrain_sectors[z][x].number_of_surface_changes_bytes, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			if((z == map_height - 1) && (x == map_width - 1)){

				// this method for when data is the last chunk of the ffp file

				i = 0;

				//printf("normal changes fp = %d\n", (int)ftell(ffp_in));

				while(!feof(ffp_in)){

					//fread((normal_changes_buf + i), sizeof(unsigned char), 1, ffp_in);

					normal_changes_buf[i] = getc(ffp_in);

					i++;

					//printf("i = %d		%d\n", i,normal_changes_buf[i]);
					//if(i > 50)exit(1);

				} // end while

				terrain_sectors[z][x].number_of_normal_changes = i;

				if(!(terrain_sectors[z][x].normal_changes = malloc(i))){
					printf("No memory for normal_changes sector %d x %d\n",z,x);
					exit(1);
				}

				for(i = 0; i < terrain_sectors[z][x].number_of_normal_changes; i++){
					*(terrain_sectors[z][x].normal_changes + i) = normal_changes_buf[i];
				}

			} // end if
			else{

				// this method for when data is NOT the last chunk of the ffp file

				// need to get the next ffp file offset for the size 
				// of normal changes data calculation.
				// this doesn't work for original maps because they may jump back in the ffp
				// file to re-use a previous sector that I assume must be identical

				fseek(sec_in, 24, SEEK_CUR);
				fread(&offset, sizeof(int), 1, sec_in); // sector file
				fseek(sec_in, -28, SEEK_CUR);

				//printf("\nSector %d x %d\n\n", z, x);

				terrain_sectors[z][x].number_of_normal_changes = 
					offset - normal_changes_offset;

				//printf("normal changes fp = %d\n", (int)ftell(ffp_in));

				//printf("next points_y offset = %d\n", offset);

				//printf("number_of_normal_changes = %d\n", 
				//	terrain_sectors[z][x].number_of_normal_changes); 

				if(!(terrain_sectors[z][x].normal_changes = 
					malloc(terrain_sectors[z][x].number_of_normal_changes))){
					printf("No memory for normal_changes sector %d x %d\n",z,x);
					exit(1);
				}

				
				fread(terrain_sectors[z][x].normal_changes,	
					sizeof(*terrain_sectors[z][x].normal_changes),
					terrain_sectors[z][x].number_of_normal_changes, ffp_in);

			} // end else

			// read in color offsets from sector file

			fread(&terrain_sectors[z][x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[z][x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[z][x].colour_indices = malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[z][x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n",z,x);
				exit(1);
			}

			fread(terrain_sectors[z][x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[z][x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[z][x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[z][x].number_of_points; i++){ 
				
				// there's one colour index per point

				if((terrain_sectors[z][x].colour_indices + i)->index > 
					terrain_sectors[z][x].max_colour_index){

					terrain_sectors[z][x].max_colour_index = 
						(terrain_sectors[z][x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[z][x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[z][x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain
*/
//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void save_terrain_sector_as_lw_3d_object ( int x, int z, const char *filename )
{

	FILE
		*fp;

	char
		id[8];

	terrain_3d_sector
		*sector;

	int
		count,
		size,
		number_of_triangles,
		number_of_point_references;

	terrain_3d_face
		*polygon;

	terrain_3d_point_xz
		*points_xz;

	terrain_3d_point_y
		*points_y;

	
	if(( fp = fopen(filename,"wb"))==NULL){
		printf("cannot open %s\n", filename);
		exit(1);
	}
	else printf("%s opened ...\n", filename );

	sector = &terrain_sectors[z][x];

	points_y = sector->points_y;

	//points_xz = ( terrain_3d_point_xz * ) points_y + sector->number_of_points;

	points_xz = sector->points_xz; // Craig changed from line above

	fwrite ( "FORM", 4, 1, fp );

	//
	// Work out the size of the file.
	//

	//
	//
	//
	// LWOB identifier
	//			4 bytes
	//
	// Surface names:
	//			8 bytes + 8 * number of surfaces
	//
	// Surface descriptions:
	//			26 * number of surfaces  (26 = SURF + FOLLOWING DATA)
	//			my change
	//			34 * number of surfaces  (34 = SURF + FOLLOWING DATA + LUMI + DATA)
	//
	// Points:
	//			8 bytes + 12 * number of points
	//
	// Faces:
	//			8 bytes + 4 * number of faces + 2 * number of point references
	//
	// Total:
	//			28 + 12 * Points + 4 * faces + 2 * point refs
	//

	polygon = sector->polygons;

	number_of_point_references = 0;

	number_of_triangles = 0;

	
	
	for ( count = 0; count < sector->number_of_polygons; count++ )
	{
		int
			number_of_polygon_triangles;

		number_of_polygon_triangles = ( polygon->number_of_points - 2 );

		number_of_triangles += number_of_polygon_triangles;

		number_of_point_references += 3 * number_of_polygon_triangles;

		polygon++;
	}

	size = 28 +
			//( 26 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 34 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 8 * ( TERRAIN_TYPE_LAST - 1 ) ) +
			( 12 * sector->number_of_points ) +
			( 4 * number_of_triangles ) +
			( 2 * number_of_point_references );

	iff_int_write ( size, fp );

	fwrite ( "LWOB", 4, 1, fp );

	//
	// Detail the surface names
	//

	fwrite ( "SRFS", 4, 1, fp );

	iff_int_write ( ( ( TERRAIN_TYPE_LAST -1 ) * 8 ), fp );

	for ( count = 1; count < TERRAIN_TYPE_LAST; count++ )
	{

		sprintf ( id, "%07d", count );

		//printf("%07d", count ); // Craig

		fwrite ( id, 8, 1, fp );
	}

	//
	// Describe the 3d points
	//

	fwrite ( "PNTS", 4, 1, fp );

	iff_int_write ( ( 12 * sector->number_of_points ), fp );

	for ( count = 0; count < sector->number_of_points; count++ )
	{

		float
			x,
			y,
			z;

		x =  points_xz[count].x;
		y =  points_y[count].y;
		z =  points_xz[count].z;
/*
		x *= ( 2048.0 / 65535.0 );
		z *= ( 2048.0 / 65535.0 );
		y = ( sector->maximum_height - sector->minimum_height );

		if ( y == 0 )
		{

			y = sector->maximum_height;
		}
		else
		{

			y = sector->minimum_height + ( ( float ) points_y[count].y / 32767.0 ) * y;
		}
*/

		iff_float_write ( x, fp );
		iff_float_write ( y, fp );
		iff_float_write ( z, fp );

		//printf("%d  %f  %f  %f\n", count,  x, z, y); // Craig
	}

	//
	// Describe the polygons
	//

	{

		int
			points_offset;

		terrain_3d_surface
			*surface;

		fwrite ( "POLS", 4, 1, fp );

		iff_int_write ( ( ( 4 * number_of_triangles ) + ( 2 * number_of_point_references ) ), fp );

		points_offset = 0;

		polygon = sector->polygons;

		surface = sector->surface_changes;

		for ( count = 0; count < sector->number_of_polygons; count++ )
		{

			int
				point_count;

			if ( polygon->surface_change )
			{

				surface++;
			}

			if ( terrain_type_information[surface->surface_id].polygon_type == POLYGON_TYPE_FAN )
			{

				if ( sector->number_of_points < 257 )
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						iff_short_int_write ( 3, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset].point, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
						iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
						iff_short_int_write ( ( short int ) surface->surface_id, fp );
					}
				}
				else
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						iff_short_int_write ( 3, fp );
						iff_short_int_write ( sector->point_word_references[points_offset].point, fp );
						iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
						iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
						iff_short_int_write ( ( short int ) surface->surface_id, fp );
					}
				}
			}
			else
			{

				if ( sector->number_of_points < 257 )
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						if ( point_count & 1 )
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
						else
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( sector->point_byte_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
					}
				}
				else
				{

					for ( point_count = 0; point_count < ( polygon->number_of_points - 2 ); point_count++ )
					{

						if ( point_count & 1 )
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
						else
						{

							iff_short_int_write ( 3, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 0].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 1].point, fp );
							iff_short_int_write ( sector->point_word_references[points_offset + point_count + 2].point, fp );
							iff_short_int_write ( ( short int ) surface->surface_id, fp );
						}
					}
				}
			}



			points_offset += polygon->number_of_points;

			polygon++;
		}
	}

	//
	// Describe the surface colours
	//

	for ( count = 1; count < TERRAIN_TYPE_LAST; count++ )
	{

		unsigned char
			value;

		fwrite ( "SURF", 4, 1, fp );

		//iff_int_write ( 18, fp ); // total bytes = 18

		iff_int_write ( 26, fp ); // increased by 8 for LUMI chunk

		sprintf ( id, "%07d", count );

		fwrite ( id, 8, 1, fp ); // 8 bytes (NULL added to 7 byte string to make 8)

		fwrite ( "COLR", 4, 1, fp ); // 4 bytes

		iff_short_int_write ( 4, fp ); // 2 bytes

		value = terrain_type_information[count].red; // 1 byte
		fwrite ( &value, 1, 1, fp );

		value = terrain_type_information[count].green; // 1 byte
		fwrite ( &value, 1, 1, fp );

		value = terrain_type_information[count].blue; // 1 byte
		fwrite ( &value, 1, 1, fp );

		value = 0;
		fwrite ( &value, 1, 1, fp ); // 1 byte

		//
		// I'm adding the below to add a luminosity value
		//

		fwrite ( "LUMI", 4, 1, fp ); // 4 bytes

		iff_short_int_write ( 2, fp ); // 2 bytes

		iff_short_int_write ( 170, fp ); // 2 bytes
	}

	fclose ( fp );




} // end save terrain

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void iff_float_write ( float value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[4];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[3];
	buffer[1] = ptr[2];
	buffer[2] = ptr[1];
	buffer[3] = ptr[0];

	fwrite ( buffer, 4, 1, fp );
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void iff_int_write ( int value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[4];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[3];
	buffer[1] = ptr[2];
	buffer[2] = ptr[1];
	buffer[3] = ptr[0];

	fwrite ( buffer, 4, 1, fp );
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void iff_short_int_write ( short int value, FILE *fp )
{

	unsigned char
		*ptr,
		buffer[2];

	ptr = ( unsigned char * ) &value;

	buffer[0] = ptr[1];
	buffer[1] = ptr[0];

	fwrite ( buffer, 2, 1, fp );
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void set_terrain_rendering_routines ( terrain_types type, terrain_polygon_type polygon_type)
													//void ( *render_word_clipped ) ( int ),
													//void ( *render_byte_clipped ) ( int ),
													//void ( *render_word_unclipped ) ( int ),
													//void ( *render_byte_unclipped ) ( int ) )
{

	terrain_type_information[type].initialised = TRUE;

	terrain_type_information[type].polygon_type = polygon_type;

	//terrain_type_information[type].render_word_clipped = render_word_clipped;
	//terrain_type_information[type].render_byte_clipped = render_byte_clipped;
	//terrain_type_information[type].render_word_unclipped = render_word_unclipped;
	//terrain_type_information[type].render_byte_unclipped = render_byte_unclipped;
}

void initialize_terrain_type_information(void){

	set_terrain_rendering_routines ( TERRAIN_TYPE_SEA, POLYGON_TYPE_FAN );

   set_terrain_rendering_routines ( TERRAIN_TYPE_BEACH, POLYGON_TYPE_STRIP);

   set_terrain_rendering_routines ( TERRAIN_TYPE_LAND, POLYGON_TYPE_FAN );

	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_FLOOR, POLYGON_TYPE_FAN );

	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_MID_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_TOP_X, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_MID_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_SIDE_TOP_Z, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_FOREST_TOP, POLYGON_TYPE_FAN );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA1, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA2, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA3, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_BUILT_UP_AREA4, POLYGON_TYPE_FAN );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_ROAD, POLYGON_TYPE_STRIP );
 	
	set_terrain_rendering_routines ( TERRAIN_TYPE_TRACK, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RIVER, POLYGON_TYPE_STRIP );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RESERVOIR, POLYGON_TYPE_FAN );
	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RAIL, POLYGON_TYPE_STRIP );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_ROAD_BANK, POLYGON_TYPE_STRIP );
 	
	set_terrain_rendering_routines ( TERRAIN_TYPE_RIVER_BANK, POLYGON_TYPE_STRIP );
   
	set_terrain_rendering_routines ( TERRAIN_TYPE_RAIL_BANK, POLYGON_TYPE_STRIP );

	//
	// Field types
	//

   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD1, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD2, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD3, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD4, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD5, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD6, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD7, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD8, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD9, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD10, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_FIELD11, POLYGON_TYPE_FAN );

	//
	// Varied Land Types (e.g. snow, rock etc.)
	//
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND1, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND2, POLYGON_TYPE_FAN );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_ALTERED_LAND3, POLYGON_TYPE_FAN );

	//
	// Hedges / Walls
	//

   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_TOP, POLYGON_TYPE_STRIP );
  
   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_SIDE_X, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_HEDGE_SIDE_Z, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_TOP, POLYGON_TYPE_STRIP );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_SIDE_X, POLYGON_TYPE_STRIP );
  
   set_terrain_rendering_routines ( TERRAIN_TYPE_WALL_SIDE_Z, POLYGON_TYPE_STRIP );
   
   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH_SIDE_X, POLYGON_TYPE_STRIP );

   set_terrain_rendering_routines ( TERRAIN_TYPE_TRENCH_SIDE_Z, POLYGON_TYPE_STRIP );

   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////
   // now do the terrain_type_textures
   /////////////////////////////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////

   set_terrain_type_textures ( TERRAIN_TYPE_SEA, 4, 103, 115 );

   set_terrain_type_textures ( TERRAIN_TYPE_BEACH, 199, 172, 120 );
   set_terrain_type_textures ( TERRAIN_TYPE_LAND,  61, 135, 75);

	//
	// Forest textures
	//

	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_FLOOR, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_MID_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_TOP_X, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_MID_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_SIDE_TOP_Z, 80, 60, 30 );
	set_terrain_type_textures ( TERRAIN_TYPE_FOREST_TOP, 190, 170, 15 );

	//
	// Builtup areas
	//


   set_terrain_type_textures ( TERRAIN_TYPE_ROAD, 255, 255, 255 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRACK, 255, 255, 255 );


	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA1, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA2, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA3, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_BUILT_UP_AREA4, 255, 255, 255 );


   set_terrain_type_textures ( TERRAIN_TYPE_RIVER, 4, 103, 115 );
   set_terrain_type_textures ( TERRAIN_TYPE_RESERVOIR, 4, 103, 115 );
   set_terrain_type_textures ( TERRAIN_TYPE_RAIL, 38, 38, 38 );
   set_terrain_type_textures ( TERRAIN_TYPE_ROAD_BANK, 61, 135, 75 );
   set_terrain_type_textures ( TERRAIN_TYPE_RIVER_BANK, 141, 122, 84 );
   set_terrain_type_textures ( TERRAIN_TYPE_RAIL_BANK, 141, 122, 84 );

	//
	// Field types
	//

   set_terrain_type_textures ( TERRAIN_TYPE_FIELD1, 255, 255, 80 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD2, 100, 70, 40 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD3, 140, 70, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD4, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD5, 170, 140, 0 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD6, 81, 165, 85 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD7, 41, 115, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD8, 255, 255, 80 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD9, 100, 70, 40 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD10, 140, 70, 50 );
   set_terrain_type_textures ( TERRAIN_TYPE_FIELD11, 190, 170, 15 );

	//
	// Varied Land Types (e.g. snow, rock etc.)
	//
	
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND1, 170, 140, 0 );
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND2, 81, 165, 85 );
	set_terrain_type_textures ( TERRAIN_TYPE_ALTERED_LAND3, 255, 255, 255 );


	//
	// Hedges / Walls
	//

   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_TOP, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_SIDE_X, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_HEDGE_SIDE_Z, 190, 170, 15 );

	set_terrain_type_textures ( TERRAIN_TYPE_WALL_TOP, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_WALL_SIDE_X, 255, 255, 255 );
	set_terrain_type_textures ( TERRAIN_TYPE_WALL_SIDE_Z, 255, 255, 255 );

   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH_SIDE_X, 190, 170, 15 );
   set_terrain_type_textures ( TERRAIN_TYPE_TRENCH_SIDE_Z, 190, 170, 15 );

	//
	// All the types that are used in the terrain generation tool
	//

   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_BEACH_MID,			0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_FRINGE,				0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_TELEPHONE_LINE,	0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_SEA_BANK,			0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );
   //set_terrain_type_textures ( TEMP_TERRAIN_TYPE_COASTAL_RIVER,	0, 0, 256, 256, 0, 0, 255, 0, 255, 0 );



} // end initialize_terrain_type_information(void)

void print_sector_data(int z, int x){

	int
		number_of_points,
		number_of_polys,
		point_loop,
		poly_loop;

	number_of_points =  terrain_sectors[z][x].number_of_points;

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		printf("%d  %d  %d  %d\n", point_loop,
			terrain_sectors[z][x].points_xz[point_loop].x,
			terrain_sectors[z][x].points_xz[point_loop].z,
			terrain_sectors[z][x].points_y[point_loop].y);
	}


}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void set_terrain_type_textures ( terrain_types type, int red, int green, int blue )
{

	terrain_type_information[type].red = red;
	terrain_type_information[type].green = green;
	terrain_type_information[type].blue = blue;

}


