// Power lines
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "power_lines.h"

// the pylon stuff is found in py_creat.c

////////////////////////////////////////
// Globals
////////////////////////////////////////

/////////////////////////////////////////////////
// *** start of Global Lists
/////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// holds geometry data terrain_sectors[z][x]
////////////////////////////////////////////////////////////////////////////

terrain_3d_sector		terrain_sectors[MAP_HEIGHT][MAP_WIDTH];

/////////////////////////////////////////////////////////////////////////////
// Holds up to 128 roads and up to 512 quads per road
/////////////////////////////////////////////////////////////////////////////

road	roads[128];

////////////////////////////////////////////////////
// *** end of Global Lists
////////////////////////////////////////////////////

int
					number_of_roads,
					max_road_poly_points,
					min_road_poly_points,
					max_road_polys_in_sector[3],
					number_of_junctions;

float	terrain_3d_map_minimum_height,
		terrain_3d_map_maximum_height;

double	terrain_3d_map_scaled_height_difference;

/////////////////////////////////////////
// functions
/////////////////////////////////////////

void read_in_util_files(void);

void process_power_lines(void);

int InsidePolygon(simple_point *polygon,int N,simple_point p);

int get_y_value(int sec_x, int sec_z, int x, int z);

void add_intermediate_pylons(void);


int main(int argc, char ** argv){

	
	FILE				*sec_in,
						*ffp_in,
						*fp;
	int					terrain_version,
						x,
						z,
						offset,
						next_offset,
						ffp_4byte_header,
						four_byte_zero_word,
						sec_z,
						sec_x;
	float				min_height,
						max_height,
						terrain_3d_map_scaled_height_difference;
	unsigned short int	map_width,
						map_height,
						sec_number_of_points,
						sec_number_of_polys,
						sec_radius,
						normal_changes_word;
	short int			sec_min_height,
						sec_max_height,
						sec_avg_height,
						pad_word_zero,
						pad_word_1536;
	unsigned char		pad_byte_zero,
						junk_byte_zero;

	////////////////////////////////////////////////////////////////////////////
	// initialize variables
	////////////////////////////////////////////////////////////////////////////


	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");

	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");

	

	////////////////////////////////////////////////////////
	// read in and write out map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);

	fread(&map_width, sizeof(unsigned short int), 1, sec_in);

	fread(&map_height, sizeof(unsigned short int), 1, sec_in);

	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);

	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);
	//fwrite(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_out);

	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	
	for( z=0 ; z < map_height ; z++){
		for( x=0 ; x < map_width ; x++){

			//printf("sector %d x %d\n", z, x);

			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[z][x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);
		
			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[z][x].pad_byte, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			//printf("%d  %d  %d\n", terrain_sectors[z][x].number_of_polygons, x, z); 

			fread(&terrain_sectors[z][x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[z][x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[z][x].number_of_points, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[z][x].number_of_points, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(terrain_sectors[z][x].number_of_points <= 256){

				fread(terrain_sectors[z][x].point_byte_references,
					(sizeof(terrain_3d_point_byte_reference) * 3),
					terrain_sectors[z][x].number_of_polygons, ffp_in);
			}
			else{

				fread(terrain_sectors[z][x].point_word_references,
					(sizeof(terrain_3d_point_word_reference) * 3),
					terrain_sectors[z][x].number_of_polygons, ffp_in);
			}

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[z][x].number_of_polygons, ffp_in);

			if(terrain_sectors[z][x].number_of_polygons % 2 != 0){
				fread(&junk_byte_zero, 1, 1, ffp_in);
			}

			fread(&offset, sizeof(int), 1, sec_in); // sector file
			fread(&next_offset, sizeof(int), 1, sec_in); // sector file

			// for a variable number of surface ids

			fread(terrain_sectors[z][x].surface_ids,
				sizeof(terrain_3d_surface),	(next_offset - offset), ffp_in);

			// find how many surface ids
			// if the number of surface ids is odd then a zero pad byte is added at the end
			// to make it even

			//printf("%d\n", next_offset - offset - 1);

			if(terrain_sectors[z][x].surface_ids[next_offset - offset - 1].surface_id == 0){
				terrain_sectors[z][x].number_of_surface_ids = next_offset - offset - 1;
			}
			else{
				terrain_sectors[z][x].number_of_surface_ids = next_offset - offset;
			}

/*			printf("Sector %d X %d surface ids = %d\n", z, x, terrain_sectors[z][x].number_of_surface_ids);

			for(i=0; i < terrain_sectors[z][x].number_of_surface_ids; i++){
				printf("%d ", terrain_sectors[z][x].surface_ids[i].surface_id);
			}
			printf("\n");
*/  


			//fread(&offset, sizeof(int), 1, sec_in); // sector file


			//fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			fread(&terrain_sectors[z][x].normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// read in color offsets from sector file

			fread(&terrain_sectors[z][x].colour_indices_offset,
				sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[z][x].point_colours_offset,
				sizeof(int), 1, sec_in); // sector file


		} // end for
	} // end for

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	printf("Terrain data has been read in\n");

	//read_in_util_files();

	process_power_lines();


	return 0;

} // end main

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void process_power_lines(){

	FILE			*road_fp,
					*out_pylon_node_file_ptr,
					*out_pylon_link_file_ptr,
					*text_out;


	int
					i,
					j,
					k,
					l,
					weird_int_one,
					weird_int_two,
					number_of_points_for_this_road,
					number_of_quads,
					x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
					z_int_pos,
					return_value,
					type = 48,
					node_count,
					path_count,
					node_loop,
					path_loop,
					start,
					end,
					link_count,
					link_loop;

	double // double convert
					ax,
					az,
					bx,
					bz;

	double
					theta;

	simple_point
					p1,
					p2,
					p3,
					p4,
					p_intersect;

	vec3d			pos;

	printf("processing power line roads\n");

	max_road_poly_points = 0;
	min_road_poly_points = 6;
	max_road_polys_in_sector[0] = 0;


	///////////////////////////////////////////////////////
	// read in the *.ROAD file
	///////////////////////////////////////////////////////


	if(( road_fp = fopen("powerlines.ROAD","rb"))==NULL){
		printf("cannot open powerlines.ROAD\n");
		exit(1);
	}
	else printf("powerlines.ROAD opened\n");

	if(( out_pylon_node_file_ptr = fopen("utils.nde","wb"))==NULL){
		printf("cannot open utils.nde\n");
		exit(1);
	}
	else printf("utils.nde opened\n");

	if(( out_pylon_link_file_ptr = fopen("utils.wp","wb"))==NULL){
		printf("cannot open utils.wp\n");
		exit(1);
	}
	else printf("utils.wp opened\n");

	if(( text_out = fopen("Text_output.log","wb"))==NULL){
		printf("cannot open Text_output.log\n");
		exit(1);
	}
	else printf("Text_output.log opened\n");


	//////////////////////////////////////////////////////////////////////
	// read in all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fread ( &number_of_roads, sizeof ( int ), 1, road_fp );

	//printf("number of roads = %d\n\n", number_of_roads);

	for(i = 0; i < number_of_roads; i++){

		fread ( &weird_int_one, sizeof ( int ), 1, road_fp );
		fread ( &weird_int_two, sizeof ( int ), 1, road_fp );
		fread ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );

		//printf("number of points for this road = %d\n\n",
		//	number_of_points_for_this_road);
		
		roads[i].number_of_center_points = number_of_points_for_this_road;

		roads[i].number_of_quads = number_of_points_for_this_road - 1;

		for(j = 0; j < number_of_points_for_this_road; j++){

			fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			roads[i].center_points[j].global_x = (double)x_int_pos / 100.0;
			roads[i].center_points[j].global_z = (double)z_int_pos / 100.0;
		}
	}

	
	fclose ( road_fp );

	////////////////////////////////////////////////////////////////////////
	// print the roads points before adding intermediate points
	////////////////////////////////////////////////////////////////////////

	for(i = 0; i < number_of_roads; i++){

		number_of_points_for_this_road = roads[i].number_of_center_points;

		fprintf(text_out, "Road Points Before Adding Intermediate Points\n\n");

		fprintf(text_out, "Road %d has %d points\n\n", i, number_of_points_for_this_road);

		for(j = 0; j < number_of_points_for_this_road; j++){

			fprintf(text_out, "%d   %f   %f   %f\n", j,
				roads[i].center_points[j].global_x,
				roads[i].center_points[j].global_z,
				roads[i].center_points[j].global_y);
		} // end for

		fprintf(text_out, "\n");

	} // end for


	///////////////////////////////////////////////////////////////////////////
	// add pylons to fill gaps over 500 meters
	///////////////////////////////////////////////////////////////////////////

	add_intermediate_pylons();

	
	///////////////////////////////////////////////////////////////////////////
	// do calculations on the center points
	///////////////////////////////////////////////////////////////////////////

	// the global value of terrain_3d_map_scaled_height_difference is set here

	terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	for(i = 0; i < number_of_roads; i++){

		number_of_points_for_this_road = roads[i].number_of_center_points;

		for(j = 0; j < number_of_points_for_this_road; j++){

			/////////////////////////////////////////////////////////////
			// find the sector
			/////////////////////////////////////////////////////////////

			roads[i].center_points[j].x_sector = (int)roads[i].center_points[j].global_x / 2048;
			roads[i].center_points[j].z_sector = (int)roads[i].center_points[j].global_z / 2048;

			//////////////////////////////////////////////////////////////
			// find the sector coordinates
			//////////////////////////////////////////////////////////////

			roads[i].center_points[j].sec_x = (int)((roads[i].center_points[j].global_x -
				(roads[i].center_points[j].x_sector * 2048 + 1024)) * 16);

			roads[i].center_points[j].sec_z = (int)((roads[i].center_points[j].global_z -
				(roads[i].center_points[j].z_sector * 2048 + 1024)) * 16);

			//printf("%f    %d    %d\n", roads[i].center_points[j].global_x,
			//	roads[i].center_points[j].x_sector,
			//	roads[i].center_points[j].sec_x);

			///////////////////////////////////////////////////////////////
			// get the y value in sector coordinates
			///////////////////////////////////////////////////////////////

			roads[i].center_points[j].sec_y = get_y_value(
				roads[i].center_points[j].x_sector,
				roads[i].center_points[j].z_sector,
				roads[i].center_points[j].sec_x,
				roads[i].center_points[j].sec_z);

			////////////////////////////////////////////////////////////////
			// calculate the global y value
			////////////////////////////////////////////////////////////////

			roads[i].center_points[j].global_y = ((double)terrain_3d_map_minimum_height + 
				((double)roads[i].center_points[j].sec_y * terrain_3d_map_scaled_height_difference ));
	

		} // end for points per road
	} // end for roads

	////////////////////////////////////////////////////////////////////////
	// print the roads points after adding intermediate pylons
	////////////////////////////////////////////////////////////////////////

	for(i = 0; i < number_of_roads; i++){

		number_of_points_for_this_road = roads[i].number_of_center_points;

		fprintf(text_out, "Road Points After Adding Intermediate Points\n\n");

		fprintf(text_out, "Road %d has %d points\n\n", i, number_of_points_for_this_road);

		for(j = 0; j < number_of_points_for_this_road; j++){

			fprintf(text_out, "%d   %f   %f   %f\n", j,
				roads[i].center_points[j].global_x,
				roads[i].center_points[j].global_z,
				roads[i].center_points[j].global_y);
		} // end for

		fprintf(text_out, "\n");

	} // end for

	fprintf(text_out, "\n\n");

	////////////////////////////////////////////////////////////////////////
	// write out the utils files
	//
	// Node file format:
	// 1 int for the number of nodes (node_count)
	// 1 vec3d for every node
	//    these are the start and end points for each string of pylons I use
	//    the first and last points of each road
	//
	// Link or wp file format:
	// 1 int for the number of paths (path_count) (path = string of pylons)
	// for each path there is:
	// 1 int for start - pointer into the list of nodes (1st node of path)
	// 1 int for end - pointer into the list of nodes (last node of path)
	// 1 int for type = 48 for power lines
	// 1 int for number of links (link_count) number of links for this path
	//    I use road points 1 - next to last
	// 1 vec3d for every link in this path 
	// 
	// the path has every position between the start and end positions
	////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////
	// write the node file
	////////////////////////////////////////////////////////////////////////

	fprintf(text_out, "*** Power Line Data ***\n\n");
	
	node_count = number_of_roads * 2;

	fprintf(text_out, "There are %d nodes\n\n", node_count);

	fwrite(&node_count, sizeof(int), 1, out_pylon_node_file_ptr);

	for(node_loop = 0; node_loop < number_of_roads; node_loop++){

		number_of_points_for_this_road = roads[node_loop].number_of_center_points;

		// write the 1st road point position

		pos.x = roads[node_loop].center_points[0].global_x;
		pos.z = roads[node_loop].center_points[0].global_z;
		pos.y = roads[node_loop].center_points[0].global_y;

		fwrite(&pos, sizeof(vec3d), 1, out_pylon_node_file_ptr);

		fprintf(text_out, "%d   %f   %f   %f\n", node_loop * 2, pos.x, pos.z, pos.y);

		// write the last road point position

		pos.x = roads[node_loop].center_points[number_of_points_for_this_road - 1].global_x;
		pos.z = roads[node_loop].center_points[number_of_points_for_this_road - 1].global_z;
		pos.y = roads[node_loop].center_points[number_of_points_for_this_road - 1].global_y;

		fwrite(&pos, sizeof(vec3d), 1, out_pylon_node_file_ptr);

		fprintf(text_out, "%d   %f   %f   %f\n", node_loop * 2 + 1, pos.x, pos.z, pos.y);
	} // end for

	fprintf(text_out, "\n\n");

	////////////////////////////////////////////////////////////////////////
	// write the link file
	////////////////////////////////////////////////////////////////////////

	path_count = number_of_roads;

	fwrite(&path_count, sizeof(int), 1, out_pylon_link_file_ptr);

	fprintf(text_out, "*** Link Data *** there are %d paths\n\n", path_count);

	for(path_loop = 0; path_loop < path_count; path_loop++){

		number_of_points_for_this_road = roads[path_loop].number_of_center_points;
	
		link_count = number_of_points_for_this_road - 2;

		start = path_loop * 2;
		end = start + 1;
			
		fwrite(&start, sizeof(int), 1, out_pylon_link_file_ptr);
		fwrite(&end, sizeof(int), 1, out_pylon_link_file_ptr);
		fwrite(&type, sizeof(int), 1, out_pylon_link_file_ptr);
		fwrite(&link_count, sizeof(int), 1, out_pylon_link_file_ptr);

		fprintf(text_out, "*** Path *** %d\n\nstart = %d  end = %d  type = %d  number of links = %d\n\n",
			path_loop, start, end, type, link_count);

		for(link_loop = 0; link_loop < link_count; link_loop++){

			pos.x = roads[path_loop].center_points[link_loop + 1].global_x;
			pos.z = roads[path_loop].center_points[link_loop + 1].global_z;
			pos.y = roads[path_loop].center_points[link_loop + 1].global_y;

			fwrite(&pos, sizeof(vec3d), 1, out_pylon_link_file_ptr);

			fprintf(text_out, "%d    %f  %f  %f   ", link_loop, pos.x, pos.z, pos.y);

			if (link_loop == 0){

				fprintf(text_out, "ENTITY_SUB_TYPE_FIXED_ELECTRICITY_PYLON_START_TERMINATOR\n");
			}
			else if (link_loop == link_count - 1){

				fprintf(text_out, "ENTITY_SUB_TYPE_FIXED_ELECTRICITY_PYLON_END_TERMINATOR\n");
			}
			else {

				fprintf(text_out, "ENTITY_SUB_TYPE_FIXED_ELECTRICITY_PYLON_STANDARD\n");
			}
		} // end for

		fprintf(text_out, "\n");

	} // end for


	fclose (out_pylon_node_file_ptr);
	fclose (out_pylon_link_file_ptr);
	fclose (text_out);

} // end process_power_lines

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


int get_y_value(int sec_x, int sec_z, int x, int z){

	// This function returns the y value in sector coordinates
	// for the sector and x z sector coord pair

	int		i,
			polys_this_sector,
			poly_counter,
			poly_number,
			y_value;

	long long	k, // plane constant
				triangle_points[3][3], // [][0]=x, [][1]=z, [][2]=y
				Ax,Az,Ay,Bx,Bz,By,	// vectors
				Nx,Nz,Ny;			// normal vector

	double	terrain_3d_map_scaled_height_difference;

	simple_point	inside_poly_array[3],
					test_point;

	terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height )
		* TERRAIN_3D_Y_SCALE;


	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	test_point.x = x;
	test_point.z = z;

	// Find which polygon of this sector the point is on
	// by testing for point_in_poly with all polys until it is found

	for(poly_counter = 0; poly_counter < polys_this_sector; poly_counter++){

		poly_number = 1000;

		triangle_points[0][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][0].point].x,
		triangle_points[1][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][1].point].x,
		triangle_points[2][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][2].point].x,
		triangle_points[0][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][0].point].z,
		triangle_points[1][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][1].point].z,
		triangle_points[2][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][2].point].z,
		triangle_points[0][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][0].point].y,
		triangle_points[1][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][1].point].y,
		triangle_points[2][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_counter][2].point].y;

		// assign points to inside_poly_array[]

		for(i = 0; i < 3; i++){
			inside_poly_array[i].x = (double)triangle_points[i][0];
			inside_poly_array[i].z = (double)triangle_points[i][1]; 
		}
/*
		printf("%d  inside_poly array  %f  %f x %f  %f x %f  %f  test_point  %f  %f\n",
			poly_counter,
			inside_poly_array[0].x, inside_poly_array[0].z,
			inside_poly_array[1].x, inside_poly_array[1].z,
			inside_poly_array[2].x, inside_poly_array[2].z,
			test_point.x, test_point.z);
*/
		if(InsidePolygon(inside_poly_array, 3, test_point)){

			poly_number = poly_counter;
			break;
		}

	} // end for poly loop

	if(poly_number == 1000){
		printf("can't find poly for point %d x %d in sector %d x %d\n", x, z, sec_x, sec_z);
		exit(1);
	}

	// vectors
	Ax = triangle_points[0][0] - triangle_points[1][0]; // ax - bx
	Ay = triangle_points[0][2] - triangle_points[1][2]; // ay - by
	Az = triangle_points[0][1] - triangle_points[1][1]; // az - bz
	Bx = triangle_points[2][0] - triangle_points[1][0]; // cx - bx
	By = triangle_points[2][2] - triangle_points[1][2]; // cy - by
	Bz = triangle_points[2][1] - triangle_points[1][1]; // cz - bz

	// normal
	Nx = Ay * Bz - Az * By;
	Ny = Az * Bx - Ax * Bz;
	Nz = Ax * By - Ay * Bx;
				
	// plane constant
	//k = ax * Nx + ay * Ny + az * Nz;
	k = triangle_points[0][0] * Nx + triangle_points[0][2] * Ny + triangle_points[0][1] * Nz;

	y_value = (int)((k - x * Nx - z * Nz) / Ny);

	//printf("%d   %d   %d   %d   %d\n", poly_number, (int)triangle_points[0][2],
	//	(int)triangle_points[1][2], (int)triangle_points[2][2], y_value);


	return y_value;
} // end get_y_value()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(simple_point *polygon,int N,simple_point p)
{
  int counter = 0;
  int i;
  double xinters;
  simple_point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.z > MIN(p1.z,p2.z)) {
      if (p.z <= MAX(p1.z,p2.z)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.z != p2.z) {
            xinters = (p.z-p1.z)*(p2.x-p1.x)/(p2.z-p1.z)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon()


void read_in_util_files(void){

	int
		type,
		node_count,
		*node_link_count,
		path_count,
		node_loop,
		path_loop,
		start,
		end,
		link_count,
		link_loop;

	vec3d
		*node_positions,
		pos;


	FILE
		*pylon_node_file_ptr,
		*pylon_link_file_ptr;


	if(( pylon_node_file_ptr = fopen("utils.nde","rb"))==NULL){
		printf("cannot open utils.nde\n");
		exit(1);
	}
	else printf("utils.nde opened\n");

	if(( pylon_link_file_ptr = fopen("utils.wp","rb"))==NULL){
		printf("cannot open utils.wp\n");
		exit(1);
	}
	else printf("utils.wp opened\n");



	fread (&node_count, sizeof (int), 1, pylon_node_file_ptr);

	printf("node_count = %d\n\n", node_count);

	node_link_count = malloc (sizeof (int) * node_count);

	for (node_loop = 0 ; node_loop < node_count ; node_loop ++){

		fread (&pos, sizeof (vec3d), 1, pylon_node_file_ptr);

		printf("%d      %f  %f  %f\n", node_loop, pos.x, pos.z, pos.y);

		node_link_count [node_loop] = 0;
	} // end for

	printf("\n**** link stuff ****\n");

	fread (&path_count, sizeof (int), 1, pylon_link_file_ptr);

	printf("path_count = %d\n\n", path_count);

	for (path_loop = 0 ; path_loop < path_count ; path_loop ++){

		printf("** Start of path %d **\n", path_loop);

		fread (&start, sizeof (int), 1, pylon_link_file_ptr);

		fread (&end, sizeof (int), 1, pylon_link_file_ptr);

		fread (&type, sizeof (int), 1, pylon_link_file_ptr);

		fread (&link_count, sizeof (int), 1, pylon_link_file_ptr);
		
		node_link_count [start] += 1;
		node_link_count [end] += 1;

		printf("start = %d  end = %d  type = %d  link_count = %d\n\n",
			start, end, type, link_count);

		if (link_count > 0){

			for (link_loop = 0 ; link_loop < link_count ; link_loop ++){

				fread (&pos, sizeof (vec3d), 1, pylon_link_file_ptr);

				printf("%d   %f  %f  %f\n", link_loop, pos.x, pos.z, pos.y);

				if (link_loop == 0)
				{
					printf("ENTITY_SUB_TYPE_FIXED_ELECTRICITY_PYLON_START_TERMINATOR\n");
									}
				else if (link_loop == link_count - 1)
				{
					printf("ENTITY_SUB_TYPE_FIXED_ELECTRICITY_PYLON_END_TERMINATOR\n");
				}
				else 
				{
					printf("ENTITY_SUB_TYPE_FIXED_ELECTRICITY_PYLON_STANDARD\n");
				}
			} // end for
			printf("\n");
		} // end if
	} // end for

} // end read_in_util_files(void)

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void add_intermediate_pylons(void){

	int
		a,
		b,
		c,
		d,
		road_loop,
		point_loop,
		points_this_road,
		temp_points_this_road,
		max_distance_ok,
		while_runaway_check,
		for_runaway_check;

	double
		distance,
		max_distance = 500.0,
		x,
		x1,
		z,
		z1;

	floating_point
		temp_points_1[1024],
		temp_points_2[1024],
		middle_point,
		original_last_point;

	printf("Adding intermediate pylons\n");

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		printf("road loop = %d\n", road_loop);

		while_runaway_check = 0;
		max_distance_ok = 0;

		points_this_road = roads[road_loop].number_of_center_points - 2; // don't worry about 1st and last points

		// copy 2nd through next to last points into temp array

		for(a = 0; a < points_this_road; a++){

			temp_points_1[a].x = (float)roads[road_loop].center_points[a + 1].global_x;
			temp_points_1[a].z = (float)roads[road_loop].center_points[a + 1].global_z;
		} // end for

		// save the original last point

		original_last_point.x = 
			(float)roads[road_loop].center_points[roads[road_loop].
			number_of_center_points - 1].global_x;
		
		original_last_point.z = 
			(float)roads[road_loop].center_points[roads[road_loop].
			number_of_center_points - 1].global_z;

		//////////////////////////////////////////////////////////////////
		// check distance between all points and add inbetween point
		// if distance is too far
		// repeat loop until distance checks okay for all distances
		//////////////////////////////////////////////////////////////////

		max_distance_ok = 0; // not okay

		temp_points_this_road = points_this_road;

		while(!max_distance_ok){

			while_runaway_check ++;

			if(while_runaway_check > 4000){

				printf("while runaway check blew past 4000 while adding inbetween pylons\n");

				exit(1);
			}

			for_runaway_check = 0;

			for(point_loop = 0; point_loop < temp_points_this_road - 2; point_loop++){

				for_runaway_check ++;

				if(for_runaway_check > 1024){

					printf("point_loop blew past 1024 while adding pylons\n");

					exit(1);
				}

				x = temp_points_1[point_loop].x;
				x1 = temp_points_1[point_loop + 1].x;
				z = temp_points_1[point_loop].z;
				z1 = temp_points_1[point_loop + 1].z;
				
				distance = sqrt(((x1 - x) * ((x1 - x))) + ((z1 - z) * (z1 - z)));

				if(distance <= max_distance && (point_loop == (temp_points_this_road - 3))){

					max_distance_ok = 1;

					break;
				} 
				else if(distance <= max_distance){

					continue;
				}
				else{

					//////////////////////////////////////////////////////
					// need to add another point inbetween
					//////////////////////////////////////////////////////

					middle_point.x = (x + x1) / 2.0;
					middle_point.z = (z + z1) / 2.0;

					/////////////////////////////////////////////////////////////
					// copy temp_points_1 into temp_points_2 adding middle point
					/////////////////////////////////////////////////////////////

					for(a = 0; a <= point_loop; a++){

						temp_points_2[a] = temp_points_1[a];
					} // end for

					temp_points_2[a] = middle_point;

					for( ; a < temp_points_this_road; a++){

						temp_points_2[a + 1] = temp_points_1[a];
					} // end for

					temp_points_this_road ++;

					/////////////////////////////////////////////////////////////
					// copy temp_points_2 back into temp_points_1
					// with new middle point and number of points increased by 1
					/////////////////////////////////////////////////////////////

					for(a = 0; a < temp_points_this_road; a++){

						temp_points_1[a] = temp_points_2[a];
					} // end for

				} // end else

				break;

			} // end for
			
		} // end while

		//////////////////////////////////////////////////////////////////////
		// copy the new temp array with added points back to global data
		//////////////////////////////////////////////////////////////////////

		// first point doesn't need to be changed

		for(a = 1, b = 0; a < temp_points_this_road; a++, b++){

			roads[road_loop].center_points[a].global_x = (double)temp_points_1[b].x;
			roads[road_loop].center_points[a].global_z = (double)temp_points_1[b].z;
		} // end for

		roads[road_loop].center_points[a].global_x = (double)temp_points_1[b].x;
		roads[road_loop].center_points[a].global_z = (double)temp_points_1[b].z;

		roads[road_loop].center_points[a + 1].global_x = (double)original_last_point.x;
		roads[road_loop].center_points[a + 1].global_z = (double)original_last_point.z;

		roads[road_loop].number_of_center_points = temp_points_this_road + 2;
	} // end for road_loop

} // end add_intermediate_pylons()

