// Grand Roads
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rob_fox_roads.h"

////////////////////////////////////////
// Globals
////////////////////////////////////////

/////////////////////////////////////////////////
// *** start of Global Lists
/////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Holds up to 256 roads and up to 512 quads per road
/////////////////////////////////////////////////////////////////////////////

road
	roads[256],
	temp_roads[256];

bridge
	bridges[256];



////////////////////////////////////////////////////
// *** end of Global Lists
////////////////////////////////////////////////////

int
	number_of_roads,
	max_road_poly_points,
	min_road_poly_points,
	number_of_bridges,
	global_number_of_appending_points = 0;

int
	appending_points [256][2];  // x = [][0]  z = [][1]

////////////////////////////////////////////
// Original terraform globals
////////////////////////////////////////////

float	terrain_3d_map_minimum_height,
		terrain_3d_map_maximum_height;

double	terrain_3d_map_scaled_height_difference;

/////////////////////////////////////////
// functions
/////////////////////////////////////////

static int solve_array(double *a,				 /* input array of wide by high */
		       double *b,				 /* result array of size high */
		       int wide,				 /* width of array */
		       int high);				 /* height of array */

simple_point	get_line_intersection(simple_point p1,
							  simple_point p2,
							  simple_point p3,
							  simple_point p4);

void read_in_road_file(void);

void build_road_quads(void);

void print_road_quads(void);

void write_lcov_file(void);

int read_in_bridge_file(void);

void modify_road_data_for_bridges(void);

int is_between(vec3d br_start, vec3d p, vec3d br_end);

void print_road_file(void);

void load_scrn_data (void);

void write_new_road_file (void);

int file_exist (char *filename);

int main(int argc, char ** argv)
{
	read_in_road_file();

	load_scrn_data ();

	write_new_road_file ();

	//print_road_file();




/*
	if(read_in_bridge_file()){

		modify_road_data_for_bridges();
	}

	build_road_quads();

	print_road_quads();

	write_lcov_file();
*/
	return 0;


} // end main

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// read in Grand.ROAD file
//
// Data Format
//
// 4 bytes - number of roads
// loop number of roads
//    4 bytes width
//    4 bytes color
//    4 bytes number of points for this road
//    loop number of points
//        x global ( Rob Fox uses ints for floats (int/100 = float))
//        z global

void read_in_road_file(){

	FILE
		*road_fp;

	int
		road_loop,
		point_loop,
		width,
		color,
		number_of_points_for_this_road,
		number_of_quads,
		x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
		z_int_pos,
		return_value;
	
	printf("reading road file\n");

	if(( road_fp = fopen("C:\\aGrand\\Grand.ROAD","rb"))==NULL){
		printf("cannot open Grand.ROAD\n");
		exit(1);
	}
	else printf("Grand.ROAD opened\n");

	//////////////////////////////////////////////////////////////////////
	// read in all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fread ( &number_of_roads, sizeof ( int ), 1, road_fp );

	//printf("number of roads = %d\n\n", number_of_roads);

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		fread ( &width, sizeof ( int ), 1, road_fp );
		fread ( &color, sizeof ( int ), 1, road_fp );
		fread ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );

		roads[road_loop].width = width;

		roads[road_loop].color = color;
		
		roads[road_loop].number_of_center_points = number_of_points_for_this_road;

		roads[road_loop].number_of_quads = number_of_points_for_this_road - 1;

		for(point_loop = 0; point_loop < number_of_points_for_this_road; point_loop++){

			fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			roads[road_loop].center_points[point_loop].global_x = (double)x_int_pos / 100.0;
			roads[road_loop].center_points[point_loop].global_z = (double)z_int_pos / 100.0;
		}
	}
	
	fclose ( road_fp );

} // end read_in_road_file()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// this function solves a system of N variables with N equations using gaussian elimination

static int solve_array(double *a,				 /* input array of wide x high */
		       double *b,				 /* result array of size high */
		       int wide,				 /* width of array */
		       int high)				 /* height of array */
{
  int i, j, k;							 /* loop variables */
  double t;							 /* temporary (scaling) value */
  
  /*
   * Step 1: Make array upper triangular
   */
  for (i = 0; i < high; i++) {					 /* for each row in the array */
    t = m(i,i);							 /* pick the scale factor */
    if (t == 0.0) {						 /* if it is 0.0, the array is singular */
      printf("Singular: Error in the Solve_array() function\n");
      //print_array(a, wide, high);
      return 0;							 /* error here */
    }
    for (j = i; j < wide; j++)					 /* scale the row until diagonal point is 1.0 */
      m(j,i) = m(j,i)/t;					 /* narrow the loop to reduce load since left of i = 0.0 */
    
    for (j = i+1; j < high; j++) {				 /* zero out all lower entries in this column */
      t = m(i, j);						 /* factor to solve by */
      for (k = 0; k < wide; k++)				 /* subtract ith row from jth row t times */
	m(k,j) = m(k,j) - t * m(k,i);				 /* to zero out next column */
    }
  }
  /* 
   * Step 2: Make array into identity matrix by eliminating non-diagonal entries in upper triangle.
   */
  for (i = high - 1; i > 0; i--) {				 /* solve for identity matrix */
    for (j = i - 1; j >= 0; j--) {				 /* take away one column */
      t = m(i,j);						 /* get the factor */
      for (k = 0; k < wide; k++)				 /* subtract the t times the i'th equation from the j'th equation*/
	m(k,j) = m(k,j) - t * m(k,i);
    }
  }
  /*
   * Step 3: Get the last column which contains the solution (since the
   *         array is now the identity matrix).
   */
  for (i = 0; i < high; i++)					 
    b[i] = m(wide-1,i);
  return 1;							 /* return success */
} // end solve_array()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


simple_point get_line_intersection(simple_point p1,
							  simple_point p2,
							  simple_point p3,
							  simple_point p4){

	int		return_value;

	simple_point	result,
					temp;

	/////////////////////////////////////////////////////////////////////
	// arrays are used for input/output to solve array()
	// which uses gausian elimination to solve two
	// simultaneous equations
	/////////////////////////////////////////////////////////////////////

	double input[6];
	double output[2];

	double	x1,
			y1,
			x2,
			y2,
			a,
			b,
			c,
			d,
			e,
			f;

	///////////////////////////////////////////////////////////////////////
	// test to see if both lines are vertical because you can't
	// divide by zero
	///////////////////////////////////////////////////////////////////////

	if((p1.x == p2.x) && (p3.x == p4.x)){

		result.x = 0.0;
		result.z = 0.0;
		return result;
	}

	///////////////////////////////////////////////////////////////////////
	// test to see if lines are parallel
	///////////////////////////////////////////////////////////////////////

	if(((p2.z - p1.z) / (p2.x - p1.x)) == ((p4.z - p3.z) / (p4.x - p3.x))){

		result.x = 0.0;
		result.z = 0.0;
		return result;
	}

	///////////////////////////////////////////////////////////////////////////
	// check for special case if first line is vertical
	// if x2 == x1 slope is infinite and you can't divide by zero
	// if this is the case we convert the other line to the slope-intercept
	// form y = ax + b where b is the y-intercept and a is the slope
	///////////////////////////////////////////////////////////////////////////

	if(p1.x == p2.x){

		a = (p4.z - p3.z) / (p4.x - p3.x); // slope

		////////////////////////////////////////////////////////////////////////
		// now just need to transpose a point of line 2 to the left
		// by p1.x and plug the new values into y = ax + b and solve for b
		// b = y - a*x
		////////////////////////////////////////////////////////////////////////

		temp = p3;
		
		temp.x = temp.x - p1.x;

		b = temp.z - a * temp.x;

		result.x = p1.x;
		result.z = b;

		return result;

	} // end if

	///////////////////////////////////////////////////////////////////////////
	// check for special case if second line is vertical
	///////////////////////////////////////////////////////////////////////////

	if(p3.x == p4.x){

		a = (p2.z - p1.z) / (p2.x - p1.x); // slope

		////////////////////////////////////////////////////////////////////////
		// now just need to transpose a point of line 2 to the left
		// by x1 and plug the new values into y = ax + b and solve for b
		// b = y - a*x
		////////////////////////////////////////////////////////////////////////

		temp.x = p1.x;
		temp.z = p1.z;
	
		temp.x = temp.x - p3.x;

		b = temp.z - a * temp.x;

		result.x = p3.x;
		result.z = b;

		return result;

	} // end if

	///////////////////////////////////////////////////////////////////////////
	// this section converts a line formula of the form
	// y - y1 = ((y2 - y1) / (x2 - x1)) * (x - x1) to the form
	// d * y + e * x = f
	// d, e, and f are the matrix values for the gaussian elimination
	// method of solving two variables with two equations
	///////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////////
	// convert the first line
	///////////////////////////////////////////////////////////////////////////

	x1 = p1.x;
	y1 = p1.z;
	x2 = p2.x;
	y2 = p2.z;


	a = y1;
	b = (y2 - y1) / (x2 - x1); // slope
	c = x1;

	d = 1;
	e = -1 * b;
	f = a + (-1 * b * c);

	input[0] = d;
	input[1] = e;
	input[2] = f;

	//////////////////////////////////////////////////////////////////////////////
	// convert the second line
	//////////////////////////////////////////////////////////////////////////////

	x1 = p3.x;
	y1 = p3.z;
	x2 = p4.x;
	y2 = p4.z;

	a = y1;
	b = (y2 - y1) / (x2 - x1);  // slope
	c = x1;

	d = 1;
	e = -1 * b;
	f = a + (-1 * b * c);

	input[3] = d;
	input[4] = e;
	input[5] = f;

	return_value = solve_array(input, output, 3, 2);

	result.x = output[1];
	result.z = output[0];

	return result;
} // end get_line_intersection()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

//
//             Quad
//
//   S -------------------------- P
//	  |                          |
//	  |       1  <-----  0       |
//	  |                          |
//	  |                          |
//	 R -------------------------- Q
//

void build_road_quads(void){

	int
		road_loop,
		point_loop,
		number_of_points_for_this_road,
		number_of_quads;

	double 
		ax,
		az,
		bx,
		bz;

	double
		theta;

	simple_point
		p1,
		p2,
		p3,
		p4,
		p_intersect;


	////////////////////////////////////////////////////////////////////////
	// do calculations on center points to get four points (quad) for
	// each road section
	////////////////////////////////////////////////////////////////////////

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		number_of_quads = roads[road_loop].number_of_quads;

		//printf("Road %d\n\n", i);

		for(point_loop = 0; point_loop < number_of_quads; point_loop++){

			///////////////////////////////////////////////////////////
			// find the quad points p, q, r, s in world coordinates
			// a and b are the center points for each road section
			///////////////////////////////////////////////////////////

			ax = roads[road_loop].center_points[point_loop].global_x;
			az = roads[road_loop].center_points[point_loop].global_z;
			bx = roads[road_loop].center_points[point_loop + 1].global_x;
			bz = roads[road_loop].center_points[point_loop + 1].global_z;

			theta = atan((ax - bx) / (bz - az));

			if(az <= bz){

				roads[road_loop].quads[point_loop].p.global_x = WIDTH * (cos(theta)) + ax;
				roads[road_loop].quads[point_loop].p.global_z = WIDTH * (sin(theta)) + az;

				roads[road_loop].quads[point_loop].q.global_x = ax - WIDTH * (cos(theta));
				roads[road_loop].quads[point_loop].q.global_z = az - WIDTH * (sin(theta));

				roads[road_loop].quads[point_loop].r.global_x = bx - WIDTH * (cos(theta));
				roads[road_loop].quads[point_loop].r.global_z = bz - WIDTH * (sin(theta));

				roads[road_loop].quads[point_loop].s.global_x = WIDTH * (cos(theta)) + bx;
				roads[road_loop].quads[point_loop].s.global_z = WIDTH * (sin(theta)) + bz;
			}
			else{ // need different calculations for cases 2, 4, 6

				roads[road_loop].quads[point_loop].p.global_x = ax - WIDTH * (cos(theta));
				roads[road_loop].quads[point_loop].p.global_z = az - WIDTH * (sin(theta));

				roads[road_loop].quads[point_loop].q.global_x = WIDTH * (cos(theta)) + ax;
				roads[road_loop].quads[point_loop].q.global_z = WIDTH * (sin(theta)) + az;

				roads[road_loop].quads[point_loop].r.global_x = WIDTH * (cos(theta)) + bx;
				roads[road_loop].quads[point_loop].r.global_z = WIDTH * (sin(theta)) + bz;

				roads[road_loop].quads[point_loop].s.global_x = bx - WIDTH * (cos(theta));
				roads[road_loop].quads[point_loop].s.global_z = bz - WIDTH * (sin(theta));
			}


		} // end for points for this road
	} // end for number of roads

	/////////////////////////////////////////////////////////////////////
	// change all quad points except the end points to
	// the intersections of adjacent road section lines
	/////////////////////////////////////////////////////////////////////

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		number_of_quads = roads[road_loop].number_of_quads;

		for(point_loop = 0; point_loop < number_of_quads - 1; point_loop++){

			// line 1 points p and s

			p1.x = roads[road_loop].quads[point_loop].p.global_x;
			p1.z = roads[road_loop].quads[point_loop].p.global_z;
			p2.x = roads[road_loop].quads[point_loop].s.global_x;
			p2.z = roads[road_loop].quads[point_loop].s.global_z;

			// line 2 points p and s

			p3.x = roads[road_loop].quads[point_loop + 1].p.global_x;
			p3.z = roads[road_loop].quads[point_loop + 1].p.global_z;
			p4.x = roads[road_loop].quads[point_loop + 1].s.global_x;
			p4.z = roads[road_loop].quads[point_loop + 1].s.global_z;

			// intersection of line 1 and line 2

			p_intersect = get_line_intersection(p1, p2, p3, p4);

			//////////////////////////////////////////////////////////////////////////////////
			// check if lines were parallel
			// if they are we don't want to change the intersection points
			// because they are already correct because right angle
			// cross lines meet flush together
			// the returned value of the intersection algorithm is 0 anyway
			//////////////////////////////////////////////////////////////////////////////////

			if(p_intersect.x == 0 && p_intersect.z == 0){
				printf("Road %d Sections %d and %d are parallel\n\n", road_loop, point_loop, point_loop+1);
				//exit (1);
				continue; // leave these cross point alone and go to next loop
			}

			// change quad point s of first section
			// and quad point p of next section

			roads[road_loop].quads[point_loop].s.global_x = p_intersect.x;
			roads[road_loop].quads[point_loop].s.global_z = p_intersect.z;

			roads[road_loop].quads[point_loop + 1].p.global_x = p_intersect.x;
			roads[road_loop].quads[point_loop + 1].p.global_z = p_intersect.z;


			///////////////////////////////////////////////////////////////
			// do the same thing for the other lines of the two quads
			///////////////////////////////////////////////////////////////

			// line 1 points q and r

			p1.x = roads[road_loop].quads[point_loop].q.global_x;
			p1.z = roads[road_loop].quads[point_loop].q.global_z;
			p2.x = roads[road_loop].quads[point_loop].r.global_x;
			p2.z = roads[road_loop].quads[point_loop].r.global_z;

			// line 2 points q and r

			p3.x = roads[road_loop].quads[point_loop + 1].q.global_x;
			p3.z = roads[road_loop].quads[point_loop + 1].q.global_z;
			p4.x = roads[road_loop].quads[point_loop + 1].r.global_x;
			p4.z = roads[road_loop].quads[point_loop + 1].r.global_z;

			// intersection of line 1 and line 2

			p_intersect = get_line_intersection(p1, p2, p3, p4);

			// change quad point r of first section
			// and quad point q of next section

			roads[road_loop].quads[point_loop].r.global_x = p_intersect.x;
			roads[road_loop].quads[point_loop].r.global_z = p_intersect.z;

			roads[road_loop].quads[point_loop + 1].q.global_x = p_intersect.x;
			roads[road_loop].quads[point_loop + 1].q.global_z = p_intersect.z;


		} // end for points for this road
	} // end for number of roads

} // end build_road_quads

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void print_road_quads(void){

	FILE
		*fp;

	int
		road_loop,
		quad_loop,
		number_of_quads;

	if(( fp = fopen("quad_points_text.log","wb"))==NULL){
		printf("cannot open quad_points_text.log\n");
		exit(1);
	}
	else printf("quad_points_text.log opened\n");

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		fprintf(fp, "Road %d\n\n", road_loop);

		number_of_quads = roads[road_loop].number_of_quads;

		for(quad_loop = 0; quad_loop < number_of_quads; quad_loop++){

			fprintf(fp, "quad %d\n\n", quad_loop);

			fprintf(fp, "p = %f  %f\n", 
				roads[road_loop].quads[quad_loop].p.global_x,
				roads[road_loop].quads[quad_loop].p.global_z);

			fprintf(fp, "q = %f  %f\n", 
				roads[road_loop].quads[quad_loop].q.global_x,
				roads[road_loop].quads[quad_loop].q.global_z);

			fprintf(fp, "r = %f  %f\n", 
				roads[road_loop].quads[quad_loop].r.global_x,
				roads[road_loop].quads[quad_loop].r.global_z);

			fprintf(fp, "s = %f  %f\n", 
				roads[road_loop].quads[quad_loop].s.global_x,
				roads[road_loop].quads[quad_loop].s.global_z);

			fprintf(fp, "\n");
		} // end for quad loop
	} // end for road_loop
} // end print_road_quads

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// write map.LCOV file
//
// Data Format
//
// 4 bytes - number of land covers
// loop number of land covers
//    1 byte R
//    1 byte G
//    1 byte B
//    4 bytes number of points for this land cover
//    loop number of points
//        x global ( Rob Fox uses ints for floats (int/100 = float))
//        z global
//
//             Quad
//
//   S -------------------------- P
//	  |                          |
//	  |       1  <-----  0       |
//	  |                          |
//	  |                          |
//	 R -------------------------- Q
//

void write_lcov_file(void){

	FILE
		*fp;

	int
		road_loop,
		quad_loop,
		number_of_quads,
		number_of_land_covers,
		number_of_land_cover_points,
		lcov_pt_count,
		lcov_pt_loop,
		terrain_type = 10,
		value;

	int
		land_cover_points[2048][2];   // [][0] = x  [][1] = z

	color
		road_color;

	if(( fp = fopen("map.LCOV","wb"))==NULL){
		printf("map.LCOV\n");
		exit(1);
	}
	else printf("map.LCOV opened\n");

	road_color.r = 128;
	road_color.b = 64;
	road_color.g = 0;
	road_color.pad = 0;

	number_of_land_covers = number_of_roads;

	fwrite(&number_of_land_covers, 4, 1, fp);

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		number_of_quads = roads[road_loop].number_of_quads;

		number_of_land_cover_points = 2 * number_of_quads + 2;

		fwrite(&terrain_type, 4, 1, fp);

		fwrite(&road_color, 4, 1, fp);

		fwrite(&number_of_land_cover_points, 4, 1, fp);

		lcov_pt_count = 0;

		/////////////////////////////////////////////////////////////
		// assign values of land cover points array
		// need to go in clockwise order
		/////////////////////////////////////////////////////////////

		//////////////////////////////////////
		// do left side first going forward
		/////////////////////////////////////

		for(quad_loop = 0; quad_loop < number_of_quads; quad_loop++){

			if(quad_loop != (number_of_quads - 1)){

				land_cover_points[lcov_pt_count][0] = 
					(int)(roads[road_loop].quads[quad_loop].q.global_x * 100.0);

				land_cover_points[lcov_pt_count][1] = 
					(int)(roads[road_loop].quads[quad_loop].q.global_z * 100.0);

				lcov_pt_count ++;
			}
			else{

				land_cover_points[lcov_pt_count][0] = 
					(int)(roads[road_loop].quads[quad_loop].q.global_x * 100.0);

				land_cover_points[lcov_pt_count][1] = 
					(int)(roads[road_loop].quads[quad_loop].q.global_z * 100.0);

				lcov_pt_count ++;

				land_cover_points[lcov_pt_count][0] = 
					(int)(roads[road_loop].quads[quad_loop].r.global_x * 100.0);

				land_cover_points[lcov_pt_count][1] = 
					(int)(roads[road_loop].quads[quad_loop].r.global_z * 100.0);

				lcov_pt_count ++;
			} // end if else
		} // end quad loop

		//////////////////////////////////////
		// do right side next going backwards
		//////////////////////////////////////

		for(quad_loop = number_of_quads - 1; quad_loop >= 0; quad_loop --){

			if(quad_loop != 0){

				land_cover_points[lcov_pt_count][0] = 
					(int)(roads[road_loop].quads[quad_loop].s.global_x * 100.0);

				land_cover_points[lcov_pt_count][1] = 
					(int)(roads[road_loop].quads[quad_loop].s.global_z * 100.0);

				lcov_pt_count ++;
			}
			else{

				land_cover_points[lcov_pt_count][0] = 
					(int)(roads[road_loop].quads[quad_loop].s.global_x * 100.0);

				land_cover_points[lcov_pt_count][1] = 
					(int)(roads[road_loop].quads[quad_loop].s.global_z * 100.0);

				lcov_pt_count ++;

				land_cover_points[lcov_pt_count][0] = 
					(int)(roads[road_loop].quads[quad_loop].p.global_x * 100.0);

				land_cover_points[lcov_pt_count][1] = 
					(int)(roads[road_loop].quads[quad_loop].p.global_z * 100.0);

				lcov_pt_count ++;
			} // end if else
		} // end quad loop

		/////////////////////////////////////////////////////////////
		// write the points to file for each land cover
		/////////////////////////////////////////////////////////////

		for(lcov_pt_loop = 0; lcov_pt_loop < lcov_pt_count; lcov_pt_loop ++){

			value = land_cover_points[lcov_pt_loop][0];

			fwrite(&value, 4, 1, fp);

			value = land_cover_points[lcov_pt_loop][1];

			fwrite(&value, 4, 1, fp);
		} // end for lcov pt loop

		
	} // end for road loop

} // end write lcov file

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int read_in_bridge_file(void){

	int
		type,
		local_number_of_bridges,
		start_node,
		end_node,
		loop,
		span_length;

	float
		max_leg_height;

	vec3d
		end1,
		end2;


	FILE
		*fp;


	if(( fp = fopen("bridge.dat","rb"))==NULL){
		printf("cannot open bridge.dat\n");
		return 0;
	}
	else printf("bridge.dat opened\n");

	fread (&local_number_of_bridges, sizeof (int), 1, fp);

	for (loop = 0 ; loop < local_number_of_bridges ; loop ++){

		fread (&end1, sizeof (vec3d), 1, fp);

		bridges[loop].end1.x = end1.x;
		bridges[loop].end1.z = end1.z;

		fread (&end2, sizeof (vec3d), 1, fp);

		bridges[loop].end2.x = end2.x;
		bridges[loop].end2.z = end2.z;

		fread (&start_node, sizeof (int), 1, fp);

		fread (&end_node, sizeof (int), 1, fp);

		fread (&type, sizeof (int), 1, fp);

		fread (&max_leg_height, sizeof (float), 1, fp);

		//span_length = get_span_length(end1, end2);

		//if(span_length > longest_span_length){

		//	longest_span_length = span_length;
		//} // end if

	} // end for bridge loop

	close (fp);

	// set global value of number of bridges

	number_of_bridges = local_number_of_bridges;

	printf("Bridge data read in\n");

	return 1;

} // end read_in_bridge_file

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

/*
	This function cuts the bridge area out of the land covers
	each bridge will divide a single land cover into two
	a land cover is equivalent to a road for this program
*/

void modify_road_data_for_bridges(void){

	int
		bridge_loop,
		road_loop,
		point_loop,
		points_for_this_road,
		temp_points_count,
		temp_roads_count;

	vec3d
		road_this_test_point,
		road_next_test_point;

	printf("Original road count = %d\n", number_of_roads);

	
	temp_roads_count = -1;

	for(road_loop = 0; road_loop < number_of_roads; road_loop ++){

		temp_roads_count ++;

		points_for_this_road = roads[road_loop].number_of_center_points;

		temp_points_count = 0;


		for(point_loop = 0; point_loop < points_for_this_road; point_loop ++){

			// copy this point to temp_roads

			temp_roads[temp_roads_count].center_points[temp_points_count].global_x =
				roads[road_loop].center_points[point_loop].global_x;

			temp_roads[temp_roads_count].center_points[temp_points_count].global_z =
				roads[road_loop].center_points[point_loop].global_z;

			temp_points_count ++;

			if(point_loop == points_for_this_road - 1){

				temp_roads[temp_roads_count].number_of_center_points = temp_points_count;

				temp_points_count = 0;

				break;

			}
			else{ // setup for testing for bridge in next loop

				road_this_test_point.x = roads[road_loop].center_points[point_loop].global_x;
				road_this_test_point.z = roads[road_loop].center_points[point_loop].global_z;

				road_next_test_point.x = roads[road_loop].center_points[point_loop + 1].global_x;
				road_next_test_point.z = roads[road_loop].center_points[point_loop + 1].global_z;
			}

			
			for(bridge_loop = 0; bridge_loop < number_of_bridges; bridge_loop ++){

				if(point_loop == points_for_this_road - 1){

					// this is the last point for this road

					//temp_roads[temp_roads_count].number_of_center_points = temp_points_count + 1;

					//temp_points_count = 0;
				}
				else if(is_between(road_this_test_point,
					bridges[bridge_loop].end1,
					road_next_test_point)){

					// this will be the new last point for this road

					temp_roads[temp_roads_count].number_of_center_points = temp_points_count + 1;

					temp_roads[temp_roads_count].center_points[temp_points_count].global_x =
						(double)bridges[bridge_loop].end1.x;

					temp_roads[temp_roads_count].center_points[temp_points_count].global_z =
						(double)bridges[bridge_loop].end1.z;

					// start new road

					temp_roads_count ++;

					temp_points_count = 0;

					temp_roads[temp_roads_count].center_points[temp_points_count].global_x =
						(double)bridges[bridge_loop].end2.x;

					temp_roads[temp_roads_count].center_points[temp_points_count].global_z =
						(double)bridges[bridge_loop].end2.z;

					temp_points_count ++;
				}
				else{

					// do nothing this bridge isn't between these two points
				}
					
			} // end for bridge loop
		} // end for point loop
	} // end for road loop

	////////////////////////////////////////////////////////////////////////////////	
	// print original road data
	////////////////////////////////////////////////////////////////////////////////

	for(road_loop = 0; road_loop < number_of_roads; road_loop ++){

		printf("Road %d\n\n", road_loop);

		points_for_this_road = roads[road_loop].number_of_center_points;

		for(point_loop = 0; point_loop < points_for_this_road; point_loop ++){

			printf("%d  %f  %f\n", point_loop, 
				roads[road_loop].center_points[point_loop].global_x,
				roads[road_loop].center_points[point_loop].global_z);

		} // end for point loop
		printf("\n");
	} // end for road loop

	////////////////////////////////////////////////////////////////////////////////
	// print bridge data
	////////////////////////////////////////////////////////////////////////////////

	for(bridge_loop = 0; bridge_loop < number_of_bridges; bridge_loop ++){

		printf("Bridge %d\n\n", bridge_loop);

		printf("end1 = %f  %f\n", bridges[bridge_loop].end1.x, bridges[bridge_loop].end1.z);

		printf("end2 = %f  %f\n", bridges[bridge_loop].end2.x, bridges[bridge_loop].end2.z);

		printf("\n");
	} // end for bridge loop

	/////////////////////////////////////////////////////////////////////////////////
	// print new road data
	/////////////////////////////////////////////////////////////////////////////////

	for(road_loop = 0; road_loop < temp_roads_count + 1; road_loop ++){

		printf("Road %d\n\n", road_loop);

		points_for_this_road = temp_roads[road_loop].number_of_center_points;

		for(point_loop = 0; point_loop < points_for_this_road; point_loop ++){

			printf("%d  %f  %f\n", point_loop, 
				temp_roads[road_loop].center_points[point_loop].global_x,
				temp_roads[road_loop].center_points[point_loop].global_z);

		} // end for point loop
		printf("\n");
	} // end for road loop

	////////////////////////////////////////////////////////////////////////////////
	// copy new road data back to the primary road data
	////////////////////////////////////////////////////////////////////////////////

	printf("The new road count is %d\n", temp_roads_count + 1);

	number_of_roads = temp_roads_count + 1;

	for(road_loop = 0; road_loop < number_of_roads; road_loop ++){

		roads[road_loop].number_of_center_points = temp_roads[road_loop].number_of_center_points;

		roads[road_loop].number_of_quads = temp_roads[road_loop].number_of_center_points - 1;

		points_for_this_road = temp_roads[road_loop].number_of_center_points;

		for(point_loop = 0; point_loop < points_for_this_road; point_loop ++){

			roads[road_loop].center_points[point_loop].global_x = 
				temp_roads[road_loop].center_points[point_loop].global_x;

			roads[road_loop].center_points[point_loop].global_z = 
				temp_roads[road_loop].center_points[point_loop].global_z;

		} // end for point loop
	} // end for road loop

} // end modify_road_data_for_bridges

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int is_between(vec3d br_start, vec3d p, vec3d br_end){

	float
		largest_x,
		smallest_x,
		largest_z,
		smallest_z;

	//printf("is_between has been called\n");

	if(br_start.x >= br_end.x){

		largest_x = br_start.x;
		smallest_x = br_end.x;
	}
	else{

		largest_x = br_end.x;
		smallest_x = br_start.x;
	}

	if(br_start.z >= br_end.z){

		largest_z = br_start.z;
		smallest_z = br_end.z;
	}
	else{

		largest_z = br_end.z;
		smallest_z = br_start.z;
	}

	if(smallest_x <= p.x &&
		p.x <= largest_x &&
		smallest_z <= p.z &&
		p.z <= largest_z){

		return 1;
	}
	else{

		return 0;
	}
} // end is_between

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void print_road_file(void){

	FILE
		*fp;

	int
		road_loop,
		point_loop,
		width,
		color,
		number_of_points_for_this_road,
		number_of_quads,
		x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
		z_int_pos,
		return_value;

	float
		x_value,
		z_value;
	
	printf("printing road file\n");

	if(( fp = fopen("ROAD.log","wb"))==NULL){
		printf("cannot open ROAD.log\n");
		exit(1);
	}
	else printf("ROAD.log opened\n");

	//////////////////////////////////////////////////////////////////////
	// write to file all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fprintf ( fp, "%d roads\n\n", number_of_roads );

	//printf("number of roads = %d\n\n", number_of_roads);

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		fprintf ( fp, "road %d has %d points\n\n", road_loop,
			roads[road_loop].number_of_center_points );

		//fread ( &width, sizeof ( int ), 1, road_fp );
		//fread ( &color, sizeof ( int ), 1, road_fp );
		//fread ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );

		//roads[road_loop].width = width;

		//roads[road_loop].color = color;
		
		//roads[road_loop].number_of_center_points = number_of_points_for_this_road;

		//roads[road_loop].number_of_quads = number_of_points_for_this_road - 1;

		number_of_points_for_this_road = roads[road_loop].number_of_center_points;

		for(point_loop = 0; point_loop < number_of_points_for_this_road; point_loop++){

			x_value = (float)roads[road_loop].center_points[point_loop].global_x;

			z_value = (float)roads[road_loop].center_points[point_loop].global_z;

			fprintf ( fp,"  %f   %f \n", x_value, z_value);


			//fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			//fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			//roads[road_loop].center_points[point_loop].global_x = (double)x_int_pos / 100.0;
			//roads[road_loop].center_points[point_loop].global_z = (double)z_int_pos / 100.0;
		}

		fprintf(fp, "\n");
	}
	
	fclose ( fp );
} // end print road file

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void load_scrn_data (void){

	
	FILE
		*in_fp;
		//*out_fp;

	int
		x_int,
		z_int,
		y_int,
		sec_x,
		sec_z,
		count = 0,
		file_number = 0,
		point_count = 0;

	float
		x_float,
		z_float;

	char
		ch,
		x_string [16],
		y_string [16],
		z_string [16],
		sec_x_string [8],
		sec_z_string [8],
		filename [64];

	
/*
	if(( out_fp = fopen("points.log","w"))==NULL){
		printf("cannot open points.log\n");
		exit(1);
	}
	else printf("points.log opened\n");
*/
	while( 1 ){

		sprintf(filename, "C:\\Razorworks\\cohokum\\scrnshot\\IMAGE%03d.TXT", file_number); 

		if( file_exist ( filename ) ){

			if(( in_fp = fopen(filename,"r"))==NULL){
				printf("cannot open %s\n", filename);
				exit(1);
			}
			else printf("%s opened\n", filename);

			fseek( in_fp, 66, SEEK_CUR );

			//
			// get the x value string
			//

			count = 0;

			do{

				ch = getc( in_fp );

				x_string[ count ] = ch;

				count ++;

			} while ( ch != '\n');

			x_string[ count ] = NULL;

			//printf("%s\n", x_string);

			//
			// get the y value string
			//

			fseek( in_fp, 16, SEEK_CUR );

			count = 0;

			do{

				ch = getc( in_fp );

				y_string[ count ] = ch;

				count ++;

			} while ( ch != '\n');

			y_string[ count ] = NULL;

			//printf("%s\n", y_string);

			//
			// get the z value string
			//

			fseek( in_fp, 16, SEEK_CUR );

			count = 0;

			do{

				ch = getc( in_fp );

				z_string[ count ] = ch;

				count ++;

			} while ( ch != '\n');

			z_string[ count ] = NULL;

			//printf("%s\n", z_string);

			//
			// get the sec_x_string
			//

			fseek( in_fp, 16, SEEK_CUR );

			count = 0;

			do{

				ch = getc( in_fp );

				sec_x_string[ count ] = ch;

				count ++;

			} while ( ch != '\n');

			sec_x_string[ count ] = NULL;

			//
			// get the sec_z_string
			//

			fseek( in_fp, 16, SEEK_CUR );

			count = 0;

			do{

				ch = getc( in_fp );

				sec_z_string[ count ] = ch;

				count ++;

			} while ( ch != '\n');

			sec_z_string[ count ] = NULL;




			x_int = (int) atof (x_string);

			z_int = (int) atof (z_string);

			y_int = (int) atof (y_string);

			sec_x = (int) atof (sec_x_string);

			sec_z = (int) atof (sec_z_string);

			appending_points [ point_count ][ 0 ] = x_int;

			appending_points [ point_count ][ 1 ] = z_int;

			point_count ++;


			//fprintf(out_fp, "%d - %d  %d ( %d )  %d - %d x %d\n", file_number,
			//	x_int, z_int, 126976 - z_int, y_int, sec_x, sec_z); 



			
			
		} // end if file_exist
		else{

			break;
		}

		fclose ( in_fp );

		file_number ++;
		
	} // end while

	global_number_of_appending_points = point_count;

	//fclose (out_fp);


} // end load scrn data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void write_new_road_file (void){

		FILE
		*road_fp;

	int
		road_loop,
		point_loop,
		width,
		color,
		number_of_points_for_this_road,
		number_of_quads,
		x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
		z_int_pos,
		return_value,
		local_number_of_roads;
	
	printf("writing road file\n");

	if(( road_fp = fopen("Grand.ROAD","wb"))==NULL){
		printf("cannot open Grand.ROAD\n");
		exit(1);
	}
	else printf("Grand.ROAD opened\n");

	//////////////////////////////////////////////////////////////////////
	// write all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	local_number_of_roads = number_of_roads + 1;

	fwrite ( &local_number_of_roads, sizeof ( int ), 1, road_fp );

	//printf("number of roads = %d\n\n", number_of_roads);

	for(road_loop = 0; road_loop < number_of_roads; road_loop++){

		width = roads[road_loop].width;

		color = roads[road_loop].color;

		number_of_points_for_this_road = roads[road_loop].number_of_center_points;

		fwrite ( &width, sizeof ( int ), 1, road_fp );
		fwrite ( &color, sizeof ( int ), 1, road_fp );
		fwrite ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );

		for(point_loop = 0; point_loop < number_of_points_for_this_road; point_loop++){

			x_int_pos = (int) roads[road_loop].center_points[point_loop].global_x * 100.0;

			z_int_pos = (int) roads[road_loop].center_points[point_loop].global_z * 100.0;

			fwrite ( &x_int_pos, sizeof ( int ), 1, road_fp );
			fwrite ( &z_int_pos, sizeof ( int ), 1, road_fp );

		}

	}

	////////////////////////////////////////////////////////////////////
	// add the appending road
	////////////////////////////////////////////////////////////////////

	fwrite ( &width, sizeof ( int ), 1, road_fp );
	fwrite ( &color, sizeof ( int ), 1, road_fp );
	fwrite ( &global_number_of_appending_points, sizeof ( int ), 1, road_fp );

	for(point_loop = 0; point_loop < global_number_of_appending_points; point_loop++){

		x_int_pos = appending_points [ point_loop ][ 0 ] * 100.0;

		z_int_pos = appending_points [ point_loop ][ 1 ] * 100.0;

		fwrite ( &x_int_pos, sizeof ( int ), 1, road_fp );
		fwrite ( &z_int_pos, sizeof ( int ), 1, road_fp );

	}


	fclose ( road_fp );

} // end write new road file

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

int file_exist (char *filename)
{
	char fn[32];

	FILE
		*file_ptr;

	if (file_ptr = fopen (filename, "r"))
	{

		fclose (file_ptr);

		return TRUE;
	}

	return FALSE;
}

