// Puerto Rico

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include "tree_plant.h"

////////////////////////////////////////
// Globals
////////////////////////////////////////

// holds geometry data terrain_sectors[z][x]
terrain_3d_sector	terrain_sectors[MAP_HEIGHT][MAP_WIDTH];

float	terrain_3d_map_minimum_height,
		terrain_3d_map_maximum_height;

double
	global_lowest_elevation,
	global_highest_elevation;

int
	global_min_slope,
	global_max_slope,
	global_max_polys = 0;

/////////////////////////////////////////
// functions
/////////////////////////////////////////

double normal_angle_with_y(int sec_z, int sec_x, int poly);
int normal_angle_with_x(int sec_z, int sec_x, int poly);
int normal_angle_with_z(int sec_z, int sec_x, int poly);
int facing_direction(int sec_z, int sec_x, int poly);
int InsidePolygon(Point *polygon,int N,Point p);
int edge_test_ok(long long x, long long z, long long min_space, long long poly[][3], int num_points);
void create_tree_files2(int max_z, int max_x);
void get_terrain_info(int sec_z, int sec_x);

void exit_function(void);

int main(int argc, char ** argv)
{
	FILE				*sec_in,
						*sec_out,
						*ffp_in,
						*ffp_out,
						*fp,
						*rgb_in,
						*rgb_out;
	int					terrain_version,
						x,
						z,
						offset,
						next_offset,
						rgb_4byte_header,
						ffp_4byte_header,
						four_byte_zero_word,
						i,
						j;
	float				min_height,
						max_height,
						terrain_3d_map_scaled_height_difference;
	unsigned short int	map_width,
						map_height,
						sec_number_of_points,
						sec_number_of_polys,
						sec_radius,
						normal_changes_word;
	short int			sec_min_height,
						sec_max_height,
						sec_avg_height,
						pad_word_zero,
						pad_word_1536;
	unsigned char		pad_byte_zero = 0,
						junk_byte_zero;
	long				long_offset;

	char
		input;



	////////////////////////////////////////////////////////////////////////////
	// initialize variables
	////////////////////////////////////////////////////////////////////////////

	printf("\n\nThis program will overwrite tree_pos.dat and tree_pos.sec!\n");
	printf("Do You Know What You Are Doing? y/n ");

	scanf("%c", &input);

	if(input == 'y'){
	}
	else if (input == 'n'){

		printf("\nAborting program!\n\nPlease come back when you know what you are doing :D\n");

		fflush( stdin );

		exit_function();
	}
	else{

		printf("\nSorry I did not understand your input!\n\n");
		printf("I need a small case y or n to know for sure whether to continue\n");
		printf("Please try again.\n");

		fflush( stdin );

		exit_function();
	}

	

	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("ERROR: cannot open default.sec\n");
		exit_function();
	}
	else printf("default.sec opened\n");

	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("ERROR: cannot open terrain.ffp\n");
		fclose(sec_in);
		fclose(sec_out);
		exit_function();
	}
	else printf("terrain.ffp opened\n");

	if(( rgb_in = fopen("default.rgb","rb"))==NULL){
		printf("ERROR: cannot open default.rgb\n");
		exit_function();
	}
	else printf("default.rgb opened\n");

	printf("Reading in terrain files ...\n");

	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);
/*
	if(map_width != 150 && map_height != 68){

		printf("\nERROR: map dimensions do not equal Puerto Rico dimensions!\n");

		exit_function();
	}
*/	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);


	for( z=0 ; z < map_height ; z++){
		for( x=0 ; x < map_width ; x++){

			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[z][x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);
		
			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[z][x].pad_byte, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			//if(terrain_sectors[z][x].number_of_points > 289){
			//	printf("Sector %d x %d  points = %d  polys = %d\n", z, x, terrain_sectors[z][x].number_of_points, terrain_sectors[z][x].number_of_polygons);
			//}

			fread(&terrain_sectors[z][x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[z][x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[z][x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[z][x].number_of_points, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[z][x].number_of_points, ffp_in);

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(terrain_sectors[z][x].number_of_points <= 256){

				fread(terrain_sectors[z][x].point_byte_references,
					(sizeof(terrain_3d_point_byte_reference) * 3),
					terrain_sectors[z][x].number_of_polygons, ffp_in);
			}
			else{

				fread(terrain_sectors[z][x].point_word_references,
					(sizeof(terrain_3d_point_word_reference) * 3),
					terrain_sectors[z][x].number_of_polygons, ffp_in);
			}

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(terrain_sectors[z][x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[z][x].number_of_polygons, ffp_in);

			if(terrain_sectors[z][x].number_of_polygons % 2 != 0){
				fread(&junk_byte_zero, 1, 1, ffp_in);
			}

			fread(&offset, sizeof(int), 1, sec_in); // sector file
			fread(&next_offset, sizeof(int), 1, sec_in); // sector file

			// this map has variable number of surface ids

			fread(terrain_sectors[z][x].surface_ids,
				sizeof(terrain_3d_surface),	(next_offset - offset), ffp_in);

			// find how many surface ids

			//printf("%d\n", next_offset - offset - 1);

			if(terrain_sectors[z][x].surface_ids[next_offset - offset - 1].surface_id == 0){
				terrain_sectors[z][x].number_of_surface_ids = next_offset - offset - 1;
			}
			else{
				terrain_sectors[z][x].number_of_surface_ids = next_offset - offset;
			}

/*			printf("Sector %d X %d surface ids = %d\n", z, x, terrain_sectors[z][x].number_of_surface_ids);

			for(i=0; i < terrain_sectors[z][x].number_of_surface_ids; i++){
				printf("%d ", terrain_sectors[z][x].surface_ids[i].surface_id);
			}
			printf("\n");
*/  


			//fread(&offset, sizeof(int), 1, sec_in); // sector file

			fread(&terrain_sectors[z][x].normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// read in color offsets from sector file

			fread(&terrain_sectors[z][x].colour_indices_offset,
				sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[z][x].point_colours_offset,
				sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////

			fread(terrain_sectors[z][x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[z][x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[z][x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[z][x].number_of_points; i++){ // there's one colour index per point

				if(terrain_sectors[z][x].colour_indices[i].index > terrain_sectors[z][x].max_colour_index){

					terrain_sectors[z][x].max_colour_index = terrain_sectors[z][x].colour_indices[i].index;
				}
			} // end for

			// read in the colours

			fread(terrain_sectors[z][x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[z][x].max_colour_index + 1, rgb_in);

			// assign the surface_id_each_face values

			terrain_sectors[z][x].surface_id_each_face[0].surface_id = terrain_sectors[z][x].surface_ids[0].surface_id;

			for(i = 1, j = 0; i < terrain_sectors[z][x].number_of_polygons; i++){
				if(terrain_sectors[z][x].polygons[i].surface_change){
					j++;
					terrain_sectors[z][x].surface_id_each_face[i].surface_id = 
						terrain_sectors[z][x].surface_ids[j].surface_id;
				}
				else{
					terrain_sectors[z][x].surface_id_each_face[i].surface_id = 
						terrain_sectors[z][x].surface_ids[j].surface_id;
				}
			} // end for

		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(sec_in);
	fclose(ffp_in);
	fclose(rgb_in);

	printf("Getting terrain information ...\n");

	for(z = 0; z < map_height; z++){
		for(x = 0; x < map_width; x++){

			get_terrain_info(z, x);
			
		}
	}

	printf("Planting trees ...\n");
	

	create_tree_files2(map_height, map_width);

	//printf("\nCheck your TREE_PLANT.log for everything you never wanted\n");
	//printf("     to know about the trees on your map\n\n");

	//printf("\nelevation %d - %d\n", (int)global_lowest_elevation, (int)global_highest_elevation);
	//printf("slope      %d - %d\n", global_min_slope, global_max_slope);
	//printf("max polys per sector = %d\n", global_max_polys);
	printf("\nFINISHED !\n");

	printf("\nCheck your TREE_PLANT.log for all the \"dirty\" details\n");

	printf("\nThank you for testing the tree plant program\n");

	printf("\nI hope it works :D\n");

	exit_function();

	return 0;

} // end main

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void exit_function(void){

	char
		input;

	printf("\nPress Enter to exit ");

	fflush( stdin );

	scanf("%c", &input);

	//input = getch();

	//if(input == 'x'){

		exit(1);
	//}

} // end exit function

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

double normal_angle_with_y(int sec_z, int sec_x, int poly){

	// returns the angle between the poly normal and the Y axis (bigger = steeper)

	// points

	double ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	double abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	//int angle;
	double	radian_angle,
			pi = 3.1415926535;

	double	normalized_ny,
			angle;

	if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

		ax = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].x;
		bx = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].x;
		cx = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].x;
		az = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].z;
		bz = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].z;
		cz = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].z;
		ay = (double)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].y;
		by = (double)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].y;
		cy = (double)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].y;
	}
	else{

		ax = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].x;
		bx = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].x;
		cx = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].x;
		az = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].z;
		bz = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].z;
		cz = (double)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].z;
		ay = (double)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].y;
		by = (double)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].y;
		cy = (double)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].y;
	}

	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_ny = ny / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_ny *= -1.0;
	
	radian_angle = acos(normalized_ny); // normalized_ny / 1 = arcos of angle with y axis
	
	angle = (180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_y()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int normal_angle_with_x(int sec_z, int sec_x, int poly){

	// returns the angle between the normal and the x axis

	// points

	int ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	int abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	int angle;
	double	radian_angle,
			pi = 3.1415926535;

	float normalized_nx;

	if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

		ax = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].x;
		bx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].x;
		cx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].x;
		az = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].z;
		bz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].z;
		cz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].z;
		ay = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].y;
		by = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].y;
		cy = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].y;
	}
	else{

		ax = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].x;
		bx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].x;
		cx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].x;
		az = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].z;
		bz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].z;
		cz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].z;
		ay = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].y;
		by = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].y;
		cy = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].y;
	}

	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_nx = nx / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_nx *= -1.0;
	
	radian_angle = acos(normalized_nx); // normalized_ny / 1 = arcos of angle with y axis
	
	angle = (int)(180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_x()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int normal_angle_with_z(int sec_z, int sec_x, int poly){

	// returns the angle between the normal and the z axis

	// points

	int ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	int abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	int angle;
	double	radian_angle,
			pi = 3.1415926535;

	float normalized_nz;

	if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

		ax = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].x;
		bx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].x;
		cx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].x;
		az = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].z;
		bz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].z;
		cz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].z;
		ay = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][0].point].y;
		by = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][1].point].y;
		cy = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly][2].point].y;
	}
	else{

		ax = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].x;
		bx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].x;
		cx = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].x;
		az = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].z;
		bz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].z;
		cz = (int)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].z;
		ay = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][0].point].y;
		by = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][1].point].y;
		cy = (int)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly][2].point].y;
	}

	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_nz = nz / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_nz *= -1.0;
	
	radian_angle = acos(normalized_nz); // normalized_nz / 1 = arcos of angle with z axis
	
	angle = (int)(180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_z()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(Point *polygon,int N,Point p)
{
  int counter = 0;
  int i;
  double xinters;
  Point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.y > MIN(p1.y,p2.y)) {
      if (p.y <= MAX(p1.y,p2.y)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.y != p2.y) {
            xinters = (p.y-p1.y)*(p2.x-p1.x)/(p2.y-p1.y)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

// function returns 1 if point is at least min_space/2 from all edges of n-sided polygon
// otherwise returns 0

int edge_test_ok(long long x, long long z, long long min_space, long long poly[][3], int num_points){

	int	distance_ok,
		i;

	long long	a,
				b,
				c,
				d,
				distance;

	distance_ok = 1;

	for(i = 0; i < num_points; i++){

		a = poly[i][0];
		b = poly[i][1];
		c = poly[(i+1) % num_points][0];
		d = poly[(i+1) % num_points][1];

		distance = (long long)(labs((long)((x - a) * (d - b) - (z - b) * (c - a))) /
			(long)sqrt((double)((c - a) * (c - a) + (d - b) * (d - b))));

		if(distance < (min_space / 2)) distance_ok = 0;
	} // end for

	return distance_ok;

} // end edge_test

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void create_tree_files2(int max_z, int max_x){

	FILE	*tree_sec,
			*tree_data,
			*fp;

	int		offset_counter_next = 0,
			offset_counter_last = 0,
			sec_x,
			sec_z,
			i,
			j,
			while_runaway_check,
			tempPoint,
			points_this_poly,
			point_ref_counter,
			poly_loop,
			polys_this_sector,
			trees_this_sector,
			trees_this_poly,
			x,
			z,
			min_y,
			max_y,
			average_y,
			slope,
			x_bound_max,
			x_bound_min,
			z_bound_max,
			z_bound_min,
			tree_index,
			space_ok,
			num_trees,
			face_orientation,
			locality_count,
			dissolve_ratio_factor,
			dissolve_trees_this_sector,
			solid_trees_this_sector,
			highest_tree_count,
			highest_x_sector,
			highest_z_sector,
			dissolve_ratio_counter,
			elevation_and_slope_okay,
			number_of_sectors_with_max_trees;

	//unsigned long long
	unsigned int	
			total_tree_count = 0;
			
	long long	k, // plane constant			
				triangle_points[3][3], // [][0]=x, [][1]=z, [][2]=y
				px,pz,	// randon point
				Ax,Az,Ay,Bx,Bz,By, // vectors
				Nx,Nz,Ny, // normal vector
				min_space = 400,
				density_factor = 10, // bigger = less trees
				space,
				tree_points[512][3]; // [][0]=x, [][1]=z, [][2]=y
			

	float	terrain_3d_map_scaled_height_difference,
			elevation;
	
	terrain_3d_tree_data	local_tree_data;

	Point	inside_poly_array[3],
			random_point;

	////////////////////////////////////////////////////
	// end of variables
	////////////////////////////////////////////////////

	number_of_sectors_with_max_trees = 0;

	dissolve_ratio_factor = DISSOLVE_TO_SOLID_RATIO + 1;

	highest_tree_count = 0;

	dissolve_ratio_counter = 0;

	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;

	local_tree_data.colour = 6;
	local_tree_data.height = 35;
	local_tree_data.type = 1;

	//////////////////////////////////////////////////////////////////////////////////
	// my output files
	//////////////////////////////////////////////////////////////////////////////////
/*
	if(( tree_sec = fopen("C:\\Razorworks\\common\\maps\\map15\\terrain\\tree_pos.sec","wb"))==NULL){
		printf("cannot open C:\\Razorworks\\common\\maps\\map15\\terrain\\tree_pos.sec\n");
		return;
	}
	else printf("C:\\Razorworks\\common\\maps\\map15\\terrain\\tree_pos.sec opened\n");

	if(( tree_data = fopen("C:\\Razorworks\\common\\maps\\map15\\terrain\\tree_pos.dat","wb"))==NULL){
		fclose(tree_sec);
		printf("cannot open C:\\Razorworks\\common\\maps\\map15\\terrain\\tree_pos.dat\n");
		return;
	}
	else printf("C:\\Razorworks\\common\\maps\\map15\\terrain\\tree_pos.dat opened\n");
*/
	/////////////////////////////////////////////////////////////////////////////////
	// team output files
	/////////////////////////////////////////////////////////////////////////////////

	if(( tree_sec = fopen("tree_pos.sec","wb"))==NULL){
		printf("cannot open tree_pos.sec\n");
		return;
	}
	else printf("tree_pos.sec opened\n");

	if(( tree_data = fopen("tree_pos.dat","wb"))==NULL){
		fclose(tree_sec);
		printf("cannot open tree_pos.dat\n");
		return;
	}
	else printf("tree_pos.dat opened\n");



	if(( fp = fopen("TREE_PLANT.log","w"))==NULL){
		printf("cannot open TREE_PLANT.log\n");
		return;
	}
	else printf("TREE_PLANT.log opened\n");

	//
	// write map size to tree sector file
	//

	fwrite(&max_x, sizeof(int), 1, tree_sec);
	fwrite(&max_z, sizeof(int), 1, tree_sec);



	fprintf(fp, "%-34s%-10d\n", "CHANCE_OF_GROVE", CHANCE_OF_GROVE);
	fprintf(fp, "%-34s%-10d\n", "NUMBER_OF_POLYS_PER_GROVE", NUMBER_OF_POLYS_PER_GROVE);
	fprintf(fp, "%-34s%-10d\n", "MAX_GROVE_TREES_PER_POLY", MAX_GROVE_TREES_PER_POLY);
	fprintf(fp, "%-34s%-10d\n", "CHANCE_OF_SINGLES", CHANCE_OF_SINGLES);
	fprintf(fp, "%-34s%-10d\n", "NUMBER_OF_SINGLES_PER_POLY", NUMBER_OF_SINGLES_PER_POLY);
	fprintf(fp, "%-34s%-10d\n", "CHANCE_OF_SHORE_SINGLES", CHANCE_OF_SHORE_SINGLES);
	fprintf(fp, "%-34s%-10d\n", "NUMBER_OF_SHORE_SINGLES_PER_POLY", NUMBER_OF_SHORE_SINGLES_PER_POLY);
	fprintf(fp, "%-34s%-10d\n", "MAX_ELEVATION", MAX_ELEVATION);
	fprintf(fp, "%-34s%-10d\n", "MIN_ELEVATION", MIN_ELEVATION);
	fprintf(fp, "%-34s%-10d\n", "MAX_SLOPE", MAX_SLOPE);
	fprintf(fp, "%-34s%-10d\n", "MIN_SLOPE", MIN_SLOPE);
	fprintf(fp, "%-34s%-10d\n", "MAX_SHORE_ELEVATION", MAX_SHORE_ELEVATION);
	fprintf(fp, "%-34s%-10d\n", "MAX_SHORE_SLOPE", MAX_SHORE_SLOPE);
	fprintf(fp, "%-34s%-10d\n", "MIN_SHORE_SLOPE", MIN_SHORE_SLOPE);
	fprintf(fp, "%-34s%-10d\n", "DISSOLVE_TO_SOLID_RATIO", DISSOLVE_TO_SOLID_RATIO);
	fprintf(fp, "%-34s%-10d\n", "MAX_TREES_PER_SECTOR", MAX_TREES_PER_SECTOR);
	fprintf(fp, "\n");
	fprintf(fp, "%-15s%-15s%-15s%-15s\n\n",
		"Sector X x Z", "Total Trees", "Solid Trees", "Dissolve Trees");

		
	//////////////////////////////////////////////////////////////////////////
	// Loop through all sectors
	//////////////////////////////////////////////////////////////////////////

	for(sec_z = 0; sec_z < max_z; sec_z++){

		if(sec_z == 15){

			printf("\nPlease wait! I'm working ...\n");
		}

		if(sec_z == 25){

			printf("\n%d trees have been planted\n", total_tree_count );
			printf("\nPlanting more ...\n");
		}

		if(sec_z == 30){

			printf("\n%d trees have been planted\n", total_tree_count );
			printf("\nPlanting more ...\n");
		}

		if(sec_z == 40){

			printf("\n%d trees have been planted\n", total_tree_count );
			printf("\nPlanting more ...\n");
		}

		if(sec_z == 45){

			printf("\n%d trees have been planted\n", total_tree_count );
			printf("\nPlanting more ...\n");
		}

		if(sec_z == 50){

			printf("\nAlmost finished ...\n");
		}



		for(sec_x = 0; sec_x < max_x; sec_x++){

	//for(sec_z = 45; sec_z < 46; sec_z++){
		//for(sec_x = 52; sec_x < 53; sec_x++){

			


			trees_this_sector = 0;
			dissolve_trees_this_sector = 0;
			solid_trees_this_sector = 0;
			locality_count = 0;

			polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

			//////////////////////////////////////////////////////////////////////////
			//////////////////////////////////////////////////////////////////////////
			// START Polygon Loop
			//////////////////////////////////////////////////////////////////////////
			//////////////////////////////////////////////////////////////////////////

			for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop++){

				if(locality_count == NUMBER_OF_POLYS_PER_GROVE){
					locality_count = 0;
				}

				if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){
					triangle_points[0][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][0].point].x,
					triangle_points[1][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][1].point].x,
					triangle_points[2][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][2].point].x,
					triangle_points[0][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][0].point].z,
					triangle_points[1][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][1].point].z,
					triangle_points[2][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][2].point].z,
					triangle_points[0][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][0].point].y,
					triangle_points[1][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][1].point].y,
					triangle_points[2][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_byte_references[poly_loop][2].point].y;
				}
				else{
					triangle_points[0][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][0].point].x,
					triangle_points[1][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][1].point].x,
					triangle_points[2][0] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][2].point].x,
					triangle_points[0][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][0].point].z,
					triangle_points[1][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][1].point].z,
					triangle_points[2][1] = (long long)terrain_sectors[sec_z][sec_x].points_xz[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][2].point].z,
					triangle_points[0][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][0].point].y,
					triangle_points[1][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][1].point].y,
					triangle_points[2][2] = (long long)terrain_sectors[sec_z][sec_x].points_y[(int)terrain_sectors[sec_z][sec_x].point_word_references[poly_loop][2].point].y;
				}

				min_y = triangle_points[0][2];
				if(triangle_points[1][2] < min_y) min_y = triangle_points[1][2];
				if(triangle_points[2][2] < min_y) min_y = triangle_points[2][2];
				max_y = triangle_points[0][2];
				if(triangle_points[1][2] > max_y) max_y = triangle_points[1][2];
				if(triangle_points[2][2] > max_y) max_y = triangle_points[2][2];

				average_y = min_y + max_y / 2;

				elevation = terrain_3d_map_minimum_height + average_y * terrain_3d_map_scaled_height_difference;

				slope = normal_angle_with_y(sec_z, sec_x, poly_loop);

				if(elevation > MIN_ELEVATION &&
					elevation < MAX_ELEVATION &&
					slope > MIN_SLOPE && 
					slope < MAX_SLOPE){

					elevation_and_slope_okay = 1;
				}
				else{

					elevation_and_slope_okay = 0;
				}

				//////////////////////////////////////////////////////////////////////
				// Determine how many trees for this polygon
				//////////////////////////////////////////////////////////////////////

				trees_this_poly = 0;
				
				if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id == 
					TERRAIN_TYPE_LAND && trees_this_sector <= MAX_TREES_PER_SECTOR){

					if(elevation < MAX_SHORE_ELEVATION && 
						slope < MAX_SHORE_SLOPE &&
						slope >= MIN_SHORE_SLOPE){

						locality_count = 0;

						if(rand() % CHANCE_OF_SHORE_SINGLES == 0){

							trees_this_poly = (rand() % NUMBER_OF_SHORE_SINGLES_PER_POLY) + 1;					
						} 
					} // end if (shore single trees)
					else if(elevation_and_slope_okay){

						if(locality_count){
							trees_this_poly = MAX_GROVE_TREES_PER_POLY;
							
							if(locality_count < NUMBER_OF_POLYS_PER_GROVE){
								locality_count ++;
							}
							
						}
						else{
							if(rand() % CHANCE_OF_GROVE == 0){ // if true, a grove is started
								trees_this_poly = MAX_GROVE_TREES_PER_POLY;
								
								locality_count = 1;
							}
							else if(rand() % CHANCE_OF_SINGLES == 0){
								trees_this_poly = (rand() % NUMBER_OF_SINGLES_PER_POLY) + 1;
								
							}
						} // end if else (locality_count > 0)
					} // end if elevation and slope are okay
					
				} // end if not sea terrain

				//printf("%d  locality %d trees %d\n", poly_loop, locality_count, trees_this_poly);

				///////////////////////////////////////////////////////////////////////////////////
				// start calculations and planting trees if we want any
				///////////////////////////////////////////////////////////////////////////////////

				if(trees_this_poly){

					// vectors
					Ax = triangle_points[0][0] - triangle_points[1][0]; // ax - bx
					Ay = triangle_points[0][2] - triangle_points[1][2]; // ay - by
					Az = triangle_points[0][1] - triangle_points[1][1]; // az - bz
					Bx = triangle_points[2][0] - triangle_points[1][0]; // cx - bx
					By = triangle_points[2][2] - triangle_points[1][2]; // cy - by
					Bz = triangle_points[2][1] - triangle_points[1][1]; // cz - bz

					// normal
					Nx = Ay * Bz - Az * By;
					Ny = Az * Bx - Ax * Bz;
					Nz = Ax * By - Ay * Bx;
				
					// plane constant
					//k = ax * Nx + ay * Ny + az * Nz;
					k = triangle_points[0][0] * Nx + triangle_points[0][2] * Ny + triangle_points[0][1] * Nz;

					// find the bounding rectangle for the triangle

					x_bound_min = (int)triangle_points[0][0];
					x_bound_max = (int)triangle_points[0][0];
					z_bound_min = (int)triangle_points[0][1];
					z_bound_max = (int)triangle_points[0][1];

					for(i = 1; i < 3; i++){
						if((int)triangle_points[i][0] < x_bound_min) x_bound_min = (int)triangle_points[i][0];
						if((int)triangle_points[i][0] > x_bound_max) x_bound_max = (int)triangle_points[i][0];
						if((int)triangle_points[i][1] < z_bound_min) z_bound_min = (int)triangle_points[i][1];
						if((int)triangle_points[i][1] > z_bound_max) z_bound_max = (int)triangle_points[i][1];
					}

					// assign points to inside_poly_array[]

					for(i = 0; i < 3; i++){
						inside_poly_array[i].x = (double)triangle_points[i][0];
						inside_poly_array[i].y = (double)triangle_points[i][1];
					}

					// find how many trees for this size of triangle

					//num_trees = ((x_bound_max - x_bound_min) * (z_bound_max - z_bound_min)) /
					//	(min_space * min_space * density_factor);

					num_trees = trees_this_poly;

					while_runaway_check = 0;

					tree_index = 0;

					while(tree_index < num_trees){

						// get a random point

						px = x_bound_min + (rand() % (x_bound_max - x_bound_min));
						pz = z_bound_min + (rand() % (z_bound_max - z_bound_min));

						random_point.x = (double)px;
						random_point.y = (double)pz;

						// check to see if this point is a good one

						if(InsidePolygon(inside_poly_array, 3, random_point)){

							if(edge_test_ok(px, pz, min_space, triangle_points, 3)){

								// check for min_space against tree list

								space_ok = 1;

								for(x = 0; x < tree_index; x++){

									space = (long long)sqrt(((px - tree_points[x][0]) * (px - tree_points[x][0])) + 
										((pz - tree_points[x][1]) * (pz - tree_points[x][1])));

									if(space < min_space){

										space_ok = 0;
									}
								} // end for
					
								// add point to tree list if space_ok

								if(space_ok){

									tree_points[tree_index][0] = px;
									tree_points[tree_index][1] = pz;
									tree_points[tree_index][2] = ((k - px * Nx - pz * Nz) / Ny);
									tree_index++;
								}

							} // end if edge_test_ok
						} // end if Inside Polygon

						if(while_runaway_check > 500)break;
						while_runaway_check++;

					} // end while
						
				} // end if trees_this_poly

				//////////////////////////////////////////////////////
				//////////////////////////////////////////////////////
				// write out tree data
				//////////////////////////////////////////////////////
				//////////////////////////////////////////////////////

				if(trees_this_poly){

					//dissolve_ratio_counter ++;

					if(locality_count){

					////////////////////////////////////////////////////
					// make some of the trees disappearing type
					// if they are in a grove
					////////////////////////////////////////////////////

						for(i = 0; i < tree_index; i++){

							dissolve_ratio_counter ++;

							if(dissolve_ratio_counter % dissolve_ratio_factor == 0){ 
							//if(rand() % 2 == 0){ 
							//if(trees_this_sector % dissolve_ratio_factor == 0){
							//if(0){
								
								// these are the solid trees
								
								local_tree_data.height = (rand() % 40) + 80;
								local_tree_data.type = 1;
								local_tree_data.x = (short int) tree_points[i][0] * 2;
								local_tree_data.z = (short int) tree_points[i][1] * 2;
								local_tree_data.y = (unsigned short int) tree_points[i][2] * 2;
								fwrite(&local_tree_data, sizeof(terrain_3d_tree_data), 1, tree_data);
								trees_this_sector += 1;
								offset_counter_next += 1;
								solid_trees_this_sector += 1;
							} // end if 
							else{
								
								// these are the dissolving trees

								local_tree_data.height = (rand() % 20) + 40;
								local_tree_data.type = 0;
								local_tree_data.x = (short int) tree_points[i][0] * 2;
								local_tree_data.z = (short int) tree_points[i][1] * 2;
								local_tree_data.y = (unsigned short int) tree_points[i][2] * 2;
								fwrite(&local_tree_data, sizeof(terrain_3d_tree_data), 1, tree_data);
								trees_this_sector += 1;
								offset_counter_next += 1;
								dissolve_trees_this_sector += 1;
							}

						} // end for tree index

					} // end if for locality count (doing a grove here)
					else{  // doing single trees here

						for(i = 0; i < tree_index; i++){

							local_tree_data.height = (rand() % 60) + 60;
							local_tree_data.type = 1;

							local_tree_data.x = (short int) tree_points[i][0] * 2;
							local_tree_data.z = (short int) tree_points[i][1] * 2;
							local_tree_data.y = (unsigned short int) tree_points[i][2] * 2;
							fwrite(&local_tree_data, sizeof(terrain_3d_tree_data), 1, tree_data);
							trees_this_sector += 1;
							solid_trees_this_sector += 1;
							offset_counter_next += 1;
						} // end for tree index

					} // end else (doing single trees here)

				} // end if trees this poly

			} // end for POLY LOOP

			/////////////////////////////////////////////////
			// END POLY LOOP
			/////////////////////////////////////////////////

			total_tree_count += trees_this_sector;

			if(highest_tree_count < trees_this_sector){

				highest_tree_count = trees_this_sector;
				highest_x_sector = sec_x;
				highest_z_sector = sec_z;
			}

			if(trees_this_sector > MAX_TREES_PER_SECTOR){

				number_of_sectors_with_max_trees ++;
			}

			if(trees_this_sector > 0){	
				fprintf(fp, "%3d x %-3d      %-15d%-15d%-15d\n",
					sec_x, sec_z, trees_this_sector,
					solid_trees_this_sector, dissolve_trees_this_sector);
			}

            ///////////////////////////////////////////////////////
			// write out the tree sector file
			///////////////////////////////////////////////////////

			fwrite(&trees_this_sector, sizeof(int), 1, tree_sec);
			fwrite(&offset_counter_last, sizeof(int), 1, tree_sec);
			
			offset_counter_last = offset_counter_next;

		} // end x loop
	} // end z loop

	fprintf(fp, "\nHighest tree count is %d in sector %d x %d\n", 
		highest_tree_count, highest_x_sector, highest_z_sector);

	fprintf(fp, "\n%d Sectors out of %d have maximum trees\n",
		number_of_sectors_with_max_trees, max_x * max_z);

	fprintf(fp, "\nTotal trees is %u\n", total_tree_count);

	printf("\n\tTotal trees is %u\n", total_tree_count);
	printf("\t%d Sectors out of %d have the maximum trees of %d per sector\n",
		number_of_sectors_with_max_trees, max_x * max_z, MAX_TREES_PER_SECTOR + 8);

		
	fclose(tree_sec);
	fclose(tree_data);
	fclose(fp);

} // end create_tree_files2

void get_terrain_info(int sec_z, int sec_x){

	int		max_slope = 0,
			min_slope = 90,
			slope_differential,
			slopes_over_20 = 0,
			slopes_over_30 = 0,
			slopes_over_40 = 0,
			slopes_over_50 = 0,
			elevation;

	int x,
		polys_this_sector,
		slope,
		y_point0,
		y_point1,
		y_point2;

	double	highest_elevation = 0.0,
			lowest_elevation = 50000.0,
			min_elevation,
			max_elevation,
			y0,
			y1,
			y2;

	float	terrain_3d_map_scaled_height_difference;

	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;



	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(x = 0; x < polys_this_sector; x++){

		if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

			y0 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_byte_references[x][0].point].y * terrain_3d_map_scaled_height_difference ));
			y1 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_byte_references[x][1].point].y * terrain_3d_map_scaled_height_difference ));
			y2 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_byte_references[x][2].point].y * terrain_3d_map_scaled_height_difference ));
		}
		else{

			y0 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_word_references[x][0].point].y * terrain_3d_map_scaled_height_difference ));
			y1 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_word_references[x][1].point].y * terrain_3d_map_scaled_height_difference ));
			y2 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_word_references[x][2].point].y * terrain_3d_map_scaled_height_difference ));
		}

		if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

			y_point0 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_byte_references[x][0].point].y;
			y_point1 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_byte_references[x][1].point].y;
			y_point2 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_byte_references[x][2].point].y;
		}
		else{

			y_point0 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_word_references[x][0].point].y;
			y_point1 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_word_references[x][1].point].y;
			y_point2 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].point_word_references[x][2].point].y;
		}

		// find the average elevation of the poly

		min_elevation = y0;
		if(y1 < min_elevation) min_elevation = y1;
		if(y2 < min_elevation) min_elevation = y2;

		max_elevation = y0;
		if(y1 > max_elevation) max_elevation = y1;
		if(y2 > max_elevation) max_elevation = y2;


		//elevation = (int)(min_elevation + max_elevation) / 2;
		//elevation = (int)min_elevation;

		slope = normal_angle_with_y(sec_z, sec_x, x);

		if(max_elevation > highest_elevation) highest_elevation = max_elevation;
		if(min_elevation < lowest_elevation) lowest_elevation = min_elevation;
		if(max_elevation > global_highest_elevation) global_highest_elevation = max_elevation;
		if(min_elevation < global_lowest_elevation) global_lowest_elevation = min_elevation;
		if(slope > 20) slopes_over_20 ++;
		if(slope > 30) slopes_over_30 ++;
		if(slope > 40) slopes_over_40 ++;
		if(slope > 50) slopes_over_50 ++;
		if(slope < min_slope) min_slope = slope;
		if(slope > max_slope) max_slope = slope;
		if(slope > global_max_slope) global_max_slope = slope;
		if(slope < global_min_slope) global_min_slope = slope;
		if(polys_this_sector > global_max_polys) global_max_polys = polys_this_sector;
		
	}

	//printf("\nSector %d x %d number of polys = %d\n", sec_z, sec_x, polys_this_sector);
	//printf("elevation %d - %d\n", (int)lowest_elevation, (int)highest_elevation);
	//printf("slope     %d - %d\n", min_slope, max_slope);
	//printf("slopes over 20 30 40 50 - %d %d %d %d\n", slopes_over_20, slopes_over_30, slopes_over_40, slopes_over_50);

} // end get_terrain_info()





