// Puerto Rico Tree Plant

#define MAP_WIDTH 150
#define MAP_HEIGHT 68


#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define INSIDE 1
#define OUTSIDE 0

#define TERRAIN_DETAIL_4	9
#define TERRAIN_TYPE_LAND	9
#define TERRAIN_SCRUB		21
#define TERRAIN_BARE_ROCK2	22
#define TERRAIN_SNOW		23
#define TERRAIN_SEA			7
#define TERRAIN_BEACH		8
#define TERRAIN_GRG_CITY4	37  // BUILT_UP_AREA1
#define TERRAIN_GRG_CITY3	38  // BUILT_UP_AREA2

// Tree parameters
// smaller numbers = greater chance for CHANCE parameters
// larger numbers = more trees for NUMBER parameters

#define CHANCE_OF_GROVE						500		// 1 out of X
#define NUMBER_OF_POLYS_PER_GROVE			15
#define MAX_GROVE_TREES_PER_POLY			4
#define CHANCE_OF_SINGLES					200		// 1 out of X
#define NUMBER_OF_SINGLES_PER_POLY			1
#define CHANCE_OF_SHORE_SINGLES				5		// 1 out of X
#define NUMBER_OF_SHORE_SINGLES_PER_POLY	1
#define MAX_ELEVATION						99999
#define MIN_ELEVATION						16
#define MAX_SLOPE							30
#define MIN_SLOPE							5
#define MAX_SHORE_ELEVATION					15
#define MAX_SHORE_SLOPE						10
#define MIN_SHORE_SLOPE						1
#define DISSOLVE_TO_SOLID_RATIO				0		// X to 1	0 = no dissolve trees
#define MAX_TREES_PER_SECTOR				400


#define TERRAIN_3D_Y_SCALE ( 1.0 / 32767.0 )

typedef struct {
   double x,y;
} Point;


struct TERRAIN_3D_POINT_XZ
{

	short int
		x,
		z;
};

typedef struct TERRAIN_3D_POINT_XZ terrain_3d_point_xz;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

#pragma pack(1)

struct TERRAIN_3D_POINT_Y
{

	union
	{

		struct
		{

			unsigned short int
				y:15,
				normal_change:1;
		};

		unsigned short int
			whole_value;

		short int
			whole_signed_value;
	};
};

typedef struct TERRAIN_3D_POINT_Y terrain_3d_point_y;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR
{

	unsigned char
		red,
		green,
		blue;
};

typedef struct TERRAIN_3D_COLOUR terrain_3d_colour;

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_COLOUR_INDEX terrain_3d_colour_index;


#pragma pack(4)

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_WORD_REFERENCE
{

	unsigned short int
		point;
};

typedef struct TERRAIN_3D_POINT_WORD_REFERENCE terrain_3d_point_word_reference;

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_BYTE_REFERENCE
{

	unsigned char
		point;
};

typedef struct TERRAIN_3D_POINT_BYTE_REFERENCE terrain_3d_point_byte_reference;

///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////

struct TERRAIN_3D_FACE
{

	unsigned char
		number_of_points:7,
		surface_change:1;
};

typedef struct TERRAIN_3D_FACE terrain_3d_face;

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

struct TERRAIN_3D_SURFACE
{

	unsigned char
		surface_id;
};

typedef struct TERRAIN_3D_SURFACE terrain_3d_surface;

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

struct TERRAIN_3D_SECTOR
{

	unsigned short int
		number_of_points,
		number_of_polygons,
		radius,
		normal_changes_word;

	//unsigned char
	//	pad_byte_zero;

	short int
		minimum_height,
		maximum_height,
		average_height,
		pad_byte,
		number_of_surface_ids;

	int
		colour_indices_offset,
		point_colours_offset,
		max_colour_index;

	struct TERRAIN_3D_POINT_XZ
		points_xz[1024];

	struct TERRAIN_3D_POINT_Y
		points_y[1024];

	struct TERRAIN_3D_FACE
		polygons[1024];

	union
	{

		struct TERRAIN_3D_POINT_WORD_REFERENCE
			point_word_references[1024][3];

		struct TERRAIN_3D_POINT_BYTE_REFERENCE
			point_byte_references[1024][3];
	};


	struct TERRAIN_3D_SURFACE
		surface_ids[256];

	struct TERRAIN_3D_SURFACE
		surface_id_each_face[1024];

	struct TERRAIN_3D_COLOUR
		point_colours[128];

	struct TERRAIN_3D_COLOUR_INDEX
		colour_indices[1024];
};

typedef struct TERRAIN_3D_SECTOR terrain_3d_sector;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define TERRAIN_3D_MAX_TREE_HEIGHT	(35)


struct TERRAIN_3D_TREE_DATA
{

	signed short int
		x,
		z;

	unsigned short int
		y;

	unsigned char
		colour;

	unsigned char
		height:7,
		type:1;
};

typedef struct TERRAIN_3D_TREE_DATA terrain_3d_tree_data;


