// write_road_files
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "write_road_files.h"

// the bridge stuff is found in bridge.c
/* Files Required

	default.sec
	terrain.ffp
	bridges.txt
	Grand.ROAD
	

   Files Produced

	bridge.dat
	ROADS.DAT
	ROADS.NDE
	ROADS.WP
*/

////////////////////////////////////////
// Globals
////////////////////////////////////////

/////////////////////////////////////////////////
// *** start of Global Lists
/////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// holds geometry data terrain_sectors[z][x]
////////////////////////////////////////////////////////////////////////////

terrain_3d_sector
	**terrain_sectors;

/////////////////////////////////////////////////////////////////////////////
// Holds up to 128 roads and up to 512 quads per road
/////////////////////////////////////////////////////////////////////////////

road
	roads[128];

/////////////////////////////////////////////////////////////////////////////
// Holds up to 128 bridges
/////////////////////////////////////////////////////////////////////////////

bridge
	bridges[128];

/////////////////////////////////////////////////////////////////////////////
// Holds all the road data
/////////////////////////////////////////////////////////////////////////////

contour_path
	*contour_map_road_paths;

vec3d
	*contour_map_road_nodes,
	*contour_map_road_path_positions,
	*current_position;

vec3d
	start_road_nodes [ 128 ],
	end_road_nodes [ 128 ],
	total_road_nodes [ 256 ],
	temp_path_points [ 1024 ];

node_data
	global_node_data [ 256 ];


road_path
	road_paths [ 128 ], // this is used when using simple paths
	sub_divided_road_paths [ 256 ]; // this is used when using all paths between nodes

//////////////////////////////////////////////////////////////////////////////
// tree data for trouble shooting road points
//////////////////////////////////////////////////////////////////////////////

global_coord_tree_data
	trees_each_sector [ 62 ][ 106 ]; // [z][x]
		

////////////////////////////////////////////////////
// *** end of Global Lists
////////////////////////////////////////////////////

int
	global_number_of_roads,
	terrain_version,
	rgb_4byte_header,
	ffp_4byte_header,
	terrain_3d_map_width,
	terrain_3d_map_height,
	number_of_bridges,
	contour_map_number_of_road_nodes,
	contour_map_number_of_road_paths,
	longest_span_length = 0,
	global_total_number_of_nodes = 0,
	global_number_of_divided_paths = 0;

short int
	pad_word_zero;

unsigned short int
	normal_changes_word;



float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;


//FILE
//	*changed_road_pts;

/////////////////////////////////////////
// functions
/////////////////////////////////////////



void load_3d_terrain (void);

void read_in_ROAD_file(void);

void convert_ROAD_data_to_eech_road_data(void);

void convert_eech_road_data_to_eech_road_data_with_junctions(void);

void write_eech_road_dat_file(void);

int node_in_list(vec3d p);

void build_global_node_data(void);

int calculate_cost (int path);

void write_eech_road_nde_file(void);

void write_eech_road_wp_file(void);

float get_global_y_value(vec3d pt);

int get_sector_y_value(int sec_x, int sec_z, int x, int z);

int InsidePolygon(simple_point *polygon,int N,simple_point p);

void reverse_backward_paths(void);

void plant_trees_on_road_points(void);

sector_tree_point   get_sector_tree_point(vec3d global_point);

void add_intermediate_points_to_paths(void);

int get_span_length(vec3d p, vec3d p1);

int get_best_point_distance ( int dist, int max_dist );

vec3d get_point_at_distance_d(vec3d p, vec3d p1, float d);


int main(int argc, char ** argv){

	load_3d_terrain ();

	read_in_ROAD_file();

	convert_ROAD_data_to_eech_road_data();

	convert_eech_road_data_to_eech_road_data_with_junctions();

	build_global_node_data();

	reverse_backward_paths();

	add_intermediate_points_to_paths();

	write_eech_road_dat_file();

	write_eech_road_nde_file();

	write_eech_road_wp_file();

	plant_trees_on_road_points();

	return 0;

} // end main

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////
////////////////////////////////////

int get_span_length(vec3d p, vec3d p1){

	int
		span_length;

	span_length = (int)sqrt(((p1.x - p.x) * (p1.x - p.x)) + ((p1.z - p.z) * (p1.z - p.z)));

	return span_length;

} // end get_span_length

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void write_new_roads_wp_file(void){

	FILE			*fp;

	int
		path_loop,
		point_loop;

	if(( fp = fopen("NEW_ROADS.WP","wb"))==NULL){
		printf("NEW_ROADS.WP\n");
		exit(1);
	}
	else printf("NEW_ROADS.WP opened\n");

	fwrite ( &contour_map_number_of_road_paths, sizeof ( int ), 1, fp );

	for(path_loop = 0; path_loop < contour_map_number_of_road_paths; path_loop++){

		fwrite ( &contour_map_road_paths[path_loop].from, sizeof ( int ), 1, fp );
		fwrite ( &contour_map_road_paths[path_loop].to, sizeof ( int ), 1, fp );
		fwrite ( &contour_map_road_paths[path_loop].type, sizeof ( int ), 1, fp );
		fwrite ( &contour_map_road_paths[path_loop].count, sizeof ( int ), 1, fp );

		for(point_loop = 0; point_loop < contour_map_road_paths[path_loop].count; point_loop++){

			fwrite ( &contour_map_road_paths[path_loop].points[point_loop], sizeof ( vec3d ), 1, fp);

		} // end for point loop

	} // end for path loop

	fclose ( fp );


} // end void write_new_roads_wp_file(void);

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// read in map.ROAD file
//
// Data Format
//
// 4 bytes - number of roads
// loop number of roads
//    4 bytes width
//    4 bytes color
//    4 bytes number of points for this road
//    loop number of points
//        x global ( Rob Fox uses ints for floats (int/100 = float))
//        z global

void read_in_ROAD_file(){

	FILE
		*road_fp;

	int
		road_loop,
		point_loop,
		width,
		color,
		number_of_points_for_this_road,
		number_of_quads,
		x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
		z_int_pos,
		return_value;
	
	printf("reading road file\n");

	if(( road_fp = fopen("Grand.ROAD","rb"))==NULL){
		printf("cannot open Grand.ROAD\n");
		exit(1);
	}
	else printf("Grand.ROAD opened\n");

	//////////////////////////////////////////////////////////////////////
	// read in all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fread ( &global_number_of_roads, sizeof ( int ), 1, road_fp );

	//printf("number of roads = %d\n\n", global_number_of_roads);

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop++){

		fread ( &width, sizeof ( int ), 1, road_fp );
		fread ( &color, sizeof ( int ), 1, road_fp );
		fread ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );

		roads[road_loop].width = width;

		roads[road_loop].color = color;
		
		roads[road_loop].number_of_center_points = number_of_points_for_this_road;

		roads[road_loop].number_of_quads = number_of_points_for_this_road - 1;

		for(point_loop = 0; point_loop < number_of_points_for_this_road; point_loop++){

			fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			roads[road_loop].center_points[point_loop].global_x = (double)x_int_pos / 100.0;
			roads[road_loop].center_points[point_loop].global_z = (double)z_int_pos / 100.0;
		}
	}
	
	fclose ( road_fp );

} // end read_in_ROAD_file()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void convert_ROAD_data_to_eech_road_data(void){

	int
		road_loop,
		point_loop,
		number_of_points_for_this_road;

	printf("converting Grand.ROAD data to EECH road data\n");

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop++){

		number_of_points_for_this_road = roads[ road_loop ].number_of_center_points;

		//
		// node data - start by making a node from the first and last point of each road
		//

		start_road_nodes [ road_loop ].x = 
			(float)roads[ road_loop ].center_points[ 0 ].global_x;

		start_road_nodes [ road_loop ].z = 
			(float)roads[ road_loop ].center_points[ 0 ].global_z;

		start_road_nodes [ road_loop ].y = 
			get_global_y_value ( start_road_nodes [ road_loop ] );

		end_road_nodes [ road_loop ].x = (float)roads[ road_loop ].
			center_points[ number_of_points_for_this_road - 1 ].global_x;

		end_road_nodes [ road_loop ].z = (float)roads[ road_loop ].
			center_points[ number_of_points_for_this_road - 1 ].global_z;

		end_road_nodes [ road_loop ].y = 
			get_global_y_value ( end_road_nodes [ road_loop ] );

		//
		// path data - list of points between nodes, one path is one link between two nodes
		//

		road_paths [ road_loop ].from = road_loop * 2;

		road_paths [ road_loop ].to = road_loop * 2 + 1;

		//road_paths [ road_loop ].type = 41;
		road_paths [ road_loop ].type = 28;

		road_paths [ road_loop ].count = number_of_points_for_this_road - 2;

		for(point_loop = 1; point_loop < number_of_points_for_this_road - 1; point_loop ++){

			road_paths [ road_loop ].points [ point_loop - 1 ].x = 
				(float)roads[ road_loop ].center_points[ point_loop ].global_x;

			road_paths [ road_loop ].points [ point_loop - 1 ].z = 
				(float)roads[ road_loop ].center_points[ point_loop ].global_z;

			road_paths [ road_loop ].points [ point_loop - 1 ].y = 
				get_global_y_value ( road_paths [ road_loop ].points [ point_loop - 1 ] );

			//printf("%d  %f\n", point_loop, roads[ road_loop ].center_points[ point_loop ].global_x);


		} // end for point loop
	} // end road loop


} // end convert_ROAD_data_to_eech_road_data()

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////


// loaded in ai_route.c 
// also in terrmap.c in a little different format
//
// Data format (ROADS.NDE)
//
// Number of nodes
// list of vec3d positions
//
// Data format (ROADS.WP)
// 
// number of paths
//
// From
// To
// Type
// Count
// List of node_link_data (vec3d) for each position and cost. 
//		(I don't think the cost is in the file and I think it's a distance factor,
//		really just vec3d of all points between nodes)
//
// it looks like I got it right see ProgramDescriptions.txt
// just need to change the data to include junctions


void convert_eech_road_data_to_eech_road_data_with_junctions(void){

	int
		road_loop,		
		point_loop,
		node_loop,
		temp_points_loop,
		number_of_points_this_path,
		total_paths = 0,
		temp_path_points_count = 0,
		temp_start_node_int,
		temp_end_node_int,
		this_point_int;

	vec3d
		this_point,
		end_node_point;
		
	printf("subdividing paths at junction points\n");


	////////////////////////////////////////////////////////
	// get a single list of nodes with no duplicates
	////////////////////////////////////////////////////////

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop ++){

		if( node_in_list( start_road_nodes [ road_loop ] ) == -1 ){ // not already in list

			total_road_nodes [ global_total_number_of_nodes ] = start_road_nodes [ road_loop ];

			global_total_number_of_nodes ++;
		}

		if( node_in_list( end_road_nodes [ road_loop ] ) == -1 ){ // not already in list

			total_road_nodes [ global_total_number_of_nodes ] = end_road_nodes [ road_loop ];

			global_total_number_of_nodes ++;
		}

	} // end for node loop

	//////////////////////////////////////////////////
	// print section
	//////////////////////////////////////////////////
/*
	printf("%d nodes with junctions\n\n", global_total_number_of_nodes);

	for(node_loop = 0; node_loop < global_total_number_of_nodes; node_loop ++){

		printf("%f  %f  %f\n",  
			total_road_nodes [ node_loop ].x,
			total_road_nodes [ node_loop ].z,
			total_road_nodes [ node_loop ].y);
	} // end for node loop

	//////////////////////////////////////////////////
	// end print section
	//////////////////////////////////////////////////
*/

	///////////////////////////////////////////////////////////
	// subdivide all road paths to paths from node - to - node,
	// this makes a network of paths between nodes with junctions
	// being nodes
	///////////////////////////////////////////////////////////

	total_paths = 0;

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop ++){
	
		number_of_points_this_path = road_paths [ road_loop ].count; // road points - 2

		this_point = start_road_nodes [ road_loop ];

		temp_start_node_int = node_in_list ( this_point );

		for(point_loop = 0; point_loop < number_of_points_this_path; point_loop ++){

			this_point = road_paths [ road_loop ].points [ point_loop ];

			this_point_int = node_in_list ( this_point );

			if(point_loop == number_of_points_this_path - 1){
			
				//////////////////////////////////////////////////////
				// this is the last point on the original
				// road path, so make a path to the end node
				//////////////////////////////////////////////////////
				
				end_node_point = end_road_nodes [ road_loop ];

				temp_end_node_int = node_in_list ( end_node_point );

				//////////////////////////////////////////////////////
				// copy temp path data to permanent global data
				//////////////////////////////////////////////////////

				sub_divided_road_paths [ total_paths ].from = temp_start_node_int;

				sub_divided_road_paths [ total_paths ].to = temp_end_node_int;

				if(temp_path_points_count == 0){

					//sub_divided_road_paths [ total_paths ].type = 59;
					sub_divided_road_paths [ total_paths ].type = 28;
				}
				else{

					//sub_divided_road_paths [ total_paths ].type = 41;
					sub_divided_road_paths [ total_paths ].type = 28;
				}
				
				sub_divided_road_paths [ total_paths ].count = temp_path_points_count + 1;

				for(temp_points_loop = 0; temp_points_loop < temp_path_points_count; temp_points_loop ++){

					sub_divided_road_paths [ total_paths ].points [ temp_points_loop ] = 
						temp_path_points [ temp_points_loop ];
				}

				//////////////////////////////////////////////////////
				// copy the last point
				//////////////////////////////////////////////////////

				sub_divided_road_paths [ total_paths ].points [ temp_points_loop ] = this_point;

				temp_path_points_count = 0;

				total_paths ++;
			}
			else if( this_point_int != -1 ){ // point is a node

				temp_end_node_int = this_point_int;

				/////////////////////////////////////////////////////////
				// copy temp path data to permanent global data
				/////////////////////////////////////////////////////////

				sub_divided_road_paths [ total_paths ].from = temp_start_node_int;

				sub_divided_road_paths [ total_paths ].to = temp_end_node_int;

				if(temp_path_points_count == 0){

					//sub_divided_road_paths [ total_paths ].type = 59;
					sub_divided_road_paths [ total_paths ].type = 28;
				}
				else{

					//sub_divided_road_paths [ total_paths ].type = 41;
					sub_divided_road_paths [ total_paths ].type = 28;
				}

				sub_divided_road_paths [ total_paths ].count = temp_path_points_count;

				for(temp_points_loop = 0; temp_points_loop < temp_path_points_count; temp_points_loop ++){

					sub_divided_road_paths [ total_paths ].points [ temp_points_loop ] = 
						temp_path_points [ temp_points_loop ];
				}

				//
				// this is the end of one path
				// since the last node of the last path wasn't the last point of the road
				// reset it to a start node and find next path on this road
				//

				if( temp_path_points_count != number_of_points_this_path){

					temp_start_node_int = temp_end_node_int;
				}


				temp_path_points_count = 0;

				total_paths ++;
			}
			else{ // add another point to this path

				temp_path_points [ temp_path_points_count ] = this_point;

				temp_path_points_count ++;
			}
			
		} // end for point loop

	} // end for road loop

	global_number_of_divided_paths = total_paths;

} // end convert_ROAD_data_to_eech_road_data_with_junctions(void)

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void reverse_backward_paths(void){

	//
	// the from node always needs to be a smaller number
	// than the to node, so reverse the path if not
	//

	int
		path_loop,
		number_of_paths,
		point_loop,
		number_of_points,
		temp_from_node,
		temp_to_node;

	number_of_paths = global_number_of_divided_paths;

	for(path_loop = 0; path_loop < number_of_paths; path_loop ++){

		temp_from_node = sub_divided_road_paths [ path_loop ].from;

		temp_to_node = sub_divided_road_paths [ path_loop ].to;

		if(temp_from_node > temp_to_node){

			number_of_points = sub_divided_road_paths [ path_loop ].count;

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				temp_path_points [ point_loop ] = sub_divided_road_paths [ path_loop ].points [ point_loop ];

			} // end for point loop

			//
			// copy back to global data in reverse order
			//

			for(point_loop = 0; point_loop < number_of_points; point_loop ++){

				sub_divided_road_paths [ path_loop ].points [ point_loop ] = temp_path_points [ number_of_points - point_loop - 1 ];

			} // end for point loop

			sub_divided_road_paths [ path_loop ].from = temp_to_node;

			sub_divided_road_paths [ path_loop ].to = temp_from_node;

		}
	} // end path loop
} // end reverse backward paths

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

int node_in_list(vec3d p){

	// if point p is in the node list
	//		return the node index number
	// else
	//		return -1

	int
		node_loop;

	for(node_loop = 0; node_loop < global_total_number_of_nodes; node_loop ++){

		if(p.x == total_road_nodes [ node_loop ].x &&
			p.z == total_road_nodes [ node_loop ].z){

			return node_loop;
		}
	} // end for

	return -1;

} // end node_in_list

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void build_global_node_data(void){

	int
		temp_link_list,
		temp_link_count,
		total_node_count,
		total_path_count,
		start_path_node,
		end_path_node,
		node_loop,
		path_loop;

	printf("building global node data\n");

	total_node_count = global_total_number_of_nodes;

	total_path_count = global_number_of_divided_paths;

	for(node_loop = 0; node_loop < total_node_count; node_loop ++){

		temp_link_count = 0;

		for(path_loop = 0; path_loop < total_path_count; path_loop ++){

			start_path_node = sub_divided_road_paths [ path_loop ].from;

			end_path_node = sub_divided_road_paths [ path_loop ].to;

			if(node_loop == start_path_node){
				
				global_node_data [ node_loop ].
					list_of_links [ temp_link_count ] = end_path_node;

				global_node_data [ node_loop ].
					list_of_cost [ temp_link_count ] = calculate_cost( path_loop );

				temp_link_count ++;

			}
			if(node_loop == end_path_node){

				global_node_data [ node_loop ].
					list_of_links [ temp_link_count ] = start_path_node;

				global_node_data [ node_loop ].
					list_of_cost [ temp_link_count ] = calculate_cost( path_loop );

				temp_link_count ++;
			}
		} // end path loop

		global_node_data [ node_loop ].node_id = node_loop;

		global_node_data [ node_loop ].safe_radius = 28;

		global_node_data [ node_loop ].number_of_links = temp_link_count;

	} // end node loop

} // end build global node data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

int calculate_cost (int path){

	int
		total_distance,
		point_loop,
		point_count;

	vec3d
		start_point,
		end_point;

	point_count = sub_divided_road_paths [ path ].count;

	total_distance = 0;

	for(point_loop = 0; point_loop < point_count - 1; point_loop ++){

		start_point = sub_divided_road_paths [ path ].points [ point_loop ];

		end_point = sub_divided_road_paths [ path ].points [ point_loop + 1 ];

		total_distance += get_span_length (start_point, end_point);
	}

	return total_distance;

} // end calculate_cost

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

//
// Data format:
//
// int map size x
// int map size z
// int total number of nodes
// loop 	{
//				node_id,			4 bytes
//				safe_radius			4 bytes
//				number_of_links		4 bytes
//				loop - list of links
//							
//						4 bytes - node
//						4 bytes - cost
//			}
//

void write_eech_road_dat_file(void){

	FILE
		*fp;

	int
		total_number_of_road_nodes,
		node_loop,
		link_loop,
		node,
		safe_radius,
		number_of_links,
		link,
		cost;

	float
		local_terrain_3d_map_width_float,
		local_terrain_3d_map_height_float;

	printf("writing EECH roads.dat file\n");

	if(( fp = fopen("ROADS.DAT","wb"))==NULL){
		printf("cannot open ROADS.DAT\n");
		exit(1);
	}
	else printf("ROADS.DAT opened\n");

	total_number_of_road_nodes = global_total_number_of_nodes;

	local_terrain_3d_map_width_float = (float) terrain_3d_map_width * 2048;

	local_terrain_3d_map_height_float = (float) terrain_3d_map_height * 2048;

	//fwrite( &terrain_3d_map_width, sizeof (int), 1, fp );

	//fwrite( &terrain_3d_map_height, sizeof (int), 1, fp );

	fwrite( &local_terrain_3d_map_width_float, sizeof (float), 1, fp );

	fwrite( &local_terrain_3d_map_height_float, sizeof (float), 1, fp );

	fwrite( &total_number_of_road_nodes, sizeof (int), 1, fp );

	for(node_loop = 0; node_loop < total_number_of_road_nodes; node_loop ++){

		fwrite( &node_loop, sizeof (int), 1, fp );

		safe_radius = global_node_data [ node_loop ].safe_radius;

		fwrite( &safe_radius, sizeof (int), 1, fp );

		number_of_links = global_node_data [ node_loop ].number_of_links;

		fwrite( &number_of_links, sizeof (int), 1, fp );

		for(link_loop = 0; link_loop < number_of_links; link_loop ++){

			link = global_node_data [ node_loop ].list_of_links [ link_loop ];

			fwrite( &link, sizeof (int), 1, fp );

			cost = global_node_data [ node_loop ].list_of_cost [ link_loop ];

			//cost = 100;

			fwrite( &cost, sizeof (int), 1, fp );

		} // end link loop
	} // end node loop

	fclose(fp);

} // end write_eech_road_dat_file

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

//
// Data format (ROADS.NDE)
//
// Number of nodes
// list of vec3d positions
//

void write_eech_road_nde_file(void){

	FILE
		*fp;

	int
		total_number_of_road_nodes,
		node_loop;

	vec3d
		node;

	printf("writing EECH roads.nde file\n");

	if(( fp = fopen("ROADS.NDE","wb"))==NULL){
		printf("cannot open ROADS.NDE\n");
		exit(1);
	}
	else printf("ROADS.NDE opened\n");

	total_number_of_road_nodes = global_total_number_of_nodes;

	fwrite( &total_number_of_road_nodes, sizeof (int), 1, fp );

	for(node_loop = 0; node_loop < total_number_of_road_nodes; node_loop ++ ){

		fwrite( &total_road_nodes [ node_loop ], sizeof (vec3d), 1, fp);
	} // end for node loop

} // end write_eech_road_nde_file

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

//
// Data format (ROADS.WP)
// 
// number of paths
//
// From
// To
// Type
// Count
// List of node_link_data (vec3d) for each position and cost. 
//		(I don't think the cost is in the file and I think it's a distance factor,
//		really just vec3d of all points between nodes)
//
// it looks like I got it right see ProgramDescriptions.txt
// just need to change the data to include junctions
//

void write_eech_road_wp_file(void){

	FILE
		*fp;

	int
		path_loop,
		point_loop,
		number_of_paths,
		number_of_points;

	printf("writing EECH roads.wp file\n");

	if(( fp = fopen("ROADS.WP","wb"))==NULL){
		printf("cannot open ROADS.WP\n");
		exit(1);
	}
	else printf("ROADS.WP opened\n");

	number_of_paths = global_number_of_divided_paths;

	fwrite( &number_of_paths, sizeof (int), 1, fp );

	for(path_loop = 0; path_loop < number_of_paths; path_loop ++){

		number_of_points = sub_divided_road_paths [ path_loop ].count;

		fwrite( &sub_divided_road_paths [ path_loop ].from, sizeof (int), 1, fp );

		fwrite( &sub_divided_road_paths [ path_loop ].to, sizeof (int), 1, fp );

		fwrite( &sub_divided_road_paths [ path_loop ].type, sizeof (int), 1, fp );

		fwrite( &sub_divided_road_paths [ path_loop ].count, sizeof (int), 1, fp );

		for(point_loop = 0; point_loop < number_of_points; point_loop ++){

			fwrite( &sub_divided_road_paths [ path_loop ].points [ point_loop ],
				sizeof (vec3d), 1, fp);
		}
	}

	fclose (fp);

} // end void write_eech_road_wp_file


////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

float get_global_y_value(vec3d pt){

	int		x_sector,	// the sector
			z_sector,
			sec_x,		// the point value in sector coordinates
			sec_z,
			sec_y;

	float
		global_y_value;

	/////////////////////////////////////////////////////////////
	// find the sector
	/////////////////////////////////////////////////////////////

	x_sector = (int)pt.x / 2048;
	z_sector = (int)pt.z / 2048;

	//////////////////////////////////////////////////////////////
	// find the sector coordinates
	//////////////////////////////////////////////////////////////

	sec_x = (int)((pt.x - (x_sector * 2048 + 1024)) * 16);

	sec_z = (int)((pt.z - (z_sector * 2048 + 1024)) * 16);

	///////////////////////////////////////////////////////////////
	// get the y value in sector coordinates
	///////////////////////////////////////////////////////////////

	sec_y = get_sector_y_value(x_sector, z_sector, sec_x, sec_z);

	////////////////////////////////////////////////////////////////
	// calculate the global y value
	////////////////////////////////////////////////////////////////

	//global_y_value = (float)((double)terrain_3d_map_minimum_height + 
	//			((double)sec_y * terrain_3d_map_scaled_height_difference ));

	global_y_value = (float)(terrain_3d_map_minimum_height + 
				((float)sec_y * terrain_3d_map_scaled_height_difference ));

	//printf("%f\n", global_y_value);

	return global_y_value;

} // end float get_global_y_value

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


int get_sector_y_value(int sec_x, int sec_z, int x, int z){

	// This function returns the y value in sector coordinates
	// for the sector and x z sector coord pair

	int		i,
			polys_this_sector,
			poly_counter,
			poly_number,
			y_value;

	long long	k, // plane constant
				triangle_points[3][3], // [][0]=x, [][1]=z, [][2]=y
				Ax,Az,Ay,Bx,Bz,By,	// vectors
				Nx,Nz,Ny;			// normal vector

	double	terrain_3d_map_scaled_height_difference;

	simple_point	inside_poly_array[3],
					test_point;

	terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height )
		* TERRAIN_3D_Y_SCALE;


	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	test_point.x = x;
	test_point.z = z;

	// Find which polygon of this sector the point is on
	// by testing for point_in_poly with all polys until it is found

	for(poly_counter = 0; poly_counter < polys_this_sector; poly_counter++){

		poly_number = 1000;

		triangle_points[0][0] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 ].point].x;

		triangle_points[1][0] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 1 ].point].x;

		triangle_points[2][0] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 2 ].point].x;

		triangle_points[0][1] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 ].point].z;

		triangle_points[1][1] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 1 ].point].z;

		triangle_points[2][1] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 2 ].point].z;

		triangle_points[0][2] = (long long)terrain_sectors[sec_z][sec_x].
			points_y[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 ].point].y;

		triangle_points[1][2] = (long long)terrain_sectors[sec_z][sec_x].
			points_y[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 1 ].point].y;

		triangle_points[2][2] = (long long)terrain_sectors[sec_z][sec_x].
			points_y[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 2 ].point].y;

		// assign points to inside_poly_array[]

		for(i = 0; i < 3; i++){
			inside_poly_array[i].x = (double)triangle_points[i][0];
			inside_poly_array[i].z = (double)triangle_points[i][1]; 
		}

		//printf("%d  inside_poly array  %f  %f x %f  %f x %f  %f  test_point  %f  %f\n",
		//	poly_counter,
		//	inside_poly_array[0].x, inside_poly_array[0].z,
		//	inside_poly_array[1].x, inside_poly_array[1].z,
		//	inside_poly_array[2].x, inside_poly_array[2].z,
		//	test_point.x, test_point.z);

		if(InsidePolygon(inside_poly_array, 3, test_point)){

			poly_number = poly_counter;
			break;
		}

	} // end for poly loop

	if(poly_number == 2048){
		printf("can't find poly for point %d x %d in sector %d x %d\n", x, z, sec_x, sec_z);
		exit(1);
	}

	// vectors
	Ax = triangle_points[0][0] - triangle_points[1][0]; // ax - bx
	Ay = triangle_points[0][2] - triangle_points[1][2]; // ay - by
	Az = triangle_points[0][1] - triangle_points[1][1]; // az - bz
	Bx = triangle_points[2][0] - triangle_points[1][0]; // cx - bx
	By = triangle_points[2][2] - triangle_points[1][2]; // cy - by
	Bz = triangle_points[2][1] - triangle_points[1][1]; // cz - bz

	// normal
	Nx = Ay * Bz - Az * By;
	Ny = Az * Bx - Ax * Bz;
	Nz = Ax * By - Ay * Bx;
				
	// plane constant
	//k = ax * Nx + ay * Ny + az * Nz;
	k = triangle_points[0][0] * Nx + triangle_points[0][2] * Ny + triangle_points[0][1] * Nz;

	y_value = (int)((k - x * Nx - z * Nz) / Ny);

	//printf("%d   %d   %d   %d   %d\n", poly_number, (int)triangle_points[0][2],
	//	(int)triangle_points[1][2], (int)triangle_points[2][2], y_value);


	return y_value;
} // end get_sector_y_value()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(simple_point *polygon,int N,simple_point p)
{
  int counter = 0;
  int i;
  double xinters;
  simple_point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.z > MIN(p1.z,p2.z)) {
      if (p.z <= MAX(p1.z,p2.z)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.z != p2.z) {
            xinters = (p.z-p1.z)*(p2.x-p1.x)/(p2.z-p1.z)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon()


////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		sec_x,
		sec_z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		four_byte_zero_word,
		i,
		print_loop,		
		surface_changes_index,
		poly_loop,
		number_of_polys;

	float
		min_height,
		max_height;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius;
		//normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height;

	unsigned char
		pad_byte_zero,
		current_surface_id,
		normal_changes_buf[1024];
	
	long
		current_file_position;


	if(( sec_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[sec_z][sec_x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[sec_z][sec_x].points_y );
			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[sec_z][sec_x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				if(!(terrain_sectors[sec_z][sec_x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[sec_z][sec_x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[sec_z][sec_x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface changes - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			// there is always a zero pad byte at the end of the surface ids
			// if you are not going to recalculate the number of surface ids
			// for every sector then you need to take this into account

			terrain_sectors[sec_z][sec_x].number_of_surface_ids = 
				normal_changes_offset - surface_changes_offset - 1;
			
			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].surface_ids, sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids + 1, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// initialize the surface_id_each_face array
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].surface_id_each_face = 
				malloc(sizeof(terrain_3d_surface) * terrain_sectors[sec_z][sec_x].number_of_polygons))){
				printf("No memory for surface_id_each_face sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

			current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

			terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id = current_surface_id;
			
			surface_changes_index = 0;

			for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

				if(terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change == 1){

					surface_changes_index ++;

					current_surface_id = terrain_sectors[sec_z][sec_x].
						surface_ids[surface_changes_index].surface_id;

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;
				}
				else{

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;

				} // end if else
			} // end for poly loop

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// read in color offsets from sector file

			fread(&terrain_sectors[sec_z][sec_x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[sec_z][sec_x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[sec_z][sec_x].colour_indices = 
				malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			fread(terrain_sectors[sec_z][sec_x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[sec_z][sec_x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[sec_z][sec_x].number_of_points; i++){ 
				
				// there is a colour index for each point

				if((terrain_sectors[sec_z][sec_x].colour_indices + i)->index > 
					terrain_sectors[sec_z][sec_x].max_colour_index){

					terrain_sectors[sec_z][sec_x].max_colour_index = 
						(terrain_sectors[sec_z][sec_x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[sec_z][sec_x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[sec_z][sec_x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

//
// Tree file format:
//
// tree_pos.sec
//
//	int map size x
//	int map size z
//	loop z
//		loop x
//			int trees this sector
//			int offset into tree_pos.dat
//
// tree_pos.dat
//
//  	one terrain_3d_tree_data structure per tree
//

void plant_trees_on_road_points(void){

	FILE
		*tree_sec,
		*tree_data;

	int
		path_loop,
		point_loop,
		node_loop,
		sec_x_loop,
		sec_z_loop,
		path_count,
		point_count,
		node_count,
		x_sector,
		z_sector,
		x_loop,
		z_loop,
		offset_counter_next = 0,
		offset_counter_last = 0,
		trees_this_sector,
		tree_loop;

	vec3d
		pt;

	terrain_3d_tree_data
		local_tree_data;

	sector_tree_point
		sec_pt;

	local_tree_data.colour = 6;
	local_tree_data.height = 100;
	local_tree_data.type = 1;

	if(( tree_sec = fopen("tree_pos.sec","wb"))==NULL){
		printf("cannot open tree_pos.sec\n");
		return;
	}
	else printf("tree_pos.sec opened\n");

	if(( tree_data = fopen("tree_pos.dat","wb"))==NULL){
		fclose(tree_sec);
		printf("cannot open tree_pos.dat\n");
		return;
	}
	else printf("tree_pos.dat opened\n");

	//
	// initialize data
	//

	for(z_loop = 0; z_loop < terrain_3d_map_height; z_loop ++){
		for(x_loop = 0; x_loop < terrain_3d_map_width; x_loop ++){

			trees_each_sector [ z_loop ][ x_loop ].tree_count = 0;
		}
	}

	//
	// add all the nodes to global tree data
	//
/*
	for(z_loop = 0; z_loop < terrain_3d_map_height; z_loop ++){
		for(x_loop = 0; x_loop < terrain_3d_map_width; x_loop ++){

			printf("%d  %d   %d\n", z_loop, x_loop, trees_each_sector [ z_loop ][ x_loop ].tree_count);
		}
	}
*/
	node_count = global_total_number_of_nodes;

	//printf("node count %d\n", node_count);

	for(node_loop = 0; node_loop < node_count; node_loop ++){

		pt = total_road_nodes [ node_loop ];

		x_sector = (int)pt.x / 2048;
		z_sector = (int)pt.z / 2048;

		//printf("sector nodes  %d   %d\n", x_sector, z_sector);

		trees_each_sector [ z_sector ][ x_sector ].
			trees [ trees_each_sector [ z_sector ][ x_sector ].tree_count ] = pt;

		trees_each_sector [ z_sector ][ x_sector ].tree_count ++;

	} // end for node loop
/*
	for(z_loop = 0; z_loop < terrain_3d_map_height; z_loop ++){
		for(x_loop = 0; x_loop < terrain_3d_map_width; x_loop ++){

			printf("%d  %d   %d\n", z_loop, x_loop, trees_each_sector [ z_loop ][ x_loop ].tree_count);
		}
	}
*/
	//
	// add the path points to global tree data
	//

	path_count = global_number_of_divided_paths;

	for(path_loop = 0; path_loop < path_count; path_loop ++){

		point_count = sub_divided_road_paths [ path_loop ].count;

		for(point_loop = 0; point_loop < point_count; point_loop ++){

			pt = sub_divided_road_paths [ path_loop ].points [ point_loop ];

			x_sector = (int)pt.x / 2048;
			z_sector = (int)pt.z / 2048;

			trees_each_sector [ z_sector ][ x_sector ].
				trees [ trees_each_sector [ z_sector ][ x_sector ].tree_count ] = pt;

			trees_each_sector [ z_sector ][ x_sector ].tree_count ++;

		} // end for point loop
	} // end for path loop

	/////////////////////////////////////////////////
	// write map size to tree sector file
	/////////////////////////////////////////////////

	fwrite(&terrain_3d_map_width, sizeof(int), 1, tree_sec);
	fwrite(&terrain_3d_map_height, sizeof(int), 1, tree_sec);

	/////////////////////////////////////////////////
	// write the tree data file and sector file
	/////////////////////////////////////////////////

	for(z_loop = 0; z_loop < terrain_3d_map_height; z_loop ++){
		for(x_loop = 0; x_loop < terrain_3d_map_width; x_loop ++){

			trees_this_sector = trees_each_sector [ z_loop ][ x_loop ].tree_count;

			if(trees_this_sector){

				//printf("trees this sector  %d   %d   %d\n", z_loop, x_loop, trees_this_sector);

				///////////////////////////////////////////////////
				// write tree data file
				///////////////////////////////////////////////////

				for(tree_loop = 0; tree_loop < trees_this_sector; tree_loop ++){

					pt = trees_each_sector [ z_loop ][ x_loop ].
						trees [ tree_loop ];

					sec_pt = get_sector_tree_point( pt );

					local_tree_data.x = sec_pt.x * 2;
					local_tree_data.z = sec_pt.z * 2;
					local_tree_data.y = sec_pt.y * 2;

					fwrite(&local_tree_data, sizeof(terrain_3d_tree_data), 1, tree_data);
					
					offset_counter_next += 1;

					//printf("%d  %d  %d  %d  %d\n", z_loop, x_loop, local_tree_data.x, local_tree_data.z, local_tree_data.y);

					//printf("offset counter next = %d\n", offset_counter_next);


				} // end for tree loop

			} // end if trees this sector

			///////////////////////////////////////////////////////
			// write tree sector file
			///////////////////////////////////////////////////////

			fwrite(&trees_this_sector, sizeof(int), 1, tree_sec);
			fwrite(&offset_counter_last, sizeof(int), 1, tree_sec);
			
			offset_counter_last = offset_counter_next;

		} // end for x loop
	} // end for z loop

	fclose(tree_sec);
	fclose(tree_data);

} // end plant trees on road points

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

sector_tree_point   get_sector_tree_point(vec3d    global_point){

	int
		x_sector,
		z_sector;

	sector_tree_point
		sec_tree_pt;

	//////////////////////////////////////////////////////////////
	// find the sector coordinates
	//////////////////////////////////////////////////////////////

	x_sector = global_point.x / 2048;

	z_sector = global_point.z / 2048;

	sec_tree_pt.x = (signed short int)((global_point.x - (x_sector * 2048 + 1024)) * 16);

	sec_tree_pt.z = (signed short int)((global_point.z - (z_sector * 2048 + 1024)) * 16);

	sec_tree_pt.y = (unsigned short int)((global_point.y - terrain_3d_map_minimum_height) / terrain_3d_map_scaled_height_difference);

	
	return sec_tree_pt;

} // end get sector point

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void add_intermediate_points_to_paths(void){

	int
		path_loop,
		point_loop,
		add_loop,
		temp_path_loop,
		path_count,
		point_count,
		temp_path_point_count,
		temp_path_point_index,
		distance,
		slope,
		this_point_y_value,
		next_point_y_value,
		max_distance,
		best_point_distance,
		number_of_points_to_add;

	vec3d
		this_point,
		next_point,
		add_point;

	float
		increment_dist;

	printf("adding intermediate path points ...\n");

	path_count = global_number_of_divided_paths;

	////////////////////////////////////
	// print section all paths all points
	///////////////////////////////////
/*
	printf("\n  %d paths\n\n", path_count);

	for(path_loop = 0; path_loop < path_count; path_loop ++){
	//for(path_loop = 0; path_loop < 1; path_loop ++){

		point_count = sub_divided_road_paths [ path_loop ].count;

		printf("\npath %d   has  %d  points\n\n", path_loop, point_count);

		for(point_loop = 0; point_loop < point_count; point_loop ++){

			printf(" %d   %f   %f   %f\n", point_loop,
				sub_divided_road_paths [ path_loop ].points [ point_loop ].x,
				sub_divided_road_paths [ path_loop ].points [ point_loop ].z,
				sub_divided_road_paths [ path_loop ].points [ point_loop ].y);

		} // end point loop
	} // end path loop
*/
	for(path_loop = 0; path_loop < path_count; path_loop ++){
	//for(path_loop = 0; path_loop < 1; path_loop ++){

		point_count = sub_divided_road_paths [ path_loop ].count;

		temp_path_point_index = 0;

		if(point_count == 0) continue;

		for(point_loop = 0; point_loop < point_count - 1; point_loop ++){

			this_point = sub_divided_road_paths [ path_loop ].points [ point_loop ] ;

			next_point = sub_divided_road_paths [ path_loop ].points [ point_loop + 1 ] ;

			temp_path_points [ temp_path_point_index ] = this_point;

			temp_path_point_index ++;

			distance = get_span_length( this_point, next_point );
			
			this_point_y_value = (int) get_global_y_value ( this_point );

			next_point_y_value = (int) get_global_y_value ( next_point );

			slope = abs ( ( ( this_point_y_value - next_point_y_value) * 100 ) / distance );

			//printf("slope %d  distance %d\n", slope, distance);

			if( slope > MAX_LEVEL_ROAD_SLOPE &&
				distance > MAX_SLOPED_ROAD_WP_DISTANCE ){
				
				//printf("\n* slope * distance %d  \n\n", distance);

				best_point_distance = get_best_point_distance ( distance, MAX_SLOPED_ROAD_WP_DISTANCE );

				number_of_points_to_add = distance / best_point_distance - 1;
				
			}
			else if( slope > MAX_LEVEL_ROAD_SLOPE &&
				distance <= MAX_SLOPED_ROAD_WP_DISTANCE ){

				number_of_points_to_add = 0;

				best_point_distance = 0;
			}
			else if( distance > MAX_LEVEL_ROAD_WP_DISTANCE ){
				
				best_point_distance = MAX_LEVEL_ROAD_WP_DISTANCE;

				number_of_points_to_add = distance / best_point_distance - 1;
			}
			else{

				number_of_points_to_add = 0;

				best_point_distance = 0;
			}

			//printf("dist %d    pt dist %d    num pts %d\n", distance, best_point_distance, number_of_points_to_add);

			///////////////////////////////////////////////////////////////////////////////
			// add the intermediate points to temp array if we want any
			///////////////////////////////////////////////////////////////////////////////

			if( number_of_points_to_add > 0 ){

				increment_dist = (float) best_point_distance;

				for(add_loop = 0; add_loop < number_of_points_to_add; add_loop ++){

					add_point = get_point_at_distance_d ( this_point, next_point, increment_dist * ( 1 + add_loop ) );

					add_point.y = get_global_y_value ( add_point );

					temp_path_points [ temp_path_point_index ] = add_point; // this is the killer

					//printf("%d   %f   %f   %f\n", temp_path_point_index, add_point.x, add_point.z, add_point.y);

					temp_path_point_index ++;


				} // end for add loop
			} // end if points to add

		} // end for point loop

		///////////////////////////////////////////////////
		// add the last point to temp array
		///////////////////////////////////////////////////

		//temp_path_points [ temp_path_point_count - 1 ] = sub_divided_road_paths [ path_loop ].points [ point_count - 1 ];
		temp_path_points [ temp_path_point_index ] = sub_divided_road_paths [ path_loop ].points [ point_count - 1 ];


		temp_path_point_count = temp_path_point_index + 1;

		////////////////////////////////////////////////////////////////
		// copy the temp path points to final global data
		////////////////////////////////////////////////////////////////

		//printf("\nPath %d  Count %d\n\n", path_loop, temp_path_point_count);

		sub_divided_road_paths [ path_loop ].count = temp_path_point_count;

		for(temp_path_loop = 0; temp_path_loop < temp_path_point_count; temp_path_loop ++){

			sub_divided_road_paths [ path_loop ].points [ temp_path_loop ] = temp_path_points [ temp_path_loop ];

		} // end for temp path loop

	} // end for path loop

	////////////////////////////////////
	// print section all paths all points
	///////////////////////////////////

	printf("\n  %d paths\n\n", path_count);

	for(path_loop = 0; path_loop < path_count; path_loop ++){
	//for(path_loop = 0; path_loop < 1; path_loop ++){

		point_count = sub_divided_road_paths [ path_loop ].count;

		printf("\npath %d   has  %d  points\n\n", path_loop, point_count);

		for(point_loop = 0; point_loop < point_count; point_loop ++){

			printf(" %d   %f   %f   %f\n", point_loop,
				sub_divided_road_paths [ path_loop ].points [ point_loop ].x,
				sub_divided_road_paths [ path_loop ].points [ point_loop ].z,
				sub_divided_road_paths [ path_loop ].points [ point_loop ].y);

		} // end point loop
	} // end path loop

	
} // end add_intermediate_points_to_paths

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////
  
int get_best_point_distance ( int dist, int max_dist ){

	int
		loop,
		remainder,
		number_of_points,
		best_point_distance,
		local_max_distance;

	
	for(loop = 0; loop < 100; loop ++){

		local_max_distance = max_dist - loop * 1;

		best_point_distance = local_max_distance;

		number_of_points = dist / best_point_distance;

		remainder = dist - number_of_points * best_point_distance;

		//printf("remainder %d    point distance %d\n", remainder, best_point_distance);

		if( remainder < 10 ){

			break;
		}
	}

	return best_point_distance;

} // end get_best_best_point_distance

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

vec3d get_point_at_distance_d ( vec3d p, vec3d p1, float d ){

	// gets a point on line (p, p1) at distance d from p

	float
		l;

	vec3d
		return_value;

	l = sqrt(((p1.x - p.x) * (p1.x - p.x)) + ((p1.z - p.z) * (p1.z - p.z)));

	return_value.x = (d / l) * (p1.x - p.x) + p.x;

	return_value.z = (d / l) * (p1.z - p.z) + p.z;

	return return_value;

} // end get_point_at_distance_d




