// roads_and_bridges taken from grand_terraform.h

#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define INSIDE 1
#define OUTSIDE 0

//
// Terrain geometry data scaling factors
//

#define TERRAIN_3D_XZ_SCALE ( ( ( float ) ( TERRAIN_3D_SECTOR_SIDE_LENGTH + TERRAIN_3D_SECTOR_SIDE_LENGTH_ADJUSTMENT ) ) / 65536.0 )

#define TERRAIN_3D_Y_SCALE ( 1.0 / 32767.0 )

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

typedef int terrain_types;	//enum TERRAIN_TYPES terrain_types;

#define  TRUE 1
#define  FALSE 0

#define  EVEN 0
#define  ODD 1

#define  MAX_LEVEL_ROAD_WP_DISTANCE		512
#define  MAX_SLOPED_ROAD_WP_DISTANCE	60
#define  MAX_LEVEL_ROAD_SLOPE			10

////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// Bridge Stuff
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////

struct VEC3D
{

	float
		x,
		y,
		z;
};

typedef struct VEC3D vec3d;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct CONTOUR_PATH
{

	int
		from,
		to,
		type,
		count;

	float
		xmin,
		xmax,
		zmin,
		zmax;

	vec3d
		*points;
};

typedef struct CONTOUR_PATH contour_path;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct NODE_DATA
{
	int
		node_id,
		safe_radius,
		number_of_links,
		list_of_links [ 4 ],
		list_of_cost [ 4 ];

};

typedef struct NODE_DATA node_data;

struct ROAD_PATH
{

	int
		from,
		to,
		type,
		count;

	vec3d
		points [ 512 ];
};

typedef struct ROAD_PATH road_path;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct BRIDGE
{
	vec3d
		end1,
		end2;

	int
		road,
		start_node,
		end_node,
		type;

	float
		max_leg_height;
};

typedef struct BRIDGE bridge;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_TREE_DATA
{

	signed short int
		x,
		z;

	unsigned short int
		y;

	unsigned char
		colour;

	unsigned char
		height:7,
		type:1;
};

typedef struct TERRAIN_3D_TREE_DATA terrain_3d_tree_data;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct SECTOR_TREE_POINT
{

	signed short int
		x,
		z;

	unsigned short int
		y;
	
};

typedef struct SECTOR_TREE_POINT sector_tree_point;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

// there are one of these structures for each sector

struct GLOBAL_COORD_TREE_DATA
{
	int
		tree_count;

	vec3d
		trees [ 64 ];
};

typedef struct GLOBAL_COORD_TREE_DATA global_coord_tree_data;

////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
// terrain data structures
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////


struct TERRAIN_3D_POINT_XZ
{

	short int
		x,
		z;
};

typedef struct TERRAIN_3D_POINT_XZ terrain_3d_point_xz;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct FLOATING_POINT
{
	float
		x,
		z;
};

typedef struct FLOATING_POINT floating_point;


///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

#pragma pack(1)

struct TERRAIN_3D_POINT_Y
{

	union
	{

		struct
		{

			unsigned short int
				y:15,
				normal_change:1;
		};

		unsigned short int
			whole_value;

		short int
			whole_signed_value;
	};
};

typedef struct TERRAIN_3D_POINT_Y terrain_3d_point_y;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR
{

	unsigned char
		red,
		green,
		blue;
};

typedef struct TERRAIN_3D_COLOUR terrain_3d_colour;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_COLOUR_INDEX terrain_3d_colour_index;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_NORMAL_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_NORMAL_INDEX terrain_3d_normal_index;

#pragma pack(4)

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_WORD_REFERENCE
{

	unsigned short int
		point;
};

typedef struct TERRAIN_3D_POINT_WORD_REFERENCE terrain_3d_point_word_reference;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_BYTE_REFERENCE
{

	unsigned char
		point;
};

typedef struct TERRAIN_3D_POINT_BYTE_REFERENCE terrain_3d_point_byte_reference;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_FACE
{

	unsigned char
		number_of_points:7,
		surface_change:1;
};

typedef struct TERRAIN_3D_FACE terrain_3d_face;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_SURFACE
{

	unsigned char
		surface_id;
};

typedef struct TERRAIN_3D_SURFACE terrain_3d_surface;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_SECTOR
{

	unsigned short int
		number_of_points,
		number_of_polygons,
		radius;

	unsigned char
		end_pad_byte_polygons,
		end_pad_byte_surface_changes,
		*normal_changes;

	short int
		minimum_height,
		maximum_height,
		average_height,
		pad_2_bytes,
		//number_of_surface_changes_bytes,
		number_of_surface_ids,
		number_of_normal_changes;

	int
		colour_indices_offset,
		point_colours_offset,
		max_colour_index,
		number_of_point_references;



	struct TERRAIN_3D_POINT_XZ
		*points_xz;

	struct TERRAIN_3D_POINT_Y
		*points_y;

	struct TERRAIN_3D_FACE
		*polygons;

	union
	{

		struct TERRAIN_3D_POINT_WORD_REFERENCE
			*point_word_references;

		struct TERRAIN_3D_POINT_BYTE_REFERENCE
			*point_byte_references;
	};


	//struct TERRAIN_3D_SURFACE
	//	*surface_changes;

	//struct TERRAIN_3D_SURFACE
	//	surface_changes [32];

	struct TERRAIN_3D_SURFACE
		surface_ids[256]; // without &

	struct TERRAIN_3D_SURFACE
		*surface_id_each_face;

	//struct TERRAIN_3D_COLOUR
	//	*point_colours;

	struct TERRAIN_3D_COLOUR
		point_colours[512]; // with &

	struct TERRAIN_3D_COLOUR_INDEX
		*colour_indices; // without &

};

typedef struct TERRAIN_3D_SECTOR terrain_3d_sector;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// my stuff for road polys
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////

struct SECTOR_POINT					
{
	terrain_3d_point_xz		xz_point;

	terrain_3d_point_y		y_point;
};

typedef struct SECTOR_POINT sector_point;

//////////////////////////////////////////////////////////////////////

struct SIMPLE_POINT
{
	//////////////////////////////////////////////////////////////////
	// the letter p-s is the same as the quad structure points p-s
	// the letter a-d is the sector corner points starting with 
	//	  a = the lower left and going counter clockwise
	// the letter R is for intersections with sector boundaries and the
	//    right side of the road
	// the letter L is for intersections with sector boundaries and the
	//    left side of the road
	// the letter e is for intersections with sector boundaries and the
	//    cross section line at the a end of the quad
	// the letter f is for intersections with sector boundaries and the
	//    cross section line at the b end of the quad
	// the letter j is for the three or four corner points of road junctions
	///////////////////////////////////////////////////////////////////
	char
		letter;

	int
		road,
		quad;

	double
		x,
		z,
		y;

	sector_point
		sec_pt;
};

typedef struct SIMPLE_POINT simple_point;

///////////////////////////////////////////////////////////////////

typedef struct {
   double x,y;
} Point;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct POINT
{

	double
		global_x,
		global_z,
		global_y;
	
};

typedef struct POINT point;

//////////////////////////////////////////////////////////////////////

struct QUAD
{
	point
		p,
		q,
		r,
		s;
};

typedef struct QUAD quad;

//////////////////////////////////////////////////////////////////////


struct ROAD
{
	int
		width,
		color,
		number_of_center_points,
		number_of_quads;

	point
		center_points[512];

	quad
		quads[512];
};

typedef struct ROAD road;


