// check_terrain_for_errors
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "check_terrain_for_errors.h"



////////////////////////////////////////
// Globals
////////////////////////////////////////

/////////////////////////////////////////////////
// *** start of Global Lists
/////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// holds geometry data terrain_sectors[z][x]
////////////////////////////////////////////////////////////////////////////

terrain_3d_sector
	**terrain_sectors;


////////////////////////////////////////////////////
// *** end of Global Lists
////////////////////////////////////////////////////

int
	terrain_version,
	rgb_4byte_header,
	ffp_4byte_header,
	terrain_3d_map_width,
	terrain_3d_map_height;

short int
	pad_word_zero;

unsigned short int
	normal_changes_word;



float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;


/////////////////////////////////////////
// functions
/////////////////////////////////////////

int InsidePolygon(simple_point *polygon,int N,simple_point p);

void load_3d_terrain (void);

void check_for_non_triangles_and_16384_errors(int sec_z, int sec_x);


int main(int argc, char ** argv){

	int
		sec_x,
		sec_z;

	load_3d_terrain ();

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			check_for_non_triangles_and_16384_errors( sec_z, sec_x );
			
		}// end for z loop
	} // end for x loop

	return 0;

} // end main

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void check_for_non_triangles_and_16384_errors(int sec_z, int sec_x){

	int poly_loop,
		polys_this_sector,
		point_loop,
		points_this_sector,
		points_this_poly,
		x,
		z;

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	points_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_points;
	
	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		points_this_poly = terrain_sectors[sec_z][sec_x].polygons [ poly_loop ].number_of_points;

		if(points_this_poly != 3){

			printf("sector %d x %d  poly %d has %d points\n", sec_x, sec_z, poly_loop, points_this_poly);
		}
	}

	for(point_loop = 0; point_loop < points_this_sector; point_loop ++){

		x = terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x;

		z =	terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z;
			
		if(x > 16380 && x < 16388 && x != 16384){

			printf("sector %d x %d  point %d  equals %d x %d\n", sec_x, sec_z, point_loop, x, z);
		}

		if(x < -16380 && x > -16388 && x != -16384){

			printf("sector %d x %d  point %d  equals %d x %d\n", sec_x, sec_z, point_loop, x, z);
		}

		if(z > 16380 && z < 16388 && z != 16384){

			printf("sector %d x %d  point %d  equals %d x %d\n", sec_x, sec_z, point_loop, x, z);
		}

		if(z < -16380 && z > -16388 && z != -16384){

			printf("sector %d x %d  point %d  equals %d x %d\n", sec_x, sec_z, point_loop, x, z);
		}
	}
	
} // end check_for_non_triangles_and_16384_errors

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(simple_point *polygon,int N,simple_point p)
{
  int counter = 0;
  int i;
  double xinters;
  simple_point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.z > MIN(p1.z,p2.z)) {
      if (p.z <= MAX(p1.z,p2.z)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.z != p2.z) {
            xinters = (p.z-p1.z)*(p2.x-p1.x)/(p2.z-p1.z)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		sec_x,
		sec_z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		four_byte_zero_word,
		i,
		print_loop,		
		surface_changes_index,
		poly_loop,
		number_of_polys;

	float
		min_height,
		max_height;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius;
		//normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height;

	unsigned char
		pad_byte_zero,
		current_surface_id,
		normal_changes_buf[1024];
	
	long
		current_file_position;


	if(( sec_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[sec_z][sec_x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[sec_z][sec_x].points_y );
			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[sec_z][sec_x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				if(!(terrain_sectors[sec_z][sec_x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[sec_z][sec_x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[sec_z][sec_x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface changes - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			// there is always a zero pad byte at the end of the surface ids
			// if you are not going to recalculate the number of surface ids
			// for every sector then you need to take this into account

			terrain_sectors[sec_z][sec_x].number_of_surface_ids = 
				normal_changes_offset - surface_changes_offset - 1;
			
			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].surface_ids, sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids + 1, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// initialize the surface_id_each_face array
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].surface_id_each_face = 
				malloc(sizeof(terrain_3d_surface) * terrain_sectors[sec_z][sec_x].number_of_polygons))){
				printf("No memory for surface_id_each_face sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

			current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

			terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id = current_surface_id;
			
			surface_changes_index = 0;

			for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

				if(terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change == 1){

					surface_changes_index ++;

					current_surface_id = terrain_sectors[sec_z][sec_x].
						surface_ids[surface_changes_index].surface_id;

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;
				}
				else{

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;

				} // end if else
			} // end for poly loop

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// read in color offsets from sector file

			fread(&terrain_sectors[sec_z][sec_x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[sec_z][sec_x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[sec_z][sec_x].colour_indices = 
				malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			fread(terrain_sectors[sec_z][sec_x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[sec_z][sec_x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[sec_z][sec_x].number_of_points; i++){ 
				
				// there is a colour index for each point

				if((terrain_sectors[sec_z][sec_x].colour_indices + i)->index > 
					terrain_sectors[sec_z][sec_x].max_colour_index){

					terrain_sectors[sec_z][sec_x].max_colour_index = 
						(terrain_sectors[sec_z][sec_x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[sec_z][sec_x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[sec_z][sec_x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////
