#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "grand_terraform.h"

//////////////////////////////
// global data
//////////////////////////////


terrain_3d_sector
	**terrain_sectors;

adjacent_poly_list
	adjacent_list[1024];

contour_lake
	*contour_lakes;	

vec3d
	*contour_lake_vertices;

int
	lake_polys[1337][3],   // [][0] = x sector  [][1] = z sector  [][2] = polygon number

used_texture_list[] = {
	
	TERRAIN_TYPE_RIVER,
	TERRAIN_TYPE_LAND,
	TERRAIN_TYPE_ALTERED_LAND1,
	TERRAIN_TYPE_ALTERED_LAND2,
	TERRAIN_TYPE_ALTERED_LAND3,
	TERRAIN_TYPE_FIELD1,
	TERRAIN_TYPE_BUILT_UP_AREA1,
    TERRAIN_TYPE_BUILT_UP_AREA2,
    TERRAIN_TYPE_BUILT_UP_AREA3,
    TERRAIN_TYPE_BUILT_UP_AREA4
	
};

int
	number_of_textures_used = 10,
	colour_file_offset_counter = 0,
	terrain_3d_map_width,
	terrain_3d_map_height,
	rgb_4byte_header,
	ffp_4byte_header,
	terrain_version,
	number_of_colors = 13,
	global_max_polys = 0,
	global_max_polys_sector[2]; // [0] = x  [1] = z
	global_min_slope,
	global_max_slope,
	global_slopes_10_20 = 0,
	global_slopes_20_30 = 0,
	global_slopes_30_40 = 0,
	global_slopes_40_50 = 0,
	global_slopes_50_60 = 0,
	global_slopes_60_70 = 0,
	global_slopes_70_80 = 0,
	global_slopes_over_80 = 0,
	odd_ball_count = 0,
	number_of_contour_lakes = 1,
	number_of_lake_triangles = 0,
	global_map_height;

	

short int
	pad_word_zero;

unsigned short int
	normal_changes_word;



float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;

double
	global_lowest_elevation,
	global_highest_elevation;


///////////////////////////////
// Functions
///////////////////////////////

void load_3d_terrain (void);

void print_sector_data(int sec_z, int sec_x);

void sort_sector_polys_by_surface( int sec_z, int sec_x );

void write_new_terrain_files(void);

void assign_colour_data(int sec_z, int sec_x);

void assign_surface_ids(int sec_z, int sec_x);

double normal_angle_with_y(int sec_z, int sec_x, int poly);

int normal_angle_with_x(int sec_z, int sec_x, int poly);

int normal_angle_with_z(int sec_z, int sec_x, int poly);

int facing_direction(int sec_z, int sec_x, int poly);

int InsidePolygon(Point *polygon,int N,Point p);

void get_terrain_info(int sec_z, int sec_x);

void post_process_surface_ids(int sec_z, int sec_x); 

void create_lakes_file(void);

void read_lakes_file(void);

//int qsort_compare(const world_coord_triangle *t1, const world_coord_triangle *t2);

void find_lake_triangles(int sec_z, int sec_x);

void print_terrain_info(void);

void print_surface_ids_of_sector(int sec_z, int sec_x);

void print_terrain_data_of_sector(int sec_z, int sec_x);

void equalize_colors_at_sector_junctions ( void );

void print_colors_of_sector(int sec_z, int sec_x);



int main(int argc, char ** argv)
{
	int
		sec_x,
		sec_z;

	char
		filename [16];

	load_3d_terrain ();

	printf("Assigning surface IDs ...\n");

	//print_surface_ids_of_sector( 56,  99);

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			assign_surface_ids( sec_z, sec_x );
			
		}// end for z loop
	} // end for x loop

	//print_surface_ids_of_sector( 56,  99);

	printf("Post processing surface IDs ...\n");

	
	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			post_process_surface_ids( sec_z, sec_x );

		}// end for z loop
	} // end for x loop

	printf("Sorting by surface ...\n");

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			sort_sector_polys_by_surface ( sec_z, sec_x );

		}// end for z loop
	} // end for x loop

	printf("Assigning color data ...\n");

	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			assign_colour_data( sec_z,  sec_x);

		}// end for z loop
	} // end for x loop

	printf("Equalizing colors ...\n");

	//print_colors_of_sector(55, 100);

	equalize_colors_at_sector_junctions ( );

	//print_colors_of_sector(55, 100);

	//print_surface_ids_of_sector(33, 85);

	write_new_terrain_files();

	printf("%d odd polygons fixed\n", odd_ball_count);

	
	return 0;
} // end main

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void assign_surface_ids(int sec_z, int sec_x){

	// this function is called once for each sector

	int poly_loop,
		polys_this_sector,
		slope,
		y_point0, //sector coordinates
		y_point1,
		y_point2,
		field11_found_for_this_sector = 0,
		sector_bool;

	double	elevation,
			min_elevation,
			max_elevation,
			y0,  // world coordinates, actual elevation in meters
			y1,
			y2;

	float	terrain_3d_map_scaled_height_difference;
	
	terrain_3d_map_scaled_height_difference = ( terrain_3d_map_maximum_height - 
		terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;

	//
	// for making checker board pattern with the sectors
	//

	if( sec_z % 2 == 0 ){ // even row

		if( sec_x % 2 == 0){ // even column

			sector_bool = EVEN;
		}
		else{

			sector_bool = ODD;
		}
	}
	else{ // odd row

		if( sec_x % 2 == 0){ // even column

			sector_bool = ODD;
		}
		else{

			sector_bool = EVEN;
		}
	}

	//////////////////////////////////////////////////////////////////////
	// loop thru all polys and assign surface ids
	//////////////////////////////////////////////////////////////////////

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){
		
		// real world elevation

		y0 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y1 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y2 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point].y * 
			terrain_3d_map_scaled_height_difference ));
		
		// sector coordinate elevation
	
		y_point0 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point].y;

		y_point1 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point].y;

		y_point2 = terrain_sectors[sec_z][sec_x].points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point].y;

		// find the average elevation of the poly

		min_elevation = y0;
		if(y1 < min_elevation) min_elevation = y1;
		if(y2 < min_elevation) min_elevation = y2;

		max_elevation = y0;
		if(y1 > max_elevation) max_elevation = y1;
		if(y2 > max_elevation) max_elevation = y2;


		//elevation = (int)(min_elevation + max_elevation) / 2;
		elevation = min_elevation;

		slope = normal_angle_with_y(sec_z, sec_x, poly_loop);

		//////////////////////////////////////////////////////////////////////////////
		// this 'if else' chain is where you select what id you want for each face
		// the order is important ... think about what you want eliminated from the
		// selection first, the surface_id_each_face[ poly_loop ] array isn't 
		// used by EECH, it's only temporary for this program
		//////////////////////////////////////////////////////////////////////////////

		//printf("%d   %d\n", poly_loop, 
		//	terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
		//		surface_id);

		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_RIVER){ // for processing importer files

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND3;
		}

		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ROAD){ 

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_FIELD1; 
		}

		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_FIELD1){ 

			// no change 
		}

		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND3){ 

			// no change , for re-processing grand_terraform files
		}

		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND2){ 

			// no change, used for the arch

		}

		//
		// this is for the normal terrain
		//

		
        else if(slope > 35 ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND1;
		}

		else{

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_LAND;
		}
/*
		//
		// temporary for trying experiments
		//

		else if( sec_x == 48 && sec_z == 42 ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_BUILT_UP_AREA1;
		}

		else if( sec_x == 15 && sec_z == 26 ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_BUILT_UP_AREA2;
		}

		else if( sec_x == 50 && sec_z == 18 ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_BUILT_UP_AREA3;
		}

		else if( sec_x == 75 && sec_z == 11 ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_BUILT_UP_AREA4;
		}

		//
		// this is for the checkered board pattern
		//

		else if( sector_bool == EVEN ){

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_ALTERED_LAND2;
		}
		else{

			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
				surface_id = TERRAIN_TYPE_LAND;
		}
*/
	} // end for poly loop

	
} // end assign_surface_ids

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////


void assign_colour_data(int sec_z, int sec_x){

	int
		poly_loop,
		number_of_polys,
		average_y, // real elevation in meters
		y0,
		y1,
		y2,
		flat_dark_rand_counter,
		flat_medium_rand_counter,
		flat_light_rand_counter,
		steep_dark_rand_counter,
		steep_medium_rand_counter,
		steep_light_rand_counter;

	terrain_3d_colour
		surface_0_colour,	
		surface_1_colour,
		surface_2_colour,
		surface_3_colour,
		surface_4_colour,	
		surface_5_colour,
		surface_6_colour,
		surface_7_colour,	
		surface_8_colour,
		tree_colour_0,
		tree_colour_1,
		tree_colour_2,
		tree_colour_3;

	/////////////////////////////////////////////////////////////

	surface_0_colour.red = 216; // lighter flat terrain color
	surface_0_colour.green = 185;
	surface_0_colour.blue = 156;

	surface_1_colour.red = 206; // middle flat terrain color
	surface_1_colour.green = 180;
	surface_1_colour.blue = 152;

	surface_2_colour.red = 196; // darker flat terrain color
	surface_2_colour.green = 165;
	surface_2_colour.blue = 136;
/*
	surface_0_colour.red = 100; // lighter flat terrain color
	surface_0_colour.green = 0;
	surface_0_colour.blue = 0;

	surface_1_colour.red = 0; // middle flat terrain color
	surface_1_colour.green = 100;
	surface_1_colour.blue = 0;

	surface_2_colour.red = 0; // darker flat terrain color
	surface_2_colour.green = 0;
	surface_2_colour.blue = 100;

*/
	//////////////////////////////////////////////////////////////


	surface_3_colour.red = 222; // lighter steep terrain color
	surface_3_colour.green = 171;
	surface_3_colour.blue = 140;

	surface_4_colour.red = 194; // middle steep terrain color
	surface_4_colour.green = 143;
	surface_4_colour.blue = 112;

	surface_5_colour.red = 194; // darker steep terrain color
	surface_5_colour.green = 127;
	surface_5_colour.blue = 86;

	surface_6_colour.red = 255; // white
	surface_6_colour.green = 255;
	surface_6_colour.blue = 255;

	//surface_7_colour.red = 150; // grey for road
	//surface_7_colour.green = 150;
	//surface_7_colour.blue = 150;

	surface_7_colour.red = 201; // grey for road
	surface_7_colour.green = 174;
	surface_7_colour.blue = 147;


	//surface_8_colour.red = 255; // spare
	//surface_8_colour.green = 255;
	//surface_8_colour.blue = 255;

	surface_8_colour.red = 0; // water
	surface_8_colour.green = 79;
	surface_8_colour.blue = 105;

	tree_colour_0.red = 184; // green
	tree_colour_0.green = 225;
	tree_colour_0.blue = 131;

	tree_colour_1.red = 246; // orange
	tree_colour_1.green = 163;
	tree_colour_1.blue = 8;

	tree_colour_2.red = 246; // red
	tree_colour_2.green = 58;
	tree_colour_2.blue = 8;

	tree_colour_3.red = 246; // yellow
	tree_colour_3.green = 240;
	tree_colour_3.blue = 8;

	terrain_sectors[sec_z][sec_x].point_colours[0] = surface_0_colour;
	terrain_sectors[sec_z][sec_x].point_colours[1] = surface_1_colour;
	terrain_sectors[sec_z][sec_x].point_colours[2] = surface_2_colour;
	terrain_sectors[sec_z][sec_x].point_colours[3] = surface_3_colour;
	terrain_sectors[sec_z][sec_x].point_colours[4] = surface_4_colour;
	terrain_sectors[sec_z][sec_x].point_colours[5] = surface_5_colour;
	terrain_sectors[sec_z][sec_x].point_colours[6] = surface_6_colour;
	terrain_sectors[sec_z][sec_x].point_colours[7] = surface_7_colour;
	terrain_sectors[sec_z][sec_x].point_colours[8] = surface_8_colour;
	terrain_sectors[sec_z][sec_x].point_colours[9] = tree_colour_0;
	terrain_sectors[sec_z][sec_x].point_colours[10] = tree_colour_1;
	terrain_sectors[sec_z][sec_x].point_colours[11] = tree_colour_2;
	terrain_sectors[sec_z][sec_x].point_colours[12] = tree_colour_3;

	////////////////////////////////////////////////////////////////////////
	// loop through all polygons
	////////////////////////////////////////////////////////////////////////

	flat_dark_rand_counter = 0;
	flat_medium_rand_counter = 0;
	flat_light_rand_counter = 0;

	steep_dark_rand_counter = 0;
	steep_medium_rand_counter = 0;
	steep_light_rand_counter = 0;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){ 

		y0 = (int)(terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[poly_loop * 3].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y1 = (int)(terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[poly_loop * 3 + 1].point].y * 
			terrain_3d_map_scaled_height_difference ));

		y2 = (int)(terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[terrain_sectors[sec_z][sec_x].
			point_word_references[poly_loop * 3 + 2].point].y * 
			terrain_3d_map_scaled_height_difference ));

		average_y = (int)(y0 + y1 + y2) / 3;

		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND1){ // steep

			if(steep_dark_rand_counter > 0 ){

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 5; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 5;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 5;

				if(steep_dark_rand_counter == 7){

					steep_dark_rand_counter = 0;
				}
				else{

					steep_dark_rand_counter ++;
				}
			}

			else if( (rand() % 8) == 0 ){ // darker
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 5; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 5;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 5;

				steep_dark_rand_counter = 1;
			}
			
			else if( (rand() % 4) == 0 ){ // medium
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 4; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 4;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 4;
			}
			else { // lightest
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 3; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 3;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 3;
			}
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_LAND){ // flat
			
			if(flat_dark_rand_counter > 0 ){

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 2; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 2;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 1;

				if(flat_dark_rand_counter == 7){

					flat_dark_rand_counter = 0;
				}
				else{

					flat_dark_rand_counter ++;
				}
			}

			else if( (rand() % 32) == 0 ){ // darker
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 2; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 2;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 1;

				flat_dark_rand_counter = 1;
			}
			else if( (rand() % 4) == 0 ){ // medium
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 1; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 1;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 1;
			}
			else { // lightest
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 0; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 0;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 0;
			}
	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND2){ // arch

			// same as steep color

			if( (rand() % 8) == 0 ){ // darker
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 5; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 5;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 5;
			}
			else if( (rand() % 4) == 0 ){ // medium
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 4; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 4;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 4;
			}
			else { // lightest
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 3; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 3;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 3;
			}
/*
			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 6;
*/	
		}
		else if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].
			surface_id == TERRAIN_TYPE_ALTERED_LAND3){ // river

			// the colors doesn't do anything for alt land3 but I guess every point
			// needs a color asignment so better assign something to be safe
			// need to change this later

			// same as steep color

			if( (rand() % 8) == 0 ){ // darker
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 5; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 5;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 5;
			}
			else if( (rand() % 4) == 0 ){ // medium
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 4; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 4;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 4;
			}
			else { // lightest
			
				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 ].
					point  ].index = 3; // 3

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 1 ].
					point  ].index = 3;

				terrain_sectors[sec_z][sec_x].
					colour_indices[  terrain_sectors[sec_z][sec_x].
					point_word_references[ poly_loop * 3 + 2 ].
					point  ].index = 3;
			}
/*
			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 6;
*/	
		}
		else{	// TERRAIN_TYPE_RIVER not being used
				// TERRAIN_TYPE_FIELD1 used for TERRAIN_TYPE_ROAD for now
				// and anything else not specifically assigned

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 1 ].
				point  ].index = 6;

			terrain_sectors[sec_z][sec_x].
				colour_indices[  terrain_sectors[sec_z][sec_x].
				point_word_references[ poly_loop * 3 + 2 ].
				point  ].index = 6;
		}

	} // end for poly loop

	//////////////////////////////////////////////////////////////
	// assign colour file offset values
	//////////////////////////////////////////////////////////////

	terrain_sectors[sec_z][sec_x].colour_indices_offset = colour_file_offset_counter;

	colour_file_offset_counter += terrain_sectors[sec_z][sec_x].number_of_points
		* sizeof(terrain_3d_colour_index);

	terrain_sectors[sec_z][sec_x].point_colours_offset = colour_file_offset_counter;

	// multiple needs to be the number of colors used

	colour_file_offset_counter += 13 * sizeof(terrain_3d_colour); 

} // end assign colour data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void write_new_terrain_files(void){

	FILE
		*sec_out,
		*ffp_out,
		*rgb_out;

	int
		sec_x,
		sec_z,
		offset,
		four_byte_zero_word = 0;

	unsigned short int
		map_width,
		map_height;

	unsigned char
		pad_byte_zero = 0;

	/////////////////////////////////////////////////////////

	printf("Writing new terrain files ...\n");

	map_width = (unsigned short int) terrain_3d_map_width;
	map_height = (unsigned short int) terrain_3d_map_height;

	//colour_file_offset_counter = 0;

	if(( sec_out = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.sec","wb"))==NULL){
		printf("cannot open C:\\Razorworks\\common\\maps\\map31\\terrain\\default.sec\n");
		exit(1);
	}
	else printf("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.sec opened\n");

	if(( ffp_out = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\terrain.ffp","wb"))==NULL){
		printf("cannot open C:\\Razorworks\\common\\maps\\map31\\terrain\\terrain.ffp\n");
		exit(1);
	}
	else printf("C:\\Razorworks\\common\\maps\\map31\\terrain\\terrain.ffp opened\n");

	if(( rgb_out = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.rgb","wb"))==NULL){
		printf("cannot open C:\\Razorworks\\common\\maps\\map31\\terrain\\default.rgb\n");
		exit(1);
	}
	else printf("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.rgb opened\n");

	fwrite(&terrain_version, sizeof(int), 1, sec_out);
	fwrite(&map_width, sizeof(unsigned short int), 1, sec_out);
	fwrite(&map_height, sizeof(unsigned short int), 1, sec_out);
	fwrite(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_out);
	fwrite(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_out);
	fwrite(&ffp_4byte_header, sizeof(int), 1, ffp_out);


	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){

	
			//printf("\nSECTOR  %d x %d\n\n", sec_x, sec_z);
		

			/////////////////////////////////////////////////////////////////
			// write all the stats for each sector to sector file
			/////////////////////////////////////////////////////////////////

			fwrite(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_out);
		
			//printf("%d points\n", terrain_sectors[sec_z][sec_x].number_of_points); 

			fwrite(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_out);

			//printf("%d pad_2_bytes\n", terrain_sectors[sec_z][sec_x].pad_2_bytes); 

			fwrite(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_out);

			//printf("%d polygons\n", terrain_sectors[sec_z][sec_x].number_of_polygons); 

			fwrite(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_out);

			//printf("%d radius\n", terrain_sectors[sec_z][sec_x].radius);

			fwrite(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_out);

			//printf("%d minimum_height\n", terrain_sectors[sec_z][sec_x].minimum_height);

			fwrite(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_out);

			//printf("%d maximum_height\n", terrain_sectors[sec_z][sec_x].maximum_height);

			fwrite(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_out);

			//printf("%d average_height\n", terrain_sectors[sec_z][sec_x].average_height);

			fwrite(&pad_word_zero, sizeof(short int), 1, sec_out);

			///////////////////////////////////////////////////////////////////
			// write the geometry data to the ffp file
			///////////////////////////////////////////////////////////////////

			offset = (int)ftell(ffp_out);

			//printf("offset to points y= %d\n", offset);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			fwrite(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

			offset = (int)ftell(ffp_out);

			//printf("offset to points_xz = %d\n", offset);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			fwrite(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_out);

			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to point_word_references = %d\n", offset);

			fwrite(terrain_sectors[sec_z][sec_x].point_word_references,
				(sizeof(terrain_3d_point_word_reference) * 3),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_out);

			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to polygons = %d\n", offset);

			fwrite(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face), // num points & surface changes
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_out); 
			
			//
			// if number of polygons is odd need to add a 0 pad byte
			//

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){

				fwrite( &pad_byte_zero, 1, 1, ffp_out);

			}
			
			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to surface_ids = %d\n", offset);

			fwrite(terrain_sectors[sec_z][sec_x].surface_ids, // better check if & is needed
				sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids,
				ffp_out);

			//printf("number of surface ids = %d\n", 
			//	terrain_sectors[sec_z][sec_x].number_of_surface_ids);

			// write the zero byte at the end of surface changes

			// it seems to work either way with or without the pad_byte_zero

			fwrite(&pad_byte_zero, sizeof(pad_byte_zero), 1, ffp_out);

			offset = (int)ftell(ffp_out);

			fwrite(&offset, sizeof(int), 1, sec_out); // sector file

			//printf("offset to normal_changes = %d\n", offset);
			
			fwrite(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_out);


			// write color offsets to sector file

			//printf("colour indices offset %d\n", 
			//	terrain_sectors[sec_z][sec_x].colour_indices_offset);

			//printf("point colours offset %d\n", 
			//	terrain_sectors[sec_z][sec_x].point_colours_offset);




			fwrite(&terrain_sectors[sec_z][sec_x].colour_indices_offset,
				sizeof(int), 1, sec_out); // sector file

			fwrite(&terrain_sectors[sec_z][sec_x].point_colours_offset,
				sizeof(int), 1, sec_out); // sector file


			// write out colour file

			// had a bad bug because I had a & in front of terrain_sectors ... colour_indices
			// some day I need to figure this & thing out !!!!



			fwrite(terrain_sectors[sec_z][sec_x].colour_indices, 
				sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_out);

			fwrite(&terrain_sectors[sec_z][sec_x].point_colours, 
				sizeof(terrain_3d_colour),	13, rgb_out);
			// multiple needs to be the number of colors used ( 13 )


		} // end for sec_x loop
	} // end for sec_z loop

	// write out the last four bytes of sector file (four bytes of zeros)

	fwrite(&four_byte_zero_word, sizeof(int), 1, sec_out);

	
	fclose(sec_out);
	
	fclose(ffp_out);

	fclose(rgb_out);



} // end write new terrain files

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void sort_sector_polys_by_surface( int sec_z, int sec_x ){

	int
		texture_loop,
		poly_loop,
		number_of_polys,
		temp_poly_array_index = 0,
		word_ref_index = 0,
		current_surface_id,
		//surface_changes_count,
		surface_ids_count;

	terrain_3d_face
		temp_sorted_polygons[1024];

	terrain_3d_point_word_reference
		temp_sorted_word_ref[3072];

	terrain_3d_surface
		temp_sorted_surface_id_each_face[1024];

	number_of_polys = terrain_sectors [sec_z][sec_x].number_of_polygons;

	for(texture_loop = 0; texture_loop < number_of_textures_used; texture_loop ++){

		for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

			if(terrain_sectors [sec_z][sec_x].surface_id_each_face[poly_loop].surface_id == 
				used_texture_list [ texture_loop ]){

				temp_sorted_polygons [ temp_poly_array_index ].number_of_points =
					terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].number_of_points;

				temp_sorted_polygons [ temp_poly_array_index ].surface_change =
					terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].surface_change;

				////////////////////////////////////////

				temp_sorted_surface_id_each_face [ temp_poly_array_index ].surface_id = 
					terrain_sectors [sec_z][sec_x].surface_id_each_face [ poly_loop ].surface_id;

				////////////////////////////////////////

				temp_poly_array_index ++;

				////////////////////////////////////////

				temp_sorted_word_ref [ word_ref_index ].point = 
					terrain_sectors [sec_z][sec_x].
					point_word_references [ poly_loop * 3 ].point;

				word_ref_index ++;

				temp_sorted_word_ref [ word_ref_index ].point = 
					terrain_sectors [sec_z][sec_x].
					point_word_references [ poly_loop * 3 + 1 ].point;

				word_ref_index ++;

				temp_sorted_word_ref [ word_ref_index ].point = 
					terrain_sectors [sec_z][sec_x].
					point_word_references [ poly_loop * 3 + 2 ].point;

				word_ref_index ++;
		
			} // end if
		} // end for poly loop 
	} // end for texture loop

	/////////////////////////////////////////////////////////
	// copy temp arrays back to primary data
	/////////////////////////////////////////////////////////

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].number_of_points =
			temp_sorted_polygons [ poly_loop ].number_of_points;

		terrain_sectors [sec_z][sec_x].polygons [ poly_loop ].surface_change =
			temp_sorted_polygons [ poly_loop ].surface_change;

		/////////////////////////////////////////////

		terrain_sectors [sec_z][sec_x].surface_id_each_face[poly_loop].surface_id = 
			temp_sorted_surface_id_each_face[poly_loop].surface_id;

		/////////////////////////////////////////////

		terrain_sectors [sec_z][sec_x].point_word_references[poly_loop * 3].point =
			temp_sorted_word_ref[poly_loop * 3].point;

		terrain_sectors [sec_z][sec_x].point_word_references[poly_loop * 3 + 1].point =
			temp_sorted_word_ref[poly_loop * 3 + 1].point;

		terrain_sectors [sec_z][sec_x].point_word_references[poly_loop * 3 + 2].point =
			temp_sorted_word_ref[poly_loop * 3 + 2].point;

	} // end for poly loop

	/////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////
	// add surface id's and change the surface change bits at proper spot
	/////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////

	terrain_sectors[sec_z][sec_x].number_of_surface_ids = 1;

	terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id = 
		terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id;

	current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

	surface_ids_count = 0;

	//////////////////////////////////////////////////////////////////////////////////////
	// loop thru polys 1-polys_this_sector
	// find surface changes
	// set terrain_sectors[sec_z][sec_x].surface_ids[surface_ids_count].surface_id
	// toggle terrain_sectors[sec_z][sec_x].polygons[x].surface_change
	//////////////////////////////////////////////////////////////////////////////////////

	//
	// it's weird I don't know why the first surface_change for the
	// imported sectors is getting set to 1 so I'm setting it back to 0 here
	// it causes the game to crash to desktop
	//

	terrain_sectors[sec_z][sec_x].polygons[ 0 ].surface_change = 0;

	for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

		if(terrain_sectors[sec_z][sec_x].
			surface_id_each_face[poly_loop].surface_id != current_surface_id){

			current_surface_id = terrain_sectors[sec_z][sec_x].
				surface_id_each_face[poly_loop].surface_id;

			terrain_sectors[sec_z][sec_x].number_of_surface_ids++;

			surface_ids_count++;

			terrain_sectors[sec_z][sec_x].
				surface_ids[surface_ids_count].surface_id = current_surface_id;

			terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change = 1;
		} // end if
		else{

			// get rid of original surface_change bits set because order is certain to be different

			terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change = 0;
		}
	} // end for

} // end sort sector polys by surface


//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		sec_x,
		sec_z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		four_byte_zero_word,
		i,
		print_loop,		
		surface_changes_index,
		poly_loop,
		number_of_polys;

	float
		min_height,
		max_height;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius;
		//normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height;

	unsigned char
		pad_byte_zero,
		current_surface_id,
		normal_changes_buf[1024];
	
	long
		current_file_position;

	////////////////////////////////////////////////////////////////////////////
	// initialize variables
	////////////////////////////////////////////////////////////////////////////

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	if(( sec_in = fopen("default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[sec_z][sec_x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[sec_z][sec_x].points_y );
			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[sec_z][sec_x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				if(!(terrain_sectors[sec_z][sec_x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[sec_z][sec_x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[sec_z][sec_x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface changes - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			// there is always a zero pad byte at the end of the surface ids
			// if you are not going to recalculate the number of surface ids
			// for every sector then you need to take this into account

			terrain_sectors[sec_z][sec_x].number_of_surface_ids = 
				normal_changes_offset - surface_changes_offset - 1;
			
			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].surface_ids, sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids + 1, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// initialize the surface_id_each_face array
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].surface_id_each_face = 
				malloc(sizeof(terrain_3d_surface) * terrain_sectors[sec_z][sec_x].number_of_polygons))){
				printf("No memory for surface_id_each_face sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

			current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

			terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id = current_surface_id;
			
			surface_changes_index = 0;

			for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

				if(terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change == 1){

					surface_changes_index ++;

					current_surface_id = terrain_sectors[sec_z][sec_x].
						surface_ids[surface_changes_index].surface_id;

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;
				}
				else{

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;

				} // end if else
			} // end for poly loop

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);
/*
			if((sec_z == map_height - 1) && (sec_x == map_width - 1)){

				// this method for when data is the last chunk of the ffp file

				i = 0;

				//printf("normal changes fp = %d\n", (int)ftell(ffp_in));

				while(!feof(ffp_in)){

					//fread((normal_changes_buf + i), sizeof(unsigned char), 1, ffp_in);

					normal_changes_buf[i] = getc(ffp_in);

					i++;

					//printf("i = %d		%d\n", i,normal_changes_buf[i]);
					//if(i > 50)exit(1);

				} // end while

				terrain_sectors[sec_z][sec_x].number_of_normal_changes = i;

				if(!(terrain_sectors[sec_z][sec_x].normal_changes = malloc(i))){
					printf("No memory for normal_changes sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				for(i = 0; i < terrain_sectors[sec_z][sec_x].number_of_normal_changes; i++){
					*(terrain_sectors[sec_z][sec_x].normal_changes + i) = normal_changes_buf[i];
				}

			} // end if
			else{

				// this method for when data is NOT the last chunk of the ffp file

				// need to get the next ffp file offset for the size 
				// of normal changes data calculation.
				// this doesn't work for original maps because they may jump back in the ffp
				// file to re-use a previous sector that I assume must be identical

				fseek(sec_in, 24, SEEK_CUR);
				fread(&offset, sizeof(int), 1, sec_in); // sector file
				fseek(sec_in, -28, SEEK_CUR);

				//printf("offset = %d		normal_changes_offset = %d\n", offset, normal_changes_offset);

				terrain_sectors[sec_z][sec_x].number_of_normal_changes = offset - normal_changes_offset;

				//printf("number_of_normal_changes = %d\n", 
				//	terrain_sectors[sec_z][sec_x].number_of_normal_changes); 

				if(!(terrain_sectors[sec_z][sec_x].normal_changes = 
					malloc(terrain_sectors[sec_z][sec_x].number_of_normal_changes))){
					printf("No memory for normal_changes sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("normal changes fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].normal_changes,	
					sizeof(*terrain_sectors[sec_z][sec_x].normal_changes),
					terrain_sectors[sec_z][sec_x].number_of_normal_changes, ffp_in);

			} // end else
*/
			// read in color offsets from sector file

			fread(&terrain_sectors[sec_z][sec_x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[sec_z][sec_x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[sec_z][sec_x].colour_indices = 
				malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			fread(terrain_sectors[sec_z][sec_x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[sec_z][sec_x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[sec_z][sec_x].number_of_points; i++){ 
				
				// there is a colour index for each point

				if((terrain_sectors[sec_z][sec_x].colour_indices + i)->index > 
					terrain_sectors[sec_z][sec_x].max_colour_index){

					terrain_sectors[sec_z][sec_x].max_colour_index = 
						(terrain_sectors[sec_z][sec_x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[sec_z][sec_x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[sec_z][sec_x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void print_sector_data(int sec_z, int sec_x){

	int
		number_of_points,
		number_of_polys,
		point_loop,
		poly_loop,
		color_index;

	number_of_points =  terrain_sectors[sec_z][sec_x].number_of_points;

	number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

	printf("\nSector %d x %d --- %d points  %d polygons \n\n",
		sec_x, sec_z, number_of_points, number_of_polys);

	printf("Point  X  Z  Y  Color index  Color\n\n");

	for(point_loop = 0; point_loop < number_of_points; point_loop ++){

		color_index = terrain_sectors[sec_z][sec_x].colour_indices [ point_loop ].index;

		printf("%d    %d  %d  %d     %d     %d %d %d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].x,
			terrain_sectors[sec_z][sec_x].points_xz[point_loop].z,
			terrain_sectors[sec_z][sec_x].points_y[point_loop].y,
			color_index,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].red,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].green,
			terrain_sectors[sec_z][sec_x].point_colours[ color_index ].blue);
	}

	printf("\nPolygon  Surface  Reference 0  Reference 1  Reference 2\n\n");

	for(poly_loop = 0; poly_loop < number_of_polys; poly_loop ++){

		printf("%d  %d  %d  %d  %d\n", poly_loop,
			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id,
			terrain_sectors[sec_z][sec_x].point_word_references[poly_loop * 3].point,
			terrain_sectors[sec_z][sec_x].point_word_references[poly_loop * 3 + 1].point,
			terrain_sectors[sec_z][sec_x].point_word_references[poly_loop * 3 + 2].point);

	} // end for poly loop


} // end print sector data

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void get_terrain_info(int sec_z, int sec_x){

	int
		max_slope = 0,
		min_slope = 90,
		slope_differential,
		slopes_10_20 = 0,
		slopes_20_30 = 0,
		slopes_30_40 = 0,
		slopes_40_50 = 0,
		slopes_50_60 = 0,
		slopes_60_70 = 0,
		slopes_70_80 = 0,
		slopes_over_80 = 0,
		elevation;

	int
		poly_loop,
		polys_this_sector,
		slope,
		y_point0,
		y_point1,
		y_point2;

	double
		highest_elevation = 0.0,
		lowest_elevation = 50000.0,
		min_elevation,
		max_elevation,
		y0,
		y1,
		y2;

	float
		terrain_3d_map_scaled_height_difference;

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * 
		TERRAIN_3D_Y_SCALE;



	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		// real world elevation

		y0 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].
			point  ].y * terrain_3d_map_scaled_height_difference ));

		y1 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].
			point  ].y * terrain_3d_map_scaled_height_difference ));

		y2 = (terrain_3d_map_minimum_height + (terrain_sectors[sec_z][sec_x].
			points_y[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].
			point  ].y * terrain_3d_map_scaled_height_difference ));

		// sector coordinate elevation
		
		y_point0 = terrain_sectors[sec_z][sec_x].
			points_y[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point  ].y;

		y_point1 = terrain_sectors[sec_z][sec_x].
			points_y[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point  ].y;

		y_point2 = terrain_sectors[sec_z][sec_x].
			points_y[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point  ].y;

		// find the average elevation of the poly

		min_elevation = y0;
		if(y1 < min_elevation) min_elevation = y1;
		if(y2 < min_elevation) min_elevation = y2;

		max_elevation = y0;
		if(y1 > max_elevation) max_elevation = y1;
		if(y2 > max_elevation) max_elevation = y2;


		//elevation = (int)(min_elevation + max_elevation) / 2;
		//elevation = (int)min_elevation;

		slope = normal_angle_with_y(sec_z, sec_x, poly_loop);

		if(max_elevation > highest_elevation) highest_elevation = max_elevation;
		if(min_elevation < lowest_elevation) lowest_elevation = min_elevation;
		if(max_elevation > global_highest_elevation) global_highest_elevation = max_elevation;
		if(min_elevation < global_lowest_elevation) global_lowest_elevation = min_elevation;

		if(slope > 10 && slope <= 20){ slopes_10_20 ++; global_slopes_10_20 ++; }
		if(slope > 20 && slope <= 30){ slopes_20_30 ++; global_slopes_20_30 ++; }
		if(slope > 30 && slope <= 40){ slopes_30_40 ++; global_slopes_30_40 ++; }
		if(slope > 40 && slope <= 50){ slopes_40_50 ++; global_slopes_40_50 ++; }
		if(slope > 50 && slope <= 60){ slopes_50_60 ++; global_slopes_50_60 ++; }
		if(slope > 60 && slope <= 70){ slopes_60_70 ++; global_slopes_60_70 ++; }
		if(slope > 70 && slope <= 80){ slopes_70_80 ++; global_slopes_70_80 ++; }
		if(slope > 80 ){ slopes_over_80 ++; global_slopes_over_80 ++; }
		if(slope < min_slope) min_slope = slope;
		if(slope > max_slope) max_slope = slope;
		if(slope > global_max_slope) global_max_slope = slope;
		if(slope < global_min_slope) global_min_slope = slope;
		
		if(polys_this_sector > global_max_polys){

			global_max_polys = polys_this_sector;

			global_max_polys_sector[0] = sec_x;
			global_max_polys_sector[1] = sec_z;
		}

		//if(sec_x == 100 && sec_z == 33){ 

		//	printf("%d  %d  %d  %d  %d  %d\n",
		//		x, terrain_sectors[sec_z][sec_x].surface_id_each_face[ poly_loop ].surface_id,
		//		slope, y_point0, y_point1, y_point2);
		//}
		
	} // end for poly loop

	//printf("\nEnd of sector info\n\n");

	////////////////////////////////////////////////////////////
	// end poly loop
	////////////////////////////////////////////////////////////

	printf("\nSector %d x %d number of polys = %d\n", sec_x, sec_z, polys_this_sector);
	printf("elevation %d - %d\n", (int)lowest_elevation, (int)highest_elevation);
	printf("slope     %d - %d\n", min_slope, max_slope);
	printf("slopes 10 to 20 %d\n", slopes_10_20 );
	printf("slopes 20 to 30 %d\n", slopes_20_30 );
	printf("slopes 30 to 40 %d\n", slopes_30_40 );
	printf("slopes 40 to 50 %d\n", slopes_40_50 );
	printf("slopes 50 to 60 %d\n", slopes_50_60 );
	printf("slopes 60 to 70 %d\n", slopes_60_70 );
	printf("slopes 70 to 80 %d\n", slopes_70_80 );
	printf("slopes over 80 %d\n", slopes_over_80 );

} // end get_terrain_info()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void post_process_surface_ids(int sec_z, int sec_x){ 

	///////////////////////////////////////////////////////////////////////////////
	// post processing to get rid of oddball polys that are
	// surrounded by different terrain types
	///////////////////////////////////////////////////////////////////////////////

	int poly_loop,
		compare_loop,
		adjacent_loop,
		//i,
		j,
		k,
		polys_this_sector,
		y_point0,
		y_point1,
		y_point2,
		temp_surface_id_each_face[1024],
		this_poly_point_ref[3],
		compare_to_poly_point_ref[3],
		number_of_adjacent_polys,
		number_of_points_in_common,
		number_of_id_matches,
		terrain_altered_land1_count,
		terrain_altered_land2_count, 
		terrain_altered_land3_count,
		terrain_land_count,
		highest_terrain_type_count,
		highest_counted_terrain_type;


	///////////////////////////////////////////////////////////////////////////////
	// find the adjacent polys for each poly
	///////////////////////////////////////////////////////////////////////////////

	polys_this_sector = terrain_sectors[sec_z][sec_x].number_of_polygons;

	//
	// first print the original surface ids and point numbers for polys
	//
/*	 
	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		printf("%d = %d  %d %d  %d %d  %d %d\n", poly_loop,
			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id,

			terrain_sectors[sec_z][sec_x].points_xz[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point  ].x,

			terrain_sectors[sec_z][sec_x].points_xz[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point  ].z,

			terrain_sectors[sec_z][sec_x].points_xz[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point  ].x,

			terrain_sectors[sec_z][sec_x].points_xz[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3  + 1 ].point  ].z,

			terrain_sectors[sec_z][sec_x].points_xz[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point  ].x,

			terrain_sectors[sec_z][sec_x].points_xz[  terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point  ].z);
		
	}
*/
	//////////////////////////////////////////////////////////////////////////////
	// start the actual post processing
	//////////////////////////////////////////////////////////////////////////////

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		//
		// get the point references for this poly
		//

		
		this_poly_point_ref[0] = (int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 ].point;

		this_poly_point_ref[1] = (int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 1 ].point;

		this_poly_point_ref[2] = (int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_loop * 3 + 2 ].point;
	

		//
		// loop thru all other polys to find common points and thus an adjacent poly
		//

		adjacent_list[poly_loop].number_of_adjacent_polys = 0;
		
		for(compare_loop = 0; compare_loop < polys_this_sector; compare_loop ++){ 

			if( compare_loop == poly_loop ) continue;

			compare_to_poly_point_ref[0] = (int)terrain_sectors[sec_z][sec_x].
				point_word_references[ compare_loop * 3 ].point;

			compare_to_poly_point_ref[1] = (int)terrain_sectors[sec_z][sec_x].
				point_word_references[ compare_loop * 3 + 1].point;

			compare_to_poly_point_ref[2] = (int)terrain_sectors[sec_z][sec_x].
				point_word_references[ compare_loop * 3 + 2 ].point;
						

			number_of_points_in_common = 0;
			
			for(j = 0; j < 3; j++){
				for(k = 0; k < 3; k++){

					if(compare_loop != poly_loop && this_poly_point_ref[j] == compare_to_poly_point_ref[k]){

						number_of_points_in_common ++;
					}
				}
			}

			if(number_of_points_in_common == 2){

				adjacent_list[ poly_loop ].number_of_adjacent_polys++;
			}

		//	if(adjacent_list[ poly_loop ].number_of_adjacent_polys){
			if(number_of_points_in_common == 2){

				adjacent_list[ poly_loop ].adjacent_poly_numbers[adjacent_list[ poly_loop ].
					number_of_adjacent_polys - 1] = compare_loop;

				adjacent_list[ poly_loop ].adjacent_poly_surface_ids[adjacent_list[ poly_loop ].
					number_of_adjacent_polys - 1] = 
					terrain_sectors[sec_z][sec_x].surface_id_each_face[compare_loop].surface_id;
			}

			//if(number_of_points_in_common == 2){
			//	printf("%d  %d\n", poly_loop, compare_loop);
			//}

 
		} // end for compare loop
/*		
		printf("poly=%d  id=%d  adjPolys=%d", poly_loop, 
			terrain_sectors[sec_z][sec_x].surface_id_each_face[ poly_loop ].surface_id,
			adjacent_list[ poly_loop ].number_of_adjacent_polys);

		for(j=0; j < adjacent_list[ poly_loop ].number_of_adjacent_polys; j++){
			printf("  %d = %d", adjacent_list[ poly_loop ].adjacent_poly_numbers[j],
				adjacent_list[ poly_loop ].adjacent_poly_surface_ids[j]);
		}
		printf("\n");
*/
	} // end for poly loop

	//////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////
	//
	// go thru each poly and look for oddballs
	// and change the surface id to that of the surrounding surface ids
	//
	//////////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////////

	//
	// assign temporary surface id list to equal original
	//

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		temp_surface_id_each_face[ poly_loop ] = 
			(int)terrain_sectors[sec_z][sec_x].
			surface_id_each_face[ poly_loop ].surface_id;
	}

	//
	// find polys that are oddballs
	//

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		number_of_id_matches = 0;

		for(adjacent_loop = 0; 
			adjacent_loop < adjacent_list[ poly_loop ].number_of_adjacent_polys; 
			adjacent_loop ++){

			if((int)terrain_sectors[sec_z][sec_x].
				surface_id_each_face[ poly_loop ].surface_id ==
				adjacent_list[ poly_loop ].adjacent_poly_surface_ids[ adjacent_loop ]){

				number_of_id_matches ++;
			}
		} // end for adjacent list

		if(number_of_id_matches < 1){

			adjacent_list[ poly_loop ].oddball_bool = 1;

		}
		else{

			adjacent_list[ poly_loop ].oddball_bool = 0;
		}
		
	} // end for poly list

	//
	// change oddball ids to match the surrounding ids
	// Antonio this is the start of the area where I made changes to fix the bug

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		terrain_land_count = 0;
		terrain_altered_land1_count = 0;
		terrain_altered_land2_count = 0;
		terrain_altered_land3_count = 0;
		
		
		if(adjacent_list[ poly_loop ].oddball_bool){

			for(adjacent_loop = 0;
				adjacent_loop < adjacent_list[ poly_loop ].number_of_adjacent_polys; 
				adjacent_loop ++){

				if(adjacent_list[ poly_loop ].adjacent_poly_surface_ids[ adjacent_loop ] == 
					TERRAIN_TYPE_LAND) terrain_land_count ++;

				else if(adjacent_list[ poly_loop ].adjacent_poly_surface_ids[ adjacent_loop ] == 
					TERRAIN_TYPE_ALTERED_LAND1) terrain_altered_land1_count ++;

				else if(adjacent_list[ poly_loop ].adjacent_poly_surface_ids[ adjacent_loop ] == 
					TERRAIN_TYPE_ALTERED_LAND2) terrain_altered_land2_count ++;

				else if(adjacent_list[ poly_loop ].adjacent_poly_surface_ids[ adjacent_loop ] == 
					TERRAIN_TYPE_ALTERED_LAND3) terrain_land_count++;

			} // end for adjacent_loop

			// find what the highest terrain type count is

			highest_terrain_type_count = 0;

			if(terrain_land_count > highest_terrain_type_count){
				highest_terrain_type_count = terrain_land_count;
			}

			if(terrain_altered_land1_count > highest_terrain_type_count){
				highest_terrain_type_count = terrain_altered_land1_count;
			}

			if(terrain_altered_land2_count > highest_terrain_type_count){
				highest_terrain_type_count = terrain_altered_land2_count;
			}
						
			if(terrain_altered_land3_count > highest_terrain_type_count){
				highest_terrain_type_count = terrain_altered_land3_count;
			}
			
			// find which terrain type had the highest count

			if(terrain_land_count == highest_terrain_type_count){
				highest_counted_terrain_type = TERRAIN_TYPE_LAND;
			}

			if(terrain_altered_land1_count == highest_terrain_type_count){
				highest_counted_terrain_type = TERRAIN_TYPE_ALTERED_LAND1;
			}

			if(terrain_altered_land2_count == highest_terrain_type_count){
				highest_counted_terrain_type = TERRAIN_TYPE_ALTERED_LAND2;
			}

			if(terrain_altered_land3_count == highest_terrain_type_count){
				highest_counted_terrain_type = TERRAIN_TYPE_ALTERED_LAND3;
			}

			// change surface id if it's not in a sector corner
			// and the only adjacent terrain is scrub
			// below I used on Mars
			
			//if(!(highest_terrain_type_count == 1 && highest_counted_terrain_type == TERRAIN_SCRUB)){
			//	temp_surface_id_each_face[x] = highest_counted_terrain_type;
			//}

			// change the oddball surface id to the highest adjacent id count
			// but don't want to mess with the water either way

			if( ! (highest_counted_terrain_type == TERRAIN_TYPE_ALTERED_LAND3 ||
				temp_surface_id_each_face[ poly_loop ] == TERRAIN_TYPE_ALTERED_LAND3) ){

				temp_surface_id_each_face[ poly_loop ] = highest_counted_terrain_type;
			}


		} // end if
	} // end for poly loop

	//
	// write temporary surface ids back to global data
	//

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[ poly_loop ].surface_id != 
			(unsigned char)temp_surface_id_each_face[ poly_loop ]){

			odd_ball_count ++;

		//	printf("%d  %d  %d  %d  %d\n", sec_z, sec_x, x,
		//		terrain_sectors[sec_z][sec_x].surface_id_each_face[ poly_loop ].surface_id,
		//		(unsigned char)temp_surface_id_each_face[ poly_loop ]);

			terrain_sectors[sec_z][sec_x].surface_id_each_face[ poly_loop ].surface_id = 
				(unsigned char)temp_surface_id_each_face[ poly_loop ];

		}

	}

} // end post_process_surface_ids()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

// Lake file format
// int	number_of_contour_lakes
// loop number_of_contour_lakes
//		int		number_of_lake_triangles
//		loop	number_of_lake_triangles
//			vec3d * 3
/*
void create_lakes_file(void){

	//  lake_polys[1337][3];	// [][0] = x sector  [][1] = z sector  [][2] = polygon number


	FILE
		*fp;

	int
		lake_loop,
		triangle_loop,
		point_loop,
		sector_coord_x,
		sector_coord_z,
		sector_coord_y,
		sec_x,
		sec_z,
		polygon_index,
		point_index,
		y_point0,
		y_point1,
		y_point2;

	float
		global_x,
		global_z,
		global_y,
		sector_to_world_coord_factor = 1024.0 / 16384.0,
		terrain_3d_map_scaled_height_difference;

	float
		terrain_3d_max_map_z = 2048.0 * global_map_height;

	vec3d
		vec3d_value;

	world_coord_triangle
		lake_triangles[1337];

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	if(( fp = fopen("LAKES.PLY","wb"))==NULL){
		printf("cannot open LAKES.PLY\n");
		exit(1);
	}
	else printf("LAKES.PLY opened\n");

	fwrite ( &number_of_contour_lakes, sizeof ( int ), 1, fp );

	for(lake_loop = 0; lake_loop < number_of_contour_lakes; lake_loop ++){

		fwrite ( &number_of_lake_triangles, sizeof ( int ), 1, fp );

		for(triangle_loop = 0; triangle_loop < number_of_lake_triangles; triangle_loop ++){

			sec_x = lake_polys[triangle_loop][0];
			sec_z = lake_polys[triangle_loop][1];

			polygon_index = lake_polys[triangle_loop][2];
			
			for(point_loop = 0; point_loop < 3; point_loop ++){

				// calculate the world coordinates

				sector_coord_x = (int)terrain_sectors[sec_z][sec_x].
					points_xz[  (int)terrain_sectors[sec_z][sec_x].
					point_word_references[ polygon_index * 3 + point_loop ].point  ].x;

				sector_coord_z = (int)terrain_sectors[sec_z][sec_x].
					points_xz[  (int)terrain_sectors[sec_z][sec_x].
					point_word_references[ polygon_index * 3 + point_loop ].point  ].z;

				sector_coord_y = terrain_sectors[sec_z][sec_x].
					points_y[  terrain_sectors[sec_z][sec_x].
					point_word_references[ polygon_index * 3 + point_loop ].point  ].y;

				//////////////////////////////////////////////////////////
				// calculate global values
				//////////////////////////////////////////////////////////

				global_x = sec_x * 2048.0 + 1024.0 + sector_coord_x * sector_to_world_coord_factor;

				global_z = sec_z * 2048.0 + 1024.0 + sector_coord_z * sector_to_world_coord_factor;

				global_y = terrain_3d_map_minimum_height + sector_coord_y * 
					terrain_3d_map_scaled_height_difference;

				//////////////////////////////////////////////////////////
				// copy into array for sorting
				//////////////////////////////////////////////////////////

				lake_triangles[ triangle_loop ].pts[ point_loop ].x = global_x;
				lake_triangles[ triangle_loop ].pts[ point_loop ].z = global_z;
				lake_triangles[ triangle_loop ].pts[ point_loop ].y = global_y;

				
			} // end for point loop

			lake_triangles[ triangle_loop ].avg_x = ( lake_triangles[ triangle_loop ].pts[0].x +
				lake_triangles[ triangle_loop ].pts[1].x +
				lake_triangles[ triangle_loop ].pts[2].x ) / 3.0;

			lake_triangles[ triangle_loop ].avg_z = ( lake_triangles[ triangle_loop ].pts[0].z +
				lake_triangles[ triangle_loop ].pts[1].z +
				lake_triangles[ triangle_loop ].pts[2].z ) / 3.0;

		} // end for triangle_loop

		qsort(lake_triangles, number_of_lake_triangles, sizeof(world_coord_triangle), qsort_compare);

		////////////////////////////////////////////////////////////////
		// write triangles to file
		////////////////////////////////////////////////////////////////

		for(triangle_loop = 0; triangle_loop < number_of_lake_triangles; triangle_loop ++){

			// point 0

			vec3d_value.x = lake_triangles[ triangle_loop ].pts[0].x;
			vec3d_value.z = lake_triangles[ triangle_loop ].pts[0].z;
			vec3d_value.y = lake_triangles[ triangle_loop ].pts[0].y;

			// reverse the z coordinate

			vec3d_value.z = terrain_3d_max_map_z - vec3d_value.z;

			fwrite (&vec3d_value, sizeof (vec3d), 1, fp);

			// point 1

			vec3d_value.x = lake_triangles[ triangle_loop ].pts[1].x;
			vec3d_value.z = lake_triangles[ triangle_loop ].pts[1].z;
			vec3d_value.y = lake_triangles[ triangle_loop ].pts[1].y;

			// reverse the z coordinate

			vec3d_value.z = terrain_3d_max_map_z - vec3d_value.z;

			fwrite (&vec3d_value, sizeof (vec3d), 1, fp);

			// point 2

			vec3d_value.x = lake_triangles[ triangle_loop ].pts[2].x;
			vec3d_value.z = lake_triangles[ triangle_loop ].pts[2].z;
			vec3d_value.y = lake_triangles[ triangle_loop ].pts[2].y;

			// reverse the z coordinate

			vec3d_value.z = terrain_3d_max_map_z - vec3d_value.z;

			fwrite (&vec3d_value, sizeof (vec3d), 1, fp);

		} // end for poly loop writing file
	} // end for lake loop

	

	fclose(fp);

} // end create lakes file
*/
/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////
/*
int qsort_compare(const world_coord_triangle *t1, const world_coord_triangle *t2){

	if(t1->avg_x == t2->avg_x && t1->avg_z == t2->avg_z){

		return 0;
	}
	else if(t1->avg_z > t2->avg_z){

		return 1;
	}
	else if(t1->avg_z < t2->avg_z){

		return -1;
	}
	else if(t1->avg_z == t2->avg_z){

		if(t1->avg_x < t2->avg_x){

			return -1;
		}
		else{

			return 1;
		}
	}

	return 0;
} // end qsort_compare
*/
/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void read_lakes_file(void){

	FILE
		*fp,
		*fp_out;

	//contour_lake
	//	*contour_lakes;

	//vec3d
	//	*contour_lake_vertices;

	int
		number_of_lake_triangles,
		total_triangles,
		count,
		number_of_contour_lakes,
		point;

	float
		terrain_3d_max_map_z = 2048.0 * global_map_height;

	vec3d
		*vertices;

	if(( fp = fopen("LAKES.PLY","rb"))==NULL){
		printf("cannot open LAKES.PLY\n");
		exit(1);
	}
	else printf("LAKES.PLY opened\n");

	if(( fp_out = fopen("lakes_text_out.log","wb"))==NULL){
		printf("cannot open lakes_text_out.log\n");
		exit(1);
	}
	else printf("lakes_text_out.log opened\n");

	fprintf(fp_out, "terrain_3d_max_map_z = %f\n", terrain_3d_max_map_z);

	//
	// Assess the number of lakes
	//

	number_of_contour_lakes = 0;

	total_triangles = 0;

	fread ( &number_of_contour_lakes, sizeof ( int ), 1, fp );

	for ( count = 0; count < number_of_contour_lakes; count++ )
	{

		fread ( &number_of_lake_triangles, sizeof ( int ), 1, fp );

		total_triangles += number_of_lake_triangles;

		fseek ( fp, sizeof ( vec3d ) * 3 * number_of_lake_triangles, SEEK_CUR );
	}

	//
	// Now read in the data properly
	//

	contour_lake_vertices = malloc ( sizeof ( vec3d ) * 3 * total_triangles );

	contour_lakes = malloc ( sizeof ( contour_lake ) * number_of_contour_lakes );

	fseek ( fp, 4, SEEK_SET );

	vertices = contour_lake_vertices;

	for ( count = 0; count < number_of_contour_lakes; count++ )
	{

		int
			point;

		fread ( &number_of_lake_triangles, sizeof ( int ), 1, fp );
	
		fread ( vertices, sizeof ( vec3d ) * 3, number_of_lake_triangles, fp );

		contour_lakes[count].number_of_triangles = number_of_lake_triangles;

		contour_lakes[count].triangle_points = vertices;

		//for ( point = 0; point < number_of_lake_triangles; point++ )
		//{

		//	vertices[point*3+0].z = terrain_3d_max_map_z - vertices[point*3+0].z;
		//	vertices[point*3+1].z = terrain_3d_max_map_z - vertices[point*3+1].z;
		//	vertices[point*3+2].z = terrain_3d_max_map_z - vertices[point*3+2].z;
		//}

		vertices += ( number_of_lake_triangles * 3 );
	}

	fclose ( fp );

	///////////////////////////////////////////////////
	// write out the data in text
	///////////////////////////////////////////////////

	fprintf (fp_out, "number_of_contour_lakes = %d\n\n", number_of_contour_lakes );

	for ( count = 0; count < number_of_contour_lakes; count++ )
	{
		number_of_lake_triangles = contour_lakes[count].number_of_triangles;

		fprintf (fp_out, "Lake - %d   number_of_lake_triangles = %d\n\n",
			count, contour_lakes[count].number_of_triangles );

		

		for ( point = 0; point < number_of_lake_triangles; point++ )
		{
			fprintf(fp_out, "\n\tTriangle %d\n\n", point);

			fprintf(fp_out, "%f  %f  %f -  %f  %f  %f -  %f  %f  %f\n",
				contour_lakes[count].triangle_points[point*3+0].x,
				contour_lakes[count].triangle_points[point*3+0].z,
				contour_lakes[count].triangle_points[point*3+0].y,
				contour_lakes[count].triangle_points[point*3+1].x,
				contour_lakes[count].triangle_points[point*3+1].z,
				contour_lakes[count].triangle_points[point*3+1].y,
				contour_lakes[count].triangle_points[point*3+2].x,
				contour_lakes[count].triangle_points[point*3+2].z,
				contour_lakes[count].triangle_points[point*3+2].y);

			
		} // end for triangle loop
	}

	fclose( fp_out );

} // end read lakes file

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void find_lake_triangles(int sec_z, int sec_x){

	int poly_loop,
		polys_this_sector,
		slope,
		y_point0, //sector coordinates
		y_point1,
		y_point2;

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		if(terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id == TERRAIN_TYPE_LAND){

			lake_polys[number_of_lake_triangles][0] = sec_x;
			lake_polys[number_of_lake_triangles][1] = sec_z;
			lake_polys[number_of_lake_triangles][2] = poly_loop;

			number_of_lake_triangles ++;

		} // end if
	} // end for poly loop
} // end find_lake_triangles

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

double normal_angle_with_y(int sec_z, int sec_x, int poly){

	// returns the angle between the poly normal and the Y axis (bigger = steeper)

	// points

	double ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	double abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	//int angle;
	double	radian_angle,
			pi = 3.1415926535;

	double	normalized_ny,
			angle;

	ax = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].x;

	bx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].x;

	cx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].x;

	az = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].z;

	bz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].z;

	cz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].z;

	ay = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].y;

	by = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].y;

	cy = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].y;


	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_ny = ny / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_ny *= -1.0;
	
	radian_angle = acos(normalized_ny); // normalized_ny / 1 = arcos of angle with y axis
	
	angle = (180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_y()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int normal_angle_with_x(int sec_z, int sec_x, int poly){

	// returns the angle between the normal and the x axis

	// points

	int ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	int abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	int angle;
	double	radian_angle,
			pi = 3.1415926535;

	float normalized_nx;

	ax = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].x;

	bx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].x;

	cx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].x;

	az = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].z;

	bz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].z;

	cz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].z;

	ay = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].y;

	by = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].y;

	cy = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].y;


	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_nx = nx / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_nx *= -1.0;
	
	radian_angle = acos(normalized_nx); // normalized_ny / 1 = arcos of angle with y axis
	
	angle = (int)(180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_x()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

int normal_angle_with_z(int sec_z, int sec_x, int poly){

	// returns the angle between the normal and the z axis

	// points

	int ax, bx, cx, az, bz, cz, ay, by, cy;

	// vectors

	int abx, abz, aby, acx, acz, acy, nx, nz, ny;
	
	int angle;
	double	radian_angle,
			pi = 3.1415926535;

	float normalized_nz;

	ax = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].x;

	bx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].x;

	cx = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].x;

	az = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].z;

	bz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].z;

	cz = (double)terrain_sectors[sec_z][sec_x].
		points_xz[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].z;

	ay = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 ].point  ].y;

	by = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 1 ].point  ].y;

	cy = (double)terrain_sectors[sec_z][sec_x].
		points_y[  (int)terrain_sectors[sec_z][sec_x].
		point_word_references[ poly * 3 + 2 ].point  ].y;


	abx = bx - ax;
	abz = bz - az;
	aby = by - ay;
	acx = cx - ax;
	acz = cz - az;
	acy = cy - ay;
	nx = abz * acy - aby * acz;
	nz = aby * acx - abx * acy;
	ny = abx * acz - abz * acx;

	normalized_nz = nz / (sqrt((pow(nx, 2) + pow(nz, 2) + pow(ny, 2))));
	
	normalized_nz *= -1.0;
	
	radian_angle = acos(normalized_nz); // normalized_nz / 1 = arcos of angle with z axis
	
	angle = (int)(180.0 * radian_angle / pi);


	return angle;

} // end normal_angle_with_z()

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void print_terrain_info(void){

	printf("\nMax polys per sector = %d in sector %d x %d\n", global_max_polys,
		global_max_polys_sector[0], global_max_polys_sector[1]);
	printf("Elevation %d - %d\n", (int)global_lowest_elevation, (int)global_highest_elevation);
	printf("Slope      %d - %d\n", global_min_slope, global_max_slope);
	printf("Slopes 10 - 20 %d\n", global_slopes_10_20);
	printf("Slopes 20 - 30 %d\n", global_slopes_20_30);
	printf("Slopes 30 - 40 %d\n", global_slopes_30_40);
	printf("Slopes 40 - 50 %d\n", global_slopes_40_50);
	printf("Slopes 50 - 60 %d\n", global_slopes_50_60);
	printf("Slopes 60 - 70 %d\n", global_slopes_60_70);
	printf("Slopes 70 - 80 %d\n", global_slopes_70_80);
	printf("Slopes over 80 %d\n", global_slopes_over_80);
	printf("Oddballs polygons fixed = %d\n", odd_ball_count);

} // end print terrain info

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////

void print_surface_ids_of_sector(int sec_z, int sec_x){

	int poly_loop,
		polys_this_sector;

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	printf("\nSector %d x %d\n\n", sec_x, sec_z);

	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		printf("%d\n", terrain_sectors[sec_z][sec_x].
			surface_id_each_face[poly_loop].surface_id);
	}

} // end print surface ids of sector

/////////////////////////////////////
/////////////////////////////////////
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
//                                 //
/////////////////////////////////////
/////////////////////////////////////


void print_terrain_data_of_sector(int sec_z, int sec_x){

	int poly_loop,
		polys_this_sector,
		point_loop,
		points_this_sector;

	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	points_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_points;

	printf("\nSector %d x %d\n\n", sec_x, sec_z);

	printf("%d surface ids\n", (int)terrain_sectors[sec_z][sec_x].number_of_surface_ids);

	printf("Polygon  Surface Id  Word References\n\n");
	
	for(poly_loop = 0; poly_loop < polys_this_sector; poly_loop ++){

		printf("%d  %d      %d  %d  %d\n", poly_loop,
			terrain_sectors[sec_z][sec_x].surface_id_each_face[poly_loop].surface_id,
			terrain_sectors[sec_z][sec_x].point_word_references[ poly_loop * 3 ].point,
			terrain_sectors[sec_z][sec_x].point_word_references[ poly_loop * 3 + 1 ].point,
			terrain_sectors[sec_z][sec_x].point_word_references[ poly_loop * 3 + 2 ].point);
	}

	printf("\n\nPoint  x  z  y\n\n");

	for(point_loop = 0; point_loop < points_this_sector; point_loop ++){

		printf("%d  %d  %d  %d\n", point_loop,
			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x,
			terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z,
			terrain_sectors[sec_z][sec_x].points_y[ point_loop ].y);
	}
	
} // end print terrain data of sector

void equalize_colors_at_sector_junctions ( void ){

	int
		point_loop,
		sec_x,
		sec_z,
		point_x,
		point_z,
		color_index,
		number_of_pivot_sector_points,
		number_of_north_sector_points,
		number_of_east_sector_points,
		pivot_sec_north_points[17],
		pivot_sec_east_points[17],
		north_sec_south_points[17],
		east_sec_west_points[17];

		
	for(sec_z = 0; sec_z < terrain_3d_map_height - 1; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width - 1; sec_x ++){

			number_of_pivot_sector_points = terrain_sectors[ sec_z ][ sec_x ].number_of_points;
			number_of_north_sector_points = terrain_sectors[ sec_z + 1 ][ sec_x ].number_of_points;
			number_of_east_sector_points = terrain_sectors[ sec_z ][ sec_x + 1 ].number_of_points;


			//////////////////////////////////////////////////////////////////////
			// If all three sectors have 289 points (not modified with Lightwave)
			// then the point numbers will be the same for all the edge points.
			// Otherwise, need to find the edge points by brute force.
			//////////////////////////////////////////////////////////////////////

			if( number_of_pivot_sector_points == 289 && 
				number_of_north_sector_points == 289 &&
				number_of_east_sector_points == 289){

				//
				// set the south edge of (sec_x, sec_z + 1) equal to
				// north edge of (sec_x, sec_z)
				//

				terrain_sectors[ sec_z + 1 ][ sec_x ].
					colour_indices[ 0 ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ 2 ].index;

				terrain_sectors[ sec_z + 1 ][ sec_x ].
					colour_indices[ 1 ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ 153 ].index;

				for(point_loop = 0; point_loop < 14; point_loop ++){

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					colour_indices[ 3 + point_loop ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ 154 + point_loop ].index;
				} // end for point loop

				//
				// set the west edge of (sec_x + 1, sec_z) equal to
				// east edge of (sec_x, sec_z)
				//

				terrain_sectors[ sec_z ][ sec_x + 1 ].
					colour_indices[ 0 ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ 1 ].index;

				terrain_sectors[ sec_z ][ sec_x + 1 ].
					colour_indices[ 2 ].index = 
					terrain_sectors[ sec_z ][ sec_x + 1].
					colour_indices[ 153 ].index;

				for(point_loop = 0; point_loop < 14; point_loop ++){

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					colour_indices[ 33 + point_loop ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ 169 + point_loop ].index;
				} // end for point loop
			}
			else{ // brute force method

				//
				// pivot sector 
				//

				//printf("\n%d x %d\n%d x %d\n%d x %d\n\n", sec_x, sec_z, sec_x, sec_z + 1,
				//	sec_x + 1, sec_z);

				for(point_loop = 0; point_loop < number_of_pivot_sector_points; point_loop ++){

					//color_index = terrain_sectors[ sec_z ][ sec_x ].
					//	colour_indices[ point_loop ].index;

					point_x = terrain_sectors[ sec_z ][ sec_x ].points_xz[ point_loop ].x;
					point_z = terrain_sectors[ sec_z ][ sec_x ].points_xz[ point_loop ].z;

					//
					// north edge
					//

					if(point_z == 16384){

						if(point_x == -16384){

							pivot_sec_north_points[0] = point_loop;
						}
						else if(point_x == -14336){

							pivot_sec_north_points[1] = point_loop;
						}
						else if(point_x == -12288){

							pivot_sec_north_points[2] = point_loop;
						}
						else if(point_x == -10240){

							pivot_sec_north_points[3] = point_loop;
						}
						else if(point_x == -8192){

							pivot_sec_north_points[4] = point_loop;
						}
						else if(point_x == -6144){

							pivot_sec_north_points[5] = point_loop;
						}
						else if(point_x == -4096){

							pivot_sec_north_points[6] = point_loop;
						}
						else if(point_x == -2048){

							pivot_sec_north_points[7] = point_loop;
						}
						else if(point_x == 0){

							pivot_sec_north_points[8] = point_loop;
						}
						else if(point_x == 2048){

							pivot_sec_north_points[9] = point_loop;
						}
						else if(point_x == 4096){

							pivot_sec_north_points[10] = point_loop;
						}
						else if(point_x == 6144){

							pivot_sec_north_points[11] = point_loop;
						}
						else if(point_x == 8192){

							pivot_sec_north_points[12] = point_loop;
						}
						else if(point_x == 10240){

							pivot_sec_north_points[13] = point_loop;
						}
						else if(point_x == 12288){

							pivot_sec_north_points[14] = point_loop;
						}
						else if(point_x == 14336){

							pivot_sec_north_points[15] = point_loop;
						}
						else if(point_x == 16384){ // north east corner

							pivot_sec_north_points[16] = point_loop;

							pivot_sec_east_points[16] = point_loop;
						}

					} // end if

					//
					// east edge
					//

					else if(point_x == 16384){

						if(point_z == -16384){

							pivot_sec_east_points[0] = point_loop;
						}
						else if(point_z == -14336){

							pivot_sec_east_points[1] = point_loop;
						}
						else if(point_z == -12288){

							pivot_sec_east_points[2] = point_loop;
						}
						else if(point_z == -10240){

							pivot_sec_east_points[3] = point_loop;
						}
						else if(point_z == -8192){

							pivot_sec_east_points[4] = point_loop;
						}
						else if(point_z == -6144){

							pivot_sec_east_points[5] = point_loop;
						}
						else if(point_z == -4096){

							pivot_sec_east_points[6] = point_loop;
						}
						else if(point_z == -2048){

							pivot_sec_east_points[7] = point_loop;
						}
						else if(point_z == 0){

							pivot_sec_east_points[8] = point_loop;
						}
						else if(point_z == 2048){

							pivot_sec_east_points[9] = point_loop;
						}
						else if(point_z == 4096){

							pivot_sec_east_points[10] = point_loop;
						}
						else if(point_z == 6144){

							pivot_sec_east_points[11] = point_loop;
						}
						else if(point_z == 8192){

							pivot_sec_east_points[12] = point_loop;
						}
						else if(point_z == 10240){

							pivot_sec_east_points[13] = point_loop;
						}
						else if(point_z == 12288){

							pivot_sec_east_points[14] = point_loop;
						}
						else if(point_z == 14336){

							pivot_sec_east_points[15] = point_loop;
						}
						else if(point_z == 16384){

							pivot_sec_east_points[16] = point_loop;
						}

					} // end else if

				} // end for point loop pivot sector
/*
				printf("\npivot sec north edge  %d x %d\n\n", sec_x, sec_z);

				for(point_loop = 0; point_loop < 17; point_loop ++){

					color_index = terrain_sectors[ sec_z ][ sec_x ].
						colour_indices[ pivot_sec_north_points[point_loop] ].index;

					printf("%d  point %d  color index %d  %d x %d  %d %d %d\n", point_loop,

					pivot_sec_north_points[point_loop],

					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ pivot_sec_north_points[point_loop] ].index,

					terrain_sectors[ sec_z ][ sec_x ].
					points_xz[ pivot_sec_north_points[point_loop] ].x,

					terrain_sectors[ sec_z ][ sec_x ].
					points_xz[ pivot_sec_north_points[point_loop] ].z,

					terrain_sectors[ sec_z ][ sec_x ].
					point_colours[color_index].red,
					
					terrain_sectors[ sec_z ][ sec_x ].
					point_colours[color_index].green,
					
					terrain_sectors[ sec_z ][ sec_x ].
					point_colours[color_index].blue);

				}

				printf("\npivot sec east edge  %d x %d\n\n", sec_x, sec_z);

				for(point_loop = 0; point_loop < 17; point_loop ++){

					color_index = terrain_sectors[ sec_z ][ sec_x ].
						colour_indices[ pivot_sec_east_points[point_loop] ].index;

					printf("%d  point %d  color index %d  %d x %d  %d %d %d\n", point_loop,

					pivot_sec_east_points[point_loop],
					
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ pivot_sec_east_points[point_loop] ].index,

					terrain_sectors[ sec_z ][ sec_x ].
					points_xz[ pivot_sec_east_points[point_loop] ].x,

					terrain_sectors[ sec_z ][ sec_x ].
					points_xz[ pivot_sec_east_points[point_loop] ].z,
					
					terrain_sectors[ sec_z ][ sec_x ].
					point_colours[color_index].red,
					
					terrain_sectors[ sec_z ][ sec_x ].
					point_colours[color_index].green,
					
					terrain_sectors[ sec_z ][ sec_x ].
					point_colours[color_index].blue);

				}
*/
				//
				// north sector , south edge
				//

				for(point_loop = 0; point_loop < number_of_north_sector_points; point_loop ++){

					//color_index = terrain_sectors[ sec_z + 1 ][ sec_x ].
					//	colour_indices[ point_loop ].index;

					point_x = terrain_sectors[ sec_z + 1 ][ sec_x ].points_xz[ point_loop ].x;
					point_z = terrain_sectors[ sec_z + 1 ][ sec_x ].points_xz[ point_loop ].z;

					if(point_z == -16384){

						if(point_x == -16384){

							north_sec_south_points[0] = point_loop;
						}
						else if(point_x == -14336){

							north_sec_south_points[1] = point_loop;
						}
						else if(point_x == -12288){

							north_sec_south_points[2] = point_loop;
						}
						else if(point_x == -10240){

							north_sec_south_points[3] = point_loop;
						}
						else if(point_x == -8192){

							north_sec_south_points[4] = point_loop;
						}
						else if(point_x == -6144){

							north_sec_south_points[5] = point_loop;
						}
						else if(point_x == -4096){

							north_sec_south_points[6] = point_loop;
						}
						else if(point_x == -2048){

							north_sec_south_points[7] = point_loop;
						}
						else if(point_x == 0){

							north_sec_south_points[8] = point_loop;
						}
						else if(point_x == 2048){

							north_sec_south_points[9] = point_loop;
						}
						else if(point_x == 4096){

							north_sec_south_points[10] = point_loop;
						}
						else if(point_x == 6144){

							north_sec_south_points[11] = point_loop;
						}
						else if(point_x == 8192){

							north_sec_south_points[12] = point_loop;
						}
						else if(point_x == 10240){

							north_sec_south_points[13] = point_loop;
						}
						else if(point_x == 12288){

							north_sec_south_points[14] = point_loop;
						}
						else if(point_x == 14336){

							north_sec_south_points[15] = point_loop;
						}
						else if(point_x == 16384){ 

							north_sec_south_points[16] = point_loop;
						}
					} // end if

				} // end for point loop north sector
/*
				printf("\nnorth sec south edge  %d x %d\n\n", sec_x, sec_z + 1);

				for(point_loop = 0; point_loop < 17; point_loop ++){

					color_index = terrain_sectors[ sec_z + 1 ][ sec_x ].
						colour_indices[ north_sec_south_points[point_loop] ].index;

					printf("%d  point %d  color index %d  %d x %d  %d %d %d\n", point_loop,

					north_sec_south_points[point_loop],

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					colour_indices[ north_sec_south_points[point_loop] ].index,

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					points_xz[ north_sec_south_points[point_loop] ].x,

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					points_xz[ north_sec_south_points[point_loop] ].z,
					
					terrain_sectors[ sec_z + 1 ][ sec_x ].
					point_colours[color_index].red,
					
					terrain_sectors[ sec_z + 1 ][ sec_x ].
					point_colours[color_index].green,
					
					terrain_sectors[ sec_z + 1 ][ sec_x ].
					point_colours[color_index].blue);

				}
*/
				//
				// east sector 
				//

				for(point_loop = 0; point_loop < number_of_east_sector_points; point_loop ++){

					//color_index = terrain_sectors[ sec_z ][ sec_x + 1 ].
					//	colour_indices[ point_loop ].index;

					point_x = terrain_sectors[ sec_z ][ sec_x + 1 ].points_xz[ point_loop ].x;
					point_z = terrain_sectors[ sec_z ][ sec_x + 1 ].points_xz[ point_loop ].z;

					if(point_x == -16384){

						if(point_z == -16384){

							east_sec_west_points[0] = point_loop;
						}
						else if(point_z == -14336){

							east_sec_west_points[1] = point_loop;
						}
						else if(point_z == -12288){

							east_sec_west_points[2] = point_loop;
						}
						else if(point_z == -10240){

							east_sec_west_points[3] = point_loop;
						}
						else if(point_z == -8192){

							east_sec_west_points[4] = point_loop;
						}
						else if(point_z == -6144){

							east_sec_west_points[5] = point_loop;
						}
						else if(point_z == -4096){

							east_sec_west_points[6] = point_loop;
						}
						else if(point_z == -2048){

							east_sec_west_points[7] = point_loop;
						}
						else if(point_z == 0){

							east_sec_west_points[8] = point_loop;
						}
						else if(point_z == 2048){

							east_sec_west_points[9] = point_loop;
						}
						else if(point_z == 4096){

							east_sec_west_points[10] = point_loop;
						}
						else if(point_z == 6144){

							east_sec_west_points[11] = point_loop;
						}
						else if(point_z == 8192){

							east_sec_west_points[12] = point_loop;
						}
						else if(point_z == 10240){

							east_sec_west_points[13] = point_loop;
						}
						else if(point_z == 12288){

							east_sec_west_points[14] = point_loop;
						}
						else if(point_z == 14336){

							east_sec_west_points[15] = point_loop;
						}
						else if(point_z == 16384){

							east_sec_west_points[16] = point_loop;
						}

					} // end else if
				} // end for point loop east sector
/*
				printf("\neast sec west edge  %d x %d\n\n", sec_x + 1, sec_z);

				for(point_loop = 0; point_loop < 17; point_loop ++){

					color_index = terrain_sectors[ sec_z ][ sec_x + 1 ].
						colour_indices[ east_sec_west_points[point_loop] ].index;

					printf("%d  point %d  color index %d  %d x %d  %d %d %d\n", point_loop,

					east_sec_west_points[point_loop],

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					colour_indices[ east_sec_west_points[point_loop] ].index,

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					points_xz[ east_sec_west_points[point_loop] ].x,

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					points_xz[ east_sec_west_points[point_loop] ].z,
					
					terrain_sectors[ sec_z ][ sec_x + 1 ].
					point_colours[color_index].red,
					
					terrain_sectors[ sec_z ][ sec_x + 1 ].
					point_colours[color_index].green,
					
					terrain_sectors[ sec_z ][ sec_x + 1 ].
					point_colours[color_index].blue);

				}
*/
				////////////////////////////////////////////////////////////
				////////////////////////////////////////////////////////////
				// equalize the point color indices
				////////////////////////////////////////////////////////////
				////////////////////////////////////////////////////////////

				for(point_loop = 0; point_loop < 17; point_loop ++){

					//
					// set the south edge of (sec_x, sec_z + 1) equal to
					// north edge of (sec_x, sec_z)
					//

					//printf("%d  %d  %d  %d  %d\n", point_loop,
					//	north_sec_south_points[point_loop],
					//	pivot_sec_north_points[point_loop],
					//	east_sec_west_points[point_loop],
					//	pivot_sec_east_points[point_loop]);

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					colour_indices[ north_sec_south_points[point_loop] ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ pivot_sec_north_points[point_loop] ].index;

					//
					// set the west edge of (sec_x + 1, sec_z ) equal to
					// east edge of (sec_x, sec_z)
					//

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					colour_indices[ east_sec_west_points[point_loop] ].index = 
					terrain_sectors[ sec_z ][ sec_x ].
					colour_indices[ pivot_sec_east_points[point_loop] ].index;
				} // end for point loop
/*
				printf("\nEqualized to pivot sec %d x %d\n", sec_x, sec_z);

				printf("\nnorth sec south edge  %d x %d\n\n", sec_x, sec_z + 1);

				for(point_loop = 0; point_loop < 17; point_loop ++){

					color_index = terrain_sectors[ sec_z + 1 ][ sec_x ].
						colour_indices[ north_sec_south_points[point_loop] ].index;

					printf("%d  point %d  color index %d  %d x %d  %d %d %d\n", point_loop,

					north_sec_south_points[point_loop],

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					colour_indices[ north_sec_south_points[point_loop] ].index,

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					points_xz[ north_sec_south_points[point_loop] ].x,

					terrain_sectors[ sec_z + 1 ][ sec_x ].
					points_xz[ north_sec_south_points[point_loop] ].z,
					
					terrain_sectors[ sec_z + 1 ][ sec_x ].
					point_colours[color_index].red,
					
					terrain_sectors[ sec_z + 1 ][ sec_x ].
					point_colours[color_index].green,
					
					terrain_sectors[ sec_z + 1 ][ sec_x ].
					point_colours[color_index].blue);

				}

				printf("\neast sec west edge  %d x %d\n\n", sec_x + 1, sec_z);

				for(point_loop = 0; point_loop < 17; point_loop ++){

					color_index = terrain_sectors[ sec_z ][ sec_x + 1 ].
						colour_indices[ east_sec_west_points[point_loop] ].index;

					printf("%d  point %d  color index %d  %d x %d  %d %d %d\n", point_loop,

					east_sec_west_points[point_loop],

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					colour_indices[ east_sec_west_points[point_loop] ].index,

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					points_xz[ east_sec_west_points[point_loop] ].x,

					terrain_sectors[ sec_z ][ sec_x + 1 ].
					points_xz[ east_sec_west_points[point_loop] ].z,
					
					terrain_sectors[ sec_z ][ sec_x + 1 ].
					point_colours[color_index].red,
					
					terrain_sectors[ sec_z ][ sec_x + 1 ].
					point_colours[color_index].green,
					
					terrain_sectors[ sec_z ][ sec_x + 1 ].
					point_colours[color_index].blue);

				}
*/
			} // end else brute force method

		} // end for sec_x loop
	} // end for sec_z loop


/*
	for(sec_z = 0; sec_z < terrain_3d_map_height; sec_z ++){
		for(sec_x = 0; sec_x < terrain_3d_map_width; sec_x ++){

			points_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_points;

			printf("\nSector %d x %d\n\n", sec_x, sec_z);

			for(point_loop = 0; point_loop < points_this_sector; point_loop ++){

				if( terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].x == 16384 ){

					printf("%d  %d\n", point_loop,
						terrain_sectors[sec_z][sec_x].points_xz[ point_loop ].z);
				}
			
			} // end point loop

		} // end for x loop
	} // end for z loop
*/

} // end equalize colors at sector junctions

void print_colors_of_sector(int sec_z, int sec_x){

	int
		color_loop;

	printf("\nSector %d x %d colors\n\n", sec_x, sec_z);

	for(color_loop = 0; color_loop < number_of_colors; color_loop ++){

		printf("%d  %d %d %d\n", color_loop,

			terrain_sectors[ sec_z ][ sec_x ].
			point_colours[color_loop].red,
					
			terrain_sectors[ sec_z ][ sec_x ].
			point_colours[color_loop].green,
					
			terrain_sectors[ sec_z ][ sec_x ].
			point_colours[color_loop].blue);
	} // end for color loop

	printf("\n");

} // end print colors of sector