// grand_terraform.h

#define MIN(x,y) (x < y ? x : y)
#define MAX(x,y) (x > y ? x : y)
#define INSIDE 1
#define OUTSIDE 0


//#define TERRAIN_BARE_ROCK2	22 // COLOUR_PASS_ROCK
//#define TERRAIN_SCRUB		21 // COLOUR_PASS_MOUNTAINS
//#define TERRAIN_HIGHLAND	9 // COLOUR_PASS_SWAMP
// mistake ? #define TERRAIN_DETAIL_4	25 // COLOUR_PASS_YEMEN_BAKED2 terrain_type_road_bank

//#define TERRAIN_GRG_CITY4	37
//#define TERRAIN_GRG_CITY3	38

//#define TERRAIN_RESERVOIR	2
//#define TERRAIN_BEACH		8

//
// Terrain geometry data scaling factors
//

#define TERRAIN_3D_XZ_SCALE ( ( ( float ) ( TERRAIN_3D_SECTOR_SIDE_LENGTH + TERRAIN_3D_SECTOR_SIDE_LENGTH_ADJUSTMENT ) ) / 65536.0 )

#define TERRAIN_3D_Y_SCALE ( 1.0 / 32767.0 )

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

typedef int terrain_types;	//enum TERRAIN_TYPES terrain_types;

#define  TRUE 1
#define  FALSE 0

#define  EVEN 0
#define  ODD 1

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////


struct VEC3D
{

	float
		x,
		y,
		z;
};

typedef struct VEC3D vec3d;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_XZ
{

	short int
		x,
		z;
};

typedef struct TERRAIN_3D_POINT_XZ terrain_3d_point_xz;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

#pragma pack(1)

struct TERRAIN_3D_POINT_Y
{

	union
	{

		struct
		{

			unsigned short int
				y:15,
				normal_change:1;
		};

		unsigned short int
			whole_value;

		short int
			whole_signed_value;
	};
};

typedef struct TERRAIN_3D_POINT_Y terrain_3d_point_y;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR
{

	unsigned char
		red,
		green,
		blue;
};

typedef struct TERRAIN_3D_COLOUR terrain_3d_colour;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_COLOUR_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_COLOUR_INDEX terrain_3d_colour_index;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_NORMAL_INDEX
{

	unsigned char
		index;
};

typedef struct TERRAIN_3D_NORMAL_INDEX terrain_3d_normal_index;

#pragma pack(4)

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_WORD_REFERENCE
{

	unsigned short int
		point;
};

typedef struct TERRAIN_3D_POINT_WORD_REFERENCE terrain_3d_point_word_reference;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_POINT_BYTE_REFERENCE
{

	unsigned char
		point;
};

typedef struct TERRAIN_3D_POINT_BYTE_REFERENCE terrain_3d_point_byte_reference;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

struct TERRAIN_3D_FACE
{

	unsigned char
		number_of_points:7,
		surface_change:1;
};

typedef struct TERRAIN_3D_FACE terrain_3d_face;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_SURFACE
{

	unsigned char
		surface_id;
};

typedef struct TERRAIN_3D_SURFACE terrain_3d_surface;

///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////

struct TERRAIN_3D_SECTOR
{

	unsigned short int
		number_of_points,
		number_of_polygons,
		radius;

	unsigned char
		end_pad_byte_polygons,
		end_pad_byte_surface_changes,
		*normal_changes;

	short int
		minimum_height,
		maximum_height,
		average_height,
		pad_2_bytes,
		//number_of_surface_changes_bytes,
		number_of_surface_ids,
		number_of_normal_changes;

	int
		colour_indices_offset,
		point_colours_offset,
		max_colour_index,
		number_of_point_references;



	struct TERRAIN_3D_POINT_XZ
		*points_xz;

	struct TERRAIN_3D_POINT_Y
		*points_y;

	struct TERRAIN_3D_FACE
		*polygons;

	union
	{

		struct TERRAIN_3D_POINT_WORD_REFERENCE
			*point_word_references;

		struct TERRAIN_3D_POINT_BYTE_REFERENCE
			*point_byte_references;
	};


	//struct TERRAIN_3D_SURFACE
	//	*surface_changes;

	//struct TERRAIN_3D_SURFACE
	//	surface_changes [32];

	struct TERRAIN_3D_SURFACE
		surface_ids[256]; // without &

	struct TERRAIN_3D_SURFACE
		*surface_id_each_face;

	//struct TERRAIN_3D_COLOUR
	//	*point_colours;

	struct TERRAIN_3D_COLOUR
		point_colours[512]; // with &

	struct TERRAIN_3D_COLOUR_INDEX
		*colour_indices; // without &

};

typedef struct TERRAIN_3D_SECTOR terrain_3d_sector;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

enum TERRAIN_POLYGON_TYPE
{

	POLYGON_TYPE_FAN,
	POLYGON_TYPE_STRIP,
	POLYGON_TYPE_UNKNOWN,
};

typedef enum TERRAIN_POLYGON_TYPE terrain_polygon_type;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

enum SURFACE_TYPES
{
	SURFACE_TYPE_NONE,

	SURFACE_TYPE_DESERT,
	SURFACE_TYPE_SAND,
	SURFACE_TYPE_GRASS,
	SURFACE_TYPE_ASPHALT,
	SURFACE_TYPE_WATER,
	SURFACE_TYPE_SOIL,
	SURFACE_TYPE_FIELD_LIGHTBROWN,
	SURFACE_TYPE_FIELD_DARKBROWN,
	SURFACE_TYPE_FIELD_LIGHTGREEN,
	SURFACE_TYPE_FIELD_DARKGREEN,
	SURFACE_TYPE_ROCK,
	SURFACE_TYPE_SNOW,

	SURFACE_TYPE_LAST,
}; 

typedef enum SURFACE_TYPES surface_types;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

struct TERRAIN_TYPE_INFO
{

	int
		initialised;

	unsigned char
		red,
		green,
		blue,
		transparent;

	terrain_polygon_type
		polygon_type;


};

typedef struct TERRAIN_TYPE_INFO terrain_type_info;

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

//
// Terrain types are ordered according to their relative heights in a sector
//

enum //TERRAIN_TYPES
{

	TERRAIN_TYPE_INVALID, // 0

	// Sunken features tops
   TERRAIN_TYPE_RIVER,
   TERRAIN_TYPE_RESERVOIR,
   TERRAIN_TYPE_TRENCH,

	// Sunken features banks
   TERRAIN_TYPE_RIVER_BANK,
   TERRAIN_TYPE_TRENCH_SIDE_X,
   TERRAIN_TYPE_TRENCH_SIDE_Z,

	// Regular terrain types
   TERRAIN_TYPE_SEA,

   TERRAIN_TYPE_BEACH,

   TERRAIN_TYPE_LAND,				// 9 TERRAIN_HIGHLAND / TERRAIN_DETAIL_4	COLOUR_PASS_SWAMP / CH_TERRAIN_CP1

	TERRAIN_TYPE_FIELD1,
	TERRAIN_TYPE_FIELD2,
	TERRAIN_TYPE_FIELD3,
	TERRAIN_TYPE_FIELD4,
	TERRAIN_TYPE_FIELD5,
	TERRAIN_TYPE_FIELD6,
	TERRAIN_TYPE_FIELD7,
	TERRAIN_TYPE_FIELD8,
	TERRAIN_TYPE_FIELD9,
	TERRAIN_TYPE_FIELD10,
	TERRAIN_TYPE_FIELD11,

	TERRAIN_TYPE_ALTERED_LAND1,		// 21 TERRAIN_SCRUB			COLOUR_PASS_MOUNTAINS	
	TERRAIN_TYPE_ALTERED_LAND2,		// 22 TERRAIN_BARE_ROCK2	COLOUR_PASS_ROCK
	TERRAIN_TYPE_ALTERED_LAND3,		// 23 TERRAIN_SNOW			COLOUR_PASS_SNOW

	TERRAIN_TYPE_FOREST_FLOOR,

	// Raised features banks
   TERRAIN_TYPE_ROAD_BANK,
   TERRAIN_TYPE_RAIL_BANK, // 26

	// Raised features sides
	TERRAIN_TYPE_HEDGE_SIDE_X,
	TERRAIN_TYPE_HEDGE_SIDE_Z,
	TERRAIN_TYPE_WALL_SIDE_X,
	TERRAIN_TYPE_WALL_SIDE_Z,
   TERRAIN_TYPE_FOREST_SIDE_BOTTOM_X,
   TERRAIN_TYPE_FOREST_SIDE_BOTTOM_Z,
   TERRAIN_TYPE_FOREST_SIDE_MID_X,
   TERRAIN_TYPE_FOREST_SIDE_MID_Z,
   TERRAIN_TYPE_FOREST_SIDE_TOP_X,
   TERRAIN_TYPE_FOREST_SIDE_TOP_Z,

	// Raised features tops
   TERRAIN_TYPE_BUILT_UP_AREA1,
   TERRAIN_TYPE_BUILT_UP_AREA2,
   TERRAIN_TYPE_BUILT_UP_AREA3,
   TERRAIN_TYPE_BUILT_UP_AREA4,

   TERRAIN_TYPE_ROAD,
   TERRAIN_TYPE_TRACK,
   TERRAIN_TYPE_RAIL,
	TERRAIN_TYPE_WALL_TOP,
	TERRAIN_TYPE_HEDGE_TOP,

   TERRAIN_TYPE_FOREST_TOP, // 46

	// Temporary types used in the terrain generation, but not in the visual
	TEMP_TERRAIN_TYPE_FRINGE,
	TEMP_TERRAIN_TYPE_POWER_LINE,
	TEMP_TERRAIN_TYPE_TELEPHONE_LINE,
	TEMP_TERRAIN_TYPE_SEA_BANK,
	TEMP_TERRAIN_TYPE_COASTAL_RIVER,

	TEMP_TERRAIN_TYPE_BEACH_MID,

   TEMP_TERRAIN_TYPE_STEPPED_RIVER_BANK3,
   TEMP_TERRAIN_TYPE_STEPPED_RIVER_BANK5,
   TEMP_TERRAIN_TYPE_STEPPED_RIVER_BANK7,

   TEMP_TERRAIN_TYPE_LAKE,

   TEMP_TERRAIN_TYPE_FOREST_SIDE_X,
   TEMP_TERRAIN_TYPE_FOREST_SIDE_Z,

	TEMP_TERRAIN_TYPE_OFFROAD,
	TEMP_TERRAIN_TYPE_POLITICAL_BOUNDARY,
	TEMP_TERRAIN_TYPE_BUILT_UP_AREA_BANK,

	TERRAIN_TYPE_LAST, // 62
};

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

terrain_type_info
	terrain_type_information[TERRAIN_TYPE_LAST];

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

struct ADJACENT_POLY_LIST
{
	int
		number_of_adjacent_polys,
		adjacent_poly_numbers[16],
		adjacent_poly_surface_ids[16],
		number_of_points_in_common_with_adjacent_poly[16],
		oddball_bool;
};

typedef struct ADJACENT_POLY_LIST adjacent_poly_list;

/////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////

struct CONTOUR_LAKE
{

	int
		number_of_triangles;

	float
		xmin,
		xmax,
		zmin,
		zmax;

	vec3d
		*triangle_points;
};

typedef struct CONTOUR_LAKE contour_lake;

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

struct WORLD_COORD_TRIANGLE
{

	int
		number_of_triangles;

	float
		avg_x,
		avg_z;

	vec3d
		pts[3];
};

typedef struct WORLD_COORD_TRIANGLE world_coord_triangle;

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

typedef struct {
   double x,y;
} Point;

/*
set_3d_rain_special_snow_flag ( FALSE ); //????

	terrain_texture_sea_detail									= get_system_texture_index ( "TERRAIN_SEA_1" );
	terrain_texture_sea_colour_pass							= get_system_texture_index ( "COLOUR_PASS_SEA_LEBANON" );
	terrain_texture_beach_detail								= get_system_texture_index ( "TERRAIN_BEACH" );
	terrain_texture_beach_colour_pass						= get_system_texture_index ( "COLOUR_PASS_LEBANON" );

	terrain_texture_land_detail								= get_system_texture_index ( "TERRAIN_HIGHLAND" );
	terrain_texture_land_colour_pass							= get_system_texture_index ( "COLOUR_PASS_SWAMP" );

	terrain_texture_road_detail								= get_system_texture_index ( "TERRAIN_DETAIL_ROAD" );
	terrain_texture_road_colour_pass							= get_system_texture_index ( "COLOUR_PASS_WALL" );      //TERRAIN_THAI_TRACK" );
	terrain_texture_track_detail								= get_system_texture_index ( "TERRAIN_DETAIL_ROAD" );
	terrain_texture_track_colour_pass						= get_system_texture_index ( "COLOUR_PASS_YEMEN_BAKED2" );
	terrain_texture_river_detail								= get_system_texture_index ( "TERRAIN_RIVER" );
	terrain_texture_river_colour_pass						= get_system_texture_index ( "COLOUR_PASS_SEA_LEBANON" );
	terrain_texture_reservoir_detail							= get_system_texture_index ( "TERRAIN_RIVER" );
	terrain_texture_reservoir_colour_pass					= get_system_texture_index ( "COLOUR_PASS_SEA_LEBANON" );
	terrain_texture_road_bank_detail							= get_system_texture_index ( "TERRAIN_DETAIL_4" );
	terrain_texture_road_bank_colour_pass					= get_system_texture_index ( "COLOUR_PASS_YEMEN_BAKED2" );
	terrain_texture_river_bank_detail						= get_system_texture_index ( "TERRAIN_YEMEN_GRITTY" );
	terrain_texture_river_bank_colour_pass					= get_system_texture_index ( "COLOUR_PASS_YEMEN_BAKED2" );
	terrain_texture_altered_land1_detail					= get_system_texture_index ( "TERRAIN_SCRUB" );
	terrain_texture_altered_land1_colour_pass				= get_system_texture_index ( "COLOUR_PASS_MOUNTAINS" );
	terrain_texture_altered_land2_detail					= get_system_texture_index ( "TERRAIN_BARE_ROCK2" );
	terrain_texture_altered_land2_colour_pass				= get_system_texture_index ( "COLOUR_PASS_ROCK" );
	terrain_texture_altered_land3_detail					= get_system_texture_index ( "TERRAIN_SNOW" );
	terrain_texture_altered_land3_colour_pass				= get_system_texture_index ( "COLOUR_PASS_SNOW" );


  Georgia or default for mod maps

  
	terrain_texture_sea_detail									= get_system_texture_index ( "TERRAIN_SEA_1" );
	terrain_texture_sea_colour_pass							= get_system_texture_index ( "COLOUR_PASS_GRG_SEA" );
	terrain_texture_beach_detail								= get_system_texture_index ( "TERRAIN_BEACH" );
	terrain_texture_beach_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_land_detail								= get_system_texture_index ( "TERRAIN_DETAIL_4" );
	terrain_texture_land_colour_pass							= get_system_texture_index ( "CH_TERRAIN_CP1" );
	terrain_texture_forest_detail								= get_system_texture_index ( "TERRAIN_TREE_CANOPY_1" );
	terrain_texture_forest_colour_pass						= 0;
	terrain_texture_builtup_area1_detail					= get_system_texture_index ( "TERRAIN_GRG_CITY4" );
	terrain_texture_builtup_area1_colour_pass				= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area2_detail					= get_system_texture_index ( "TERRAIN_GRG_CITY3" );
	terrain_texture_builtup_area2_colour_pass				= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area3_detail					= get_system_texture_index ( "TERRAIN_GRG_CITY2" );
	terrain_texture_builtup_area3_colour_pass				= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area4_detail					= get_system_texture_index ( "TERRAIN_GRG_CITY1" );
	terrain_texture_builtup_area4_colour_pass				= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area1_infrared_detail		= get_system_texture_index ( "TERRAIN_BW_CITY4" );
	terrain_texture_builtup_area1_infrared_colour_pass	= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area2_infrared_detail		= get_system_texture_index ( "TERRAIN_BW_CITY3" );
	terrain_texture_builtup_area2_infrared_colour_pass	= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area3_infrared_detail		= get_system_texture_index ( "TERRAIN_BW_CITY2" );
	terrain_texture_builtup_area3_infrared_colour_pass	= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_builtup_area4_infrared_detail		= get_system_texture_index ( "TERRAIN_BW_CITY1" );
	terrain_texture_builtup_area4_infrared_colour_pass	= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_road_detail								= get_system_texture_index ( "TERRAIN_DETAIL_ROAD" );
	terrain_texture_road_colour_pass							= get_system_texture_index ( "COLOUR_PASS_WALL" );
	terrain_texture_track_detail								= get_system_texture_index ( "TERRAIN_THAI_TRACK" );
	terrain_texture_track_colour_pass						= get_system_texture_index ( "COLOUR_PASS_WALL" );
	terrain_texture_river_detail								= get_system_texture_index ( "TERRAIN_RIVER" );
	terrain_texture_river_colour_pass						= 0;
	terrain_texture_reservoir_detail							= get_system_texture_index ( "TERRAIN_RIVER" );
	terrain_texture_reservoir_colour_pass					= 0;
	terrain_texture_rail_detail								= 0;
	terrain_texture_rail_colour_pass							= 0;
	terrain_texture_road_bank_detail							= get_system_texture_index ( "TERRAIN_DETAIL_4" );
	terrain_texture_road_bank_colour_pass					= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_river_bank_detail						= get_system_texture_index ( "TERRAIN_BEACH" );
	terrain_texture_river_bank_colour_pass					= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_rail_bank_detail							= 0;
	terrain_texture_rail_bank_colour_pass					= 0;
	terrain_texture_field1_detail								= get_system_texture_index ( "TERRAIN_MUSTARDFIELD" );
	terrain_texture_field1_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field2_detail								= get_system_texture_index ( "TERRAIN_PLOUGHED1FIELD" );
	terrain_texture_field2_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field3_detail								= get_system_texture_index ( "TERRAIN_PLOUGHED2FIELD" );
	terrain_texture_field3_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field4_detail								= get_system_texture_index ( "TERRAIN_CORNFIELD" );
	terrain_texture_field4_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field5_detail								= get_system_texture_index ( "TERRAIN_WHEATFIELD" );
	terrain_texture_field5_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field6_detail								= get_system_texture_index ( "TERRAIN_LIGHTGREENFIELD" );
	terrain_texture_field6_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field7_detail								= get_system_texture_index ( "TERRAIN_DARKGREENFIELD" );
	terrain_texture_field7_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field8_detail								= get_system_texture_index ( "TERRAIN_MUSTARDFIELD" );
	terrain_texture_field8_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field9_detail								= get_system_texture_index ( "TERRAIN_PLOUGHED1FIELD" );
	terrain_texture_field9_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field10_detail							= get_system_texture_index ( "TERRAIN_PLOUGHED2FIELD" );
	terrain_texture_field10_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_field11_detail							= get_system_texture_index ( "TERRAIN_CORNFIELD" );
	terrain_texture_field11_colour_pass						= get_system_texture_index ( "COLOUR_PASS" );
	terrain_texture_altered_land1_detail					= get_system_texture_index ( "TERRAIN_SCRUB" );
	terrain_texture_altered_land1_colour_pass				= get_system_texture_index ( "COLOUR_PASS_MOUNTAINS" );
	terrain_texture_altered_land2_detail					= get_system_texture_index ( "TERRAIN_BARE_ROCK2" );
	terrain_texture_altered_land2_colour_pass				= get_system_texture_index ( "COLOUR_PASS_ROCK" );
	terrain_texture_altered_land3_detail					= get_system_texture_index ( "TERRAIN_SNOW" );
	terrain_texture_altered_land3_colour_pass				= get_system_texture_index ( "COLOUR_PASS_SNOW" );
	terrain_texture_hedge_detail								= get_system_texture_index ( "TERRAIN_TREE_CANOPY_1" );
	terrain_texture_hedge_colour_pass						= 0;
	terrain_texture_wall_detail								= get_system_texture_index ( "TERRAIN_TREE_CANOPY_1" );
	terrain_texture_wall_colour_pass							= get_system_texture_index ( "COLOUR_PASS_ROCK" );
	terrain_texture_trench_detail								= get_system_texture_index ( "TERRAIN_RIVER" );
	terrain_texture_trench_colour_pass						= 0;

*/

