// check_terrain_for_errors
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "roads_and_bridges.h"

// the bridge stuff is found in bridge.c
/* Files Required

	default.sec
	terrain.ffp
	bridges.txt
	Grand.ROAD
	

   Files Produced

	bridge.dat
	ROADS.DAT
	ROADS.NDE
	ROADS.WP
*/

////////////////////////////////////////
// Globals
////////////////////////////////////////

/////////////////////////////////////////////////
// *** start of Global Lists
/////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// holds geometry data terrain_sectors[z][x]
////////////////////////////////////////////////////////////////////////////

terrain_3d_sector
	**terrain_sectors;

/////////////////////////////////////////////////////////////////////////////
// Holds up to 128 roads and up to 512 quads per road
/////////////////////////////////////////////////////////////////////////////

road
	roads[128];

/////////////////////////////////////////////////////////////////////////////
// Holds up to 128 bridges
/////////////////////////////////////////////////////////////////////////////

bridge
	bridges[128];

/////////////////////////////////////////////////////////////////////////////
// Holds all the road data
/////////////////////////////////////////////////////////////////////////////

contour_path
	*contour_map_road_paths;

vec3d
	*contour_map_road_nodes,
	*contour_map_road_path_positions,
	*current_position;

vec3d
	start_road_nodes [ 128 ],
	end_road_nodes [ 128 ],
	total_road_nodes [ 256 ];


road_path
	road_paths [ 128 ],
	sub_divided_road_paths [ 256 ];
		

////////////////////////////////////////////////////
// *** end of Global Lists
////////////////////////////////////////////////////

int
	global_number_of_roads,
	terrain_version,
	rgb_4byte_header,
	ffp_4byte_header,
	terrain_3d_map_width,
	terrain_3d_map_height,
	number_of_bridges,
	contour_map_number_of_road_nodes,
	contour_map_number_of_road_paths,
	longest_span_length = 0,
	global_total_number_of_nodes = 0,
	global_number_of_divided_paths = 0;

short int
	pad_word_zero;

unsigned short int
	normal_changes_word;



float
	terrain_3d_map_minimum_height,
	terrain_3d_map_maximum_height,
	terrain_3d_map_scaled_height_difference;


FILE
	*changed_road_pts;

/////////////////////////////////////////
// functions
/////////////////////////////////////////

void read_in_bridge_file(void);

void process_bridges(void);

int read_in_bridge_text_file(void);

void read_in_road_data(void);

int InsidePolygon(simple_point *polygon,int N,simple_point p);

int get_sector_y_value(int sec_x, int sec_z, int x, int z);

float get_global_y_value(vec3d pt);

void get_bridge_path(int brdg);

void get_max_leg_height(int brdg);

int get_span_length(vec3d p, vec3d p1);

void adjust_y_values_on_road_for_bridges(void);

int is_between(vec3d br_start, vec3d p, vec3d br_end);

void adjust_road_y_point(int bridge, int path, int point);

void change_path_type_value(void);

void write_new_roads_wp_file(void);

vec3d get_point_at_distance_d(vec3d p, vec3d p1, float d);

void read_in_bridge_road_file(void);

void load_3d_terrain (void);

void read_in_ROAD_file(void);

void convert_ROAD_data_to_eech_road_data(void);

void write_eech_road_files(void);

void write_eech_road_dat_file(void);

int node_in_list(vec3d p);


int main(int argc, char ** argv){

	load_3d_terrain ();

	////////////////////////////////////////////////////
	// this is for writing eech road files
	////////////////////////////////////////////////////
/*
	read_in_ROAD_file();

	convert_ROAD_data_to_eech_road_data();

	write_eech_road_files();

	write_eech_road_dat_file();
*/
	///////////////////////////////////////////////////
	// this is for writing bridge files
	///////////////////////////////////////////////////
/*
	read_in_bridge_text_file();

	read_in_road_data();

	process_bridges();
*/

	read_in_ROAD_file();

	convert_ROAD_data_to_eech_road_data();

	convert_ROAD_data_to_eech_road_data_with_junctions();

	return 0;

} // end main

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void process_bridges(){

	FILE	*out_bridge_dat_file_ptr;

	int		bridge_loop,
			x_sector,
			z_sector;

	float
		max_leg_height = 0.0;

	

	printf("processing bridges\n");

	
	if(( out_bridge_dat_file_ptr = fopen("BRIDGE.DAT","wb"))==NULL){
		printf("cannot open BRIDGE.DAT\n");
		exit(1);
	}
	else printf("BRIDGE.DAT opened\n");

	// the global value of terrain_3d_map_scaled_height_difference is set here

	//terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	for(bridge_loop = 0; bridge_loop < number_of_bridges; bridge_loop++){

		///////////////////////////////////////////////////////////////////////////
		// get the y values for the end points
		///////////////////////////////////////////////////////////////////////////

		bridges[bridge_loop].end1.y = get_global_y_value(bridges[bridge_loop].end1);

		bridges[bridge_loop].end2.y = get_global_y_value(bridges[bridge_loop].end2);

		///////////////////////////////////////////////////////////////////////////
		// get the path start and end nodes
		///////////////////////////////////////////////////////////////////////////

		get_bridge_path( bridge_loop );

		//get_max_leg_height(bridge_loop);
		
	} // end for number of bridges

	///////////////////////////////////////////////////////////////////////////////
	// write the eech bridge file
	///////////////////////////////////////////////////////////////////////////////

	fwrite (&number_of_bridges, sizeof (int), 1, out_bridge_dat_file_ptr);

	for(bridge_loop = 0; bridge_loop < number_of_bridges; bridge_loop ++){

		fwrite (&bridges[bridge_loop].end1, sizeof (vec3d), 1, out_bridge_dat_file_ptr);

		fwrite (&bridges[bridge_loop].end2, sizeof (vec3d), 1, out_bridge_dat_file_ptr);

		fwrite (&bridges[bridge_loop].start_node, sizeof (int), 1, out_bridge_dat_file_ptr);

		fwrite (&bridges[bridge_loop].end_node, sizeof (int), 1, out_bridge_dat_file_ptr);

		fwrite (&bridges[bridge_loop].type, sizeof (int), 1, out_bridge_dat_file_ptr);

		//fwrite (&bridges[bridge_loop].max_leg_height, sizeof (float), 1, out_bridge_dat_file_ptr);

		fwrite (&max_leg_height, sizeof (float), 1, out_bridge_dat_file_ptr);

	} // end for bridge loop


	fclose (out_bridge_dat_file_ptr);

} // end process_bridges

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////


int get_sector_y_value(int sec_x, int sec_z, int x, int z){

	// This function returns the y value in sector coordinates
	// for the sector and x z sector coord pair

	int		i,
			polys_this_sector,
			poly_counter,
			poly_number,
			y_value;

	long long	k, // plane constant
				triangle_points[3][3], // [][0]=x, [][1]=z, [][2]=y
				Ax,Az,Ay,Bx,Bz,By,	// vectors
				Nx,Nz,Ny;			// normal vector

	double	terrain_3d_map_scaled_height_difference;

	simple_point	inside_poly_array[3],
					test_point;

	terrain_3d_map_scaled_height_difference = (double)( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height )
		* TERRAIN_3D_Y_SCALE;


	polys_this_sector = (int)terrain_sectors[sec_z][sec_x].number_of_polygons;

	test_point.x = x;
	test_point.z = z;

	// Find which polygon of this sector the point is on
	// by testing for point_in_poly with all polys until it is found

	for(poly_counter = 0; poly_counter < polys_this_sector; poly_counter++){

		poly_number = 1000;

		triangle_points[0][0] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 ].point].x;

		triangle_points[1][0] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 1 ].point].x;

		triangle_points[2][0] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 2 ].point].x;

		triangle_points[0][1] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 ].point].z;

		triangle_points[1][1] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 1 ].point].z;

		triangle_points[2][1] = (long long)terrain_sectors[sec_z][sec_x].
			points_xz[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 2 ].point].z;

		triangle_points[0][2] = (long long)terrain_sectors[sec_z][sec_x].
			points_y[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 ].point].y;

		triangle_points[1][2] = (long long)terrain_sectors[sec_z][sec_x].
			points_y[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 1 ].point].y;

		triangle_points[2][2] = (long long)terrain_sectors[sec_z][sec_x].
			points_y[(int)terrain_sectors[sec_z][sec_x].
			point_word_references[ poly_counter * 3 + 2 ].point].y;

		// assign points to inside_poly_array[]

		for(i = 0; i < 3; i++){
			inside_poly_array[i].x = (double)triangle_points[i][0];
			inside_poly_array[i].z = (double)triangle_points[i][1]; 
		}

		//printf("%d  inside_poly array  %f  %f x %f  %f x %f  %f  test_point  %f  %f\n",
		//	poly_counter,
		//	inside_poly_array[0].x, inside_poly_array[0].z,
		//	inside_poly_array[1].x, inside_poly_array[1].z,
		//	inside_poly_array[2].x, inside_poly_array[2].z,
		//	test_point.x, test_point.z);

		if(InsidePolygon(inside_poly_array, 3, test_point)){

			poly_number = poly_counter;
			break;
		}

	} // end for poly loop

	if(poly_number == 2048){
		printf("can't find poly for point %d x %d in sector %d x %d\n", x, z, sec_x, sec_z);
		exit(1);
	}

	// vectors
	Ax = triangle_points[0][0] - triangle_points[1][0]; // ax - bx
	Ay = triangle_points[0][2] - triangle_points[1][2]; // ay - by
	Az = triangle_points[0][1] - triangle_points[1][1]; // az - bz
	Bx = triangle_points[2][0] - triangle_points[1][0]; // cx - bx
	By = triangle_points[2][2] - triangle_points[1][2]; // cy - by
	Bz = triangle_points[2][1] - triangle_points[1][1]; // cz - bz

	// normal
	Nx = Ay * Bz - Az * By;
	Ny = Az * Bx - Ax * Bz;
	Nz = Ax * By - Ay * Bx;
				
	// plane constant
	//k = ax * Nx + ay * Ny + az * Nz;
	k = triangle_points[0][0] * Nx + triangle_points[0][2] * Ny + triangle_points[0][1] * Nz;

	y_value = (int)((k - x * Nx - z * Nz) / Ny);

	//printf("%d   %d   %d   %d   %d\n", poly_number, (int)triangle_points[0][2],
	//	(int)triangle_points[1][2], (int)triangle_points[2][2], y_value);


	return y_value;
} // end get_sector_y_value()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

//function returns INSIDE or OUTSIDE indicating the status of a point P with respect to a polygon with N points.

int InsidePolygon(simple_point *polygon,int N,simple_point p)
{
  int counter = 0;
  int i;
  double xinters;
  simple_point p1,p2;

  p1 = polygon[0];
  for (i=1;i<=N;i++) {
    p2 = polygon[i % N];
    if (p.z > MIN(p1.z,p2.z)) {
      if (p.z <= MAX(p1.z,p2.z)) {
        if (p.x <= MAX(p1.x,p2.x)) {
          if (p1.z != p2.z) {
            xinters = (p.z-p1.z)*(p2.x-p1.x)/(p2.z-p1.z)+p1.x;
            if (p1.x == p2.x || p.x <= xinters)
              counter++;
          }
        }
      }
    }
    p1 = p2;
  }

  if (counter % 2 == 0)
    return(OUTSIDE);
  else
    return(INSIDE);
} // end InsidePolygon()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void read_in_bridge_file(void){

	int
		type,
		local_number_of_bridges,
		start_node,
		end_node,
		loop,
		span_length;

	float
		max_leg_height;

	vec3d
		end1,
		end2;


	FILE
		*fp,
		*out_fp;


	if(( fp = fopen("bridge.dat","rb"))==NULL){
		printf("cannot open bridge.dat\n");
		exit(1);
	}
	else printf("bridge.dat opened\n");

	if(( out_fp = fopen("bridge_data_text.log","wb"))==NULL){
		printf("cannot open bridge_data_text.log\n");
		exit(1);
	}
	else printf("bridge_data_text.log opened\n");

	
	fread (&local_number_of_bridges, sizeof (int), 1, fp);

	fprintf(out_fp, "Number of bridges = %d\n\n", local_number_of_bridges);

	fprintf(out_fp, "end1 = X  Z  Y\nend2 = X  Z  Y\n\n");

	for (loop = 0 ; loop < local_number_of_bridges ; loop ++){

		fread (&end1, sizeof (vec3d), 1, fp);

		fread (&end2, sizeof (vec3d), 1, fp);

		fread (&start_node, sizeof (int), 1, fp);

		fread (&end_node, sizeof (int), 1, fp);

		fread (&type, sizeof (int), 1, fp);

		fread (&max_leg_height, sizeof (float), 1, fp);

		fprintf(out_fp, "Bridge %d\n\n", loop);

		fprintf(out_fp, "end1 = %f  %f  %f\n", end1.x, end1.z, end1.y);

		fprintf(out_fp, "end2 = %f  %f  %f\n", end2.x, end2.z, end2.y);

		span_length = get_span_length(end1, end2);

		if(span_length > longest_span_length){

			longest_span_length = span_length;
		} // end if

		fprintf(out_fp, "span = %d\n", span_length);

		fprintf(out_fp, "start node = %d\n", start_node);

		fprintf(out_fp, "end node = %d\n", end_node);

		fprintf(out_fp, "type = %d\n", type);

		fprintf(out_fp, "max leg height = %f\n\n", max_leg_height);

	} // end for bridge loop

	fprintf(out_fp, "\nlongest span length = %d\n", longest_span_length);

	close (fp);

	// set global value of number of bridges

	number_of_bridges = local_number_of_bridges;

} // end read_in_bridge_file(void)

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int read_in_bridge_text_file(void){

	FILE	*fp;

	int
		a,
		char_count,
		bridge_count = 0,
		road[256];

	char
		ch,
		road_string[256][4],
		end1x_string[256][32],
		end1z_string[256][32],
		end2x_string[256][32],
		end2z_string[256][32];

	double
		end1x[256],
		end1z[256],
		end2x[256],
		end2z[256];

	if(( fp = fopen("bridges.txt","r"))==NULL){
		return 0;
	}
	else printf("bridges.txt opened\n");

	ch = getc(fp);

	do{

		//ch = getc(fp);

		// get the road integer string

		char_count = 0;

		while(ch != ' '){

			if(char_count > 15){

				printf("Error: char_count > 15 reading 1st integer of bridge %d\n", bridge_count);
				exit(1);
			}

			road_string[bridge_count][char_count] = ch;

			char_count++;

			ch = getc(fp);
		} // end while

		road_string[bridge_count][char_count] = NULL;

		// get the x start pos string

		char_count = 0;

		ch = getc(fp);

		while(ch != ' '){

			if(char_count > 20){

				printf("Error: char_count > 20 reading x start pos of bridge %d\n", bridge_count);
				exit(1);
			}

			end1x_string[bridge_count][char_count] = ch;

			char_count++;

			ch = getc(fp);
		} // end while

		end1x_string[bridge_count][char_count] = NULL;

		// get the z start pos string

		char_count = 0;

		ch = getc(fp);

		while(ch != ' '){

			if(char_count > 20){

				printf("Error: char_count > 20 reading z start pos of bridge %d\n", bridge_count);
				exit(1);
			}

			end1z_string[bridge_count][char_count] = ch;

			char_count++;

			ch = getc(fp);
		} // end while

		end1z_string[bridge_count][char_count] = NULL;

		// get the x end pos string

		char_count = 0;

		ch = getc(fp);

		while(ch != ' '){

			if(char_count > 20){

				printf("Error: char_count > 20 reading x end pos of bridge %d\n", bridge_count);
				exit(1);
			}

			end2x_string[bridge_count][char_count] = ch;

			char_count++;

			ch = getc(fp);
		} // end while

		end2x_string[bridge_count][char_count] = NULL;

		// get the z end pos string

		char_count = 0;

		ch = getc(fp);

		while(ch != '\n' && ch != EOF){

			if(char_count > 20){

				printf("Error: char_count > 20 reading z end pos of bridge %d\n", bridge_count);
				exit(1);
			}

			end2z_string[bridge_count][char_count] = ch;

			char_count++;

			ch = getc(fp);
		} // end while

		end2z_string[bridge_count][char_count] = NULL;

		bridge_count++;

		ch = getc(fp);

	} while(ch != EOF);

	fclose(fp);

	////////////////////////////////////////////////////////////
	// convert strings to ints and floats
	// and populate global data
	////////////////////////////////////////////////////////////

	printf("bridge count = %d\n", bridge_count);

	for(a = 0; a < bridge_count; a++){

		road[a] = atoi(road_string[a]);

		end1x[a] = atof(end1x_string[a]);

		end1z[a] = atof(end1z_string[a]);

		end2x[a] = atof(end2x_string[a]);

		end2z[a] = atof(end2z_string[a]);

		bridges[a].road = road[a];

		bridges[a].end1.x = (float)end1x[a];

		bridges[a].end1.z = (float)end1z[a];

		bridges[a].end2.x = (float)end2x[a];

		bridges[a].end2.z = (float)end2z[a];
	} // end for

	//for(a = 0; a < bridge_count; a++){

	//	printf("%d  %f  %f  %f  %f\n", road[a], end1x[a], end1z[a] , end2x[a], end2z[a]);
	//} // end for

	number_of_bridges = bridge_count;

	return 1;

	
} // end read in bridge text file

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void read_in_road_data(void){

	FILE
		*fp;

	int
		x,
		total,
		count;

	///////////////////////////////////////////////////////
	// read in the nodes
	///////////////////////////////////////////////////////

	if(( fp = fopen("C:\\Razorworks\\common\\maps\\map31\\route\\ROADS.NDE","rb"))==NULL){
		printf("cannot open ROADS.NDE\n");
		exit(1);
	}
	else printf("ROADS.NDE opened\n");

	fread ( &contour_map_number_of_road_nodes, sizeof ( int ), 1, fp );

	contour_map_road_nodes = malloc ( sizeof ( vec3d ) * contour_map_number_of_road_nodes );

	fread ( contour_map_road_nodes, sizeof ( vec3d ), contour_map_number_of_road_nodes, fp );

	fclose(fp);

	/////////////////////////////////////////////////////////////
	// read in the paths
	/////////////////////////////////////////////////////////////

	if(( fp = fopen("C:\\Razorworks\\common\\maps\\map31\\route\\ROADS.WP","rb"))==NULL){
		printf("cannot open ROADS.WP\n");
		exit(1);
	}
	else printf("ROADS.WP opened\n");

	fread ( &contour_map_number_of_road_paths, sizeof ( int ), 1, fp );

	contour_map_road_paths = malloc ( sizeof ( contour_path ) * contour_map_number_of_road_paths );

	total = 0;

	for ( count = 0; count < contour_map_number_of_road_paths; count++ )
	{

		fread ( &contour_map_road_paths[count].from, sizeof ( int ), 1, fp );
		fread ( &contour_map_road_paths[count].to, sizeof ( int ), 1, fp );
		fread ( &contour_map_road_paths[count].type, sizeof ( int ), 1, fp );
		fread ( &contour_map_road_paths[count].count, sizeof ( int ), 1, fp );

		total += contour_map_road_paths[count].count;

		fseek ( fp, ( contour_map_road_paths[count].count * sizeof ( vec3d ) ), SEEK_CUR );
	}

	fseek ( fp, 4, SEEK_SET );

	contour_map_road_path_positions = malloc ( sizeof ( vec3d ) * total );

	current_position = contour_map_road_path_positions;

	for ( count = 0; count < contour_map_number_of_road_paths; count++ )
	{

		fread ( &contour_map_road_paths[count].from, sizeof ( int ), 1, fp );
		fread ( &contour_map_road_paths[count].to, sizeof ( int ), 1, fp );
		fread ( &contour_map_road_paths[count].type, sizeof ( int ), 1, fp );
		fread ( &contour_map_road_paths[count].count, sizeof ( int ), 1, fp );

 		//contour_map_road_paths[count].from = bound ( contour_map_road_paths[count].from, 0, contour_map_number_of_road_nodes );
		//contour_map_road_paths[count].to = bound ( contour_map_road_paths[count].to, 0, contour_map_number_of_road_nodes );

		contour_map_road_paths[count].points = current_position;

		fread ( current_position, sizeof ( vec3d ), contour_map_road_paths[count].count, fp );

		//
		// print path data and points
		//

		//fprintf(out_path_fp, "path %d  from node %d  to node %d  type = %d  count = %d\n\n", count,
		//	contour_map_road_paths[count].from,
		//	contour_map_road_paths[count].to,
		//	contour_map_road_paths[count].type,
		//	contour_map_road_paths[count].count);

	//	printf("xmin = %f  xmax = %f  zmin = %f  zmax = %f\n", not used it seems
	//		contour_map_road_paths[count].xmin,
	//		contour_map_road_paths[count].xmax,
	//		contour_map_road_paths[count].zmin,
	//		contour_map_road_paths[count].zmax);


		//fprintf(out_path_fp, "points  X  Z  Y\n\n");

		//for(x = 0; x < contour_map_road_paths[count].count; x++){

		//	fprintf(out_path_fp, "%d  %f  %f  %f\n", x, current_position[x].x,
		//		current_position[x].z, current_position[x].y);
		//} // end for

		//fprintf(out_path_fp, "\n");

		current_position += contour_map_road_paths[count].count;
	} // end for number of paths

	fclose ( fp );

} // end read_in_road_data

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

float get_global_y_value(vec3d pt){

	int		x_sector,	// the sector
			z_sector,
			sec_x,		// the point value in sector coordinates
			sec_z,
			sec_y;

	float
		global_y_value;

	/////////////////////////////////////////////////////////////
	// find the sector
	/////////////////////////////////////////////////////////////

	x_sector = (int)pt.x / 2048;
	z_sector = (int)pt.z / 2048;

	//////////////////////////////////////////////////////////////
	// find the sector coordinates
	//////////////////////////////////////////////////////////////

	sec_x = (int)((pt.x - (x_sector * 2048 + 1024)) * 16);

	sec_z = (int)((pt.z - (z_sector * 2048 + 1024)) * 16);

	///////////////////////////////////////////////////////////////
	// get the y value in sector coordinates
	///////////////////////////////////////////////////////////////

	sec_y = get_sector_y_value(x_sector, z_sector, sec_x, sec_z);



	////////////////////////////////////////////////////////////////
	// calculate the global y value
	////////////////////////////////////////////////////////////////

	//global_y_value = (float)((double)terrain_3d_map_minimum_height + 
	//			((double)sec_y * terrain_3d_map_scaled_height_difference ));

	global_y_value = (float)(terrain_3d_map_minimum_height + 
				((float)sec_y * terrain_3d_map_scaled_height_difference ));

	//printf("%f\n", global_y_value);

	return global_y_value;

} // end float get_global_y_value

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void get_bridge_path( int brdg ){

	int
		path_loop,
		point_loop,
		number_of_paths = contour_map_number_of_road_paths,
		number_of_points,
		end1_path,
		end2_path,
		min_distance_path;

	float
		x,
		z,
		x1,
		z1,
		distance,
		min_distance,
		min_distance_const = 2000.0;

	///////////////////////////////////////////////////////
	// find end1 path
	///////////////////////////////////////////////////////

	x = bridges[brdg].end1.x;
	z = bridges[brdg].end1.z;

	min_distance = 4096.0;

	for(path_loop = 0; path_loop < number_of_paths; path_loop++){

		number_of_points = contour_map_road_paths[path_loop].count;

		for(point_loop = 0; point_loop < number_of_points; point_loop++){

			x1 = contour_map_road_paths[path_loop].points[point_loop].x;
			z1 = contour_map_road_paths[path_loop].points[point_loop].z;

			distance = sqrt(((x1 - x) * ((x1 - x))) + ((z1 - z) * (z1 - z)));

			if(distance < min_distance){

				min_distance = distance;

				min_distance_path = path_loop;
			} // end if
			
		} // end for point_loop
	} // end for path_loop

	if(min_distance < min_distance_const){

		end1_path = min_distance_path;
	} // end if
	else{

		printf("Error: bridge %d end1 is > %f from all path points!\n", brdg, min_distance_const);

		exit(1);
	}

	///////////////////////////////////////////////////////
	// find end2 path
	///////////////////////////////////////////////////////

	x = bridges[brdg].end2.x;
	z = bridges[brdg].end2.z;

	min_distance = 4096.0;

	for(path_loop = 0; path_loop < number_of_paths; path_loop++){

		number_of_points = contour_map_road_paths[path_loop].count;

		for(point_loop = 0; point_loop < number_of_points; point_loop++){

			x1 = contour_map_road_paths[path_loop].points[point_loop].x;
			z1 = contour_map_road_paths[path_loop].points[point_loop].z;

			distance = sqrt(((x1 - x) * ((x1 - x))) + ((z1 - z) * (z1 - z)));

			if(distance < min_distance){

				min_distance = distance;

				min_distance_path = path_loop;
			} // end if
			
		} // end for point_loop
	} // end for path_loop

	if(min_distance < min_distance_const){

		end2_path = min_distance_path;
	} // end if
	else{

		printf("Error: bridge %d end1 is > %f from all path points!\n", brdg, min_distance_const);

		exit(1);
	}

	if(end1_path == end2_path){

		bridges[brdg].start_node = contour_map_road_paths[end1_path].from;

		bridges[brdg].end_node = contour_map_road_paths[end1_path].to;
	}
	else{

		printf("Error: bridge %d end1 is on path %d and end2 is on path %d!\n",
			brdg, end1_path, end2_path);

		exit(1);
	}
} // end get_bridge_path

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void get_max_leg_height(int brdg){

	int
		interval = 10,
		next_distance,
		while_runaway_check = 0;

	float
		min_y_value = 9999999.9,
		current_y_value;

	vec3d
		center_point,
		current_point,
		next_point;

	next_distance = get_span_length(bridges[brdg].end1, bridges[brdg].end2);

	if(next_distance < 50){

		center_point.x = (bridges[brdg].end1.x + bridges[brdg].end2.x) / 2;

		center_point.z = (bridges[brdg].end1.z + bridges[brdg].end2.z) / 2;

		center_point.y = get_global_y_value(center_point);

		bridges[brdg].max_leg_height = bridges[brdg].end1.y - center_point.y;
	}
	else{

		current_point = bridges[brdg].end1;

		while(next_distance > interval){

			next_point = get_point_at_distance_d(current_point, 
				bridges[brdg].end2, (float) interval);

			current_y_value = get_global_y_value(next_point);

			if(current_y_value < min_y_value){
				
				min_y_value = current_y_value;
			}

			next_distance = get_span_length(current_point, bridges[brdg].end2);

			current_point = next_point;

			if(while_runaway_check > 16384){

				printf("Error: while_runaway_check > 16384 in function get_max_leg_height\n");

				exit(1);
			}

			while_runaway_check ++;

		} // end while

		bridges[brdg].max_leg_height = ((bridges[brdg].end1.y + bridges[brdg].end2.y) / 2) 
			- min_y_value;

	} // end if else

	

/*	vec3d	center_point;

	center_point.x = (bridges[brdg].end1.x + bridges[brdg].end2.x) / 2;

	center_point.z = (bridges[brdg].end1.z + bridges[brdg].end2.z) / 2;

	center_point.y = get_global_y_value(center_point);

	bridges[brdg].max_leg_height = bridges[brdg].end1.y - center_point.y;
*/
} // end get_max_leg_height

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

vec3d get_point_at_distance_d(vec3d p, vec3d p1, float d){

	// gets a point on line (p, p1) at distance d from p

	float
		l;

	vec3d
		return_value;

	l = sqrt(((p1.x - p.x) * (p1.x - p.x)) + ((p1.z - p.z) * (p1.z - p.z)));

	return_value.x = (d / l) * (p1.x - p.x) + p.x;

	return_value.z = (d / l) * (p1.z - p.z) + p.z;

	return return_value;

} // end get_point_at_distance_d

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int get_span_length(vec3d p, vec3d p1){

	int
		span_length;

	span_length = (int)sqrt(((p1.x - p.x) * (p1.x - p.x)) + ((p1.z - p.z) * (p1.z - p.z)));

	return span_length;

} // end get_span_length

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void adjust_y_values_on_road_for_bridges(void){

	int
		bridge_loop,
		path_loop,
		point_loop;

	//printf("adjust_y_values_on_road_for_bridges has been called\n");
	//printf("number of bridges = %d\n", number_of_bridges);
	//printf("number of paths = %d\n", contour_map_number_of_road_paths); 

	if(( changed_road_pts = fopen("changed_road_points.log","wb"))==NULL){
		printf("cannot open changed_road_points.log\n");
		exit(1);
	}
	else printf("changed_road_points.log opened\n");

	for(bridge_loop = 0; bridge_loop < number_of_bridges; bridge_loop++){

		fprintf(changed_road_pts, "\nBridge %d\n\n", bridge_loop);

		for(path_loop = 0; path_loop < contour_map_number_of_road_paths; path_loop++){

			// if the bridge is on this path

			//printf("bridge start node = %d\n" , bridges[bridge_loop].start_node);
			//printf("bridge end node = %d\n" , bridges[bridge_loop].end_node); 
			//printf("path start node = %d\n" , contour_map_road_paths[path_loop].from); 
			//printf("path end node = %d\n" , contour_map_road_paths[path_loop].to); 

			if(((bridges[bridge_loop].start_node == contour_map_road_paths[path_loop].from) &&
				(bridges[bridge_loop].end_node == contour_map_road_paths[path_loop].to)) ||
				((bridges[bridge_loop].start_node == contour_map_road_paths[path_loop].to) &&
				(bridges[bridge_loop].end_node == contour_map_road_paths[path_loop].from))){

				//printf("****adjust_y_values_on_road_for_bridges if has  been entered\n");

				//printf("* point count = %d\n", contour_map_road_paths[path_loop].count);

				for(point_loop = 0; point_loop < contour_map_road_paths[path_loop].count; point_loop++){

					if(is_between(bridges[bridge_loop].end1, 
						contour_map_road_paths[path_loop].points[point_loop],
						bridges[bridge_loop].end2)){

						if(((bridges[bridge_loop].end1.x == 
							contour_map_road_paths[path_loop].points[point_loop].x) &&
							(bridges[bridge_loop].end1.z ==
							contour_map_road_paths[path_loop].points[point_loop].z)) ||
							
							((bridges[bridge_loop].end2.x == 
							contour_map_road_paths[path_loop].points[point_loop].x) &&
							(bridges[bridge_loop].end2.z ==
							contour_map_road_paths[path_loop].points[point_loop].z))){

							// do nothing
						}
						else{

							//printf("calling adjust_road_y_point\n");

							adjust_road_y_point(bridge_loop, path_loop, point_loop);
						} // end if
					} // end if
				} // end for point loop


			} // end if

		} // end for path loop

	} // end for bridge loop

	fclose (changed_road_pts);
	
} // end adjust_y_values_on_road_for_bridges 

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

int is_between(vec3d br_start, vec3d p, vec3d br_end){

	float
		largest_x,
		smallest_x,
		largest_z,
		smallest_z;

	//printf("is_between has been called\n");

	if(br_start.x >= br_end.x){

		largest_x = br_start.x;
		smallest_x = br_end.x;
	}
	else{

		largest_x = br_end.x;
		smallest_x = br_start.x;
	}

	if(br_start.z >= br_end.z){

		largest_z = br_start.z;
		smallest_z = br_end.z;
	}
	else{

		largest_z = br_end.z;
		smallest_z = br_start.z;
	}

	if(smallest_x <= p.x &&
		p.x <= largest_x &&
		smallest_z <= p.z &&
		p.z <= largest_z){

		return 1;
	}
	else{

		return 0;
	}
} // end is_between

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void adjust_road_y_point(int bridge, int path, int point){

	float
		x,
		x1,
		y,
		y1,
		px,
		py,
		original_y_value,
		new_y_value;

	x = bridges[bridge].end1.x;
	x1 = bridges[bridge].end2.x;
	y = bridges[bridge].end1.y;
	y1 = bridges[bridge].end2.y;

	px = contour_map_road_paths[path].points[point].x;
	original_y_value = contour_map_road_paths[path].points[point].y;

	new_y_value = (px - x) * ((y1 - y) / (x1 - x)) + y;

	contour_map_road_paths[path].points[point].y = new_y_value;

	// remember if bridges.exe has already been run and NEW_ROAD.WP has been renamed
	// to ROAD.WP then you will see points being changed to the same value as the original

	fprintf(changed_road_pts, "changed path %d  point %d  from  %f  to  %f\n",
		path, point, original_y_value, new_y_value);

} // end adjust_bridge_y_point

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void change_path_type_value(void){

	int
		path_loop;

	for(path_loop = 0; path_loop < contour_map_number_of_road_paths; path_loop++){

		contour_map_road_paths[path_loop].type = 41;
	} // end for path_loop
} // end void change_path_type_value(void);

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void write_new_roads_wp_file(void){

	FILE			*fp;

	int
		path_loop,
		point_loop;

	if(( fp = fopen("NEW_ROADS.WP","wb"))==NULL){
		printf("NEW_ROADS.WP\n");
		exit(1);
	}
	else printf("NEW_ROADS.WP opened\n");

	fwrite ( &contour_map_number_of_road_paths, sizeof ( int ), 1, fp );

	for(path_loop = 0; path_loop < contour_map_number_of_road_paths; path_loop++){

		fwrite ( &contour_map_road_paths[path_loop].from, sizeof ( int ), 1, fp );
		fwrite ( &contour_map_road_paths[path_loop].to, sizeof ( int ), 1, fp );
		fwrite ( &contour_map_road_paths[path_loop].type, sizeof ( int ), 1, fp );
		fwrite ( &contour_map_road_paths[path_loop].count, sizeof ( int ), 1, fp );

		for(point_loop = 0; point_loop < contour_map_road_paths[path_loop].count; point_loop++){

			fwrite ( &contour_map_road_paths[path_loop].points[point_loop], sizeof ( vec3d ), 1, fp);

		} // end for point loop

	} // end for path loop

	fclose ( fp );


} // end void write_new_roads_wp_file(void);

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void read_in_bridge_road_file(void){

	FILE
		*road_fp;

	int
		bridge_loop,
		point_loop,
		width,
		color,
		number_of_points_for_this_bridge,
		number_of_quads,
		x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
		z_int_pos,
		return_value;

	double
		x_double_pos,
		z_double_pos;
	
	printf("reading bridges road file\n");

	if(( road_fp = fopen("bridges.ROAD","rb"))==NULL){
		printf("cannot open bridges.ROAD\n");
		exit(1);
	}
	else printf("bridges.ROAD opened\n");

	//////////////////////////////////////////////////////////////////////
	// read in all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fread ( &number_of_bridges, sizeof ( int ), 1, road_fp );

	//printf("number of bridges = %d\n\n", global_number_of_roads);

	for(bridge_loop = 0; bridge_loop < number_of_bridges; bridge_loop++){

		fread ( &width, sizeof ( int ), 1, road_fp );
		fread ( &color, sizeof ( int ), 1, road_fp );
		fread ( &number_of_points_for_this_bridge, sizeof ( int ), 1, road_fp );

		if(number_of_points_for_this_bridge != 2){

			printf("Error: Bridge %d has something other than 2 points\n", bridge_loop);

			exit( 1 );
		}

		fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
		fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

		x_double_pos = (double)x_int_pos / 100.0;
		z_double_pos = (double)z_int_pos / 100.0;

		bridges[bridge_loop].end1.x = (float)x_double_pos;
		bridges[bridge_loop].end1.z = (float)z_double_pos;

		fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
		fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

		x_double_pos = (double)x_int_pos / 100.0;
		z_double_pos = (double)z_int_pos / 100.0;

		bridges[bridge_loop].end2.x = (float)x_double_pos;
		bridges[bridge_loop].end2.z = (float)z_double_pos;

		bridges[bridge_loop].type = 0;

		//roads[road_loop].width = width;

		//roads[road_loop].color = color;
		
		//roads[road_loop].number_of_center_points = number_of_points_for_this_road;

		//roads[road_loop].number_of_quads = number_of_points_for_this_road - 1;

		//for(point_loop = 0; point_loop < number_of_points_for_this_road; point_loop++){

			//fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			//fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			//roads[road_loop].center_points[point_loop].global_x = (double)x_int_pos / 100.0;
			//roads[road_loop].center_points[point_loop].global_z = (double)z_int_pos / 100.0;
		//}
	}
	
	fclose ( road_fp );

} // end read_in_bridge_road_file



//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void load_3d_terrain (void){

	FILE
		*sec_in,
		*ffp_in,
		*rgb_in;

	int
		sec_x,
		sec_z,
		offset,
		point_ref_offset,
		polygons_offset,
		surface_changes_offset,
		normal_changes_offset,
		four_byte_zero_word,
		i,
		print_loop,		
		surface_changes_index,
		poly_loop,
		number_of_polys;

	float
		min_height,
		max_height;
	
	unsigned short int
		map_width,
		map_height,
		sec_number_of_points,
		sec_number_of_polys,
		sec_radius;
		//normal_changes_word;
	
	short int
		sec_min_height,
		sec_max_height,
		sec_avg_height;

	unsigned char
		pad_byte_zero,
		current_surface_id,
		normal_changes_buf[1024];
	
	long
		current_file_position;


	if(( sec_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.sec","rb"))==NULL){
		printf("cannot open default.sec\n");
		exit(1);
	}
	else printf("default.sec opened\n");


	if(( ffp_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\terrain.ffp","rb"))==NULL){
		printf("cannot open terrain.ffp\n");
		fclose(sec_in);
		exit(1);
	}
	else printf("terrain.ffp opened\n");


	if(( rgb_in = fopen("C:\\Razorworks\\common\\maps\\map31\\terrain\\default.rgb","rb"))==NULL){
		printf("cannot open default.rgb\n");
		exit(1);
	}
	else printf("default.rgb opened\n");


	////////////////////////////////////////////////////////
	// read in map dimensions of sector file
	////////////////////////////////////////////////////////

	fread(&terrain_version, sizeof(int), 1, sec_in);
	
	fread(&map_width, sizeof(unsigned short int), 1, sec_in);
	
	fread(&map_height, sizeof(unsigned short int), 1, sec_in);
	
	fread(&terrain_3d_map_minimum_height, sizeof(float), 1, sec_in);
	
	fread(&terrain_3d_map_maximum_height, sizeof(float), 1, sec_in);

	terrain_3d_map_width = map_width;
	terrain_3d_map_height = map_height;

	terrain_3d_map_scaled_height_difference = 
		( terrain_3d_map_maximum_height - terrain_3d_map_minimum_height ) * TERRAIN_3D_Y_SCALE;


	////////////////////////////////////////////////////////
	// allocate memory for sectors
	////////////////////////////////////////////////////////

	printf("Allocating memory for sectors ...\n");

	//terrain_sectors = malloc ( terrain_3d_map_height * sizeof ( terrain_3d_sector * ) );

	terrain_sectors = malloc ( map_height * sizeof ( terrain_3d_sector * ) );

	for(sec_z = 0; sec_z < map_height; sec_z ++){

		if( ! (terrain_sectors[sec_z] = malloc( map_width * sizeof(terrain_3d_sector)))){

			printf("No memory\n");

			exit(1);

		}

	} // end for

	printf("Memory allocated ...\n");
	
	//////////////////////////////////////////////////////////
	// read in the sector data and geometry data,
	// the sector file pointer is at byte 16, which is
	// the start of the individual sector data
	//////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////
	// read in the 1st 4 bytes of ffp file
	//////////////////////////////////////////////////////////

	fread(&ffp_4byte_header, sizeof(int), 1, ffp_in);

	//////////////////////////////////////////////////////////
	// read in 1st 4 bytes of default.rgb file
	//////////////////////////////////////////////////////////

	fread(&rgb_4byte_header, sizeof(int), 1, rgb_in);
	
	for( sec_z = 0 ; sec_z < map_height ; sec_z ++){
		for( sec_x = 0 ; sec_x < map_width ; sec_x ++){
		
			/////////////////////////////////////////////////////////////////
			// read in all the stats for each sector
			/////////////////////////////////////////////////////////////////

			fread(&terrain_sectors[sec_z][sec_x].number_of_points,
				sizeof(unsigned short int), 1, sec_in);

			//fread(&pad_word_1536, sizeof(short int), 1, sec_in);
			fread(&terrain_sectors[sec_z][sec_x].pad_2_bytes, sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].number_of_polygons,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].radius,
				sizeof(unsigned short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].minimum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].maximum_height,
				sizeof(short int), 1, sec_in);

			fread(&terrain_sectors[sec_z][sec_x].average_height,
				sizeof(short int), 1, sec_in);

			fread(&pad_word_zero, sizeof(short int), 1, sec_in);

			///////////////////////////////////////////////////////////////////
			// read in the geometry data
			///////////////////////////////////////////////////////////////////

			///////////////////////////////////////////////////////////////////
			// points_y
			///////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file


			if(!(terrain_sectors[sec_z][sec_x].points_y = malloc(sizeof(terrain_3d_point_y) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_y sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf ( "terrain_sectors[%i][%i].points_y is %p\n", 
			// z, x, terrain_sectors[sec_z][sec_x].points_y );
			//printf("points y fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_y,sizeof(terrain_3d_point_y),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

			/////////////////////////////////////////////////////////////////////
			// points_xz
			/////////////////////////////////////////////////////////////////////

			fread(&offset, sizeof(int), 1, sec_in); // sector file

			if(!(terrain_sectors[sec_z][sec_x].points_xz = malloc(sizeof(terrain_3d_point_xz) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for points_xz sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("points xz fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].points_xz,sizeof(terrain_3d_point_xz),
				terrain_sectors[sec_z][sec_x].number_of_points, ffp_in);

					
			////////////////////////////////////////////////////////////////////////////
			// here need to use a different method of finding size of data because
			// the number of point_references isn't known
			////////////////////////////////////////////////////////////////////////////


			fread(&point_ref_offset, sizeof(int), 1, sec_in); // sector file
			fread(&polygons_offset, sizeof(int), 1, sec_in); // sector file
			fread(&surface_changes_offset, sizeof(int), 1, sec_in); // sector file
			fread(&normal_changes_offset, sizeof(int), 1, sec_in); // sector file

			/////////////////////////////////////////////////////////////////////
			// point references
			/////////////////////////////////////////////////////////////////////
			
			if(terrain_sectors[sec_z][sec_x].number_of_points <= 256){

				if(!(terrain_sectors[sec_z][sec_x].point_byte_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_byte_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point byte fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_byte_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_byte_reference);
			}
			else{

				if(!(terrain_sectors[sec_z][sec_x].point_word_references = 
					malloc(polygons_offset - point_ref_offset))){
					printf("No memory for point_word_references sector %d x %d\n", sec_z, sec_x);
					exit(1);
				}

				//printf("point word fp = %d\n", (int)ftell(ffp_in));

				fread(terrain_sectors[sec_z][sec_x].point_word_references, 1,
					polygons_offset - point_ref_offset, ffp_in); 

				terrain_sectors[sec_z][sec_x].number_of_point_references = 
					(polygons_offset - point_ref_offset) / 
					sizeof(terrain_3d_point_word_reference);
			}

			/////////////////////////////////////////////////////////////////////
			// polygons
			/////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].polygons = 
				malloc(surface_changes_offset - polygons_offset))){
				printf("No memory for polygons sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			//printf("polygons fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].polygons, sizeof(terrain_3d_face),
				terrain_sectors[sec_z][sec_x].number_of_polygons, ffp_in);

			// if number of polygons is odd need to read an extra byte = 0 at 
			// the end of polygon section

			if(terrain_sectors[sec_z][sec_x].number_of_polygons % 2 != 0){
				fread(&terrain_sectors[sec_z][sec_x].end_pad_byte_polygons, 
					sizeof(pad_byte_zero), 1, ffp_in);
			}

			////////////////////////////////////////////////////////////////////////
			// surface changes - list of surface ids for this sector in order
			////////////////////////////////////////////////////////////////////////

			// there is always a zero pad byte at the end of the surface ids
			// if you are not going to recalculate the number of surface ids
			// for every sector then you need to take this into account

			terrain_sectors[sec_z][sec_x].number_of_surface_ids = 
				normal_changes_offset - surface_changes_offset - 1;
			
			//printf("surface changes fp = %d\n", (int)ftell(ffp_in));

			fread(terrain_sectors[sec_z][sec_x].surface_ids, sizeof(terrain_3d_surface),
				terrain_sectors[sec_z][sec_x].number_of_surface_ids + 1, ffp_in);

			////////////////////////////////////////////////////////////////////////
			// initialize the surface_id_each_face array
			////////////////////////////////////////////////////////////////////////

			if(!(terrain_sectors[sec_z][sec_x].surface_id_each_face = 
				malloc(sizeof(terrain_3d_surface) * terrain_sectors[sec_z][sec_x].number_of_polygons))){
				printf("No memory for surface_id_each_face sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			number_of_polys = terrain_sectors[sec_z][sec_x].number_of_polygons;

			current_surface_id = terrain_sectors[sec_z][sec_x].surface_ids[0].surface_id;

			terrain_sectors[sec_z][sec_x].surface_id_each_face[0].surface_id = current_surface_id;
			
			surface_changes_index = 0;

			for(poly_loop = 1; poly_loop < number_of_polys; poly_loop ++){

				if(terrain_sectors[sec_z][sec_x].polygons[poly_loop].surface_change == 1){

					surface_changes_index ++;

					current_surface_id = terrain_sectors[sec_z][sec_x].
						surface_ids[surface_changes_index].surface_id;

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;
				}
				else{

					terrain_sectors[sec_z][sec_x].
						surface_id_each_face[poly_loop].surface_id =
						current_surface_id;

				} // end if else
			} // end for poly loop

			////////////////////////////////////////////////////////////////////////
			// normal changes
			////////////////////////////////////////////////////////////////////////

			fread(&normal_changes_word, sizeof(normal_changes_word), 1, ffp_in);

			// read in color offsets from sector file

			fread(&terrain_sectors[sec_z][sec_x].colour_indices_offset,	sizeof(int), 1, sec_in); // sector file
			fread(&terrain_sectors[sec_z][sec_x].point_colours_offset, sizeof(int), 1, sec_in); // sector file

			////////////////////////////////////////////////////////////////////////////
			// read in actual color data from the default.rgb file
			////////////////////////////////////////////////////////////////////////////


			if(!(terrain_sectors[sec_z][sec_x].colour_indices = 
				malloc(sizeof(terrain_3d_colour_index) * 
				terrain_sectors[sec_z][sec_x].number_of_points))){
				printf("No memory for colour_indices sector %d x %d\n", sec_z, sec_x);
				exit(1);
			}

			fread(terrain_sectors[sec_z][sec_x].colour_indices, sizeof(terrain_3d_colour_index),
				terrain_sectors[sec_z][sec_x].number_of_points, rgb_in);

			// find the max colour index (number of colours-1 used) for this sector

			terrain_sectors[sec_z][sec_x].max_colour_index = 0;

			for(i = 0; i < terrain_sectors[sec_z][sec_x].number_of_points; i++){ 
				
				// there is a colour index for each point

				if((terrain_sectors[sec_z][sec_x].colour_indices + i)->index > 
					terrain_sectors[sec_z][sec_x].max_colour_index){

					terrain_sectors[sec_z][sec_x].max_colour_index = 
						(terrain_sectors[sec_z][sec_x].colour_indices + i)->index;
				}
			} // end for

			// read in the colours
			
			fread(terrain_sectors[sec_z][sec_x].point_colours, sizeof(terrain_3d_colour),
				terrain_sectors[sec_z][sec_x].max_colour_index + 1, rgb_in);


		} // end for x loop
	} // end for z loop

	// read in the last four bytes of sector file (four bytes of zeros)

	fread(&four_byte_zero_word, sizeof(int), 1, sec_in);

	fclose(ffp_in);
	fclose(sec_in);
	fclose(rgb_in);

	printf("3d terrain data read in ...\n");


} // end load 3d terrain

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

// read in map.ROAD file
//
// Data Format
//
// 4 bytes - number of roads
// loop number of roads
//    4 bytes width
//    4 bytes color
//    4 bytes number of points for this road
//    loop number of points
//        x global ( Rob Fox uses ints for floats (int/100 = float))
//        z global

void read_in_ROAD_file(){

	FILE
		*road_fp;

	int
		road_loop,
		point_loop,
		width,
		color,
		number_of_points_for_this_road,
		number_of_quads,
		x_int_pos,						// Rob Fox uses ints for floats (int/100 = float)
		z_int_pos,
		return_value;
	
	printf("reading road file\n");

	if(( road_fp = fopen("C:\\aGrand\\Grand.ROAD","rb"))==NULL){
		printf("cannot open Grand.ROAD\n");
		exit(1);
	}
	else printf("Grand.ROAD opened\n");

	//////////////////////////////////////////////////////////////////////
	// read in all the road data which is x and z data only in world coords
	//////////////////////////////////////////////////////////////////////

	fread ( &global_number_of_roads, sizeof ( int ), 1, road_fp );

	//printf("number of roads = %d\n\n", global_number_of_roads);

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop++){

		fread ( &width, sizeof ( int ), 1, road_fp );
		fread ( &color, sizeof ( int ), 1, road_fp );
		fread ( &number_of_points_for_this_road, sizeof ( int ), 1, road_fp );

		roads[road_loop].width = width;

		roads[road_loop].color = color;
		
		roads[road_loop].number_of_center_points = number_of_points_for_this_road;

		roads[road_loop].number_of_quads = number_of_points_for_this_road - 1;

		for(point_loop = 0; point_loop < number_of_points_for_this_road; point_loop++){

			fread ( &x_int_pos, sizeof ( int ), 1, road_fp );
			fread ( &z_int_pos, sizeof ( int ), 1, road_fp );

			roads[road_loop].center_points[point_loop].global_x = (double)x_int_pos / 100.0;
			roads[road_loop].center_points[point_loop].global_z = (double)z_int_pos / 100.0;
		}
	}
	
	fclose ( road_fp );

} // end read_in_ROAD_file()

////////////////////////////////////
////////////////////////////////////
//                                //
//                                //
//                                //
////////////////////////////////////
////////////////////////////////////

void convert_ROAD_data_to_eech_road_data(void){

	int
		road_loop,
		point_loop,
		number_of_points_for_this_road;

	printf("converting Grand.ROAD data to EECH road data\n");

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop++){

		number_of_points_for_this_road = roads[ road_loop ].number_of_center_points;

		//
		// node data
		//

		start_road_nodes [ road_loop ].x = 
			(float)roads[ road_loop ].center_points[ 0 ].global_x;

		start_road_nodes [ road_loop ].z = 
			(float)roads[ road_loop ].center_points[ 0 ].global_z;

		start_road_nodes [ road_loop ].y = 
			get_global_y_value ( start_road_nodes [ road_loop ] );

		end_road_nodes [ road_loop ].x = (float)roads[ road_loop ].
			center_points[ number_of_points_for_this_road - 1 ].global_x;

		end_road_nodes [ road_loop ].z = (float)roads[ road_loop ].
			center_points[ number_of_points_for_this_road - 1 ].global_z;

		end_road_nodes [ road_loop ].y = 
			get_global_y_value ( end_road_nodes [ road_loop ] );

		//
		// path data
		//

		road_paths [ road_loop ].from = road_loop * 2;

		road_paths [ road_loop ].to = road_loop * 2 + 1;

		road_paths [ road_loop ].type = 41;

		road_paths [ road_loop ].count = number_of_points_for_this_road - 2;

		for(point_loop = 1; point_loop < number_of_points_for_this_road - 1; point_loop ++){

			road_paths [ road_loop ].points [ point_loop - 1 ].x = 
				(float)roads[ road_loop ].center_points[ point_loop ].global_x;

			road_paths [ road_loop ].points [ point_loop - 1 ].z = 
				(float)roads[ road_loop ].center_points[ point_loop ].global_z;

			road_paths [ road_loop ].points [ point_loop - 1 ].y = 
				get_global_y_value ( road_paths [ road_loop ].points [ point_loop - 1 ] );


		} // end for point loop
	} // end road loop


} // end convert_ROAD_data_to_eech_road_data()

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

void convert_ROAD_data_to_eech_road_data_with_junctions(void){

	int
		node_loop,
		road_loop,
		path_count,
		point_loop,
		number_of_points_this_road,
		start_node,
		current_node,
		link_count;

	vec3d
		road_start,
		this_point;

	////////////////////////////////////////////////////////
	// get a single list of nodes with no duplicates
	////////////////////////////////////////////////////////

	
	for(road_loop = 0; road_loop < global_number_of_roads; road_loop ++){

		if( node_in_list( start_road_nodes [ road_loop ] ) == -1 ){ // not already in list

			total_road_nodes [ global_total_number_of_nodes ] = start_road_nodes [ road_loop ];

			global_total_number_of_nodes ++;
		}

		if( node_in_list( end_road_nodes [ road_loop ] ) == -1 ){ // not already in list

			total_road_nodes [ global_total_number_of_nodes ] = end_road_nodes [ road_loop ];

			global_total_number_of_nodes ++;
		}

	} // end for node loop

	//////////////////////////////////////////////////
	// print section
	//////////////////////////////////////////////////
/*
	printf("%d nodes with junctions\n\n", global_total_number_of_nodes);

	for(node_loop = 0; node_loop < global_total_number_of_nodes; node_loop ++){

		printf("%f  %f  %f\n",  
			total_road_nodes [ node_loop ].x,
			total_road_nodes [ node_loop ].z,
			total_road_nodes [ node_loop ].y);
	} // end for node loop

	//////////////////////////////////////////////////
	// end print section
	//////////////////////////////////////////////////
*/

	///////////////////////////////////////////////////////////
	// subdivide all road paths to paths from node - to - node
	///////////////////////////////////////////////////////////

	path_count = 0;

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop ++){

		number_of_points_this_road = roads[ road_loop ].number_of_center_points;

		//
		// find the start node
		//

		road_start = start_road_nodes [ road_loop ];

		for(node_loop = 0; node_loop < global_total_number_of_nodes; node_loop ++){

			if(road_start.x == total_road_nodes [ node_loop ].x &&
				road_start.z == total_road_nodes [ node_loop ].z){

				start_node = node_loop;

				break;
			}
		} // end node loop

		////////////////////////////////////////////////////////
		// find the paths
		////////////////////////////////////////////////////////

		current_node = start_node;

		link_count = 0;

		sub_divided_road_paths [ path_count ].from = start_node;

		for(point_loop = 1; point_loop < number_of_points_this_road; point_loop ++){

			this_point = road_paths [ road_loop ].points [ point_loop ];

			if( node_in_list( this_point ) != -1){ // this point is a node
			}
		} // end for point loop

	} // end for road loop
} // end convert_ROAD_data_to_eech_road_data_with_junctions(void)

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

int node_in_list(vec3d p){

	// if point p is in the node list
	//		return the node index number
	// else
	//		return -1

	int
		node_loop;

	for(node_loop = 0; node_loop < global_total_number_of_nodes; node_loop ++){

		if(p.x == total_road_nodes [ node_loop ].x &&
			p.z == total_road_nodes [ node_loop ].z){

			return node_loop;
		}
	} // end for

	return -1;

} // end node_in_list

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

// loaded in ai_route.c 
//
// Data format (ROADS.NDE)
//
// Number of nodes
// list of vec3d positions
//

//
// Data format (ROADS.WP)
//
// Source
// Destination
// Number of links
// List of node_link_data for each position and cost.
//

// also in terrmap.c in a little different format

//
// Data format (ROADS.NDE)
//
// Number of nodes
// list of vec3d positions
//

//
// Data format (ROADS.WP)
// 
// number of paths
//
// From
// To
// Type
// Count
// List of node_link_data for each position and cost. (don't know what cost is, 
//		really just vec3d of all points between nodes)

void write_eech_road_files(void){

	FILE
		*node_fp,
		*path_fp;

	int
		road_loop,
		point_loop,
		count,
		number_of_nodes,
		number_of_paths;

	printf("writing EECH road files\n");

	if(( node_fp = fopen("ROADS.NDE","wb"))==NULL){
		printf("cannot open ROADS.NDE\n");
		exit(1);
	}
	else printf("ROADS.NDE opened\n");

	if(( path_fp = fopen("ROADS.WP","wb"))==NULL){
		printf("cannot open ROADS.WP\n");
		exit(1);
	}
	else printf("ROADS.WP opened\n");

	number_of_nodes = global_number_of_roads * 2;

	number_of_paths = global_number_of_roads;

	fwrite( &number_of_nodes, sizeof (int), 1, node_fp );

	fwrite( &number_of_paths, sizeof (int), 1, path_fp );

	for(road_loop = 0; road_loop < global_number_of_roads; road_loop++){

		count = road_paths [ road_loop ].count;

		fwrite(&start_road_nodes [ road_loop ], sizeof (vec3d), 1, node_fp);
		
		fwrite(&end_road_nodes [ road_loop ], sizeof (vec3d), 1, node_fp); 

		fwrite( &road_paths [ road_loop ].from, sizeof (int), 1, path_fp );

		fwrite( &road_paths [ road_loop ].to, sizeof (int), 1, path_fp );

		fwrite( &road_paths [ road_loop ].type, sizeof (int), 1, path_fp );

		fwrite( &road_paths [ road_loop ].count, sizeof (int), 1, path_fp );

		for(point_loop = 0; point_loop < count; point_loop ++){

			fwrite( &road_paths [ road_loop ].points [ point_loop ],
				sizeof (vec3d), 1, path_fp);

		} // end point loop

	} // end road loop

	fclose ( node_fp );
	fclose ( path_fp );

}// end write_eech_road_files(void)

//////////////////////////
//////////////////////////
//                      //
//                      //
//////////////////////////
//////////////////////////

//
// Data format:
//
// int map size x
// int map size z
// int total number of nodes
// loop 	{
//				node_id,			4 bytes
//				safe_radius			4 bytes
//				number_of_links		4 bytes
//				loop - list of links
//							
//						4 bytes - node
//						4 bytes - cost
//			}
//

void write_eech_road_dat_file(void){

	FILE
		*fp;

	int
		total_number_of_road_nodes,
		node_loop,
		node,
		safe_radius = 28,
		number_of_links = 1,
		node_link,
		cost = 100;

	printf("writing EECH roads.dat file\n");

	if(( fp = fopen("ROADS.DAT","wb"))==NULL){
		printf("cannot open ROADS.DAT\n");
		exit(1);
	}
	else printf("ROADS.DAT opened\n");

	total_number_of_road_nodes = global_number_of_roads * 2;

	fwrite( &terrain_3d_map_width, sizeof (int), 1, fp );

	fwrite( &terrain_3d_map_height, sizeof (int), 1, fp );

	fwrite( &total_number_of_road_nodes, sizeof (int), 1, fp );

	for(node_loop = 0; node_loop < total_number_of_road_nodes; node_loop ++){

		fwrite( &node_loop, sizeof (int), 1, fp );

		fwrite( &safe_radius, sizeof (int), 1, fp );

		fwrite( &number_of_links, sizeof (int), 1, fp );

		//
		// don't need to loop here because all roads only have two nodes
		//

		if(node_loop % 2 == 0){

			node_link = node_loop + 1;
		}
		else{

			node_link = node_loop - 1;
		}

		fwrite( &node_link, sizeof (int), 1, fp );

		fwrite( &cost, sizeof (int), 1, fp );

	} // end for node loop

	fclose(fp);

} // end write_eech_road_dat_file(void)
